<?php
require_once( "massiv.php" );
html_begin( "About" );
?>

<h3> <?php draw_dot(); ?> About the Massiv </h3>

<h5> What is the Massiv? </h5>

<p>
The Massiv is a distributed game middleware whose purpose is to simplify 
the development of distributed persistent massively multiplayer online games.
</p>

<p>
It is an object-oriented distributed system built on top of C++ and its 
standard library. It defines an "extended" object model, in a form of coding 
instructions, that allows the Core to fully manage application objects. This 
includes automatic object life time management (garbage collection), object 
serializability and full object introspection, ability to remote call object
methods and <b>migrate and replicate</b> objects. All these features are
self-contained and implemented by the Massiv core library. A potential 
application developer has only to know that these features are supported and 
how to use them properly. 
</p>

<p>
Besides the object model the Massiv also supports transparent simulation state 
archivation, dynamic download of static data (textures, models) or server load 
balancing.
</p>

<h5> What the Massiv is not </h5>

<p>
The Massiv is not one of those fancy one-purpose game creation toolkits that
could be used directly out of the box. Instead we have developed a generic
distributed programmable environment that allows for a broader range of use, 
not limited to online games only. It does not make the development of such a 
game an easy task but it is much easier to use the Massiv than other generic 
distributed systems.
</p>

<h5> Features </h5>

<ul>
    <li> Client/Server architecture </li>
    <li> Object Model ("managed" objects):
        <ul>
        <li> Classes written in accordance with object model instructions and
            described in IDL </li>
        <li> Support for object introspection, serialization </li>
        <li> Transparent object migration and replication (without users's
            assistance) </li>
        <li> Migration &amp; replication groups driven by persistent object
            references </li>
        <li> Garbage collector </li>
        <li> RPC in various flavours (asynchronous, synchronous, timed, delayed,
            ...) </li>
        <li> Exceptions </li>
        <li> Superior integration with the C++ language</li>
        </ul>
    </li>
    <li>Consistent archivation
        <ul>
        <li> Ability to automatically create a snapshot of the simulation state
          and restore the simulation from the snapshot </li>
        </ul>
    </li>
    <li> Management of static data and their distribution (textures, models)
        <ul>
        <li> Data organized in a tree-like structure </li>
        <li> Versioning </li>
        <li> When data are not available, the latest available version is
            used </li>
        <li> Can be updated on-line </li>
        </ul>
    </li>
    <li> Server load balancing
        <ul>
        <li> CPU &amp; Network traffic load balancing </li>
        </ul>
    </li>
    <li> Network
        <ul>
        <li> RSA authentization, traffic encrypted using a symmetric ciphre </li>
        <li> Optimized protocols designed to save network traffic </li>
        </ul>
    </li>
    <li> Multiplatform portable code
        <ul>
        <li> Support for Windows and Linux platforms </li>
        <li> OS dependent portions of the core library carefully isolated </li>
        </ul>
    </li>
</ul>

<h5> License </h5>

<p>
The Massiv sources and documentation are currently distributed under LGPL. There is
a talk in progress on license change for documentation and supporting a 
BSD-style license for sources.
</p>

<?php html_end(); ?>
