/** @file runin.c
 * Executes program in specified directory.
 * Usage: runin path program [arg1 arg2 ...]
 * Change directory to 'path', execute "program arg1 arg2 ..." and change directory back.
 */

/*
 * begin        : Wed Dec 12 2001
 * copyright    : (C) 2001 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <direct.h>
#include <stdlib.h>
#include <assert.h>
#include <process.h>


static char *           curr_dir;
static const char *     dest_dir;

static int              arg_c;
static const char **    arg_v;  // Command to execute.


void print_usage( void )
    {
    printf( "Usage: runin path program [arg1 arg2 ...]\n" );
    printf( "Change directory to 'path', execute \"program arg1 arg2 ...\" and change directory back.\n" );
    }


void clean_and_exit
    (
    int ret
    )
    {
    if( curr_dir )
        {
        chdir( curr_dir );
        free( curr_dir );
        curr_dir = NULL;
        }

    exit( ret );
    }


void error_exit( void )
    {
    clean_and_exit( 1 );
    }


void get_curr_dir( void )
    {
    assert( !curr_dir );
    curr_dir = getcwd( NULL, 0 );
    if( !curr_dir )
        {
        fprintf( stderr, "Unable to get current directory.\n" );
        error_exit();
        }
    }


void cd_to_dest_dir( void )
    {
    assert( dest_dir );
    if( chdir( dest_dir ) != 0 )
        {
        fprintf( stderr, "Error change directory to \"%s\".\n", dest_dir );
        error_exit();
        }
    }


int exec( void )
    {
    int ret;

    ret = spawnvp( _P_WAIT, arg_v[0], &arg_v[0] );
    if( ret == -1 )
        {
        fprintf( stderr, "Error exec program.\n" );
        error_exit();
        }

    return ret;   
    }


int main
    ( 
    int     argc, 
    char *  argv[]
    )
    {
    int ret;

    if( argc < 3 )
        {
        fprintf( stderr, "Invalid arguments count (%d).\n", argc - 1 );
        print_usage();
        error_exit();
        }

    dest_dir = argv[1];
    arg_c = argc - 2;
    arg_v = argv + 2;

    // Remember current directory.
    get_curr_dir();

    // Change directory to argv[1].
    cd_to_dest_dir();

    // Run argv[2] with argv[3] ... arguments.
    ret = exec();

    // Change directory back and exit.
    clean_and_exit( ret );
    return ret;
    }

