/** @file path_manager.cpp
 * PathManager implementation.
 */

/* 
 * begin        : Sun Nov 23 2003
 * copyright    : (C) 2003 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "data/path_manager.h"
#include "utility/debug.h"
#include "global/global.h"
#include "filesys/native_file_system.h"


namespace Massiv {
namespace Core {

// PathManagerSettings implementation.

PathManager::PathManagerSettings::PathManagerSettings() :
    Settings( "Paths" ),
    login_directory( access_string( "login_directory", "./login" ) ),
    source_data_directory( access_string( "source_data_directory", "./source_data" ) ),
    data_directory( access_string( "data_directory", "./data" ) ),
    archive_directory( access_string( "archive_directory", "./archives" ) ),
    log_directory( access_string( "log_directory", "./log" ) ),
    work_directory( access_string( "work_directory", "." ) )
    {
    }


// PathManager implementation.

#define PM_HOME         "$HOME"
    /**< Path manager home shortcut. */
    

namespace {
    
std::string replace_shortcut
    (
    const std::string & directory_path,
    const std::string & home_directory
    )
    {
    std::string home = PM_HOME;
    std::string temp = directory_path;
    if( temp.find( home ) == 0 )
        {
        temp.replace( 0, home.size(), home_directory );
        }
    return temp;
    }

}
    

PathManager::PathManager()
    {
    // Initialize home_directory attribute.
    home_directory = NativeFileSystem::get_home_directory();

    // Resolve paths.
    login_directory = replace_home_shortcut( path_manager_settings.login_directory );
    source_data_directory = replace_home_shortcut( path_manager_settings.source_data_directory );
    data_directory = replace_home_shortcut( path_manager_settings.data_directory );
    archive_directory = replace_home_shortcut( path_manager_settings.archive_directory );
    log_directory = replace_home_shortcut( path_manager_settings.log_directory );
    work_directory = replace_home_shortcut( path_manager_settings.work_directory );
    }


std::string PathManager::replace_home_shortcut
    (
    const std::string & directory_path
    )
    {
    return replace_shortcut( directory_path, home_directory );
    }


std::string PathManager::resolve_path_to_config
    (
    const std::string & directory_path
    )
    {
    return replace_shortcut
        ( directory_path, NativeFileSystem::get_home_directory() );
    }


const std::string & PathManager::get_home_directory() const
    {
    return home_directory;
    }    


const std::string & PathManager::get_login_directory() const
    {
    return login_directory;
    }


const std::string & PathManager::get_source_data_directory() const
    {
    return source_data_directory;
    }
    

const std::string & PathManager::get_data_directory() const
    {
    return data_directory;
    }


const std::string & PathManager::get_archive_directory() const
    {
    return archive_directory;
    }


const std::string & PathManager::get_log_directory() const
    {
    return log_directory;
    }


const std::string & PathManager::get_work_directory() const
    {
    return work_directory;
    }


} // namespace Massiv::Core
} // namespace Massiv
