/** @file well_known_object_id_database.h
 * Database of well known object ids.
 */

/*
 * begin        : Sun Oct 12 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_WELL_KNOWN_OBJECT_ID_DATABASE_H
#define MASSIV_WELL_KNOWN_OBJECT_ID_DATABASE_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_STATISTICS_H
#include "registry/statistics.h"
#endif

#ifndef MASSIV_OBJECTID_H
#include "pointer/objectid.h"
#endif


namespace Massiv {
namespace Core {


/**
 * Database of well known object ids.
 *
 * @ingroup unclassified
 */
class MASSIV_EXPORTED WellKnownObjectIdDatabase
    {
    NO_COPY_CONSTRUCTOR( WellKnownObjectIdDatabase );
    NO_ASSIGNMENT_OPERATOR( WellKnownObjectIdDatabase );
    
public:

    /**
     * WellKnownObjectIdDatabase settings.
     */
    class WellKnownObjectIdDatabaseSettings : public Settings
        {
    public:

        /**
         * Constructor.
         */
        WellKnownObjectIdDatabaseSettings();

    public:
    
        String &    naming_service_object;
            /**< Object id of naming service object. */

        }; // WellKnownObjectIdDatabaseSettings
        
public:

    /**
     * Constructor.
     */
    WellKnownObjectIdDatabase();
    
public:

    /**
     * Get #ObjectId of naming service object.
     * Returns null object id if naming service object id is not
     * registered in the database.
     */
    ObjectId get_naming_service_object();
    
private:

    WellKnownObjectIdDatabaseSettings settings;
        /**< Settings. */

    }; // class WellKnownObjectIdDatabase


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_WELL_KNOWN_OBJECT_ID_DATABASE_H
