#! /usr/bin/perl -w
#
# Generate metaobjects and object factories.

=head1 NAME

factgen.pl - generate metaobjects and factories from IDL description

=head1 SYNOPSIS

B<factgen.pl>
    S<[ B<-v>|B<--verbose> ]>
    S<[ B<-I>|B<--include> I<include-directory>]>
    I<file.idl>

=head1 Description

Factgen utility is used to generate metaclasses for each managed class defined
in IDL I<file.idl>. Its options are:

=over

=item B<-v> or B<--verbose>

Increase verbosity level. May be used multiple times.

=item B<--include> I<include-directory>

Add I<include-directory> to list of directories to search when "#include" or
"#import" is used in IDL. The B<-I> option does the same.

=back

For given I<file.idl>, the following files will be generated:

=over

=item I<file_rpc.h>

Interfaces of "public" metaclasses. Currently all of them are RPC-related.
Include this header when you want to use Remote pointer to class defined in
the IDL, or in any IDL it includes.

=item I<file_generated.h>

Interfaces of "private" metaclasses, such as MetaObjectImpl and ObjectFactoryImpl
for each class defined in the IDL. This header should never be included by
non-generated sources. Use generic MetaObject interface instead.

=item I<file_generated.cpp>

Implementation of all generated metaclasses.

=back

You should probably never use F<factgen.pl> directly. Use F<genmkgen.pl>
in cooperation with F<mkgen.pl> to generate makefiles that will contain
correct rules and dependencies to generate all metaclass files,
including class lists.

=head1 KNOWN BUGS

None.

=head1 SEE ALSO

Use L<genmkgen.pl> to generate makefiles.

=cut

use strict ;
use warnings ;

use Getopt::Long ;
use FileHandle ;

use FindBin ;
use lib "$FindBin::Bin/lib" ;
use SourceGen ;


###
# Global variables

my $opt_verbose = 0 ;
my @opt_includes = ( ) ;
my $file_name ;
my $parser ;


###
# Main

# Options

Getopt::Long::Configure( "bundling" ) ;
GetOptions(
    'v|verbose+'    => \$opt_verbose,
    'I|include=s'   => \@opt_includes,
) or die "Invalid options, stopped" ;

$file_name = $ARGV[ 0 ] or
    die "No file name specified, stopped" ;

# Create and initialzie parser.

$parser = SourceGen->new() ;
foreach my $include ( @opt_includes ) {
    $parser->add_include_path( $include ) ;
}

# Parse specified file.

$parser->parse( $file_name ) ;

# Generate sources.

$parser->write_generated_header() ;
$parser->write_rpc_header() ;
$parser->write_generated_source() ;


## EOF ##
