=head1 NAME

FileNames.pm - get file names of various files

=head1 SYNOPSIS

    use FileNames ;

    $base_name = FileNames::base( $idl_name ) ;
    $h_name = FileNames::header( $base_name ) ;
    $cpp_name = FileNames::generated_source( $base_name ) ;
    $h_name = FileNames::generated_header( $base_name ) ;
    $h_name = FileNames::rpc_header( $base_name ) ;

    $list_name = FileNames::idl_list() ;
    $list_name = FileNames::class_list() ;
    $cpp_name = FileNames::class_list_source() ;
    $h_name = FileNames::class_list_header() ;

    $gen_name = FileNames::makefile_idl() ;

=head1 DESCRIPTION

Use functions in this modules to get names of various files.

=head1 KNOWN BUGS

None.

=cut

package FileNames ;

use strict ;
use warnings ;


###
# base
sub base( $ ) {
    my $name = shift ;
    $name =~ s|\.[^./]*$|| ;
    return $name ;
}


###
# header
sub header( $ ) {
    my $base = shift ;
    return $base . '.h' ;
}


###
# generated_source
sub generated_source( $ ) {
    my $base = shift ;
    return $base . '_generated.cpp' ;
}


###
# generated_header
sub generated_header( $ ) {
    my $base = shift ;
    return $base . '_generated.h' ;
}


###
# rpc_header
sub rpc_header( $ ) {
    my $base = shift ;
    return $base . '_rpc.h' ;
}


###
# idl_list
sub idl_list {
    'idl.list' ;
}


###
# class_list
sub class_list {
    'class_list.list' ;
}


###
# class_list_source
sub class_list_source {
    'class_list.cpp' ;
}


###
# class_list_header
sub class_list_header {
    'class_list.h' ;
}


###
# makefile_idl
sub makefile_idl {
    'makefile_idl.gen' ;
}


1 ;


__END__
