=head1 NAME

IdlParserEnum - private enum parsing methods of C<IdlParser>.

=cut

package IdlParser ;


###
# _parse_enum
sub _parse_enum {
    my $self = shift ;

    # Skip 'enum'.

    $self->consume_token( 'enum' ) ;

    # Read enum type name and create enum symbol.

    $self->ensure_identifier() ;
    my $name = $self->token_string() ;
    my $enum = $self->_create_new_symbol( 'SymbolEnum', $name ) ;
    $$self{ ENUMS }{ $enum->full_name() } = $enum ;
    $self->_enter_symbol( $enum ) ;
    $self->next_token() ;

    # Read block.

    $self->_parse_enum_block( $enum ) ;

    # Done.

    $self->_finish_symbol( $enum ) ;
    $self->_leave_symbol() ;
}


###
# _parse_enum_block
#   enum - reference to enum symbol
sub _parse_enum_block {
    my $self = shift ;
    my $enum = shift ;

    $self->consume_token( '{' ) ;

    CONSTANT : for ( ; ; ) {
        # Read constant name.

        if ( $self->token_string() eq '}' ) {
            $self->next_token() ;
            last CONSTANT ;
        }
        $self->ensure_identifier() ;
        my $name = $self->token_value() ;

        # Check redefinition.

        my $constant = $enum->get_symbol( $name ) ;
        if ( defined( $constant ) ) {
            $self->error( {
                MESSAGE => "Redefinition of enum constant '$name'." ,
                NO_DIE => 1,
            } ) ;
            $self->error( $self->_error_symbol( $constant, "Previous definition here." ) ) ;
        }

        # Read value (before symbol is defined).

        $self->next_token() ;
        $self->consume_token( '=' ) ;
        my $value = $self->_parse_integer_value() ;

        # Create the symbol.

        $constant = $self->_create_symbol( 'SymbolConstInt', $name ) ;
        $constant->value( $value ) ;

        # End of block?

        my $token = $self->consume_token( ',', '}' ) ;
        if ( $token->token() eq '}' ) {
            last CONSTANT ;
        }
    }
}


1 ;


__END__
