=head1 NAME

IdlParserNamespace - private namespace parsing methods of C<IdlParser>.

=cut

package IdlParser ;


###
# _parse_global
sub _parse_global {
    my $self = shift ;
    $self->dispatch( {
        DISPATCH => {
            'namespace' => \&_parse_namespace,
            'enum' => \&_parse_enum,
            'classattr' => \&_parse_attribute_definition,
            'propattr' => \&_parse_attribute_definition,
            'methodattr' => \&_parse_attribute_definition,
            'attribute' => \&_parse_attribute_default,
            'class' => \&_parse_class,
            'include_header' => \&_parse_include_header,
            ';' => \&_ignore_semicolon,
            '}' => 'end',
        },
        REPEAT => 1,
    } ) ;
}


###
# _parse_namespace
sub _parse_namespace {
    my $self = shift ;

    # Skip 'namespace'.
    
    $self->consume_token( 'namespace' ) ;

    # Enter the namespace.
    
    $self->ensure_identifier() ;
    $self->_enter_namespace( $self->token_value() ) ;
    $self->next_token() ;

    # Eat opening {.

    $self->consume_token( '{' ) ;

    # Parse namespace contents.

    $self->_parse_global() ;

    # Leave the namespace.

    $self->consume_token( '}' ) ;
    $self->_leave_namespace() ;
}


###
# _enter_namespace
# Open given namespace in current namespace.
#   name - namespace to enter
sub _enter_namespace {
    my $self = shift ;
    my $name = shift ;

    my $namespace = $self->_create_symbol( 'SymbolNamespace', $name ) ;
    $self->_enter_symbol( $namespace ) ;
}


###
# _leave_namespace
# Close current namespace.
sub _leave_namespace {
    $_[ 0 ]->_leave_symbol() ;
}


1 ;


__END__
