=head1 NAME

IdlParserType - private methods of C<IdlParser> - Massiv type parsing.

=head1 TODO

Expression parsing.

=cut

package IdlParser ;


###
# _parse_type
sub _parse_type {
    my $self = shift ;

    my $token = $self->token_value() ;

    # Try builtin types.

    if ( $self->token_type eq 'I' ) {
        if ( $token =~ /^(property|value)_array$/ ) {
            return $self->_parse_type_array() ;
        }
        elsif ( $token eq 'bool' ) {
            $self->next_token() ;
            return TypeBool->new() ;
        }
        elsif ( $token eq 'color' ) {
            $self->next_token() ;
            return TypeColor->new() ;
        }
        elsif ( $token =~ /^(multi_)?dictionary$/ ) {
            return $self->_parse_type_dictionary() ;
        }
        elsif ( $token =~ /^event_handle$/ ) {
            $self->next_token() ;
            return TypeEventHandle->new() ;
        }
        elsif ( $token =~ /^float(32|64)$/ ) {
            my $float_type = $token ;
            $self->next_token() ;
            return TypeFloat->new( $float_type ) ;
        }
        elsif ( $token =~ /^floatq(32|64)$/ ) {
            return $self->_parse_type_floatq() ;
        }
        elsif ( $token =~ /^u?int(8|16|32|64)$/ ) {
            return $self->_parse_type_int() ;
        }
        elsif ( $token eq 'orientation' ) {
            $self->next_token() ;
            return TypeOrientation->new() ;
        }
        elsif ( $token =~ /^pair$/ ) {
            return $self->_parse_type_pair() ;
        }
        elsif ( $token =~ /^(strong|weak|remote)_pointer$/ ) {
            return $self->_parse_type_pointer() ;
        }
        elsif ( $token =~ /^(multi_)?set$/ ) {
            return $self->_parse_type_set() ;
        }
        elsif ( $token eq 'string' ) {
            $self->next_token() ;
            return TypeString->new() ;
        }
        elsif ( $token eq 'time' ) {
            $self->next_token() ;
            return TypeTime->new() ;
        }
        elsif ( my( $length ) = ( $token =~ /^vector(2|3)$/ ) ) {
            $self->next_token() ;
            return TypeVector->new( $length ) ;
        }
        elsif ( $token =~ /^vlu?int(8|16|32|64)$/ ) {
            my $int_type = substr( $token, 2 ) ;
            $self->next_token() ;
            return TypeVLInt->new( $int_type ) ;
        }
    }

    # Try class or enum.

    my @symbol_name = $self->_parse_symbol() ;
    my $symbol = $self->_find_symbol( @symbol_name ) ;
    if ( defined( $symbol ) ) {
        if ( ref( $symbol ) eq 'SymbolClass' ) {
            return TypeClass->new( $symbol ) ;
        }
        elsif ( ref( $symbol ) eq 'SymbolEnum' ) {
            return TypeEnum->new( $symbol ) ;
        }
        $self->error( "Symbol '" . join( '::', @symbol_name ) ."' is neither class nor enum." ) ;
    }

    my $found ;
    if ( scalar( @symbol_name ) ) {
        $found = join( '::', @symbol_name ) ;
    }
    else {
        $found = $token ;
    }
    $self->error( "Parse error. '$found' is neither builtin type name nor symbol name." ) ;
}


###
# _parse_type_array
sub _parse_type_array {
    my $self = shift ;

    # Kind.

    my( $kind ) = ( $self->token_value() =~ /^(property|value)/ ) ;
    $self->next_token() ;

    # Element type.

    $self->consume_token( '<' ) ;
    my $element_type = $self->_parse_type() ;

    # Check element type.

    if ( $kind eq 'value' ) {
        unless ( $element_type->is_stype() ) {
            my $idl_type = $element_type->idl_type() ;
            $self->error( "Can't use non-stype '$idl_type' as element type of value_array." ) ;
        }
    }

    # Done.

    $self->consume_token( '>' ) ;
    return TypeArray->new( $kind, $element_type ) ;
}


###
# _parse_type_dictionary
sub _parse_type_dictionary {
    my $self = shift ;

    # Kind.

    my $multi = ( $self->token_value() =~ /^multi_/ ) ;
    $self->next_token() ;

    # Key and value types.

    $self->consume_token( '<' ) ;
    my $key_type = $self->_parse_type() ;
    $self->consume_token( ',' ) ;
    my $value_type = $self->_parse_type() ;
    $self->consume_token( '>' ) ;

    # FIXME: Check key type.

    # Done.

    return TypeDictionary->new( $multi ? 'multi' : 'simple', $key_type, $value_type ) ;
}


###
# _parse_type_floatq
sub _parse_type_floatq {
    my $self = shift ;

    # Type.

    my $float_type = $self->token_value() ;
    $float_type =~ s/q// ;
    $self->next_token() ;

    # Traits.

    $self->consume_token( '<' ) ;
    $self->ensure_string() ;
    my $traits_name = $self->token_value() ;
    $self->next_token() ;
    $self->consume_token( '>' ) ;

    # Done.

    return TypeFloatQ->new( $float_type, $traits_name ) ;
}


###
# _parse_type_int
sub _parse_type_int {
    my $self = shift ;

    # Type.

    my $int_type = $self->token_value() ;
    $self->next_token() ;

    # Bits.

    my $bits ;
    if ( $self->token_string() eq '<' ) {
        $self->next_token() ;
        $bits = $self->_parse_integer_value() ;
        $self->consume_token( '>' ) ;
    }
    else {
        ( $bits ) = ( $int_type =~ /int(\d+)$/ ) ;
    }

    # Done.

    return TypeInt->new( $int_type, $bits ) ;
}


###
# _parse_type_pair
sub _parse_type_pair {
    my $self = shift ;

    $self->next_token() ;

    # First and second types.

    $self->consume_token( '<' ) ;
    my $first_type = $self->_parse_type() ;
    $self->consume_token( ',' ) ;
    my $second_type = $self->_parse_type() ;
    $self->consume_token( '>' ) ;

    # Done.

    return TypePair->new( $first_type, $second_type ) ;
}


###
# _parse_type_pointer
sub _parse_type_pointer {
    my $self = shift ;

    # Kind.

    my( $kind ) = ( $self->token_value() =~ /^(strong|weak|remote)/ ) ;
    $self->next_token() ;

    # Class symbol.

    $self->consume_token( '<' ) ;
    my $class = $self->_parse_and_find_symbol() ;
    if ( ref( $class ) ne 'SymbolClass' ) {
        my $type = $class->type_name() ;
        my $name = $class->full_name() ;
        $self->error( {
            MESSAGE => "Symbol '$type $name' is not a class.",
            NO_DIE => 1,
        } ) ;
        $self->error( $self->_error_symbol( $class, "Declared here." ) ) ;
    }
    $self->consume_token( '>' ) ;

    # Done.

    return TypePointer->new( $kind, $class ) ;
}


###
# _parse_type_set
sub _parse_type_set {
    my $self = shift ;

    # Kind.

    my $multi = ( $self->token_value() =~ /^multi_/ ) ;
    $self->next_token() ;

    # Value type.

    $self->consume_token( '<' ) ;
    my $value_type = $self->_parse_type() ;
    $self->consume_token( '>' ) ;

    # FIXME: Check if value_type is hashable.

    # Done.

    return TypeSet->new( $multi ? 'multi' : 'simple', $value_type ) ;
}


1 ;


__END__
