=head1 NAME

IdlParserValue - private methods of C<IdlParser> - value/expression parsing.

=head1 TODO

Expression parsing.

=cut

package IdlParser ;


###
# _parse_value
#   type - builtin type name or reference to enumeration type symbol
# Parses value of given type.
sub _parse_value {
    my $self = shift ;
    my $type = shift ;

    if ( $type eq 'bool' ) {
        return $self->_parse_boolean_value() ;
    }
    elsif ( $type eq 'int' ) {
        return $self->_parse_integer_value() ;
    }
    elsif ( $type eq 'string' ) {
        return $self->_parse_string_value() ;
    }

    if ( ! ref( $type ) || ( $type->type_name() ne 'enum' ) ) {
        $self->internal_error( "Type '$type' is neither builtin type nor enumeration type." ) ;
    }

    return $self->_parse_enum_value( $type ) ;
}


###
# _parse_boolean_value
# FIXME: allow constant boolean expressions
sub _parse_boolean_value {
    my $self = shift ;
    my $token = $self->consume_token( '1', '0', 'true', 'false' ) ;
    my $token_string = $token->token() ;
    if ( ( $token_string eq '1' ) || ( $token_string eq 'true' ) ) {
        return 1 ;
    }
    return 0 ;
}


###
# _parse_integer_value
# FIXME: allow constant integer expressions
sub _parse_integer_value {
    my $self = shift ;
    my $value = 1 ;

    while ( $self->token_string() eq '-' ) {
        $value *= -1 ;
        $self->next_token() ;
    }

    $self->ensure_integer() ;
    $value *= $self->token_value() ;
    $self->next_token() ;

    return $value ;
}


###
# _parse_string_value
# FIXME: allow concatenated strings.
sub _parse_string_value {
    my $self = shift ;

    $self->ensure_string() ;
    my $value = $self->token_value() ;
    $self->next_token() ;

    return $value ;
}


###
# _parse_enum_value
#   enum - reference to enum symbol
# FIXME: allow constant enum expressions
sub _parse_enum_value {
    my $self = shift ;
    my $enum = shift ;
    
    $self->ensure_identifier() ;
    my $name = $self->token_value() ;
    my $symbol = $enum->get_symbol( $name ) ;
    if ( ! defined( $symbol ) ) {
        my $enum_name = $enum->full_name() ;
        $self->error( {
            MESSAGE => "No '$name' defined in 'enum $enum_name'.",
            NO_DIE => 1,
        } ) ;
        $self->error( $self->_error_symbol( $enum, "'enum $enum_name' defined here." ) ) ;
    }
    if ( $symbol->type_name() ne 'const_int' ) {
        $self->internal_error( "Enumeration constant is not 'const_int'." ) ;
    }

    $self->next_token() ;
    return $symbol->value() ;
}


1 ;


__END__
