=head1 NAME

SymbolConstant - constant value symbol classes

=head1 SYNOPSIS

    use SymbolConstant ;

    ###
    # SymbolConstant

    # parent is SymbolBase

    # Get/set value.
    $x = $value->value() ;
    $value->value( $x ) ;

=cut

{
package SymbolConstant ;

our( @ISA ) ;
@ISA = qw( SymbolBase ) ;


###
# new
sub new {
    my $proto = shift ;
    my $class = ref( $proto ) || $proto ;

    # Allocate the object.

    my $self = $class->SUPER::new( @_ ) ;

    # Add data.

    $$self{ VALUE } = undef ;

    # Bless it.

    return bless( $self, $class ) ;
}


###
# type_name
sub type_name {
    'constant' ;
}


###
# dump
sub dump {
    my $self = shift ;
    my $indent = shift ;
    $self->SUPER::dump( $indent ) ;
    print ' ' x ( $indent + 2 ) . "value = $$self{ VALUE }\n" ;
}


###
# value
sub value {
    my $self = shift ;
    if ( @_ ) { $$self{ VALUE } = shift ; }
    return $$self{ VALUE } ;
}


} # package SymbolConstant


=pod

    ###
    # SymbolConstInt

    # parent is SymbolConstant

=cut

{
package SymbolConstInt ;

our( @ISA ) ;
@ISA = qw( SymbolConstant ) ;


###
# new
sub new {
    my $proto = shift ;
    my $class = ref( $proto ) || $proto ;

    # Allocate the object.

    my $self = $class->SUPER::new( @_ ) ;

    # Bless it.

    return bless( $self, $class ) ;
}


###
# type_name
sub type_name {
    'const_int' ;
}


###
# dump
sub dump {
    my $self = shift ;
    my $indent = shift ;
    $self->SUPER::dump( $indent ) ;
}


} # package SymbolConstInt


1 ;


__END__


=pod

=head1 DESCRIPTION

Package F<SymbolValue.pm> defines multiple constant value symbol types.
Currently only C<SymbolConstInt> is used as a member symbol of C<SymbolEnum>.
In the future more constant types will be defined and it will be possible to
define constants in namespace and class scope and to use them in expressions.

=over

=item SymbolConstant

Generic scalar constant symbol. It's possible to get and set its value.

=item SymbolConstInt

Integral constant symbol.

=back

=head1 KNOWN BUGS

None.

=cut
