/** @file native_file_system.h
 * Native file system abstraction.
 */

/* 
 * begin        : Thu Jun 19 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_NATIVE_FILE_SYSTEM_H
#define MASSIV_NATIVE_FILE_SYSTEM_H

#ifndef MASSIV_FILE_SYSTEM_H
#include "filesys/file_system.h"
#endif

namespace Massiv {
namespace Core {


/** @addtogroup filesys
 * @{
 */

/**
 * Native file system.
 */
class NativeFileSystem : public FileSystem
    {
public:

    /**
     * Call-back used to create FileSystem instance that accesses
     * native file system managed by the OS.
     */
    static FileSystem * instantiate();

    /**
     * Returns user's "home" directory according to customs specific to
     * the underlying OS.
     * May be called before system is properly started. That is why it
     * is static.
     */
    static std::string get_home_directory();

    }; // NativeFileSystem

/** @} */ // addtogroup filesys


} // namespace Massiv::Core
} // namespace Massiv

#endif // MASSIV_NATIVE_FILE_SYSTEM_H
