/** @file common.h
 * Common core stuff.
 * Include this before any header.
 */

/*
 * begin        : 2001
 * copyright    : (C) 2001 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_COMMON_H
#define MASSIV_COMMON_H


#if defined( _ASSERT_H ) || defined( assert )
#error "common.h must be included before assert.h."
#endif // assert.h included


// Define NDEBUG.

#ifdef OPTION_DEBUG
#undef NDEBUG
#elif !defined( NDEBUG )
#define NDEBUG
#endif // OPTION_DEBUG


// Include system-specific stuff.

#ifdef SYSTEM_WIN32
#ifndef MASSIV_COMMON_WIN32_H
#include "global/common_win32.h"
#endif
#endif // SYSTEM_WIN32

#ifdef SYSTEM_UNIX
#ifndef MASSIV_COMMON_UNIX_H
#include "global/common_unix.h"
#endif
#endif // SYSTEM_UNIX


/**
 * Massiv namespace contains all %Massiv library symbols.
 * Almost all symbols are defined in Massiv::Core namespace, there are only few
 * exceptions, such as Massiv::System class.
 */
namespace Massiv {

/**
 * This is a pretty redundant namespace.
 * It exists for weird historical reasongs. Everything could be safely moved
 * into almost-empty parent namespace (Massiv).
 */
namespace Core {

// Define undefined system stuff.

/** @def NORETURN
 * Functions/methods marked as NORETURN never return once called.
 *
 * @ingroup utility
 */
#ifndef NORETURN
#define NORETURN( Type ) Type
#endif

/** @def CURRENT_FUNCTION_NAME
 * Expands to current function name.
 * Default is NULL.
 *
 * @ingroup utility
 */
#ifndef CURRENT_FUNCTION_NAME
#define CURRENT_FUNCTION_NAME reinterpret_cast< const char * >( 0 )
#endif

#ifndef MASSIV_EXPORTED
#define MASSIV_EXPORTED
#endif

/** @def HACK_AUTO_PTR_IS_STRUCT
 * Defined if std::auto_ptr is struct, not class.
 *
 * @ingroup utility
 */
#if !defined( HACK_AUTO_PTR_IS_STRUCT ) && defined( OPTION_STLPORT )
#define HACK_AUTO_PTR_IS_STRUCT
#endif


} // namespace Massiv::Core
} // namespace Massiv


// Include garbage.

#ifndef MASSIV_COMMON_GARBAGE_H
#include "global/common_garbage.h"
#endif


// Include debug.h to define panic and hope.

#ifndef MASSIV_DEBUG_H
#include "utility/debug.h"
#endif


#endif // MASSIV_COMMON_H
