/** @file common_msvc.h
 * Common MSVC stuff.
 */

/*
 * begin        : 2001
 * copyright    : (C) 2001 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_COMMON_MSVC_H
#define MASSIV_COMMON_MSVC_H


namespace Massiv {
namespace Core {

#pragma warning ( disable : 4514 4512 4097 4786 4503 4127 4121 4290 4251 4100 4275 4660 )
/* 4514: unreferenced inline/local function has been removed */
/* 4512: 'class' : assignment operator could not be generated */
/* 4097: typedef-name 'identifier1' used as synonym for class-name 'identifier2' */
/* 4786: 'identifier' : identifier was truncated to 'number' characters in the debug information */
/* 4503: 'identifier' : decorated name length exceeded, name was truncated */
/* 4127: conditional expression is constant */
/* 4121: 'symbol' : alignment of a member was sensitive to packing */
/* 4290: C++ Exception Specification ignored */
/* 4251: 'symbol' needs to have dll-interface to be used by clients class 'class' */
/* 4275: non dll-interface class 'class1' used as base class for dll-interface class 'class2'
/* 4273: inconsistent DLL linkage. dllexport assumed */
/* 4100: 'symbol' : unreferenced formal parameter */
/* 4660: template-class specialization 'identifier' is already instantiated */
#if ( _MSC_VER < 1200 )
    #error "MSVC 6.0 or better required."
#elif ( _MSC_VER == 1200 )
    #if !defined( OPTION_STLPORT )
    #error "STLPort must be used when compiling by MSVC 6.0."
    #endif
#endif // COMPILER_MSVC


#define NORETURN( Type ) __declspec(noreturn) Type


#if ( _MSC_VER >= 1300 )
#define CURRENT_FUNCTION_NAME __FUNCSIG__
#endif


/**
 * MSVC requires that class member specializations are defined in-class.
 * Unfortunaly this is forbidden according to C++ standard (specializations
 * must be defined in namespace scope).
 */
#define HACK_MEMBER_SPECIALIZATIONS_IN_CLASS


//
// Template-related problems under MSVC6:
//
// 1) MSVC 6.0 does not compile function templates correctly:
// it does not distinguish among f<1>(), f<2>() in
// template <int i> int f();
// Workaround: use static methods of class templates instead
//
// 2) Class templates can not reference symbols using :: qualifier
// in their method declaration, if they do so (and it is the last
// method in the class) subsequent <tt>template <int i> class X;</tt> will
// cause internal compiler error.
// To prevent this template classes that reference X::Y
// should be implemented in this way:
// template <....>
// class MyTemplate
//  {
//  typedef X::Y Y;
//  void f( Y & desc );
//  // void f( X::Y & desc ); // breaks the compiler
//  };
// Unfortunatelly this holds for symbols from the std namespace too.
// The only remedy is to either avoid template classes with integral
// arguments or include them earlier (header file order).
//

//
// Namespace-related problems under MSVC6:
//
// 1) "using namespace Blah;" sometimes does not work (symbols will not
// be found)
// 2) "using symbol;" works alone as long as symbol is not a template
//
// However 1) + 2) could be combined together to workaround problem 1):
//  using namespace Blah;
//  using Blah::my_symbol1;
//  using Blah::my_symbol2;
//  using Blah::my_symbol3;
//  ...
//  // Add "using symbol;" for each symbol that compiler incorrectly can
//  // not find although "using namespace Blah;" was specified.
//  // In this case, symbol can refer to a template!
//
// Define this as a macro so that it could be easily reused wherever
// "using namespace Blah;" would be needed.
//

} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_COMMON_MSVC_H
