/** @file constraints.h
 * Global constraints used to check template parameters and similar stuff.
 * All these functions cause compile-time error if they "fail". No interesting
 * code should be generated on "success".
 */

/*
 * begin        : 2001
 * copyright    : (C) 2001 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_CONSTRAINTS_H
#define MASSIV_CONSTRAINTS_H


namespace Massiv {
namespace Core {


/**
 * Checks whether pointer to @c SourceType is default-convertible
 * to pointer to @c TargetType (causes compile-time error if not).
 * Template parameters must be specified explicitly
 * (ie. check_default_cast<Type1, Type2>()).
 *
 * @ingroup utility
 */
template <class TargetType, class SourceType>
inline void check_default_cast()
    {
    TargetType * ptr_check = reinterpret_cast<SourceType *>( 0 );
    (void) ptr_check; // get rid of "unused variable" warning
    }


} // namespace Massiv::Core
} // namespace Massiv


#endif // _CONSTRAINTS_H_
