/** @file global.cpp
 * Global core stuff.
 */

/*
 * begin        : Tue Nov 25 2001
 * copyright    : (C) 2001 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "global/global.h"
#include "database/node_database.h"
#include "time/system_time.h"
#include "logger/logger.h"

namespace Massiv {
namespace Core {


// Global stuff - singletons.
    
Logger *                    Global::g_logger = NULL;
ObjectManager *             Global::g_object_manager = NULL;
GlobalSearchManager *       Global::g_global_search_manager = NULL;
ClassManager *              Global::g_class_manager = NULL;
NodeManager *               Global::g_node_manager = NULL;
NodeActionManager *         Global::g_node_action_manager = NULL;
ArchiveManager *            Global::g_archive_manager = NULL;
SystemEventQueue *          Global::g_system_event_queue = NULL;
TimeManager *               Global::g_time_manager = NULL;
DataManager *               Global::g_data_manager = NULL;
VolumeManager *             Global::g_volume_manager = NULL;
ThreadManager *             Global::g_thread_manager = NULL;
SemaphoreManager *          Global::g_semaphore_manager = NULL;
CriticalSectionManager *    Global::g_critical_section_manager = NULL;
StackTracer *               Global::g_stack_tracer = NULL;
FileSystem *                Global::g_native_file_system = NULL;
FileAckManager *            Global::g_file_ack_manager = NULL;
ReplicaManager *            Global::g_replica_manager = NULL;
Network *                   Global::g_network = NULL;
Registry *                  Global::g_registry = NULL;
NodeDatabase *              Global::g_node_database = NULL;
ArchiveDatabase *           Global::g_archive_database = NULL;
WellKnownObjectIdDatabase * Global::g_well_known_object_id_database = NULL;
SystemTime                  Global::g_system_time = 0;
bool                        Global::g_replica_writes_allowed = false;
bool                        Global::g_log_exceptions = false;
bool                        Global::g_print_stack_traces = false;
AccountManager *            Global::g_account_manager = NULL;
PathManager *               Global::g_path_manager = NULL;
SystemContext *             Global::g_system_context = NULL;

// Cached pointers.

GarbageCollector *          Global::g_gc = NULL;
ObjectProxyManager *        Global::g_proxy_manager = NULL;
ReplicaServer *             Global::g_replica_server = NULL;
const ReplicaTime *         Global::g_replica_time_ptr = NULL;
Balancer *                  Global::g_balancer = NULL;
Property *                  Global::g_stack_root_object = NULL;

// FOR TESTING DATA MANAGEMENT
DataManager *               Global::g_data_manager_test_server = NULL;
DataManager *               Global::g_data_manager_test_client = NULL;


bool Global::log
    (
    const LogMessage & message
    )
    {
    return logger().log( message );
    }


bool Global::log
    (
    const Status &      status,
    const std::string & message
    )
    {
    return logger().log( LogMessage( status, message ) );
    }


bool Global::log_debug
    (
    Status::Facility    facility,
    Status::Priority    priority,
    const std::string & message
    )
    {
    return Global::log( STATUS_DEBUG( facility, priority ), message );
    }


bool Global::log_info
    (
    Status::Facility    facility,
    Status::Priority    priority,
    const std::string & message
    )
    {
    return Global::log( STATUS_INFO( facility, priority ), message );
    }


bool Global::log_warning
    (
    Status::Facility    facility,
    Status::Priority    priority,
    const std::string & message
    )
    {
    return Global::log( STATUS_WARNING( facility, priority ), message );
    }


bool Global::log_error
    (
    Status::Facility    facility,
    const std::string & message
    )
    {
    return Global::log( STATUS_ERROR( facility ), message );
    }


NodeId Global::get_local_node_id()
    {
    // We can't cache the node id, because it may be changed at run-time.
    NodeDatabase & node_database = Global::node_database();
    return node_database.get_local_node_id();
    }


} // namespace Massiv::Core
} // namespace Massiv


#ifndef ENABLE_INLINE
#include "global_inline.h"
#endif
