/** @file global_inline.h
 * Global core stuff (inline functions)
 */

/*
 * begin        : Tue Nov 25 2001
 * copyright    : (C) 2001 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_LOGGER_H
#include "logger/logger.h"
#endif

#ifndef MASSIV_SYSTEM_TIME_H
#include "time/system_time.h"
#endif

namespace Massiv {
namespace Core {


bool Global::logger_available()
    {
    return ( g_logger ? true : false );
    }


Logger & Global::logger()
    {
    hope( g_logger );
    return *g_logger;
    }


ObjectManager & Global::object_manager()
    {
    hope( g_object_manager );    
    return *g_object_manager;
    }
    
    
GlobalSearchManager & Global::global_search_manager()
    {
    hope( g_global_search_manager );    
    return *g_global_search_manager;
    }
    
    
ClassManager & Global::class_manager()
    {
    hope( g_class_manager );    
    return *g_class_manager;
    }


NodeManager & Global::node_manager()
    {
    hope( g_node_manager );    
    return *g_node_manager;
    }


NodeActionManager & Global::node_action_manager()
    {
    hope( g_node_action_manager );
    return *g_node_action_manager;
    }


ArchiveManager & Global::archive_manager()
    {
    hope( g_archive_manager );    
    return *g_archive_manager;
    }

SystemEventQueue & Global::system_event_queue()
    {
    hope( g_system_event_queue );    
    return *g_system_event_queue;
    }
    

TimeManager & Global::time_manager()
    {
    hope( g_time_manager );    
    return *g_time_manager;
    }


Network & Global::network()
    {
    hope( g_network );    
    return *g_network;
    }


Registry & Global::registry()
    {
    hope( g_registry );    
    return *g_registry;
    }


NodeDatabase & Global::node_database()
    {
    hope( g_node_database );
    return *g_node_database;
    }


ArchiveDatabase & Global::archive_database()
    {
    hope( g_archive_database );
    return *g_archive_database;
    }


WellKnownObjectIdDatabase & Global::well_known_object_id_database()
    {
    hope( g_well_known_object_id_database );
    return *g_well_known_object_id_database;
    }

DataManager & Global::data_manager()
    {
    hope( g_data_manager );
    return *g_data_manager;
    }


AccountManager & Global::account_manager()
    {
    hope( g_account_manager );
    return *g_account_manager;
    }


PathManager & Global::path_manager()
    {
    hope( g_path_manager );
    return *g_path_manager;
    }


ThreadManager & Global::thread_manager()
    {
    hope( g_thread_manager );
    return *g_thread_manager;
    }


CriticalSectionManager & Global::critical_section_manager()
    {
    hope( g_critical_section_manager );
    return *g_critical_section_manager;
    }


SemaphoreManager & Global::semaphore_manager()
    {
    hope( g_semaphore_manager );
    return *g_semaphore_manager;
    }


StackTracer & Global::stack_tracer()
    {
    hope( g_stack_tracer );
    return *g_stack_tracer;
    }


bool Global::stack_tracer_available()
    {
    return g_stack_tracer != NULL;
    }


FileSystem & Global::native_file_system()
    {
    hope( g_native_file_system );
    return *g_native_file_system;
    }


VolumeManager & Global::volume_manager()
    {
    hope( g_volume_manager );
    return *g_volume_manager;
    }


FileAckManager & Global::file_ack_manager()
    {
    hope( g_file_ack_manager );    
    return *g_file_ack_manager;
    }


ReplicaManager & Global::replica_manager()
    {
    hope( g_replica_manager );    
    return *g_replica_manager;
    }


ReplicaServer & Global::replica_server()
    {
    hope( g_replica_server );
    return *g_replica_server;
    }


SystemContext & Global::system_context()
    {
    hope( g_system_context );
    return *g_system_context;
    }


DataManager & Global::data_manager_test_server()
    {
    hope( g_data_manager_test_server );
    return *g_data_manager_test_server;
    }


DataManager & Global::data_manager_test_client()
    {
    hope( g_data_manager_test_client );
    return *g_data_manager_test_client;
    }


const SystemTime & Global::system_time()
    {
    g_system_time = SystemTime::current_time();
    return g_system_time;
    }


ObjectProxyManager & Global::object_proxy_manager()
    {
    hope( g_object_manager );
    hope( g_proxy_manager );

    return *g_proxy_manager;
    }


GarbageCollector & Global::garbage_collector()
    {
    hope( g_object_manager );
    hope( g_gc );

    return *g_gc;
    }


Balancer & Global::balancer()
    {
    hope( g_object_manager );
    hope( g_balancer );

    return *g_balancer;
    }


bool Global::replica_writes_allowed_hack()
    {
    return g_replica_writes_allowed;
    }


Property * Global::get_stack_root_object()
    {
    hope( g_object_manager );
    hope( g_stack_root_object );

    return g_stack_root_object;
    }

    
void Global::allow_replica_writes
    (
    bool allow
    )
    {
    g_replica_writes_allowed = allow;
    }


const ReplicaTime & Global::replica_time()
    {
    hope( g_object_manager );
    hope( g_replica_time_ptr );

    return *g_replica_time_ptr;
    }

} // namespace Massiv::Core
} // namespace Massiv
