/** @file logdest_cerr.h
 * Console error output log destination.
 */

/*
 * begin        : Thu Mar 06 2003
 * copyright    : (C) 2003 by Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_LOGDEST_CERR_H
#define MASSIV_LOGDEST_CERR_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_LOGDEST_H
#include "logger/logdest.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup logger
 * @{
 */

/**
 * Console error output destination. Destination name is @c cerr.
 */
class MASSIV_EXPORTED LogDestinationCerr : public LogDestination
    {
public:

    /**
     * Constructor.
     */
    LogDestinationCerr();

    /**
     * Destructor.
     */
    ~LogDestinationCerr();

public:

    /**
     * Gets short name describing destination. 
     * @return <tt>"cerr"</tt>
     */
    std::string get_name() const;

    /**
     * Gets short textual description of destination.
     */
    std::string get_description() const;

protected:

    /**
     * Matches the destination name.
     * @return true if the name is <tt>"cerr"</tt>.
     */
    virtual bool match_name
        (
        const std::string & name
        );

    /**
     * Logs message (writes the message to the error output).
     */
    virtual void log
        (
        const std::string & name,
        const LogMessage &  message
        );

    /**
     * Flushes output stream.
     */
    virtual void flush();
    
    /**
     * Flushes output stream hard.
     * @see LogDestination::flush_hard()
     */
    virtual void flush_hard();

    }; // class LogDestinationCerr

/** @} */ // addtogroup logger


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_LOGDEST_CERR_H
