/** @file logdest_cout.h
 * Console output log destination.
 */

/*
 * begin        : Tue Nov 25 2001
 * copyright    : (C) 2001 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_LOGDEST_COUT_H
#define MASSIV_LOGDEST_COUT_H


#ifndef MASSIV_LOGGER_H
#include "logger/logger.h"
#endif

#ifndef MASSIV_LOGDEST_H
#include "logger/logdest.h"
#endif



namespace Massiv {
namespace Core {


/** @addtogroup logger
 * @{
 */

/**
 * Console output destination. The destination name is <tt>"cout"</tt>.
 */
class MASSIV_EXPORTED LogDestinationCout : public LogDestination
    {
public:

    /**
     * Constructor.
     */
    LogDestinationCout();

    /**
     * Destructor.
     */
    ~LogDestinationCout();

public:

    /**
     * Gets short name describing the destination. 
     * @returns <tt>"cout"</tt>.
     */
    std::string get_name() const;

    /**
     * Gets short textual description of the destination.
     */
    std::string get_description() const;

protected:

    /**
     * Matches the destination name. 
     * @return true if the @a name matches <tt>"cout"</tt>.
     */
    virtual bool match_name
        (
        const std::string & name
        );

    /**
     * Logs message (writes it into the standard output).
     * @param name Ignored.
     * @param message The message to be logged.
     */
    virtual void log
        (
        const std::string & name,
        const LogMessage &  message
        );
    
    /**
     * Flushes output.
     */
    virtual void flush();
    
    /**
     * Flushes output hard.
     * @see LogDestination::flush_hard
     */
    virtual void flush_hard();

    }; // class LogDestinationCout

/** @} */ // addtogroup logger


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_LOGDEST_COUT_H
