/** @file logmessage.h
 * Log message class.
 */

/*
 * begin        : Tue Nov 6 2001
 * copyright    : (C) 2001 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_LOGMESSAGE_H
#define MASSIV_LOGMESSAGE_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_STATUS_H
#include "status/status.h"
#endif



namespace Massiv {
namespace Core {


/** @addtogroup logger
 * @{
 */

/**
 * Log %message represents a %message that can be processed by the Logger.
 *
 * It consists of Status instance and a textual %string. 
 */
class MASSIV_EXPORTED LogMessage
    {
public:

    /**
     * Constructor.
     */
    LogMessage()
        {
        }

    /**
     * Constructor.
     *
     * @param status Status.
     * @param message Custom %message. It will be combined with status %message 
     *      using Status::get_description(). @a %message may contain newline
     *      characters except the trailing one.
     */
    LogMessage
        (
        const Status &          status,
        const std::string &     message
        ) : status( status ),
            message( message )
        {
        }

    /**
     * Compares messages.
     *
     * @param log_message The %message to be compared by %this one.
     * @return true if the %messages are equal, false otherwise.
     */
    bool operator==
        (
        const LogMessage & log_message
        ) const
        {
        return status == log_message.status && message == log_message.message;
        }

public:
    
    /** @name Setters */
    //@{

    /**
     * Sets status.
     *
     * @param status Status.
     */
    void set_status
        (
        const Status &          status
        )
        {
        this->status = status;
        }

    /**
     * Sets the textual %message to be contained within this %log %message.
     *
     * @param message Custom %message. It will be combined with the status 
     *      %message.
     * @see Status
     */
     void set_message
        (
        const std::string &     message
        )
        {
        this->message = message;
        }

    //@}

public:

    /** @name Getters */
    //@{

    /**
     * Gets the %message string.
     * 
     * @return The string contained withing the log %message.
     */
    std::string get_string() const
        {
        return status.get_description( message );
        }

    //@}

public:

    Status          status;
        /**< Status. */
    std::string     message;
        /**< Message. */
    }; // class LogMessage

/** @} */ // addtogroup logger


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_LOGMESSAGE_H
