/** @file serveraddress_unix.cpp
 * Managing server address.
 */

/*
 * begin        : Thu October 09 2002
 * copyright    : (C) 2002 by Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "global/common.h"
#include "status/exception.h"
#include "net/serveraddress.h"
#include "net/arch/unix/byte_order.h"
#include <sstream>

#include <netdb.h>
#include <sys/socket.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>

namespace Massiv {
namespace Core {


UInt32 ServerAddress::string_to_ip
    (
    const std::string & string
    )
    {
    UInt32 ip = INADDR_NONE;

    ip = inet_addr( string.c_str() );

    if( ip == INADDR_NONE )
        return ( UInt32 ) IP_UNDEFINED;
    else
        return ip;
    }


UInt32 ServerAddress::resolve_ip_address
    (
    const std::string & dns_name
    )
    {
    struct hostent* host_address;
    struct in_addr  inaddr;

    host_address = gethostbyname( dns_name.c_str() );

    if( host_address )
        {
        // IP address resolved successfuly.
        memcpy
            (
            &inaddr.s_addr,
            host_address->h_addr_list[ 0 ],
            host_address->h_length
            );

        return inaddr.s_addr;
        }
    else
        {
        // IP address could not be resolved.
        std::ostringstream  s;
        s << "IP address could not be found for dns name \"" << dns_name << "\"";
        throw ExInvalidDnsName( s.str() );
        }
    }


void ServerAddress::get_ip_fields
    (
    UInt8 &  field1,
    UInt8 &  field2,
    UInt8 &  field3,
    UInt8 &  field4
    )
    {
    UInt8 * aux = ( UInt8 * ) &ip;

    // FIXME: Is not there any special routine that
    //      would make this more portable?

    field1 = *aux++;
    field2 = *aux++;
    field3 = *aux++;
    field4 = *aux++;
    }


UInt32 ServerAddress::assemble_ip
    (
    UInt8    field1,
    UInt8    field2,
    UInt8    field3,
    UInt8    field4
    )
    {
    struct in_addr  adr;
    UInt8 *         aux = ( UInt8 * ) &adr;

    // FIXME: Is not there any special routine that
    //      would make this more portable?

    *aux++ = ( UInt8 ) field1;
    *aux++ = ( UInt8 ) field2;
    *aux++ = ( UInt8 ) field3;
    *aux++ = ( UInt8 ) field4;

    return adr.s_addr;
    }


UInt16 ServerAddress::host_to_network_short
    (
    UInt16  hostshort
    )
    {
    return ByteOrder::host_to_network_16( hostshort );
    }


UInt16 ServerAddress::network_to_host_short
    (
    UInt16  netshort
    )
    {
    return ByteOrder::network_to_host_16( netshort );
    }


} // namespace Massiv::Core
} // namespace Massiv
