/** @file socket_unix.h
 * Unix sockets.
 */

/*
 * begin        : Mon May 20 2002
 * copyright    : (C) 2002 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SOCKET_UNIX_H
#define MASSIV_SOCKET_UNIX_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_SOCKET_H
#include "net/socket.h"
#endif

#include <sys/socket.h>
#include <arpa/inet.h>

namespace Massiv {
namespace Core {


/** @addtogroup net
 * @{
 */

/**
 * TCP socket implementation.
 */
class TCPSocketUnix : public TCPSocket
    {
public:

    /**
     * Create unbound socket.
     * Either @c connect() or @c start_server should be called then.
     */
    TCPSocketUnix();

    /**
     * Creates socket object bound to given @c socket handle.
     */
    TCPSocketUnix
        ( 
        int socket
        );

    /**
     * Destructor.
     */
    virtual ~TCPSocketUnix();

protected:
    
    /**
     */
    virtual bool connect
        ( 
        const ServerAddress & address 
        );

    /**
     */
    virtual void close();

    /**
     */
    virtual bool start_server
        ( 
        UInt16  port, 
        int     backlog = 0
        );

    /**
     */
    virtual Socket * accept();

    /**
     */
    virtual bool set_non_block
        (
        bool    non_blocking
        );

    /**
     */
    virtual int get_pending_data_amount();

    /**
     */
    virtual int internal_send
        (
        const UInt8 *   buf,
        int             len
        );
    
    /**
     */
    virtual int internal_receive
        (
        UInt8 * buf,
        int     len
        );

private:

    int sock;
        /**< Unix socket handle. */

    };  // class TCPSocketUnix


/**
 * UDP socket implementation.
 */
class UDPSocketUnix : public UDPSocket
    {
public:
    
    /**
     * Default constructor.
     */
    UDPSocketUnix();

    /**
     * Binds socket object to a given @c socket handle.
     */
    UDPSocketUnix
        ( 
        int socket
        );

    /**
     * Destructor.
     */
    virtual ~UDPSocketUnix();

protected:

    /**
     */
    virtual UInt16 get_bound_port();

    /**
     */
    virtual void set_destination_address
        (
        const ServerAddress & sa
        );
    
    /**
     */
    virtual bool connect
        ( 
        const ServerAddress & address 
        );

    /**
     */
    virtual void close();

    /**
     */
    virtual bool start_server
        ( 
        UInt16  port, 
        int     backlog = 0
        );

    /**
     */
    virtual bool set_non_block
        (
        bool    non_blocking
        );

    /**
     */
    virtual int get_pending_data_amount();

    /**
     */
    virtual int internal_send
        (
        const UInt8 *   buf,
        int             len
        );
    
    /**
     */
    virtual int internal_receive
        (
        UInt8 * buf,
        int     len
        );

private:

    sockaddr_in destination_address;
        /**< Address to send data to. */
    bool        udp_connected;
        /**< True if @c destination_address was filled. */
    int         sock;
        /**< Unix socket handle. */

    };  // class UDPSocketUnix

/** @} */ // addtogroup net


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_SOCKET_UNIX_H
