/** @file net_init_win32.cpp
 * Network initialization for Win32.
 */

/*
 * begin        : Mon May 20 2002
 * copyright    : (C) 2002 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "net/net.h"
#include "net/arch/win32/socket_win32.h"
#include "logger/logger.h"
#include <winsock2.h>


namespace Massiv {
namespace Core {


void Network::arch_init()
    {
    WSADATA wsd;
    int     result = WSAStartup( 0x0002, &wsd );

    if( result != 0 )
        {
        throw ExNetworkInitError( "WSAStartup() failed." );
        }
    }


void Network::arch_deinit()
    {
    int result = WSACleanup();
    
    if( result != 0 )
        {
        Global::log_debug
            (
            Status::FACILITY_NETWORK,
            Status::PRIORITY_HIGHEST,
            "WSACleanup() failed."
            );
        }
    }


UDPSocket * Network::create_udp_socket()
    {
    return new UDPSocketWin32();
    }


TCPSocket * Network::create_tcp_socket()
    {
    return new TCPSocketWin32();
    }


} // namespace Massiv::Core
} // namespace Massiv
