/** @file serveraddress_win32.cpp
 * Server address (Win32).
 */

/*
 * begin        : Thu October 09 2002
 * copyright    : (C) 2002 by Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "global/common.h"
#include "status/exception.h"
#include "net/serveraddress.h"
#include <winsock2.h>
#include <sstream>

namespace Massiv {
namespace Core {


UInt32 ServerAddress::string_to_ip
    (
    const std::string & string
    )
    {
    UInt32 ip = INADDR_NONE;

    ip = inet_addr( string.c_str() );

    if( ip == INADDR_NONE )
        return IP_UNDEFINED;
    else
        return ip;
    }


UInt32 ServerAddress::resolve_ip_address
    (
    const std::string & dns_name
    )
    {
    struct hostent *    host_address;
    struct in_addr      inaddr;

    host_address = gethostbyname( dns_name.c_str() );

    if( host_address )
        {
        // IP address resolved successfuly.
        
        memcpy
            (
            &inaddr.S_un.S_addr,
            host_address->h_addr_list[ 0 ],
            host_address->h_length
            );

        return inaddr.S_un.S_addr;
        }
    else
        {
        // IP address could not be resolved.
        std::ostringstream  s;
        s << "IP address could not be found for dns name \"" << dns_name << "\"";
        throw ExInvalidDnsName( s.str() );
        }
    }


void ServerAddress::get_ip_fields
    (
    UInt8 &  field1,
    UInt8 &  field2,
    UInt8 &  field3,
    UInt8 &  field4
    )
    {
    IN_ADDR adr;

    adr.S_un.S_addr = ip;

    field1 = adr.S_un.S_un_b.s_b1;
    field2 = adr.S_un.S_un_b.s_b2;
    field3 = adr.S_un.S_un_b.s_b3;
    field4 = adr.S_un.S_un_b.s_b4;
    }


UInt32 ServerAddress::assemble_ip
    (
    UInt8 field1,
    UInt8 field2,
    UInt8 field3,
    UInt8 field4
    )
    {
    IN_ADDR         adr;

    adr.S_un.S_un_b.s_b1 = field1;
    adr.S_un.S_un_b.s_b2 = field2;
    adr.S_un.S_un_b.s_b3 = field3;
    adr.S_un.S_un_b.s_b4 = field4;

    return adr.S_un.S_addr;
    }


UInt16 ServerAddress::host_to_network_short
    (
    UInt16  host
    )
    {
    return htons( host );
    }


UInt16 ServerAddress::network_to_host_short
    (
    UInt16  net
    )
    {
    return ntohs( net );
    }


} // namespace Massiv
} // namespace Core
