/** @file bitistream_inline.h
 * Bit-oriented input stream buffer.
 */

/*
 * begin        : Tue Dec 18 2001
 * copyright    : (C) 2001 by Ondrej Pecta, Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Massiv {
namespace Core {


BitIStream::BitIStream()
    : BitIOStream(), offset( 0 )
    {
    bind( BitIOStreamBuffer(), false );
    }


BitIStream::BitIStream
    (
    const BitIStream & stream
    )
    {
    operator=( stream );
    }


BitIStream::BitIStream
    (
    BitIOStreamBuffer   buf,
    bool                dup,
    SizeType            offset
    ) : BitIOStream()
    {
    hope( offset % 8 == 0 );

    bind( buf, dup, offset );
    }


BitIStream::~BitIStream()
    {
    }


BitIOStreamBuffer BitIStream::unbind()
    {
    return bind( BitIOStreamBuffer(), false );
    }


void BitIStream::rewind()
    {
    bit = 0;
    data = buffer.begin() + offset / 8;
    }


void BitIStream::skip
    (
    SizeType offset
    )
    {
    seek( offset, WHENCE_CURRENT );
    }


BitIOStreamBuffer BitIStream::get_buffer()
    {
    return buffer;
    }


void BitIStream::align_to_byte()
    {
    seek( ( 8 - bit ) & 0x07, WHENCE_CURRENT );
    }


BitIStream::SizeType BitIStream::bit_size() const
    {
    return ( end - buffer.begin() ) * 8 - offset;
    }


BitIStream::SizeType BitIStream::byte_size() const
    {
    return ( bit_size() + 7 ) / 8;
    }


BitIStream::SizeType BitIStream::get_position() const
    {
    return ( data - buffer.begin() ) * 8 - offset + bit;
    }


BitIStream::SizeType BitIStream::count_remaining_bits() const
    {
    return bit_size() - get_position();
    }


} // namespace Massiv::Core
} // namespace Massiv
