/** @file connection_info_impl.h
 * Connection information class inline methods implementation.
 */

/*
 * begin        : Fri Nov 15 2002
 * copyright    : (C) 2002 by Ondrej Pecta, Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_CONNECTION_INFO_IMPL_H
#define MASSIV_CONNECTION_INFO_IMPL_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_GLOBAL_H
#include "global/global.h"
#endif

#ifndef MASSIV_CONNECTION_INFO_H
#include "net/connection_info.h"
#endif

#ifndef MASSIV_NET_H
#include "net/net.h"
#endif

#ifndef MASSIV_SYSTEM_TIME_H
#include "time/system_time.h"
#endif


namespace Massiv {
namespace Core {


inline void ConnectionInfo::set_tcpsocket
    (
    Socket * socket 
    )
    {
    tcpsocket = socket;
    }

        
inline void ConnectionInfo::set_udpsocket
    (
    Socket * socket 
    ) 
    {
    udpsocket = socket;
    }


inline Socket * ConnectionInfo::get_tcpsocket()
    {
    return tcpsocket;
    }
    

inline Socket * ConnectionInfo::get_udpsocket()
    {
    return udpsocket;
    }


inline void ConnectionInfo::set_other_side_public_key
    (
    const std::string & public_key 
    ) 
    {
    other_side_public_key = public_key;
    }


inline std::string & ConnectionInfo::get_other_side_public_key()
    {
    return other_side_public_key;
    }


inline std::string & ConnectionInfo::get_blowfish_key_odd()
    {
    return blowfish_key_odd;
    }


inline std::string & ConnectionInfo::get_blowfish_key_even()
    {
    return blowfish_key_even;
    }


inline void ConnectionInfo::inc_ping_messages_sent()
    {
    ping_messages_sent++;
    }


inline void ConnectionInfo::set_server_side
    (
    bool        server_side
    )
    {
    this->server_side = server_side;
    }


inline bool ConnectionInfo::is_server_side()
    {
    return server_side;
    }


inline void ConnectionInfo::inc_idle_ticks()
    {
    idle_ticks++;
    }


inline void ConnectionInfo::reset_idle_ticks()
    {
    ping_messages_sent = 0;
    idle_ticks = 0;
    }


inline unsigned int ConnectionInfo::get_ping_messages_sent()
    {
    return ping_messages_sent;
    }


inline int ConnectionInfo::get_idle_ticks()
    {
    return idle_ticks;
    }


inline void ConnectionInfo::touch_last_message_time()
    {
    last_message_time = Global::system_time();
    }


inline SystemTime ConnectionInfo::get_last_message_time()
    {
    return last_message_time;
    }


inline unsigned int ConnectionInfo::get_max_idle_seconds()
    {
    return Global::network().settings.max_idle_seconds;
    }


inline SystemTime ConnectionInfo::get_idle_seconds()
    {
    return Global::system_time() - get_last_message_time();
    }


inline unsigned int ConnectionInfo::get_next_ping_delay()
    {
    return Global::network().settings.next_ping_delay;
    }


inline unsigned int ConnectionInfo::get_max_pings()
    {
    return Global::network().settings.max_pings;
    }


inline void ConnectionInfo::touch_blowfish_key_timestamp()
    {
    blowfish_key_timestamp = Global::system_time();
    }


inline SystemTime ConnectionInfo::get_blowfish_key_timestamp()
    {
    return blowfish_key_timestamp;
    }


inline void ConnectionInfo::set_blowfish_key_timestamp
    (
    SystemTime  timestamp_new
    )
    {
    blowfish_key_timestamp = timestamp_new;
    }


inline ConnectionInfo::BlowfishKeyPeriod 
ConnectionInfo::get_blowfish_key_period()
    {
    return blowfish_key_period;
    }


inline BlowfishEncryptor * ConnectionInfo::get_blowfish_encryptor_odd()
    {
    return benc_odd;
    }


inline BlowfishDecryptor * ConnectionInfo::get_blowfish_decryptor_odd()
    {
    return bdec_odd;
    }


inline BlowfishEncryptor * ConnectionInfo::get_blowfish_encryptor_even()
    {
    return benc_even;
    }


inline BlowfishDecryptor * ConnectionInfo::get_blowfish_decryptor_even()
    {
    return bdec_even;
    }


inline const NodeId & ConnectionInfo::get_node_id() const
    {
    return node_id;
    }


inline void ConnectionInfo::set_node_id
    (
    NodeId node
    )
    {
    node_id = node;
    }


inline bool ConnectionInfo::get_permit_auto_close()
    {
    return permit_auto_close;
    }


inline void ConnectionInfo::set_permit_auto_close
    (
    bool auto_close
    )
    {
    permit_auto_close = auto_close;
    }


inline void ConnectionInfo::set_udp_connected
    (
    bool connected
    )
    {
    udp_connected = connected;
    }


inline bool ConnectionInfo::is_udp_connected()
    {
    return udp_connected;
    };


inline void ConnectionInfo::set_tcp_confirmed
    (
    bool confirmed
    )
    {
    tcp_confirmed = confirmed;
    }

    
inline bool ConnectionInfo::is_tcp_confirmed()
    {
    return tcp_confirmed;   
    }

} // namespace Massiv::Core
} // namespace Massiv

#endif  // MASSIV_CONNECTION_INFO_IMPL_H
