/** @file cryptopp_begin.h
 * Include this header before including cryptopp headers. It "detects" the
 * cryptopp library version and disables warnings produced by many cryptopp 
 * headers. Include @c cryptopp_end.h afterwards.
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CRYPTOPP_VERSION

    #include "cryptopp/config.h"

    #ifndef CRYPTOPP_VERSION

    // Attempt to "detect" the used version of cryptopp library. This is required 
    // because its external interfaces have changed substantially since version 
    // 4.2 and the new 5.x interfaces are incompatible with the old ones. Certain 
    // portions of our code are thus written in several variants, depending on 
    // the used version of the library. Once the new version settles down, support
    // for cryptopp 4.2 will be dropped and this hackery removed.
    
        #ifdef CRYPTOPP_API
    
            // This is definitely not a 4.2 version.
            // Assuming 5.0 or newer.
    
            #define CRYPTOPP_VERSION 500
    
        #else
    
            // Assuming 4.2
    
            #define CRYPTOPP_VERSION 420
    
        #endif
        
    #endif // CRYPTOPP_VERSION

#endif // CRYPTOPP_VERSION

#ifdef COMPILER_ICC

// Note: You will still see plenty of warnings when compiling cryptopp sources
// or massiv sources that use cryptopp templates, but not when including
// network headers.

// Set level of some warnings to 4.
// 327: NULL reference is not allowed.
// 654: patrial override of overloaded function.
// 1125: hidden virtual function - override intended?
#pragma warning ( 4 327 654 1125 )

#endif // COMPILER_ICC
