/** @file massiv_blowfish.h
 * Blowfish cipher.
 */

/*
 * begin        : Tue Nov 9 2001
 * copyright    : (C) 2001 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_MASSIV_BLOWFISH_H
#define MASSIV_MASSIV_BLOWFISH_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_BLOCK_CIPHER_H
#include "net/block_cipher.h"
#endif

// CryptoPP
#include "net/cryptopp_begin.h"
#include "cryptopp/blowfish.h"
#include "net/cryptopp_end.h"

#include <string>


namespace Massiv {
namespace Core {


// Forward declaration.
template <class Engine> class BlowfishCipher;


/** @addtogroup net
 * @{
 */

/**
 * Blowfish cipher.
 * 
 * Support for block-oriented encryption. Blocks are processed using a simple 
 * electronic code book (ECB) mode.
 */
template <class Engine>
class MASSIV_EXPORTED BlowfishCipher : public BlockCipher< 8, BlowfishCipher< Engine > >
    {
public:

    typedef std::string Key;

public:

    /**
     * Initializes blowfish engine using the given key.
     */
    BlowfishCipher
        (
        const Key & key
        );

public:

    /**
     * Process block of data.
     * Called by @c BlockCipher.
     */
    void process_block
        (
        const UInt8 *   input,
        UInt8 *         output
        );

private:

    Engine  engine;
        /**< CryptoPP native engine. */

    }; // BlowfishCipher

typedef BlowfishCipher<CryptoPP::BlowfishEncryption> BlowfishEncryptor;
typedef BlowfishCipher<CryptoPP::BlowfishDecryption> BlowfishDecryptor;

/** @} */ // addtogroup net


} // namespace Massiv::Core
} // namespace Massiv

#endif // MASSIV_MASSIV_BLOWFISH_H
