/** @file massiv_checksum_impl.h
 * CRC32 checksum.
 */

/*
 * begin        : Fri Oct 17 2002
 * copyright    : (C) 2002 by Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_MASSIV_CHECKSUM_IMPL_H
#define MASSIV_MASSIV_CHECKSUM_IMPL_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_GLOBAL_H
#include "global/global.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_BITIOSTREAM_COMMON_H
#include "net/bitiostream_common.h"
#endif

#ifndef MASSIV_MASSIV_CHECKSUM_H
#include "net/massiv_checksum.h"
#endif

#include <sstream>



namespace Massiv {
namespace Core {


// CRC32 implementation
// ~~~~~~~~~~~~~~~~~~~~

template< class BufferType >
CRC32< BufferType >::CRC32
    ( 
    BufferType *    buffer 
    ) : crc32(),
        buffer( buffer ) 
    {
    init_digest();
    }


template< class BufferType >
CRC32< BufferType>::CRC32
    (
    const CRC32< BufferType > & source
    ) : crc32(),
        buffer( source.buffer )
    {
    init_digest();
    }


template< class BufferType >
CRC32< BufferType >::~CRC32()
    {
    delete [] digest;
    }


template< class BufferType >
void CRC32< BufferType >::init_digest()
    {
    unsigned int    digest_size = crc32.DigestSize();
    
    digest_size = digest_size >= sizeof( ChecksumType ) ? 
                                        digest_size : sizeof( ChecksumType );

    digest = new UInt8[ digest_size ];

    for( unsigned int i = 0; i < digest_size; i++ )
        {
        digest[ i ] = 0;
        }
    }


template< class BufferType >
typename CRC32< BufferType >::ChecksumType CRC32< BufferType >::compute_checksum()
    {
    return compute_checksum( buffer->size() );
    }


template< class BufferType >
bool CRC32< BufferType >::validate_checksum
    (
    ChecksumType    crc
    )
    {
    ChecksumType    check_crc = compute_checksum();

    if( check_crc == crc )
        {
        return true;
        }
    else
        {
        return false;
        }
    }


template< class BufferType >
bool CRC32< BufferType >::validate_checksum()
    {
    hope( digest );

    unsigned int    size    = buffer->size();
    bool            result;

    if( size < sizeof( ChecksumType ) )
        {
        result = false;
        }
    else
        {
        result = crc32.VerifyDigest
                        (
                        buffer->get_buffer() + size - sizeof( ChecksumType ),
                        buffer->get_buffer(),
                        size - sizeof( ChecksumType )
                        );
        }

    return result;
    }


template< class BufferType >
void CRC32< BufferType >::set_buffer
    (
    BufferType *    buffer
    )
    {
    this->buffer = buffer;
    }


template< class BufferType >
typename CRC32< BufferType >::ChecksumType CRC32< BufferType >::compute_checksum
    (
    unsigned int                number
    )
    {
    hope( digest );

    crc32.CalculateDigest
                        (
                        digest,
                        buffer->get_buffer(),
                        number
                        );

    return *( ( ChecksumType * ) digest );
    }

}; // namespace Core
}; // namespace Massiv

#endif // MASSIV_MASSIV_CHECKSUM_IMPL_H
