/** @file net_common.h
 * Network common types and definitions.
 */

/*
 * begin        : Mon Feb 17 2003
 * copyright    : (C) 2003 by Petr Tovarys, Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_NET_COMMON_H
#define MASSIV_NET_COMMON_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif



namespace Massiv {
namespace Core {


/** @addtogroup net
 * @{
 */

/**
 * Common network types and definitions.
 * The main reason for this is to avoid including
 * @b huge net.h in massiv headers (that would
 * result net's private headers to be included
 * in almost every file even if it is not required).
 */
namespace NetworkCommon {

    /**
     * Buffer type - reliable/unreliable.
     */
    enum StreamType
        {
        STREAM_UNRELIABLE = 0x01,
        STREAM_RELIABLE =   0x02
        };
    
    /**
     * Length of the TCP message header.
     */
    enum
        {
        NET_BUFFER_HEAD_LEN_TCP = 10
        };

    /**
     * Length of the UDP message header.
     */
    enum
        {
        NET_BUFFER_HEAD_LEN_UDP = 10
        };

    /**
     * Length of the TCP message footer.
     */
     enum
        {
        NET_BUFFER_FOOT_LEN_TCP = 0
        };

    /**
     * Length of the UDP message footer.
     */
    enum
        {
        NET_BUFFER_FOOT_LEN_UDP = 5
        };

    /**
     * Default reserved buffer size.
     */
    enum
        {
        NET_BUFFER_DEFAULT_LEN = 1024
        };

    /**
     * Maximum of data waiting on the socket for receive.
     */
    enum
        {
        MAX_WAITING_BYTES = 8192
        };
    
} // namespace Massiv::Core::NetworkCommon

/** @} */ // addtogroup net

} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_NET_COMMON_H
