/** @file net_statistics.cpp
 * Network statistics.
 */

/*
 * begin        : Wed Oct 9 2002
 * copyright    : (C) 2002 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "net/net_statistics.h"
#include "global/global.h"


namespace Massiv {
namespace Core {


NetStatistics::NetStatistics() :
    Statistics( "Network", Status::FACILITY_NETWORK, Statistics::OPEN_NORMAL ),
    consumed_time( create_float( "consumed_time" ) ),
    connections_created( create_large_integer( "connections_created" ) ),
    connections_killed( create_large_integer( "connections_killed" ) ),
    bytes_received( create_large_integer( "bytes_received" ) ),
    bytes_sent( create_large_integer( "bytes_sent" ) ),
    messages_received( create_large_integer( "messages_received" ) ),
    messages_sent( create_large_integer( "messages_sent" ) ),
    ping_timeouts( create_large_integer( "ping_timeouts" ) ),
    tcp_packets_sent( create_large_integer( "tcp_packets_sent" ) ),
    tcp_packets_received( create_large_integer( "tcp_packets_received" ) ),
    udp_packets_sent( create_large_integer( "udp_packets_sent" ) ),
    udp_packets_received( create_large_integer( "udp_packets_received" ) ),
    bad_packets_received( create_large_integer( "bad_packets_received" ) ),
    blowfish_expirations( create_large_integer( "blowfish_expirations" ) ),
    averange_throughput( create_float( "averange_throughput" ) ),
    actual_throughput( create_float( "actual_throughput" ) ),
    max_throughput( create_float( "max_throughput" ) )
    {
    }  // NetStatistics::NetStatistics()


NetStatistics::~NetStatistics()
    {
    }


void NetStatistics::inc_connections_created
    (
    int howmany
    )
    {
    connections_created = connections_created + howmany;
    }


UInt64 NetStatistics::get_connections_created() const
    {
    return connections_created;
    }


void NetStatistics::inc_connections_killed()
    {
    connections_killed++;
    }


UInt64 NetStatistics::get_connections_killed() const
    {
    return connections_killed;
    }


UInt32 NetStatistics::get_active_connections() const
    {
    return ( UInt32 ) ( connections_created - connections_killed );
    }


UInt64 NetStatistics::get_bytes_received() const
    {
    return bytes_received;
    }


void NetStatistics::inc_tcp_bytes_received
    (
    int howmany
    )
    {
    bytes_received = bytes_received + howmany;
    tcp_packets_received++;
    }


void NetStatistics::inc_udp_bytes_received
    (
    int howmany
    )
    {
    bytes_received = bytes_received + howmany;
    udp_packets_received++;
    }


UInt64 NetStatistics::get_bytes_sent
    (
    ) const
    {
    return bytes_sent;
    };


void NetStatistics::inc_tcp_bytes_sent
    (
    int howmany
    )
    {
    bytes_sent = bytes_sent + howmany;
    tcp_packets_sent++;
    }


void NetStatistics::inc_udp_bytes_sent
    (
    int howmany
    )
    {
    bytes_sent = bytes_sent + howmany;
    udp_packets_sent++;
    }


UInt64 NetStatistics::get_ping_timeouts() const
    {
    return ping_timeouts;
    }


void NetStatistics::inc_ping_timeouts()
    {
    ping_timeouts++;
    }


UInt64 NetStatistics::get_tcp_packets_received() const
    {
    return tcp_packets_received;
    }


UInt64 NetStatistics::get_udp_packets_received() const
    {
    return udp_packets_received;
    }


void NetStatistics::inc_tcp_packets_received
    (
    int howmany
    )
    {
    tcp_packets_received = tcp_packets_received + howmany;
    }


UInt64 NetStatistics::get_udp_packets_sent() const
    {
    return udp_packets_sent;
    }


void NetStatistics::inc_udp_packets_sent
    (
    int howmany
    )
    {
    udp_packets_sent = udp_packets_sent + howmany;
    }

UInt64 NetStatistics::get_tcp_packets_sent() const
    {
    return tcp_packets_sent;
    }


void NetStatistics::inc_tcp_packets_sent
    (
    int howmany
    )
    {
    tcp_packets_sent = tcp_packets_sent + howmany;
    }


UInt64 NetStatistics::get_bad_packets_received() const
    {
    return bad_packets_received;
    }


void NetStatistics::inc_bad_packets_received()
    {
    bad_packets_received++;
    }


UInt64 NetStatistics::get_blowfish_expirations() const
    {
    return blowfish_expirations;
    }


void NetStatistics::inc_blowfish_expirations()
    {
    blowfish_expirations++;
    }


UInt64 NetStatistics::get_messages_received() const
    {
    return messages_received;
    }


void NetStatistics::inc_messages_received
    (
    int howmany
    )
    {
    messages_received = messages_received + howmany;
    };


UInt64 NetStatistics::get_messages_sent() const
    {
    return messages_sent;
    }


void NetStatistics::inc_messages_sent
    (
    int howmany
    )
    {
    messages_sent = messages_sent + howmany;
    }


void NetStatistics::recompute_throughput
    (
    UInt64  bytes_delta,
    Float32 time_delta
    )
    {
    averange_throughput = static_cast<Float32>
        ( 
        ( bytes_received + bytes_sent ) / Global::system_time().get_value() 
        );
    actual_throughput = static_cast<Float32>
        (
        // Note: MSVC6 doesn't support conversion UInt64 -> Float64.
        // So we first convert the bytes_delta to Int64, because
        // Int64 -> Float64 is supported.
        static_cast< Int64 >( bytes_delta ) / time_delta 
        );

    max_throughput = ( actual_throughput > max_throughput ) ? actual_throughput : max_throughput;
    }


} // namespace Massiv::Core
} // namespace Massiv
