/** @file net_statistics.h
 * Network statistics.
 */

/*
 * begin        : Wed Oct 9 2002
 * copyright    : (C) 2002 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_NET_STATISTICS_H
#define MASSIV_NET_STATISTICS_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_STATISTICS_H
#include "registry/statistics.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup net
 * @{
 */

/**
 * Network statistics.
 * @todo add more properties to monitor.
 */
class NetStatistics : protected Statistics
    {
public:

    /**
     * Constructor.
     */
    NetStatistics();

    /**
     * Destructor.
     */
    ~NetStatistics();

public:

    //
    // Setters & getters.
    //

    void inc_connections_created
        (
        int howmany
        );

    UInt64 get_connections_created() const;

    void inc_connections_killed();

    UInt64 get_connections_killed() const;

    UInt32 get_active_connections() const;

    UInt64 get_bytes_received() const;

    void inc_tcp_bytes_received
        (
        int howmany
        );

    void inc_udp_bytes_received
        (
        int howmany
        );

    UInt64 get_bytes_sent() const;

    void inc_tcp_bytes_sent
        (
        int howmany
        );

    void inc_udp_bytes_sent
        (
        int howmany
        );

    UInt64 get_ping_timeouts() const;

    void inc_ping_timeouts();

    UInt64 get_udp_packets_sent() const;

    void inc_udp_packets_sent
        (
        int howmany
        );

    UInt64 get_tcp_packets_sent() const;

    void inc_tcp_packets_sent
        (
        int howmany
        );

    UInt64 get_tcp_packets_received() const;

    void inc_tcp_packets_received
        (
        int howmany
        );

    UInt64 get_udp_packets_received() const;

    void inc_udp_packets_received
        (
        int howmany
        );

    UInt64 get_bad_packets_received() const;

    void inc_bad_packets_received();

    UInt64 get_blowfish_expirations() const;

    void inc_blowfish_expirations();

    UInt64 get_messages_received() const;

    void inc_messages_received
        (
        int howmany
        );

    UInt64 get_messages_sent() const;

    void inc_messages_sent
        (
        int howmany
        );

    void recompute_throughput
        (
        UInt64  bytes_delta,
        Float32 time_delta
        );

public:
    Statistics::Float        & consumed_time;

private:

    Statistics::LargeInteger & connections_created;
    Statistics::LargeInteger & connections_killed;
    Statistics::LargeInteger & bytes_received;
    Statistics::LargeInteger & bytes_sent;
    Statistics::LargeInteger & messages_received;
    Statistics::LargeInteger & messages_sent;
    Statistics::LargeInteger & ping_timeouts;
    Statistics::LargeInteger & tcp_packets_sent;
    Statistics::LargeInteger & tcp_packets_received;
    Statistics::LargeInteger & udp_packets_sent;
    Statistics::LargeInteger & udp_packets_received;
    Statistics::LargeInteger & bad_packets_received;
    Statistics::LargeInteger & blowfish_expirations;
    Statistics::Float        & averange_throughput;
    Statistics::Float        & actual_throughput;
    Statistics::Float        & max_throughput;

    };  // class NetStatistics

/** @} */ // addtogroup net


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_NET_STATISTICS_H
