/** @file netibuffer.h
 * Bit-oriented network input stream with ability of decrypting messages.
 */

/*
 * begin        : Tue Apr 23 2002
 * copyright    : (C) 2002 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_NETIBUFFER_H
#define MASSIV_NETIBUFFER_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_BITISTREAM_H
#include "net/bitistream.h"
#endif

#ifndef MASSIV_MASSIV_CHECKSUM_H
#include "net/massiv_checksum.h"
#endif



namespace Massiv {
namespace Core {


class Network;
class UDPOBuffer;


/** @addtogroup net
 * @{
 */

/**
 * Bit-oriented network input stream with ability of decrypting messages.
 */
class NetIBuffer : public BitIStream
    {
    friend class Massiv::Core::Network;
    friend class Massiv::Core::Test::SocketsTest;
    friend class Massiv::Core::UDPOBuffer;

public :
    
    /**
     * Default constructor.
     * Binds empty buffer with default reserved net buffer size.
     */
    NetIBuffer();

    /**
     * Copy constructor.
     */
    NetIBuffer
        (
        const NetIBuffer &      sourcebuf
        );

    /**
     * Default destructor.
     */
    ~NetIBuffer();

protected:

    typedef CRC32< BitIOStreamBuffer >      Crc32;

    Crc32       engine_checksum;
        /**< The checksum "engine". */

public:

    UInt32      tag;
        /**< Message tag. */
    
    };  // class NetIBuffer

/** @} */ // addtogroup net


} // namespace Massiv::Core
} // namespace Massiv

#endif // MASSIV_NETIBUFFER_H
