/** @file netobuffer.cpp
 * Bit-oriented net output stream buffer.
 */

/*
 * begin        : Tue May 15 2002
 * copyright    : (C) 2002 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "net/netobuffer.h"
#include "net/bitiostream_common.h"
#include <sstream>


namespace Massiv {
namespace Core {


NetOBuffer::NetOBuffer()
    :   status( NET_BUFFER_UNLOCKED ),
        connection_info( NULL ),
        permanent( false ),
        owner( NodeId() )
    {
    }


NetOBuffer::~NetOBuffer()
    {
    }


std::string NetOBuffer::to_string() const
    {
    std::ostringstream ss;

    ss << "status = ";

    switch( status )
        {
        case NET_BUFFER_LOCKED:
            ss << "NET_BUFFER_LOCKED\n";
            break;

        case NET_BUFFER_UNLOCKED:
            ss << "NET_BUFFER_UNLOCKED\n";
            break;

        case NET_BUFFER_FLUSH_ALLOWED:
            ss << "NET_BUFFER_FLUSH_ALLOWED\n";
            break;

        case NET_BUFFER_CRYPTED:
            ss << "NET_BUFFER_CRYPTED\n";
            break;

        default:
            panic( "Invalid NetOBuffer status." );
        }

    ss << "connection_info:";

    if( connection_info != NULL )
        {
        ss << "\n" + connection_info->to_string() << "\n";
        }
    else
        {
        ss << " NULL\n";
        }

    ss << "permanent = " << permanent << "\n";
    ss << "owner = " << owner;

    return ss.str();
    }


} // namespace Massiv::Core
} // namespace Massiv
