/** @file serveraddress.cpp
 * Managing server address.
 */

/*
 * begin        : Tue May 15 2002
 * copyright    : (C) 2002 by Ondrej Pecta, Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "net/serveraddress.h"
#include <sstream>

namespace Massiv {
namespace Core {


ServerAddress::ServerAddress()
    {
    set_addr( "" );
    set_port( 0 );
    caching = true;
    }


ServerAddress::ServerAddress
    (
    std::string     address,
    UInt16          server_port,
    bool            caching
    ) : caching( caching )
    {
    set_addr( address );
    set_port( server_port );
    }


ServerAddress::ServerAddress
    (
    UInt32  ip,
    UInt16  server_port,
    bool    caching
    ) : caching( caching )
    {
    set_ip( ip );
    set_port( server_port );
    }


ServerAddress::~ServerAddress()
    {
    }


ServerAddress & ServerAddress::operator=
    (
    const ServerAddress & sa
    )
    {
    set_addr( sa.dns_name );
    
    // We can not use set_port( sa.port ) because sa.port is already
    // converted by host_to_network_short()!
    set_port( sa.get_port() );
    
    caching = sa.caching;

    return *this;
    }


UInt32 ServerAddress::resolve_ip_address() const
    {
    UInt32 result = static_cast<UInt32>( IP_UNDEFINED );

    if( dns_name.length() )
        {
        result = resolve_ip_address( dns_name );
        ip = result;
        }

    // Note: ExInvalidDnsName can go through here.

    return result;
    }


UInt16 ServerAddress::get_port() const
    {
    return network_to_host_short( port );
    }


std::string ServerAddress::get_dns_name() const
    {
    return dns_name;
    }


UInt32 ServerAddress::get_ip() const
    {
    if( caching && ( ip != static_cast<UInt32>( IP_UNDEFINED ) ) )
        {
        return ip;
        }
    else
        {
        return resolve_ip_address();
        }
    }


void ServerAddress::set_port
    (
    UInt16 new_port
    )
    {
    port = host_to_network_short( new_port );
    }


void ServerAddress::set_ip
    (
    UInt32 ip
    )
    {
    this->ip = ip;
    dns_name = ip_to_string();
    }


void ServerAddress::set_addr
    (
    const std::string & address
    )
    {
    ip = string_to_ip( address );
    dns_name = address;
    }


bool ServerAddress::set_caching
    (
    bool caching
    )
    {
    bool old_caching = this->caching;
    this->caching = caching;
    return old_caching;
    }


std::string ServerAddress::ip_to_string
    (
    )
    {
    UInt8   ip1;
    UInt8   ip2;
    UInt8   ip3;
    UInt8   ip4;
   
    get_ip_fields( ip1, ip2, ip3, ip4 );

    std::ostringstream oss;
    oss << ( UInt32 ) ip1 << "." << ( UInt32 ) ip2 
        << "." << ( UInt32 ) ip3 << "." << ( UInt32 ) ip4;
    return oss.str();
    }
    
} // namespace Massiv::Core
} // namespace Massiv
