/** @file tcpobuffer.cpp
 * Bit-oriented tcp output stream buffer with crypto
 */

/*
 * begin        : Tue Apr 23 2002
 * copyright    : (C) 2002 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "net/tcpobuffer.h"
#include "net/bitostream.h"
#include "net/connection_info_impl.h"

namespace Massiv {
namespace Core {

using namespace NetworkCommon;

TCPOBuffer::TCPOBuffer() :
    NetOBuffer()
    {
    buffer.reserve( NET_BUFFER_DEFAULT_LEN );
    crypted_buffer.reserve( NET_BUFFER_DEFAULT_LEN );
    send_immediately = false;
    }


TCPOBuffer::~TCPOBuffer()
    {
    }


std::string TCPOBuffer::to_string() const
    {
    std::ostringstream ss;

    ss  << NetOBuffer::to_string()
        << "\nsend_immediately = " << send_immediately;

    return ss.str();
    }


void TCPOBuffer::init
    (
    UInt32  tag
    )
    {
    hope( byte_size() == 0 );

    // write nr of messages - for compatibility with UDP.
    // only 1 message may be contained in TCP buffer
    crypted_buffer.resize( NET_BUFFER_HEAD_LEN_TCP );

    crypted_buffer[0] = ( UInt8 ) 1;
    crypted_buffer[1] = ( UInt8 ) ( 1 >> 8 );

    crypted_buffer[2] = ( UInt8 ) tag;
    crypted_buffer[3] = ( UInt8 ) ( tag >> 8 );
    crypted_buffer[4] = ( UInt8 ) ( tag >> 16 );
    crypted_buffer[5] = ( UInt8 ) ( tag >> 24 );
    
    // reserve space for message len
    crypted_buffer[6] = ( UInt8 ) 0;
    crypted_buffer[7] = ( UInt8 ) 0;
    crypted_buffer[8] = ( UInt8 ) 0;
    crypted_buffer[9] = ( UInt8 ) 0;
    // Buffer is ready for bit stream 
    status = NET_BUFFER_LOCKED;
    }


void TCPOBuffer::flush()
    {
    hope( status == NET_BUFFER_FLUSH_ALLOWED );
    hope( connection_info->get_blowfish_encryptor_for_tcp() );
    
    /**
     * whole buffer will be encrypted into crypted_buffer.
     * skips head in crypted_buffer - head isn't crypted!.
     */
    
    // aligns to the eight byte - neede for crypting
    int phase = connection_info->get_blowfish_key_period() % 2;
    crypted_buffer[0] = ( UInt8 ) phase;
    crypted_buffer[1] = ( UInt8 ) ( phase >> 8 );

    connection_info->get_blowfish_encryptor_for_tcp()->align( buffer );

    int size = byte_size();

    crypted_buffer[6] = ( UInt8 ) size;
    crypted_buffer[7] = ( UInt8 ) ( size >> 8 );
    crypted_buffer[8] = ( UInt8 ) ( size >> 16 );
    crypted_buffer[9] = ( UInt8 ) ( size >> 24 );

    crypted_buffer.resize( byte_size() + NET_BUFFER_HEAD_LEN_TCP );
    connection_info->get_blowfish_encryptor_for_tcp()->process_range
        ( 
        buffer.begin(), 
        buffer.end(), 
        crypted_buffer.begin() + NET_BUFFER_HEAD_LEN_TCP
        );
    
    // ready for send
    status = NET_BUFFER_CRYPTED;
    }


} // namespace Massiv::Core
} // namespace Massiv
