/** @file tcpobuffer.h
 * Bit-oriented tcp output stream buffer with crypto
 */

/*
 * begin        : Tue Apr 23 2002
 * copyright    : (C) 2002 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TCPOBUFFER_H
#define MASSIV_TCPOBUFFER_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_NETOBUFFER_H
#include "net/netobuffer.h"
#endif



namespace Massiv {
namespace Core {

class Network;


/** @addtogroup net
 * @{
 */

/**
 * Implements a TCP buffer that can be sent via the TCP protocol by lower
 * layers. Unlike UDPOBuffer, TCPOBuffer can hold no more than one message.
 * However, for compatibility with UDP, it has the "number of messages" 
 * field - this field must always contain 1.
 */
class TCPOBuffer : public NetOBuffer
    {
    friend class Massiv::Core::Network;

public :
    
    /**
     * Default constructor.
     */
    TCPOBuffer( );

    /**
     * Default destructor.
     */
    virtual ~TCPOBuffer();

    /**
     */
    virtual std::string to_string() const;

private:

    /**
     * Writes message tag and reserves space for message length.
     */
    virtual void init
        ( 
        UInt32 tag
        );

    /**
     * Finishes buffer - writes message size into the reserved space and 
     * encrypts the buffer (except of the header) into crypted_buffer
     * using blowfish cipher.
     */
    virtual void flush();

private:

    bool                                send_immediately;
        /**< TRUE causes the message to be sent immediately after 
             unlocking the buffer. */

    std::list< TCPOBuffer * >::iterator self_iterator;
        /**< Iterator to buffer list in @c Network pointing to self. */
    };  // class TCPOBuffer

/** @} */ // addtogroup net


} //namespace Massiv::Core
} //namespace Massiv


#endif // MASSIV_TCPOBUFFER_H
