/** @file node_action_id.h
 * Node action id.
 */

/*
 * begin        : Sat Oct 4 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_NODE_ACTION_ID_H
#define MASSIV_NODE_ACTION_ID_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_NODE_ACTION_H
#include "node/node_action.h"
#endif

#ifndef MASSIV_STYPE_ENUM_H
#include "serializer/stype_enum.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup node
 * @{
 */

/**
 * Serializable #NodeAction identification.
 * Simple placeholder for #NodeAction::Id.
 */
class NodeActionId
    {
public:

    /**
     * NodeActionId hash function.
     */
    struct Hash
        {

        /** Hash function. */
        INLINE size_t operator()
            (
            const NodeActionId & action_id
            ) const throw();

        }; // struct Hash

    /**
     * NodeActionId equalkey function.
     */
    struct EqualKey
        {

        /** Equalkey function. */
        INLINE bool operator()
            (
            const NodeActionId & node_id1,
            const NodeActionId & node_id2
            ) const throw();

        }; // struct EqualKey

    friend struct NodeActionId::Hash;
    friend struct NodeActionId::EqualKey;

public:

    /** Constructor. */
    INLINE NodeActionId();

    /** Copy constructor. */
    INLINE NodeActionId( const NodeActionId & other );
    
    /** Constructor. */
    INLINE NodeActionId( NodeAction::Id id );
    
    /** Destructor. */
    INLINE ~NodeActionId();
    
    /** Operator=. */
    INLINE NodeActionId & operator=( const NodeActionId & other );
    
public:

    //
    // Serializations.
    //
    
    /**
     * Serialize to bitstream.
     */
    INLINE void bitstr_write
        (
        BitOStream &                        bos,
        const Serializer::Description &     desc
        ) const;

    /**
     * Read from bitstream.
     */
    INLINE void bitstr_read
        (
        BitIStream &                        bis,
        const Serializer::Description &     desc
        );

public:

    //
    // Setters/getters.
    //

    /** Getter. */
    INLINE NodeAction::Id get() const;
        
    /** Setter. */
    INLINE NodeActionId & set( NodeAction::Id id );

private:

    SEnum< NodeAction::Id > id;
        /**< Action identification. */
    
    }; // class NodeActionId

/** @} */ // addtogroup node


} // namespace Massiv::Core
} // namespace Massiv


#ifdef ENABLE_INLINE
#include "node/node_action_id_inline.h"
#endif


#endif // MASSIV_NODE_ACTION_ID_H
