/** @file node_action_server_quit.cpp
 * SERVER_QUIT node action object (implementation).
 */

/*
 * begin        : Tue Oct 7 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "node/node_action_server_quit.h"
#include "global/global.h"
#include "global/global_callbacks.h"
#include "object/archive_manager.h"
#include "utility/debug.h"


namespace Massiv {
namespace Core {


NodeActionServerQuit::NodeActionServerQuit() :
    initiate_new_archivation( true )
    {
    }


NodeActionServerQuit::~NodeActionServerQuit()
    {
    }


NodeAction::NextTickResult NodeActionServerQuit::next_tick()
    {
    ArchiveManager & archive_manager = Global::archive_manager();
    
    // Before shutting down server nodes, archivate current simulation.
    if( initiate_new_archivation )
        {
        // Check if archivation is already in progress. In that case
        // we will only wait until the archivation is finished.
        if( archive_manager.archivation_in_progress() )
            {
            initiate_new_archivation = false;
            return RESULT_CONTINUE;
            }
        
        // Try initiate new archivation.
        switch( archive_manager.request_start_archivation() )
            {
            case ArchiveManager::RESULT_NODE_IS_NOT_SERVER:
                panic( "Local node is not server!?!" );
                break;

            case ArchiveManager::RESULT_ARCHIVATION_IN_PROGRESS:
                // Archivation is running.
                initiate_new_archivation = false;
                break;
                
            case ArchiveManager::RESULT_REQUEST_DELAYED_BECAUSE_OF_SRPC:
                // Archivation will start soon.
                // We will stay in initiate_new_archivation mode (hoping
                // archivation will run for more than one tick when started
                // so will be able to set initiate_new_archivation to false :)
                break;
            }
        
        return RESULT_CONTINUE;
        }
        
    // Wait until archivation is finished.
    if( archive_manager.archivation_in_progress() )
        {
        return RESULT_CONTINUE;
        }
    
    // Final archivation was finished. Shutdown local server.
    GlobalCallbacks::shutdown_local_node();
    
    // We never return RESULT_FINISH. There is no reason.
    return RESULT_CONTINUE;
    }


} // namespace Core
} // namespace Massiv
