/** @file node_action_server_quit.h
 * SERVER_QUIT node action object.
 */

/*
 * begin        : Tue Oct 7 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_NODE_ACTION_SERVER_QUIT_H
#define MASSIV_NODE_ACTION_SERVER_QUIT_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_NODE_ACTION_H
#include "node/node_action.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup node
 * @{
 */

/**
 * Server quit action object.
 * Shutdowns server nodes.
 *
 * First final archivation is initiated. Once the
 * the archivation is finished, the object shutdows
 * local node.
 */
class NodeActionServerQuit : public NodeAction
    {
    NO_COPY_CONSTRUCTOR( NodeActionServerQuit );
    NO_ASSIGNMENT_OPERATOR( NodeActionServerQuit );
    
public:

    /** Constructor. */
    NodeActionServerQuit();
    
    /** Destructor. */
    virtual ~NodeActionServerQuit();
    
public:

    // From NodeAction.
    virtual NextTickResult next_tick();
    
private:

    bool    initiate_new_archivation;
        /**< True if next_tick() is trying to initiate final archivation. */

    }; // class NodeActionServerQuit

/** @} */ // addtogroup node


} // namespace Core
} // namespace Massiv


#endif // MASSIV_NODE_ACTION_SERVER_QUIT_H
