/** @file node_manager_anonymous.h
 * Anonymous specialization of NodeManager::Node.
 */
 
/*
 * begin        : Sat Nov 8 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_NODE_MANAGER_ANONYMOUS_H
#define MASSIV_NODE_MANAGER_ANONYMOUS_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_NODE_MANAGER_NODE_H
#include "node/node_manager_node.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup node
 * @{
 */

/**
 * Anonymous encapsulates node manager's behaviour while
 * local node is running in anonymous mode.
 * In anonymous node only connection to service nodes
 * are permitted.
 */
class NodeManager::Anonymous : public NodeManager::Node
    {
    NO_COPY_CONSTRUCTOR( Anonymous );
    NO_ASSIGNMENT_OPERATOR( Anonymous );
    
public:

    /** Constructor. */
    Anonymous();
    
public:

    // From NodeManager::Node.
    virtual void connect();

    // From NodeManager::Node.
    virtual void disconnect();
    
    // From NodeManager::Node.
    bool is_connected();

    // From NodeManager::Node.
    virtual void next_tick();

    // From NodeManager::Node.
    virtual ConnectionEstablishedResult net_callback_connection_established
        (
        const NodeId & node_id
        );

    // From NodeManager::Node.
    virtual void net_callback_connection_closed
        (
        const NodeId & node_id
        );

    // From NodeManager::Node.
    virtual void net_callback_connection_lost
        (
        const NodeId & node_id
        );

    // From NodeManager::Node.
    virtual void net_callback_cannot_initiate_connection
        (
        const NodeId & node_id
        );

    }; // NodeManager::Anonymous

/** @} */ // addtogroup node


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_NODE_MANAGER_ANONYMOUS_H
