/** @file node_manager_anonymous_callbacks.cpp
 * Anonymous specialization of NodeManager::Node.
 */
 
/*
 * begin        : Sat Nov 9 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "node/node_manager_anonymous.h"
#include "global/global.h"
#include "global/global_callbacks.h"
#include "utility/debug.h"
#include "sysevent/simulation_sync_msg.h"
#include "sysevent/client_connection_msg.h"
#include "sysevent/system_msg.h"
#include "time/time_manager.h"
#include "database/node_database.h"
#include "net/net.h"


namespace Massiv {
namespace Core {


NodeManager::ConnectionEstablishedResult NodeManager::Anonymous::net_callback_connection_established
    (
    const NodeId & node_id
    )
    {
    // Accept connections with service nodes only.
    if( node_id.is_type( NodeId::TYPE_SERVICE ) )
        {
        return RESULT_OK;
        }
    else
        {
        std::ostringstream oss;
        oss << "Denying connection to/from " << node_id
            << " because anonymous node can connect only to service nodes.";
        Global::log_warning( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_HIGHEST, oss.str() );
        return RESULT_CLOSE_CONNECTION;
        }
    }


void NodeManager::Anonymous::net_callback_connection_closed
    (
    const NodeId & node_id
    )
    {
    // Notify core.
    GlobalCallbacks::node_disconnected( node_id );
    }


void NodeManager::Anonymous::net_callback_connection_lost
    (
    const NodeId & node_id
    )
    {
    // Notify core.
    GlobalCallbacks::node_disconnected( node_id );
    
    // Service node connection crash while in anonymous mode is fatal error.
    if( node_id.is_type( NodeId::TYPE_SERVICE ) )
        {
        throw_connection_lost( node_id );
        }
    }


void NodeManager::Anonymous::net_callback_cannot_initiate_connection
    (
    const NodeId & node_id
    )
    {
    // Failed to establish connection to a service node is fatal error during anonymous mode.
    if( node_id.is_type( NodeId::TYPE_SERVICE ) )
        {
        throw_connection_failed( node_id );
        }
    }


} // namespace Massiv::Core
} // namespace Massiv
