/** @file node_manager_callbacks.cpp
 * Node manager network callbacks.
 */

/*
 * begin        : Sun Mar 30 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "node/node_manager.h"
#include "global/global.h"
#include "system/system.h"
#include "utility/debug.h"
#include "net/net.h"
#include "node/node_manager_node.h"
#include <sstream>


namespace Massiv {
namespace Core {


NodeManager::ConnectionEstablishedResult NodeManager::net_callback_connection_established
    (
    const NodeId & node_id
    )
    {
    // Log.
        {
        std::ostringstream oss;
        oss << "Connection established to/from " << node_id << ".";
        Global::log_info( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_NORMAL, oss.str() );
        }
    
    if( !node )
        {
        std::ostringstream oss;
        oss << "Denying connection to/from " << node_id
            << ". The local node manager was not properly initialized yet.";
        Global::log_warning( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_HIGH, oss.str() );
        return RESULT_CLOSE_CONNECTION;
        }
    else
        {
        return node->net_callback_connection_established( node_id );
        }
    }


void NodeManager::net_callback_connection_closed
    (
    const NodeId & node_id
    )
    {
    // Log.
        {
        std::ostringstream oss;
        oss << "Connection closed to/from " << node_id << ".";
        Global::log_info( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_NORMAL, oss.str() );
        }
    
    if( !node )
        {
        std::ostringstream oss;
        oss << "Connection closed to/from " << node_id << " while node manager is in uninitialized state."
            << " Probably bug in massiv network layer.";
        Global::log_warning( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_HIGHEST, oss.str() );
        }
    else
        {
        node->net_callback_connection_closed( node_id );
        }
    }


void NodeManager::net_callback_connection_lost
    (
    const NodeId & node_id
    )
    {
    // Log.
        {
        std::ostringstream oss;
        oss << "Connection lost to/from " << node_id << ".";
        Global::log_warning( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_HIGHEST, oss.str() );
        }

    if( !node )
        {
        std::ostringstream oss;
        oss << "Connection lost to/from " << node_id << " while node manager is in uninitialized state."
            << " Probably bug in massiv network layer.";
        Global::log_warning( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_HIGHEST, oss.str() );
        }
    else
        {
        node->net_callback_connection_lost( node_id );
        }
    }


void NodeManager::net_callback_cannot_initiate_connection
    (
    const NodeId & node_id
    )
    {
    // Log.
        {
        std::ostringstream oss;
        oss << "Failed to establish connection to " << node_id << ".";
        Global::log_warning( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_HIGHEST, oss.str() );
        }

    if( !node )
        {
        std::ostringstream oss;
        oss << "Failed to initiate connection to " << node_id << " while node manager is in uninitialized state."
            << " Probably bug in massiv network layer.";
        Global::log_warning( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_HIGHEST, oss.str() );
        }
    else
        {
        node->net_callback_cannot_initiate_connection( node_id );
        }
    }
    
} // namespace Massiv::Core
} // namespace Massiv
