/** @file node_manager_client_messages.cpp
 * Node manager message handling for client node.
 */

/*
 * begin        : Sat Nov 8 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "node/node_manager_client.h"
#include "global/global.h"
#include "global/global_callbacks.h"
#include "utility/debug.h"
#include "sysevent/simulation_sync_msg.h"
#include "sysevent/client_connection_msg.h"
#include "sysevent/system_msg.h"
#include "time/time_manager.h"
#include "database/node_database.h"
#include "net/net.h"


namespace Massiv {
namespace Core {


SystemMessage::ExecuteResult NodeManager::Client::handle_welcome_message
    (
    const WelcomeMessage & msg
    )
    {
    DEBUG_ONLY( const NodeId &  msg_sender  = msg.get_sender(); )

    hope( msg.get_initiator() == msg_sender );          // Already verified.
    hope( msg_sender.is_type( NodeId::TYPE_SERVER ) );  // Already verified.

    hope( !get_local_node_id().is_type( NodeId::TYPE_SERVER ) ); // Already verified.

    if( phase == PHASE_WAITING_FOR_WELCOME )
        {
        // Switch to next phase.
        phase = PHASE_RUNNING;
        
        // We are client node. Synchronize time with our forward node.
            {
            /** @todo FIXME */
            TimeManager & time_manager = Global::time_manager();
            time_manager.add_node_for_synchronization( forward_node );
            time_manager.allow_synchronization();
            time_manager.unpause();
            }
        
        // Log.
        log_phase();
        }

    return SystemMessage::EXECUTE_OK;
    }


} // namespace Massiv::Core
} // namespace Massiv

