/** @file node_manager_messages.cpp
 * Node manager message handling.
 */

/*
 * begin        : Sun Mar 30 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "node/node_manager.h"
#include "sysevent/system_msg.h"
#include "sysevent/forward_msg.h"
#include "global/global.h"
#include "utility/debug.h"
#include "sysevent/simulation_sync_msg.h"
#include "sysevent/client_connection_msg.h"
#include "sysevent/system_event_queue.h"
#include "object/object_manager.h"
#include "net/net.h"
#include "time/time_manager.h"
#include "global/global_callbacks.h"
#include "node/node_manager_node.h"
#include "node/node_manager_server.h"
#include "node/node_manager_client.h"
#include <sstream>


namespace Massiv {
namespace Core {


SystemMessage::ExecuteResult NodeManager::handle_accept_client_message
    (
    const AcceptClientMessage & msg
    )
    {
    if( server )
        {
        return server->handle_accept_client_message( msg );
        }
    return SystemMessage::EXECUTE_OK;
    }


SystemMessage::ExecuteResult NodeManager::handle_client_accepted_message
    (
    const ClientAcceptedMessage & msg
    )
    {
    if( server )
        {
        return server->handle_client_accepted_message( msg );
        }
    return SystemMessage::EXECUTE_OK;
    }
    

SystemMessage::ExecuteResult NodeManager::handle_client_ready_message
    (
    const ClientReadyMessage & msg
    )
    {
    if( server )
        {
        return server->handle_client_ready_message( msg );
        }
    return SystemMessage::EXECUTE_OK;
    }

SystemMessage::ExecuteResult NodeManager::handle_client_ready_reply_message
    (
    const ClientReadyReplyMessage & msg
    )
    {
    if( server )
        {
        return server->handle_client_ready_reply_message( msg );
        }
    return SystemMessage::EXECUTE_OK;
    }


SystemMessage::ExecuteResult NodeManager::handle_kill_client_message
    (
    const KillClientMessage & msg
    )
    {
    if( server )
        {
        return server->handle_kill_client_message( msg );
        }
    return SystemMessage::EXECUTE_OK;
    }


SystemMessage::ExecuteResult NodeManager::handle_welcome_message
    (
    const WelcomeMessage & msg
    )
    {
    if( client )
        {
        return client->handle_welcome_message( msg );
        }
    return SystemMessage::EXECUTE_OK;
    }


//------------------------------------------------------------------------------
// Server to client communication.
//------------------------------------------------------------------------------


SystemMessage::ExecuteResult NodeManager::handle_forwarded_message
    (
    const ForwardedMessage & msg
    )
    {
    NodeId          local_node_id   = get_local_node_id();
    const NodeId &  dest_node       = msg.get_destination_node();
    const NodeId &  msg_sender      = msg.get_sender();
    
    hope( !dest_node.is_type( NodeId::TYPE_SERVER ) );
    
    if( local_node_id == dest_node )
        {
        // The destination is reached. Dispatch the message stored in the msg.
        
        // Note: dispatch_system_message_from_packet() require nonconst reference
        //       to bitistream.
        BitIStream  stream( msg.forwarded_message );
        
        SystemMessage::dispatch_system_message_from_packet
            (
            stream,
            msg_sender
            );
        }
    else
        {
        // Forward the message to destination node.
        NetworkCommon::StreamType  stream_type;
        
        if( msg.use_reliable_stream() )
            {
            stream_type = NetworkCommon::STREAM_RELIABLE;
            }
        else
            {
            stream_type = NetworkCommon::STREAM_UNRELIABLE;
            }
        
        msg.send( dest_node, stream_type );
        }
    
    return SystemMessage::EXECUTE_OK;
    }


} // namespace Massiv::Core
} // namespace Massiv
