/** @file node_manager_service.cpp
 * Service specialization of NodeManager::Node.
 */
 
/*
 * begin        : Sat Nov 8 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "node/node_manager_service.h"
#include "global/global.h"
#include "global/global_callbacks.h"
#include "utility/debug.h"
#include "sysevent/simulation_sync_msg.h"
#include "sysevent/client_connection_msg.h"
#include "sysevent/system_msg.h"
#include "time/time_manager.h"
#include "database/node_database.h"
#include "net/net.h"
#include "database/node_database.h"


namespace Massiv {
namespace Core {


NodeManager::Service::Service() :
    phase( PHASE_NONE )
    {
    }


void NodeManager::Service::connect()
    {
    TAG_STACK_FRAME( "::Massiv::Core::NodeManager::Service::connect()" );
    
    // Enumerate server nodes.
    initialize_server_node_table();

    // Start waiting on well known port of the service node.
    const ServerAddress sa = Global::node_database().get_node_server_address( get_local_node_id() );
    const int           port = sa.get_port();

    // Log.
        {
        std::ostringstream oss;
        oss  << "Starting service on port " << port << ".";
        Global::log_info( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_NORMAL, oss.str() );
        }

    // Initialize network.
    if( !Global::network().start_well_known_port_server( port ) )
        {
        /** @todo FIXME */
        panic( "Network::start_well_known_port_server() failed." );
        }

    // Switch phase.
    phase = PHASE_RUNNING;
    }


bool NodeManager::Service::is_connected()
    {
    return phase == PHASE_RUNNING;
    }


void NodeManager::Service::disconnect()
    {
    phase = PHASE_NONE;
    }


void NodeManager::Service::next_tick()
    {
    }


} // namespace Massiv::Core
} // namespace Massiv
