/** @file node_manager_service.h
 * Service specialization of NodeManager::Node.
 */
 
/*
 * begin        : Sat Nov 8 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_NODE_MANAGER_SERVICE_H
#define MASSIV_NODE_MANAGER_SERVICE_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_NODE_MANAGER_NODE_H
#include "node/node_manager_node.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup node
 * @{
 */

/**
 * Service encapsulates node manager's behaviour if local node is service.
 */
class NodeManager::Service : public NodeManager::Node
    {
    NO_COPY_CONSTRUCTOR( Service );
    NO_ASSIGNMENT_OPERATOR( Service);
    
public:
    
    /** Internal phase. */
    enum Phase
        {
        PHASE_NONE,
            /**< Initial phase. The service object has just been created but not initialized yet. */

        PHASE_RUNNING,
            /**< The service is running. */

        }; // Phase

public:

    /** Constructor. */
    Service();

public:

    // From NodeManager::Node.
    virtual void connect();

    // From NodeManager::Node.
    virtual void disconnect();
    
    // From NodeManager::Node.
    bool is_connected();
    
    // From NodeManager::Node.
    virtual void next_tick();

    // From NodeManager::Node.
    virtual ConnectionEstablishedResult net_callback_connection_established
        (
        const NodeId & node_id
        );

    // From NodeManager::Node.
    virtual void net_callback_connection_closed
        (
        const NodeId & node_id
        );

    // From NodeManager::Node.
    virtual void net_callback_connection_lost
        (
        const NodeId & node_id
        );

    // From NodeManager::Node.
    virtual void net_callback_cannot_initiate_connection
        (
        const NodeId & node_id
        );

private:

    Phase               phase;
        /**< Internal phase. */

    }; // NodeManager::Service

/** @} */ // addtogroup node


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_NODE_MANAGER_SERVICE_H
