/** @file node_manager_service_callbacks.cpp
 * Service specialization of NodeManager::Node.
 */
 
/*
 * begin        : Sat Nov 8 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "node/node_manager_service.h"
#include "global/global.h"
#include "global/global_callbacks.h"
#include "utility/debug.h"
#include "sysevent/simulation_sync_msg.h"
#include "sysevent/client_connection_msg.h"
#include "sysevent/system_msg.h"
#include "time/time_manager.h"
#include "database/node_database.h"
#include "net/net.h"


namespace Massiv {
namespace Core {


NodeManager::ConnectionEstablishedResult NodeManager::Service::net_callback_connection_established
    (
    const NodeId & node_id
    )
    {
    TAG_STACK_FRAME( "::Massiv::Core::NodeManager::Service::net_callback_connection_established()" );
    
    // Check if not yet initialized...
    if( phase == PHASE_NONE )
        {
        std::ostringstream oss;
        oss << "Denying connection to/from " << node_id << ". The local node "
            << "is not yet ready.";
        Global::log_warning( Status::FACILITY_NODE_MANAGEMENT, Status::PRIORITY_HIGH, oss.str() );
        return RESULT_CLOSE_CONNECTION;
        }
    
    // Insert the node into corresponding table.
    // Note: Server nodes are already in server table.
    switch( node_id.get_type() )
        {
        case NodeId::TYPE_SERVER:
            hope( get_server_state( node_id ) != NULL );
            break;
        
        case NodeId::TYPE_SERVICE:
            break;
        
        case NodeId::TYPE_CLIENT:
            break;
        
        case NodeId::TYPE_ANONYMOUS:
            break;
        }

    return RESULT_OK;
    }


void NodeManager::Service::net_callback_connection_closed
    (
    const NodeId & node_id
    )
    {
    // Notify core.
    GlobalCallbacks::node_disconnected( node_id );
    }


void NodeManager::Service::net_callback_connection_lost
    (
    const NodeId & node_id
    )
    {
    // Notify core.
    GlobalCallbacks::node_disconnected( node_id );
    }


void NodeManager::Service::net_callback_cannot_initiate_connection
    (
    const NodeId & node_id
    )
    {
    }


} // namespace Massiv::Core
} // namespace Massiv
