/** @file nodeid_inline.h
 * Node id (serializable).
 */

/*
 * begin        : Thu Apr 11 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_HASH_FUNCTION_H
#include "utility/hash_function.h"
#endif


namespace Massiv {
namespace Core {


// struct NodeId::Hash

size_t NodeId::Hash::operator()
    (
    const NodeId & node_id
    ) const throw()
    {
    return ext::hash_int( node_id.id );
    }


// struct NodeId::EqualKey

bool NodeId::EqualKey::operator()
    (
    const NodeId & node_id1,
    const NodeId & node_id2
    ) const throw()
    {
    return node_id1.id == node_id2.id;
    }


// struct NodeId::Less

bool NodeId::Less::operator()
    (
    const NodeId & node_id1,
    const NodeId & node_id2
    ) const throw()
    {
    // Note: Test "node_id1.id < node_id2.id" may be compiler/architecture dependent (?)
    
    if( node_id1.components.type < node_id2.components.type )
        {
        return true;
        }
    else if( node_id1.components.type > node_id2.components.type )
        {
        return false;
        }
    else
        {
        return node_id1.components.relative_id < node_id2.components.relative_id;
        }
    }


// class NodeId

NodeId::NodeId()
    {
    clear();
    }


NodeId::NodeId
    (
    const NodeId & node_id
    ) : id( node_id.id )
    {
    }


NodeId & NodeId::operator=
    (
    const NodeId & node_id
    )
    {
    id = node_id.id;
    return *this;
    }


bool NodeId::operator==
    (
    const NodeId & node_id
    ) const
    {
    return id == node_id.id;
    }


bool NodeId::operator!=
    (
    const NodeId & node_id
    ) const
    {
    return id != node_id.id;
    }


void NodeId::clear()
    {
    // We must clear all 32 bits!!!
    id = 0;
    hope( components.type == TYPE_SERVER );
    hope( components.relative_id == 0 );
    }


bool NodeId::is_null() const
    {
    return id == 0;
    }


NodeId::Type NodeId::get_type() const
    {
    return static_cast< Type >( components.type );
    }


UInt32 NodeId::get_relative_id() const
    {
    return components.relative_id;
    }


bool NodeId::is_type
    (
    Type type
    ) const
    {
    return get_type() == type;
    }


UInt32 NodeId::convert_to_uint32
    (
    const NodeId & node_id
    )
    {
    UInt32 type          = node_id.components.type;
    UInt32 relative_id   = node_id.components.relative_id;

    return ( relative_id | ( type << RELATIVE_ID_BITS_COUNT ) );
    }


NodeId NodeId::create_from_uint32
    (
    UInt32 num
    )
    {
    UInt32 type           = ( num >> RELATIVE_ID_BITS_COUNT );
    UInt32 mask           = ( 1 << RELATIVE_ID_BITS_COUNT ) - 1;
    UInt32 relative_id    = ( num & mask );
    
    return create_from_type_and_relative_id( type, relative_id );
    }


} // namespace Massiv::Core
} // namespace Massiv
