/** @file archivation_id.cpp
 * ArchivationId implementation.
 */

/*
 * begin        : Sun Oct 20 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "object/archivation_id.h"
#include "serializer/stype_int.h"
#include <sstream>
#include <iomanip> // IO manipulators


namespace Massiv {
namespace Core {


ArchivationId::ArchivationId()
    {
    invalidate();
    }


ArchivationId::ArchivationId
    (
    const ArchivationId & arch_id
    ) :
    id( arch_id.id )
    {
    }


ArchivationId & ArchivationId::operator=
    (
    const ArchivationId & arch_id
    )
    {
    id = arch_id.id;
    return *this;
    }


bool ArchivationId::operator==
    (
    const ArchivationId & arch_id
    ) const
    {
    return id == arch_id.id;
    }


bool ArchivationId::operator!=
    (
    const ArchivationId & arch_id
    ) const
    {
    return id != arch_id.id;
    }


bool ArchivationId::operator<
    (
    const ArchivationId & arch_id
    ) const
    {
    return id < arch_id.id;
    }
    

bool ArchivationId::operator<=
    (
    const ArchivationId & arch_id
    ) const
    {
    return id <= arch_id.id;
    }


bool ArchivationId::operator>
    (
    const ArchivationId & arch_id
    ) const
    {
    return id > arch_id.id;
    }


bool ArchivationId::operator>=
    (
    const ArchivationId & arch_id
    ) const
    {
    return id >= arch_id.id;
    }


void ArchivationId::invalidate()
    {
    id = 0;
    }


bool ArchivationId::is_invalid() const
    {
    return id == 0;
    }


void ArchivationId::increment()
    {
    id++;
    }


UInt32 ArchivationId::get_order() const
    {
    return id;
    }


std::string ArchivationId::to_string() const
    {
    std::ostringstream ss;
    ss  << "massiv_"    /** @todo FIXME Prefix. */
        << std::setfill( '0' )
        << std::setw( 10 ) // UInt32
        << id;
    ss  << ".archive";
    return ss.str();
    }
    
    
ArchivationId ArchivationId::from_string
    (
    const std::string & name
    )
    {
    /** @todo FIXME Otestovat!! */
    char                prefix[ 7 + 1 ];
    char                postfix[ 8 + 1 ];
    char                dummy_char;
    int                 i;
    UInt32              id;
    ArchivationId       new_id;
    std::istringstream  ss( name );
    
    for( i = 0; i < 7; i++ )
        {
        ss >> ( prefix[ i ] );
        }
    prefix[ 7 ] = '\0';
    
    ss  >> id;
        
    for( i = 0; i < 8; ++i )
        {
        ss >> ( postfix[ i ] );
        }
    postfix[ 8 ] = '\0';
    
    if( ss.fail() )
        {
        return new_id;
        }
    
    // Test if there are no more characters in the stream.
    ss >> dummy_char;
    if( !ss.fail() || !ss.eof() )
        {
        // We were not at the end of the stream.
        return new_id;
        }
        
    if( std::string( "massiv_" ) != prefix )
        {
        return new_id;
        }
        
    if( std::string( ".archive" ) != postfix )
        {
        return new_id;
        }
        
    new_id.id = id;
    return new_id;
    }


void ArchivationId::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    SUInt32 s_id( id );
    s_id.bitstr_write( bos, desc );
    }


void ArchivationId::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    SUInt32 s_id;
    s_id.bitstr_read( bis, desc );
    id = s_id;
    }


std::ostream & operator<<
    (
    std::ostream &          s,
    const ArchivationId &   arch_id
    )
    {
    s << arch_id.to_string();
    return s;
    }


} // namespace Massiv::Core
} // namespace Massiv
