/** @file class_manager_inline.h
 * Class manager (inline methods).
 */

/*
 * begin        : Sun May 12 2002
 * copyright    : (C) 2002 by Marek Vondrak, Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_HASH_FUNCTION_H
#include "utility/hash_function.h"
#endif



namespace Massiv {
namespace Core {


size_t ClassManager::Hash::operator()
    (
    const std::type_info * class_type_info
    ) const
    {
    #ifndef OPTION_SHARED_LIBRARIES
    
    // Type infos are singleton objects, their addresses can be used
    // as hash codes.
    
    return reinterpret_cast< size_t >( class_type_info );
    
    #else
    
    return ext::hash_string( class_type_info->name() );
    
    #endif
    }


bool ClassManager::EqualKey::operator()
    (
    const std::type_info * type_info1,
    const std::type_info * type_info2
    ) const
    {
    #ifndef OPTION_SHARED_LIBRARIES
    
    // Type infos are singleton objects, their addresses can be used to
    // compare them.
    
    return type_info1 == type_info2;
    
    #else
    
    hope( type_info1 != NULL && type_info2 != NULL );
    return ( *type_info1 == *type_info2 ) != 0;
    
    #endif
    }


} // namespace Massiv::Core
} // namespace Massiv
