/** @file class_type_id_inline.h
 * Class type id structure (inline methods).
 */

/*
 * begin        : May 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_HASH_FUNCTION_H
#include "utility/hash_function.h"
#endif

#ifndef MASSIV_STYPE_INT_H
#include "serializer/stype_int.h"
#endif



namespace Massiv {
namespace Core {


// struct ClassTypeId::Hash

size_t ClassTypeId::Hash::operator()
    (
    const ClassTypeId & class_type_id
    ) const throw()
    {
    return ext::hash_int( class_type_id.id );
    }


// struct ClassTypeId::EqualKey

bool ClassTypeId::EqualKey::operator()
    (
    const ClassTypeId & class_type_id_1,
    const ClassTypeId & class_type_id_2
    ) const throw()
    {
    return class_type_id_1.id == class_type_id_2.id;
    }


// struct ClassTypeId::Less

bool ClassTypeId::Less::operator()
    (
    const ClassTypeId & class_type_id_1,
    const ClassTypeId & class_type_id_2
    ) const throw()
    {
    return class_type_id_1.id < class_type_id_2.id;
    }


// class ClassTypeId

ClassTypeId::ClassTypeId()
      : id( 0 )
    {
    }


ClassTypeId::ClassTypeId
    (
    int the_kind,
    int the_class_id
    ) : id( 0 )
    {
    components.kind = the_kind;
    components.class_id = the_class_id;
    }
    
    
ClassTypeId::ClassTypeId
    (
    const ClassTypeId & other
    ) : id( other.id )
    {
    }


ClassTypeId & ClassTypeId::operator=
    (
    const ClassTypeId & other
    )
    {
    id = other.id;
    return *this;
    }


bool ClassTypeId::operator==
    (
    const ClassTypeId & other
    ) const
    {
    return id == other.id;
    }


bool ClassTypeId::operator!=
    (
    const ClassTypeId & other
    ) const
    {
    return id != other.id;
    }


int ClassTypeId::get_kind() const
    {
    return components.kind;
    }


int ClassTypeId::get_class_id() const
    {
    return components.class_id;
    }


} // namespace Massiv::Core
} // namespace Massiv
