/** @file collect_group.h
 * Object group collecting operations.
 */

/*
 * begin        : Tue Oct 22 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_COLLECT_GROUP_H
#define MASSIV_COLLECT_GROUP_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_OBJECT_H
#include "object/object.h"
#endif

#ifndef MASSIV_OBJECT_COMMON_H
#include "object/object_common.h"
#endif

#ifndef MASSIV_GARBAGE_COLLECTOR_H
#include "pointer/garbage_collector.h"
#endif



namespace Massiv {
namespace Core {


/** @addtogroup object_management
 * @{
 */

/**
 * Object operation which stores enumerated objects into some list.
 */
struct CollectObjectGroupOperation : public GarbageCollector::ObjectOperation
    {
public:

    /**
     * Constructor.
     * @param output_object_list List where to store enumerated objects.
     * @note The @a output_object_list is not cleared!
     */
    CollectObjectGroupOperation
        (
        ObjectShared::ObjectList &  output_object_list
        );

    /**
     * Clear object list associated with the @a CollectObjectGroupOperation.
     */
    void clear();

    /**
     * Operation.
     * Enumerated @a object is added to the stored list.
     */
    virtual void operator()
        (
        ObjectProperty * object
        );

private:

    ObjectShared::ObjectList &  objects;
        /**< List of enumerated objects. Enumerator ensures that each object
             is enumerated at most once so its sufficient to map ObjectList
             to a plain vector / array. */

    }; // CollectObjectGroupOperation

/** @} */ // addtogroup object_management


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_COLLECT_GROUP_H
