/** @file event_handle.cpp
 * Event handle.
 */

/*
 * begin        : Wed May 15 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "object/event_handle.h"
#include "object/object_manager.h"
#include "global/global.h"
#include "utility/text_reader.h"
#include "utility/text_writer.h"
#include "net/bitistream.h"
#include "net/bitostream.h"
#include <sstream>


namespace Massiv {
namespace Core {


// Implementation of EventHandle::Hash
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

std::size_t EventHandle::Hash::operator()
    (
    const EventHandle & handle
    )
    {
    return  ObjectId::Hash()( handle.object_id ) ^
            EventId::Hash()( handle.event_id );    
    }
    

// Implementation of EventHandle
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

EventHandle::EventHandle()
    {
    }


EventHandle::EventHandle
    (
    const EventHandle & h
    )
    {
    set( h.object_id, h.event_id );
    }


EventHandle & EventHandle::operator=
    (
    const EventHandle & h
    )
    {
    set( h.object_id, h.event_id );
    return *this;
    }


EventHandle::~EventHandle()
    {
    }


void EventHandle::kill()
    {
    ObjectManager & object_manager = Global::object_manager();

    object_manager.kill_event( object_id, event_id );

    clear();
    }


void EventHandle::clear()
    {
    object_id.set_null();
    event_id.set_private_null();
    }


void EventHandle::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    object_id.bitstr_write( bos, desc );
    event_id.bitstr_write( bos, desc );
    }


void EventHandle::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    object_id.bitstr_read( bis, desc );
    event_id.bitstr_read( bis, desc );
    }


void EventHandle::text_write
    (
    TextWriter &                    tw,
    const Serializer::Description & desc
    ) const
    {
    object_id.text_write( tw, desc );
    tw.write_space();
    event_id.text_write( tw, desc );
    }


void EventHandle::text_read
    (
    TextReader &                    tr,
    const Serializer::Description & desc
    )
    {
    object_id.text_read( tr, desc );
    event_id.text_read( tr, desc );
    }


std::ostream & operator<<
    (
    std::ostream &      stream,
    const EventHandle & handle
    )
    {
    stream << "[ " << handle.object_id << " : " << handle.event_id << " ]";
    return stream;
    }


void EventHandle::set
    (
    const ObjectId & oid,
    const EventId &  eid
    )
    {
    object_id = oid;
    event_id  = eid;
    }


}; // namespace Massiv::Core
}; // namespace Massiv
