/** @file eventid.cpp
 * Event identification.
 */

/*
 * begin        : Wed May 15 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "global/common.h"
#include "object/eventid.h"
#include "net/bitistream.h"
#include "net/bitostream.h"
#include "utility/text_writer.h"
#include "serializer/stype_int.h"
#include "serializer/stype_bool.h"
#include <ostream>


namespace Massiv {
namespace Core {


void EventId::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    SBool priv( is_private() );

    priv.bitstr_write( bos, desc );

    if( !priv )
        {
        SInt<Int64, RELATIVE_ID_BITS_COUNT> rel_id( components.relative_id );
        rel_id.bitstr_write( bos, desc );
        }
    }


void EventId::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    SBool priv;

    priv.bitstr_read( bis, desc );
    components.is_public = priv ? 0 : 1;

    if( !priv )
        {
        SInt<Int64, RELATIVE_ID_BITS_COUNT> rel_id;
        rel_id.bitstr_read( bis, desc );
        components.relative_id = rel_id;
        }
    else
        {
        components.relative_id = 0;
        }
    }


void EventId::text_write
    (
    TextWriter &                    tw,
    const Serializer::Description & desc
    ) const
    {
    SBool priv( is_private() );

    priv.text_write( tw, desc );

    if( !priv )
        {
        SInt<Int64, RELATIVE_ID_BITS_COUNT> rel_id( components.relative_id );
        tw.write_space();
        rel_id.text_write( tw, desc );
        }
    }


void EventId::text_read
    (
    TextReader &                    tr,
    const Serializer::Description & desc
    )
    {
    SBool priv;

    priv.text_read( tr, desc );
    components.is_public = priv ? 0 : 1;

    if( !priv )
        {
        SInt<Int64, RELATIVE_ID_BITS_COUNT> rel_id;
        rel_id.text_read( tr, desc );
        components.relative_id = rel_id;
        }
    else
        {
        components.relative_id = 0;
        }
    }


std::ostream & operator<<
    (
    std::ostream &  stream,
    const EventId & event_id
    )
    {
    stream << event_id.components.relative_id;
    stream << ( event_id.components.is_public ? "" : " (internal)" );
    return stream;
    }


} // namespace Massiv::Core
} // namespace Massiv
