/** @file lib_exceptions_helper.h
 * Lib exceptions helper.
 * Used by the core to raise lib exceptions while allowing not to include
 * the whole "core.h".
 */

/*
 * begin        : Sat Nov 1 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_LIB_EXCEPTIONS_HELPER_H
#define MASSIV_LIB_EXCEPTIONS_HELPER_H


#include "global/common.h"
#include <string>


namespace Massiv {
namespace Core {
namespace Lib {

// lib_exceptions.h

class RuntimeException;
class InvalidArgumentException;
class CanNotAccessObjectException;
class IllegalPointerConversionException;
class ObjectNotOnLocalSystemException;
class PointerPolicyViolationException;
class RemoteCallFailedException;
class UndefinedClassTypeException;
class InvalidEventException;
class AbstractClassException;


/**
 * Class with static @c raise() method that raises Core::Lib exceptions
 * of a given type.
 *
 * @ingroup exception
 */
template <class ExceptionName>
class MASSIV_EXPORTED RaiseCoreLibException
    {
public:
        
    /** Raise exception. */
    static NORETURN( void ) raise
        (
        const std::string & message
        ) /*throw( ExceptionName )*/;

    };


/**
 * Equivalent to "throw ExceptionName( message )".
 * Core uses this instead of the latter to allow not to include "core.h"
 * wherever a Core::Lib exception needs be thrown.
 *
 * @ingroup exception
 */
#define RAISE_CORE_LIB_EXCEPTION( ExceptionName, message ) \
    ::Massiv::Core::Lib::RaiseCoreLibException< ::Massiv::Core::Lib::ExceptionName >:: \
    raise( message )


} // namespace Massiv::Core::Lib
} // namespace Massiv::Core
} // namespace Massiv

#endif // MASSIV_LIB_EXCEPTIONS_HELPER_H
