/** @file metaobject_inline.h
 * Metaobject base class (inline methods).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


namespace Massiv {
namespace Core {


// class MetaObject::Attributes

MetaObject::Attributes::Attributes()
      : abstract( false ),
        assignable( false ),
        client_server_migrate( false ),
        comparable( false ),
        has_strong_pointer( false ),
        kind( KIND_SHARED ),
        no_archive( false ),
        no_balance( false ),
        no_migrate( false ),
        node_object( false ),
        root( false ),
        simulation_startup_notify( false ),
        throwable( false ),
        tracked( false ),
        value_type( false )
    {
    }


// class MetaObject

const std::string & MetaObject::get_class_name() const
    {
    return class_name;
    }


const std::type_info & MetaObject::get_class_type_info() const
    {
    hope( class_type_info );
    return *class_type_info;
    }


const std::type_info & MetaObject::get_class_pointer_type_info() const
    {
    hope( class_pointer_type_info );
    return *class_pointer_type_info;
    }


const ClassTypeId & MetaObject::get_class_type_id() const
    {
    return class_type_id;
    }


const ObjectFactory * MetaObject::get_object_factory() const
    {
    return object_factory.get();
    }

    
const MetaObject::Attributes & MetaObject::get_attributes() const
    {
    return attributes;
    }


bool MetaObject::get_attribute_abstract() const
    {
    return attributes.abstract;
    }


bool MetaObject::get_attribute_assignable() const
    {
    return attributes.assignable;
    }


bool MetaObject::get_attribute_client_server_migrate() const
    {
    return attributes.client_server_migrate;
    }


bool MetaObject::get_attribute_comparable() const
    {
    return attributes.comparable;
    }


bool MetaObject::get_attribute_has_strong_pointer() const
    {
    return attributes.has_strong_pointer;
    }


int MetaObject::get_attribute_kind() const
    {
    return attributes.kind;
    }


bool MetaObject::get_attribute_no_archive() const
    {
    return attributes.no_archive;
    }


bool MetaObject::get_attribute_no_balance() const
    {
    return attributes.no_balance;
    }


bool MetaObject::get_attribute_no_migrate() const
    {
    return attributes.no_migrate;
    }


bool MetaObject::get_attribute_node_object() const
    {
    return attributes.node_object;
    }


bool MetaObject::get_attribute_root() const
    {
    return attributes.root;
    }


bool MetaObject::get_attribute_simulation_startup_notify() const
    {
    return attributes.simulation_startup_notify;
    }


bool MetaObject::get_attribute_throwable() const
    {
    return attributes.throwable;
    }


bool MetaObject::get_attribute_tracked() const
    {
    return attributes.tracked;
    }


bool MetaObject::get_attribute_value_type() const
    {
    return attributes.value_type;
    }


const MetaObject::ComponentInfos & MetaObject::get_component_infos() const
    {
    return component_infos;
    }


const MetaObject::PropertyInfos & MetaObject::get_property_infos() const
    {
    return property_infos;
    }


const MetaObject::MethodInfos & MetaObject::get_method_infos() const
    {
    return method_infos;
    }


// class MetaObject::ComponentInfo

bool MetaObject::ComponentInfo::is_complete_component() const
    {
    return ( inherited_from == -1 );
    }


bool MetaObject::ComponentInfo::is_parent() const
    {
    return ( inherited_from == 0 );
    }


// class MetaObject::MethodInfo::Attributes

MetaObject::MethodInfo::Attributes::Attributes()
      : a_const( false ),
        a_virtual( false )
    {
    }


} // namespace Massiv::Core
} // namespace Massiv
