/** @file object_factory.cpp
 * Object factory.
 */

/*
 * begin        : Sun Dec 15 2001
 * copyright    : (C) 2001 by Marek Vondrak, Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "object/object_factory.h"
#include "object/object.h"
#include "object/metaobject.h"
#include "pointer/pointer_to_object.h"
#include "system/tick_lock.h"
#include "utility/debug.h"
#include "object/lib/lib_exceptions_helper.h"
#include <sstream>


namespace Massiv {
namespace Core {


ObjectFactory::ObjectFactory
    (
    const MetaObject &  the_metaobject
    ) : metaobject( the_metaobject )
    {
    }


ObjectFactory::~ObjectFactory()
    {
    }


void ObjectFactory::initialize_object
    (
    ObjectProperty &    object
    ) const
    {
    object.initialize_object( Object::INIT_NEW_INSTANCE, NULL );
    
    LOCK_SYSTEM();
    
    UpcallToObject upcall = &object;
    upcall.tag_upcall
        ( "::Massiv::Core::Object::object_updated( INSTANCE_CREATED )" );
    upcall->object_updated( Object::INSTANCE_CREATED );
    }
    

void ObjectFactory::initialize_object
    (
    ObjectProperty &    object,
    const ObjectId &    object_id
    ) const
    {
    object.initialize_object( Object::INIT_MIGRATED, &object_id );
    
    LOCK_SYSTEM();
    
    UpcallToObject upcall = &object;
    upcall.tag_upcall
        ( "::Massiv::Core::Object::object_updated( INSTANCE_MIGRATED )" );
    upcall->object_updated( Object::INSTANCE_MIGRATED );
    }


void ObjectFactory::initialize_replica
    (
    ObjectProperty &    object,
    const ObjectId &    object_id
    ) const
    {
    object.initialize_object( Object::INIT_REPLICA, &object_id );
    
    LOCK_SYSTEM();
    
    UpcallToObject upcall = &object;
    upcall.tag_upcall
        ( "::Massiv::Core::Object::object_updated( REPLICA_CREATED )" );
    upcall->object_updated( Object::REPLICA_CREATED );
    }


VariantPointer ObjectFactory::get_complete_object
    (
    const ObjectProperty * object
    ) const
    {
    hope( object );
    return object->get_complete_object();
    }


void ObjectFactory::_abstract_instance() const
    {
    std::ostringstream ss;
    
    ss  << "Can't create instance of abstract class "
        << metaobject.get_class_name() << ".";
        
    RAISE_CORE_LIB_EXCEPTION
        (
        AbstractClassException,
        ss.str()
        );
    }


} // namespace Massiv::Core
} // namespace Massiv
