/** @file object_info.cpp
 * ObjectInfo (implementation).
 */

/*
 * begin        : Wed Jun 11 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "object/object_info.h"
#include "utility/text_writer.h"
#include "status/exception.h"


namespace Massiv {
namespace Core {


// ObjectInfo
// ~~~~~~~~~~


void ObjectInfo::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    class_type_id.bitstr_write( bos, desc );
    archivable.bitstr_write( bos, desc );
    }


void ObjectInfo::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    class_type_id.bitstr_read( bis, desc );
    archivable.bitstr_read( bis, desc );
    }


void ObjectInfo::text_write
    (
    TextWriter &                    tw,
    const Serializer::Description & desc
    ) const
    {
    class_type_id.text_write( tw, desc );
    tw.write_space();
    archivable.text_write( tw, desc );
    }


void ObjectInfo::text_read
    (
    TextReader &                    tr,
    const Serializer::Description & desc,
    TextReadRequest                 tr_request
    )
    {
    if( tr_request == TEXT_READ_IGNORE_INVALID_CLASS_TYPE_ID )
        {
        try 
            {
            class_type_id.text_read( tr, desc );
            }
        catch( ExUnexpectedValue & )
            {
            // Clear the class type id and ignore the error.
            class_type_id = ClassTypeId();
            }
        }
    else
        {
        class_type_id.text_read( tr, desc );
        }

    archivable.text_read( tr, desc );
    }


} // namespace Core
} // namespace Massiv


#ifndef ENABLE_INLINE
#include "object/object_info_inline.h"
#endif
