/** @file object_info_inline.h
 * ObjectInfo (inline methods).
 */

/*
 * begin        : Wed Jun 11 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Massiv {
namespace Core {


// ObjectInfo
//~~~~~~~~~~~


ObjectInfo::ObjectInfo() :
    archivable( true )
    {
    }
    
    
ObjectInfo::ObjectInfo
    (
    const ObjectInfo & other
    ) :
    class_type_id( other.class_type_id ),
    archivable( other.archivable )
    {
    }
    
    
ObjectInfo::ObjectInfo
    (
    const ClassTypeId & _class_type_id,
    bool                _archivable
    ) :
    class_type_id( _class_type_id ),
    archivable( _archivable )
    {
    }
    
    
ObjectInfo & ObjectInfo::operator=
    (
    const ObjectInfo & other
    )
    {
    class_type_id   = other.class_type_id;
    archivable      = other.archivable;
    return *this;
    }


const ClassTypeId & ObjectInfo::get_class_type_id() const
    {
    return class_type_id;
    }


bool ObjectInfo::is_class_registered() const
    {
    return get_class_type_id().is_class_registered();
    }


bool ObjectInfo::is_archivable() const
    {
    return archivable;
    }


void ObjectInfo::set_class_type_id
    (
    const ClassTypeId & _class_type_id
    )
    {
    class_type_id = _class_type_id;
    }

    
void ObjectInfo::set_archivable_flag
    (
    bool    is_archivable
    )
    {
    archivable = is_archivable;
    }

    
} // namespace Core
} // namespace Massiv
