/** @file object_status.cpp
 * Object status (serializable).
 * Used by object searching protocols.
 */

/*
 * begin        : Wed May 1 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "object/object_status.h"
#include "serializer/stype_int.h"
#include <string>
#include <ostream>


namespace Massiv {
namespace Core {


void ObjectStatus::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    SInt<UInt32, STATUS_BITS_COUNT> s( status );
    s.bitstr_write( bos, desc );
    }


void ObjectStatus::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    SInt<UInt32, STATUS_BITS_COUNT> s;
    s.bitstr_read( bis, desc );
    hope( s < STATUS_COUNT );
    status = static_cast<Status>( s.read() );
    }


std::string ObjectStatus::to_string() const
    {
    switch( status )
        {
        case ALIVE:
            return "ALIVE";

        case NOT_ACCESSIBLE:
            return "NOT_ACCESSIBLE";

        case UNKNOWN:
            return "UNKNOWN";

        default:
            return "error - invalid status value";
        };
    }


std::ostream & operator<<
    (
    std::ostream &          s,
    const ObjectStatus &    status
    )
    {
    s << status.to_string();
    return s;
    }


} // namespace Massiv::Core
} // namespace Massiv


#ifndef ENABLE_INLINE
#include "object_status_inline.h"
#endif
