/** @file providerid.cpp
 * Object provider id and state (both serializable).
 */

/*
 * begin        : Thu Apr 11 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "object/providerid.h"
#include "serializer/stype_int.h"
#include <ostream>


namespace Massiv {
namespace Core {


ProviderId ProviderId::create_from_number
    (
    UInt32 new_id
    )
    {
    ProviderId  new_provider_id;
    new_provider_id.set_from_number( new_id );
    return new_provider_id;
    }


ProviderId ProviderId::create_virtual_provider_id()
    {
    ProviderId  new_provider_id;
    new_provider_id.set_from_number( VIRTUAL_PROVIDER_ID );
    return new_provider_id;
    }


ProviderId ProviderId::create_dummy_provider_id()
    {
    ProviderId  new_provider_id;
    new_provider_id.set_from_number( DUMMY_PROVIDER_ID );
    return new_provider_id;
    }


void ProviderId::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    SInt<UInt32, PROVIDER_ID_BITS_COUNT> s_id( id );
    s_id.bitstr_write( bos, desc );
    }


void ProviderId::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    SInt<UInt32, PROVIDER_ID_BITS_COUNT> s_id;
    s_id.bitstr_read( bis, desc );
    id = s_id;
    }


void ProviderId::text_write
    (
    TextWriter &                    tw,
    const Serializer::Description & desc
    ) const
    {
    SInt<UInt32, PROVIDER_ID_BITS_COUNT> s_id( id );
    s_id.text_write( tw, desc );
    }


void ProviderId::text_read
    (
    TextReader &                    tr,
    const Serializer::Description & desc
    )
    {
    SInt<UInt32, PROVIDER_ID_BITS_COUNT> s_id;
    s_id.text_read( tr, desc );
    id = s_id;
    }


void ProviderId::set_from_number
    (
    UInt32 new_id
    )
    {
    hope( ( new_id >> PROVIDER_ID_BITS_COUNT ) == 0 );
    UInt32 mask = ( ( 1 << PROVIDER_ID_BITS_COUNT ) - 1 );
    id = ( new_id & mask );
    }


std::ostream & operator<<
    (
    std::ostream &      s,
    const ProviderId &  provider_id
    )
    {
    s << provider_id.get_id_value();
    return s;
    }


} // namespace Massiv::Core
} // namespace Massiv


#ifndef ENABLE_INLINE
#include "providerid_inline.h"
#endif
