/** @file providerid_inline.h
 * Object provider id (inline methods).
 */

/*
 * begin        : Tue Mar 11 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Massiv {
namespace Core {


// Implementation of ProviderId::Hash
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

size_t ProviderId::Hash::operator()
    (
    const ProviderId & provider_id
    ) const throw()
    {
    return ( size_t ) provider_id.id;
    }


// Implementation of ProviderId::EqualKey
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

bool ProviderId::EqualKey::operator()
    (
    const ProviderId & provider_id1,
    const ProviderId & provider_id2
    ) const throw()
    {
    return provider_id1.id == provider_id2.id;
    }


// Implementation of ProviderId::Less
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

bool ProviderId::Less::operator()
    (
    const ProviderId & provider_id1,
    const ProviderId & provider_id2
    ) const throw()
    {
    return provider_id1.id < provider_id2.id;
    }


// Implementation of ProviderId
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

ProviderId::ProviderId() :
    id( NULL_ID )
    {
    }


ProviderId::ProviderId
    (
    const ProviderId & provider_id
    ) :
    id( provider_id.id )
    {
    }


ProviderId & ProviderId::operator=
    (
    const ProviderId & provider_id
    )
    {
    id = provider_id.id;
    return *this;
    }


bool ProviderId::operator==
    (
    const ProviderId & provider_id
    ) const
    {
    return id == provider_id.id;
    }


bool ProviderId::operator!=
    (
    const ProviderId & provider_id
    ) const
    {
    return id != provider_id.id;
    }


void ProviderId::set_null()
    {
    id = NULL_ID;
    }


bool ProviderId::is_null() const
    {
    return id == NULL_ID;
    }


bool ProviderId::is_virtual_provider_id() const
    {
    return id == VIRTUAL_PROVIDER_ID;
    }
    

bool ProviderId::is_dummy_provider_id() const
    {
    return id == DUMMY_PROVIDER_ID;
    }


UInt32 ProviderId::get_id_value() const
    {
    return id;
    }


} // namespace Massiv::Core
} // namespace Massiv
