/** @file schedule.h
 * Global schedule maintained by object manager.
 * To prevent cyclic c/c++ header dependency (but not limited to).
 */

/*
 * begin        : Tue Mar 12 2002
 * copyright    : (C) 2001 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_SCHEDULE_H
#define MASSIV_SCHEDULE_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_HEAP_H
#include "utility/heap.h"
#endif

#ifndef MASSIV_GLOBAL_H
#include "global/global.h"
#endif

#ifndef MASSIV_OBJECT_FWD_H
#include "object/object_fwd.h"
#endif

#ifndef MASSIV_STYPE_TIME_H
#include "serializer/stype_time.h"
#endif

#include <functional>


namespace Massiv {
namespace Core {


/** @addtogroup object_management
 * @{
 */

/** @struct GlobalScheduleIndexChanged
 * Monitors object position changes in global schedule.
 */
struct GlobalScheduleIndexChanged : public std::binary_function<ObjectProperty *, HeapIndex, void>
    {
    /**
     * Called whenever object is moved in the global schedule.
     * Inline implementation is located in schedule_impl.h file.
     */
    void operator()
        (
        ObjectProperty *    object,
        HeapIndex           new_index
        ) const;

    }; // struct GlobalScheduleIndexChanged


/** @class GlobalSchedule
 * Schedule maintained by @c ObjectManager.
 * Stores objects sorted by times of their first scheduled events.
 * These times are determined from corresponding object's LocalSchedules.
 * @see LocalSchedule.
 */
class GlobalSchedule : public Heap<STime, ObjectProperty *, STime::Less, GlobalScheduleIndexChanged>
    {
public:

    /** @typedef Index
     * Index into the global schedule.
     */
    typedef HeapIndex       Index;

    }; // GlobalSchedule

/** @} */ // addtogroup object_management


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_SCHEDULE_H
