/** @file default_cast.h
 * Default CastObject declaration.
 * The implementation is in pointer/default_cast_impl.h.
 */

/*
 * begin        : Tue Apr 23 2002
 * copyright    : (C) 2002 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_DEFAULT_CAST_H
#define MASSIV_DEFAULT_CAST_H


#ifndef MASSIV_OBJECT_FWD_H
#include "object/object_fwd.h"
#endif

#ifndef MASSIV_POINTER_PARAMETERS_H
#include "pointer/pointer_parameters.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup pointer
 * @{
 */

/**
 * Cast object implemented using dynamic_cast<> for simple object hierarchies.
 *
 * @note We need to support pointers to forward declared classes
 *       so that cross-referenced objects could be implemented easily.
 *       In order to make this work, the compiler must be able to
 *       instantiate ObjectPointer's default constructor and its cast
 *       object even in the case when the pointer type refers to
 *       a forward declared class.
 *       To ensure that the compiler will not try to instantiate
 *       DefaultCast methods when Type has not been seen yet (which will
 *       fail because of dynamic_cast<>), the implementation was moved
 *       out of class and the object must be instantiated manually later.
 *       This is done by factgen/class_list.
 */
template <class Type>
class DefaultCast : public CastObject
    {
public:

    /**
     * Casts object to Type *.
     */
    Type * do_cast
        (
        ObjectProperty * object
        ) const;

    /**
     */
    virtual VariantPointer operator()
        (
        ObjectProperty * object
        ) const;

    /**
     */
    virtual const std::type_info & get_target_type_info() const;

    }; // DefaultCast

/** @} */ // addtogroup pointer


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_DEFAULT_CAST_H
