/** @file default_cast_impl.h
 * Default CastObject implementation.
 * Factgen must ensure that DefaultCast<Type> is instantiated (explicitly)
 * for all managed objects (Types).
 * This header file can be included only if definitions of all the
 * managed classes have been included before.
 */

/*
 * begin        : Tue Apr 23 2002
 * copyright    : (C) 2002 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_DEFAULT_CAST_IMPL_H
#define MASSIV_DEFAULT_CAST_IMPL_H


#ifndef MASSIV_DEFAULT_CAST_H
#include "pointer/default_cast.h"
#endif



namespace Massiv {
namespace Core {


// Implementation of DefaultCast
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

template <class Type>
Type * DefaultCast<Type>::do_cast
    (
    ObjectProperty * object
    ) const
    {
    return dynamic_cast<Type *>( object );
    }


template <class Type>
VariantPointer DefaultCast<Type>::operator()
    (
    ObjectProperty * object
    ) const
    {
    return VariantPointer( do_cast( object ) );
    }


template <class Type>
const std::type_info & DefaultCast<Type>::get_target_type_info() const
    {
    return typeid( Type );
    }


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_DEFAULT_CAST_IMPL_H
