/** @file null_keyword.h
 * Null keyword.
 */

/*
 * begin        : Sat Feb 8 2003
 * copyright    : (C) 2003 by Marek Vondrak, Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_NULL_KEYWORD_H
#define MASSIV_NULL_KEYWORD_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup pointer
 * @{
 */

/**
 * Singleton object that represents "NULL keyword".
 * ObjectPointer is tweaked so that it recognize constructs involving @c null
 * and treats them appropriatelly.
 * The following is possible:
 * @code
 *  Pointer<SomeType>   ptr;
 *  ptr = null;     // The same behaviour as ptr.set_null().
 *  ptr == null;    // The same behaviour as ptr.is_null().
 * @endcode
 */
extern struct NullKeyword { } MASSIV_EXPORTED null;

/** @} */ // addtogroup pointer


} // namespace Massiv::Core
} // namespace Massiv

#endif // MASSIV_NULL_KEYWORD_H
