/** @file pointer_to_object_inline.h
 * Pointer to Object inline implementation.
 */

/*
 * begin        : Sat Jun 8 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_OBJECT_H
#include "object/object.h"
#endif

#ifndef MASSIV_OBJECT_PROXY_H
#include "pointer/object_proxy.h"
#endif

namespace Massiv {
namespace Core {


// Implementation of PtrToObjectBase
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Object * PtrToObjectBase::operator->() const
    {
    hope( object );
    return object;
    }
    

Object & PtrToObjectBase::operator*() const
    {
    return *operator->();
    }


PtrToObjectBase::operator Object *() const
    {
    return object;
    }


Object * PtrToObjectBase::get() const
    {
    return object;
    }

    
bool PtrToObjectBase::operator==
    (
    const Object * other
    ) const
    {
    return object == other;
    }


bool PtrToObjectBase::operator!=
    (
    const Object * other
    ) const
    {
    return !operator==( other );
    }


// Implementation of PtrToObject
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

PtrToObject::PtrToObject()
    {
    object = NULL;
    }
    

PtrToObject::PtrToObject
    (
    ObjectProperty * property
    )
    {
    if( property )
        {
        object = checked_cast<Object *>( property );
        object->ptr_pin();
        }
    else
        {
        object = NULL;
        }
    }

    
PtrToObject::PtrToObject
    (
    const LocalProxy<Object> & proxy
    )
    {
    object = proxy;
    hope( object );
    object->ptr_pin();
    }


PtrToObject::PtrToObject
    (
    const PtrToObject & ptr
    )
    {
    object = ptr.object;

    if( object )
        {
        object->ptr_pin();
        }
    }


PtrToObject::~PtrToObject()
    {
    if( object )
        {
        object->ptr_unpin();
        }
    }


PtrToObject & PtrToObject::operator=
    (
    const PtrToObject & ptr
    )
    {
    if( object )
        {
        object->ptr_unpin();
        }

    object = ptr.object;

    if( object )
        {
        object->ptr_pin();
        }

    return *this;
    }


// Implementation of UpcallToObject
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

UpcallToObject::UpcallToObject()
    {
    prev = last;
    last = this;
    
    #ifdef GC_TAG_STACK
    tag = NULL;
    #endif

    object = NULL;
    }
    

UpcallToObject::UpcallToObject
    (
    ObjectProperty * property
    )
    {
    prev = last;
    last = this;
    
    #ifdef GC_TAG_STACK
    tag = NULL;
    #endif

    if( property )
        {
        object = checked_cast<Object *>( property );
        }
    else
        {
        object = NULL;
        }
    }


UpcallToObject::UpcallToObject
    (
    const LocalProxy<Object> & proxy
    )
    {
    prev = last;
    last = this;
    
    #ifdef GC_TAG_STACK
    tag = NULL;
    #endif

    object = proxy;
    }


UpcallToObject::UpcallToObject
    (
    const UpcallToObject & ptr
    )
    {
    prev = last;
    last = this;
    
    #ifdef GC_TAG_STACK
    tag = NULL;
    #endif

    object = ptr.object;
    }


UpcallToObject::~UpcallToObject()
    {
    if( last == this )
        {
        last = prev;
        }
    else
        {
        panic( "UpcallToObject instances are not used in LIFO order." );
        }
    }


UpcallToObject & UpcallToObject::operator=
    (
    const UpcallToObject & ptr
    )
    {
    object = ptr.object;
    return *this;
    }


void UpcallToObject::tag_upcall
    (
    const char * new_tag
    )
    {
    #ifdef GC_TAG_STACK
    tag = new_tag;
    #endif
    }


} // namespace Massiv::Core
} // namespace Massiv
