/** @file rpc_stubs_inline.h
 * RPC flags and stub class (inline methods).
 */

/*
 * begin        : Nov 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Massiv {
namespace Core {


RPCStubsBase::RPCStubsBase()
      : _flags( RPC_DEFAULT )
    {
    }


void RPCStubsBase::_bind
    (
    const ObjectId & id
    )
    {
    _object_id = id;
    }


void RPCStubsBase::_param
    (
    int             flags,
    const STime &   time
    )
    {
    // FIXME: exceptions instead of hope?

    // Can't be both TIMED and DELAYED.
    hope( !flags_on( flags, RPC_TIMED | RPC_DELAYED ) );

    // No RESULTS from REPLICAS.
    hope( !flags_on( flags, RPC_REPLICAS ) || !flags_on( flags, RPC_RESULTS ) );

    // NO_ARCHIVE is set automatically for sync calls.
    hope( !flags_on( flags, RPC_NO_ARCHIVE ) );

    _flags = flags;
    _time = time;
    }


void RPCStubsBase::_sync()
    {
    // FIXME: exceptions instead of hope?

    hope( flags_on( _flags, 0, RPC_SYNC_NO ) );
    hope( _time.is_invalid() );
    _flags |= RPC_SYNC_YES;
    }


void RPCStubsBase::_check_class_argument_type
    (
    const std::type_info &  expected,
    const std::type_info &  passed
    )
    {
    if( expected != passed )
        {
        _unexpected_argument_class_type( expected, passed );
        }
    }


bool RPCStubsBase::_require_reply() const
    {
    return ( _flags & RPC_RESULTS ) ? true : false;
    }


bool RPCStubsBase::_no_archive() const
    {
    return ( _flags & RPC_NO_ARCHIVE ) ? true : false;
    }


} // namespace Massiv::Core
} // namespace Massiv
