/** @file color.cpp
 * RGBA floating-point color (implementation).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "property/color.h"
#include "property/property.h"
#include "utility/debug.h"
#include <sstream>


namespace Massiv {
namespace Core {


const Float32 ColorData::FloatQTraits::min_value = 0.0f;
const Float32 ColorData::FloatQTraits::max_value = 1.0f;


template< class Base >
Color< Base >::Color
    (
    Name                                   name,
    typename Color< Base >::Component      alpha
    )
    {
    write_requested();

    switch( name )
        {
        case BLACK:
            rgba.comp[ 0 ] = 0.0f; rgba.comp[ 1 ] = 0.0f; rgba.comp[ 2 ] = 0.0f;
            break;

        case WHITE:
            rgba.comp[ 0 ] = 1.0f; rgba.comp[ 1 ] = 1.0f; rgba.comp[ 2 ] = 1.0f;
            break;

        case RED:
            rgba.comp[ 0 ] = 1.0f; rgba.comp[ 1 ] = 0.0f; rgba.comp[ 2 ] = 0.0f;
            break;

        case GREEN:
            rgba.comp[ 0 ] = 0.0f; rgba.comp[ 1 ] = 1.0f; rgba.comp[ 2 ] = 0.0f;
            break;

        case BLUE:
            rgba.comp[ 0 ] = 0.0f; rgba.comp[ 1 ] = 0.0f; rgba.comp[ 2 ] = 1.0f;
            break;

        case YELLOW:
            rgba.comp[ 0 ] = 1.0f; rgba.comp[ 1 ] = 1.0f; rgba.comp[ 2 ] = 0.0f;
            break;

        case MAGENTA:
            rgba.comp[ 0 ] = 1.0f; rgba.comp[ 1 ] = 0.0f; rgba.comp[ 2 ] = 1.0f;
            break;

        case CYAN:
            rgba.comp[ 0 ] = 0.0f; rgba.comp[ 1 ] = 1.0f; rgba.comp[ 2 ] = 1.0f;
            break;

        default:
            panic( "Bad color name" );
        }

    rgba.comp[ 3 ] = alpha;
    }


template< class Base >
std::string Color< Base >::to_string() const
    {
    std::ostringstream oss;
    oss << "( " << red() << ", " << green() << ", " << blue() << ", " << alpha() << " )";
    return oss.str();
    }


} // namespace Massiv::Core
} // namespace Massiv


#ifndef ENABLE_INLINE
#include "property/color_inline.h"
#endif


// Instantiate templates - stype and property.

namespace Massiv {
namespace Core {

template class MASSIV_EXPORTED Color<>;
template class MASSIV_EXPORTED Color< Property >;

} // namespace Massiv::Core
} // namespace Massiv
