/** @file color_inline.h
 * RGBA floating-point color (inline methods).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TEXT_WRITER_H
#include "utility/text_writer.h"
#endif


namespace Massiv {
namespace Core {


template< class Base >
Color< Base >::Color()
    {
    rgba.comp[ 0 ] = rgba.comp[ 1 ] = rgba.comp[ 2 ] = rgba.comp[ 3 ] = 0.0f;
    default_constructed();
    }


template< class Base >
Color< Base >::Color
    (
    Component   red,
    Component   green,
    Component   blue,
    Component   alpha
    )
    {
    rgba.comp[ 0 ] = red;
    rgba.comp[ 1 ] = green;
    rgba.comp[ 2 ] = blue;
    rgba.comp[ 3 ] = alpha;
    }


template< class Base >
Color< Base >::Color
    (
    const ColorData & cd
    )
    {
    int i;
    
    for( i = 0; i < 4; i++ )
        {
        rgba.comp[ i ] = cd.comp[ i ];
        }
    }


template< class Base >
Color< Base >::Color
    (
    const Color & other
    ) : Super( other )
    {
    other.read_requested();
    rgba.comp[ 0 ] = other.rgba.comp[ 0 ];
    rgba.comp[ 1 ] = other.rgba.comp[ 1 ];
    rgba.comp[ 2 ] = other.rgba.comp[ 2 ];
    rgba.comp[ 3 ] = other.rgba.comp[ 3 ];
    }


template< class Base >
Color< Base > & Color< Base >::operator=
    (
    const Color & other
    )
    {
    write_requested();
    other.read_requested();
    rgba.comp[ 0 ] = other.rgba.comp[ 0 ];
    rgba.comp[ 1 ] = other.rgba.comp[ 1 ];
    rgba.comp[ 2 ] = other.rgba.comp[ 2 ];
    rgba.comp[ 3 ] = other.rgba.comp[ 3 ];
    return *this;
    }


template< class Base >
Color< Base > & Color< Base >::operator=
    (
    const ColorData & cd
    )
    {
    int i;

    write_requested();

    for( i = 0; i < 4; i++ )
        {
        rgba.comp[ i ] = cd.comp[ i ];
        }

    return *this;
    }


template< class Base >        
Color< Base >::operator const ColorData & () const
    {
    return read();
    } 


template< class Base >
typename Color< Base >::Component Color< Base >::red() const
    {
    read_requested();
    return rgba.comp[ 0 ];
    }


template< class Base >
typename Color< Base >::Component Color< Base >::green() const
    {
    read_requested();
    return rgba.comp[ 1 ];
    }


template< class Base >
typename Color< Base >::Component Color< Base >::blue() const
    {
    read_requested();
    return rgba.comp[ 2 ];
    }


template< class Base >
typename Color< Base >::Component Color< Base >::alpha() const
    {
    read_requested();
    return rgba.comp[ 3 ];
    }


template< class Base >
const ColorData & Color< Base >::read() const
    {
    read_requested();
    return rgba;
    };


template< class Base >
void Color< Base >::set_red
    (
    Component r
    )
    {
    write_requested();
    rgba.comp[ 0 ] = r;
    }


template< class Base >
void Color< Base >::set_green
    (
    Component g
    )
    {
    write_requested();
    rgba.comp[ 1 ] = g;
    }


template< class Base >
void Color< Base >::set_blue
    (
    Component b
    )
    {
    write_requested();
    rgba.comp[ 2 ] = b;
    }


template< class Base >
void Color< Base >::set_alpha
    (
    Component a
    )
    {
    write_requested();
    rgba.comp[ 3 ] = a;
    }


template< class Base >
bool Color< Base >::operator==
    (
    const Color & other
    ) const
    {
    read_requested();
    other.read_requested();
    return ( ( rgba.comp[ 0 ] == other.rgba.comp[ 0 ] ) &&
             ( rgba.comp[ 1 ] == other.rgba.comp[ 1 ] ) &&
             ( rgba.comp[ 2 ] == other.rgba.comp[ 2 ] ) &&
             ( rgba.comp[ 3 ] == other.rgba.comp[ 3 ] ) );
    }


template< class Base >
bool Color< Base >::operator!=
    (
    const Color & other
    ) const
    {
    return !( *this == other );
    }


template< class Base >
bool Color< Base >::operator<
    (
    const Color & other
    ) const
    {
    read_requested();
    other.read_requested();

    if( rgba.comp[ 0 ] == other.rgba.comp[ 0 ] )
        {
        if( rgba.comp[ 1 ] == other.rgba.comp[ 1 ] )
            {
            if( rgba.comp[ 2 ] == other.rgba.comp[ 2 ] )
                {
                return ( rgba.comp[ 3 ] < other.rgba.comp[ 3 ] );
                }
            else
                {
                return ( rgba.comp[ 2 ] < other.rgba.comp[ 2 ] );
                }
            }
        else
            {
            return ( rgba.comp[ 1 ] < other.rgba.comp[ 1 ] );
            }
        }
    else
        {
        return ( rgba.comp[ 0 ] < other.rgba.comp[ 0 ] );
        }
    }


template< class Base >
Color< Base > & Color< Base >::operator+=
    (
    const Color & other
    )
    {
    write_requested();
    other.read_requested();
    rgba.comp[ 0 ] += other.rgba.comp[ 0 ];
    rgba.comp[ 1 ] += other.rgba.comp[ 1 ];
    rgba.comp[ 2 ] += other.rgba.comp[ 2 ];
    rgba.comp[ 3 ] += other.rgba.comp[ 3 ];
    return *this;
    }


template< class Base >
Color< Base > & Color< Base >::operator-=
    (
    const Color & other
    )
    {
    write_requested();
    other.read_requested();
    rgba.comp[ 0 ] -= other.rgba.comp[ 0 ];
    rgba.comp[ 1 ] -= other.rgba.comp[ 1 ];
    rgba.comp[ 2 ] -= other.rgba.comp[ 2 ];
    rgba.comp[ 3 ] -= other.rgba.comp[ 3 ];
    return *this;
    }


template< class Base >
Color< Base > & Color< Base >::operator*=
    (
    Component scalar
    )
    {
    write_requested();
    rgba.comp[ 0 ] *= scalar;
    rgba.comp[ 1 ] *= scalar;
    rgba.comp[ 2 ] *= scalar;
    rgba.comp[ 3 ] *= scalar;
    return *this;
    }


template< class Base >
Color< Base > & Color< Base >::operator*=
    (
    const Color & other
    )
    {
    write_requested();
    other.read_requested();
    rgba.comp[ 0 ] *= other.rgba.comp[ 0 ];
    rgba.comp[ 1 ] *= other.rgba.comp[ 1 ];
    rgba.comp[ 2 ] *= other.rgba.comp[ 2 ];
    rgba.comp[ 3 ] *= other.rgba.comp[ 3 ];
    return *this;
    }


template< class Base >
Color< Base > Color< Base >::operator+
    (
    const Color & other
    ) const
    {
    Color result( *this );
    result += other;
    return result;
    }


template< class Base >
Color< Base > Color< Base >::operator-
    (
    const Color & other
    ) const
    {
    Color result( *this );
    result -= other;
    return result;
    }


template< class Base >
Color< Base > Color< Base >::operator*
    (
    Component scalar
    ) const
    {
    Color result( *this );
    result *= scalar;
    return result;
    }


template< class Base >
Color< Base > Color< Base >::operator*
    (
    const Color & other
    ) const
    {
    Color result( *this );
    result *= other;
    return result;
    }


template< class Base >
Color< Base > & Color< Base >::add_rgb
    (
    const Color & other
    )
    {
    write_requested();
    other.read_requested();
    rgba.comp[ 0 ] += other.rgba.comp[ 0 ];
    rgba.comp[ 1 ] += other.rgba.comp[ 1 ];
    rgba.comp[ 2 ] += other.rgba.comp[ 2 ];
    return *this;
    }


template< class Base >
Color< Base > & Color< Base >::subtract_rgb
    (
    const Color & other
    )
    {
    write_requested();
    other.read_requested();
    rgba.comp[ 0 ] -= other.rgba.comp[ 0 ];
    rgba.comp[ 1 ] -= other.rgba.comp[ 1 ];
    rgba.comp[ 2 ] -= other.rgba.comp[ 2 ];
    return *this;
    }


template< class Base >
Color< Base > & Color< Base >::multiply_rgb
    (
    Component scalar
    )
    {
    write_requested();
    rgba.comp[ 0 ] *= scalar;
    rgba.comp[ 1 ] *= scalar;
    rgba.comp[ 2 ] *= scalar;
    return *this;
    }


template< class Base >
Color< Base > & Color< Base >::multiply_rgb
    (
    const Color & other
    )
    {
    write_requested();
    other.read_requested();
    rgba.comp[ 0 ] *= other.rgba.comp[ 0 ];
    rgba.comp[ 1 ] *= other.rgba.comp[ 1 ];
    rgba.comp[ 2 ] *= other.rgba.comp[ 2 ];
    return *this;
    }


template< class Base >
Color< Base > & Color< Base >::scale
    (
    Component scalar
    )
    {
    write_requested();
    rgba.comp[ 0 ] = 0.5f + ( scalar * ( rgba.comp[ 0 ] - 0.5f ) );
    rgba.comp[ 1 ] = 0.5f + ( scalar * ( rgba.comp[ 1 ] - 0.5f ) );
    rgba.comp[ 2 ] = 0.5f + ( scalar * ( rgba.comp[ 2 ] - 0.5f ) );
    rgba.comp[ 3 ] = 0.5f + ( scalar * ( rgba.comp[ 3 ] - 0.5f ) );
    return *this;
    }


template< class Base >
Color< Base > & Color< Base >::scale_rgb
    (
    Component scalar
    )
    {
    write_requested();
    rgba.comp[ 0 ] = 0.5f + ( scalar * ( rgba.comp[ 0 ] - 0.5f ) );
    rgba.comp[ 1 ] = 0.5f + ( scalar * ( rgba.comp[ 1 ] - 0.5f ) );
    rgba.comp[ 2 ] = 0.5f + ( scalar * ( rgba.comp[ 2 ] - 0.5f ) );
    return *this;
    }


template< class Base >
typename Color< Base >::Component Color< Base >::intensity() const
    {
    read_requested();

    // Constants stolen from unknown RGB->YCbCr convertor long long ago.

    return ( 0.29900f * rgba.comp[ 0 ] ) +
           ( 0.58700f * rgba.comp[ 1 ] ) +
           ( 0.11400f * rgba.comp[ 2 ] );
    }


template< class Base >
void Color< Base >::bitstr_write
    (
    BitOStream &                    bos,
    const SerializerDescription &   desc
    ) const
    {
    read_requested();
    rgba.comp[ 0 ].bitstr_write( bos, desc );
    rgba.comp[ 1 ].bitstr_write( bos, desc );
    rgba.comp[ 2 ].bitstr_write( bos, desc );
    rgba.comp[ 3 ].bitstr_write( bos, desc );
    }


template< class Base >
void Color< Base >::bitstr_read
    (
    BitIStream &                    bis,
    const SerializerDescription &   desc
    )
    {
    write_requested();
    rgba.comp[ 0 ].bitstr_read( bis, desc );
    rgba.comp[ 1 ].bitstr_read( bis, desc );
    rgba.comp[ 2 ].bitstr_read( bis, desc );
    rgba.comp[ 3 ].bitstr_read( bis, desc );
    }


template< class Base >
void Color< Base >::text_write
    (
    TextWriter &                    tw,
    const SerializerDescription &   desc
    ) const
    {
    read_requested();
    rgba.comp[ 0 ].text_write( tw, desc );
    tw.write_space();
    rgba.comp[ 1 ].text_write( tw, desc );
    tw.write_space();
    rgba.comp[ 2 ].text_write( tw, desc );
    tw.write_space();
    rgba.comp[ 3 ].text_write( tw, desc );
    }


template< class Base >
void Color< Base >::text_read
    (
    TextReader &                    tr,
    const SerializerDescription &   desc
    )
    {
    write_requested();
    rgba.comp[ 0 ].text_read( tr, desc );
    rgba.comp[ 1 ].text_read( tr, desc );
    rgba.comp[ 2 ].text_read( tr, desc );
    rgba.comp[ 3 ].text_read( tr, desc );
    }


template< class Base >
int Color< Base >::hash() const
    {
    // Only insane people would use Color as a hash key.
    // If they want to do so, they should change this method.

    return 0;
    }


template< class Base >
int Color< Base >::memory_size() const
    {
    return sizeof( *this );
    }


template< class Base >
int Color< Base >::bitstream_size() const
    {
    return rgba.comp[ 0 ].bitstream_size() +
        rgba.comp[ 1 ].bitstream_size() +
        rgba.comp[ 2 ].bitstream_size() +
        rgba.comp[ 3 ].bitstream_size();
    }


} // namespace Massiv::Core
} // namespace Massiv
