/** @file prop_bool.h
 * Boolean property.
 */

/*
 * begin        : Mon Apr 8 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_PROP_BOOL_H
#define MASSIV_PROP_BOOL_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_PROPERTY_H
#include "property/property.h"
#endif

#ifndef MASSIV_ATTRIBUTE_H
#include "property/attribute.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif

#include <string>


namespace Massiv {
namespace Core {


/** @addtogroup property
 * @{
 */

/** @class PBool
 * Boolean property.
 */
class MASSIV_EXPORTED PBool : public BooleanAttribute<Property>
    {
    typedef BooleanAttribute<Property> Super;

public:

    /** Default constructor. */
    INLINE PBool();

    /** Constructor. */
    INLINE PBool
        (
        bool b
        );

    /** Copy constructor. */
    INLINE PBool
        (
        const PBool & b
        );

    /** Operator=. */
    INLINE PBool & operator=
        (
        bool b
        );

    /** Operator=. */
    INLINE PBool & operator=
        (
        const PBool & b
        );

public:

    /**
     */
    virtual std::string to_string() const;
    
    /**
     */
    virtual int hash() const;

    /**
     */
    virtual int memory_size() const;

    /**
     */
    virtual int bitstream_size() const;

public:

    /**
     * Serialize to bitstream.
     */
    virtual INLINE void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description &
        ) const;

    /**
     * Read from bitstream.
     */
    virtual INLINE void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description &
        );

    /**
     * Serialize to text stream.
     * Writes either "true" or "false".
     */
    virtual INLINE void text_write
        (
        TextWriter &                    tw,
        const Serializer::Description &
        ) const;

    /**
     * Read from text stream.
     */
    virtual INLINE void text_read
        (
        TextReader &                    tr,
        const Serializer::Description &
        );

    }; // class PBool

/** @} */ // addtogroup property


} // namespace Massiv::Core
} // namespace Massiv


#ifdef ENABLE_INLINE
#include "property/prop_bool_inline.h"
#endif


#endif // MASSIV_PROP_BOOL_H
