/** @file prop_event_handle.h
 * Event handle property.
 */

/*
 * begin        : Sat Mar 23 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_PROP_EVENT_HANDLE_H
#define MASSIV_PROP_EVENT_HANDLE_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_PROPERTY_H
#include "property/property.h"
#endif

#ifndef MASSIV_ATTRIBUTE_H
#include "property/attribute.h"
#endif

#ifndef MASSIV_EVENT_HANDLE_H
#include "object/event_handle.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif

#include <string>


namespace Massiv {
namespace Core {


/** @addtogroup property
 * @{
 */

/** @class PEventHandle
 * Event handle property.
 */
class MASSIV_EXPORTED PEventHandle : public ManagedAttribute<EventHandle, Property>
    {
    typedef ManagedAttribute<EventHandle, Property> Super;

public:

    /**
     * Default constructor.
     */
    INLINE PEventHandle();

    /**
     * Constructor.
     */
    INLINE PEventHandle
        (
        const EventHandle & event_handle
        );

    /**
     * Copy constructor.
     */
    INLINE PEventHandle
        (
        const PEventHandle & event_handle
        );

    /**
     * Operator=.
     */
    INLINE PEventHandle & operator=
        (
        const EventHandle & event_handle
        );

    /**
     * Operator=.
     */
    INLINE PEventHandle & operator=
        (
        const PEventHandle & event_handle
        );

public:

    /**
     */
    virtual std::string to_string() const;
    
    /**
     */
    virtual int hash() const;

    /**
     */
    virtual int memory_size() const;

    /**
     */
    virtual int bitstream_size() const;

public:

    /**
     * Kill the event this handle points to.
     * @note See @c EventHandle for more detailed description.
     */
    INLINE void kill();

    /**
     * Clear handle content.
     * @note See @c EventHandle for more detailed description.
     */
    INLINE void clear();

public:

    /**
     * Serialize to bitstream.
     */
    virtual INLINE void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description & desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual INLINE void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description & desc
        );

    /**
     * Serialize to text stream.
     * Uses EventHandle serializer.
     */
    virtual INLINE void text_write
        (
        TextWriter &                    tw,
        const Serializer::Description & desc
        ) const;

    /**
     * Read from text stream.
     */
    virtual INLINE void text_read
        (
        TextReader &                    tr,
        const Serializer::Description & desc
        );

    }; // PEventHandle

/** @} */ // addtogroup property


} // namespace Massiv::Core
} // namespace Massiv


#ifdef ENABLE_INLINE
#include "property/prop_event_handle_inline.h"
#endif


#endif // MASSIV_PROP_EVENT_HANDLE_H
