/** @file prop_string.h
 * String property.
 */

/*
 * begin        : Sun Oct 20 2002
 * copyright    : (C) 2002 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_PROP_STRING_H
#define MASSIV_PROP_STRING_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_PROPERTY_H
#include "property/property.h"
#endif

#ifndef MASSIV_ATTRIBUTE_H
#include "property/attribute.h"
#endif

#include <string>


namespace Massiv {
namespace Core {


/** @addtogroup property
 * @{
 */

/** @class PString
 * String property.
 */
class MASSIV_EXPORTED PString : public StringAttribute<Property>
    {
private:

    typedef StringAttribute<Property> Super;

public:

    /** Default constructor. */
    INLINE PString();

    /** Constructor. */
    INLINE PString
        (
        const std::string & value
        );

    /** Copy constructor. */
    INLINE PString
        (
        const PString & s
        );

    /** Operator=. */
    INLINE PString & operator=
        (
        const std::string & value
        );

    /** Operator=. */
    INLINE PString & operator=
        (
        const PString & s
        );

public:

    /**
     */
    virtual std::string to_string() const;

    /**
     */
    virtual int hash() const;
    
    /**
     */
    virtual int memory_size() const;

    /**
     */
    virtual int bitstream_size() const;

public:

    /**
     * Serialize to bitstream.
     */
    virtual INLINE void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description &
        ) const;

    /**
     * Read from bitstream.
     */
    virtual INLINE void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description & desc
        );

    /**
     * Serialize to text stream.
     */
    virtual INLINE void text_write
        (
        TextWriter &                    tw,
        const Serializer::Description & desc
        ) const;

    /**
     * Read from text stream.
     */
    virtual INLINE void text_read
        (
        TextReader &                    tr,
        const Serializer::Description & desc
        );

    }; // class PString

/** @} */ // addtogroup property


} // namespace Massiv::Core
} // namespace Massiv


#ifdef ENABLE_INLINE
#include "property/prop_string_inline.h"
#endif


#endif // MASSIV_PROP_STRING_H
