/** @file prop_time_inline.h
 * Time property implementation.
 */

/*
 * begin        : Fri Mar 22 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Massiv {
namespace Core {


template <class VTime>
PVirtualTime<VTime>::PVirtualTime()
    {
    }


template <class VTime>
PVirtualTime<VTime>::PVirtualTime
    (
    const VTime & time
    ) : Super( time )
    {
    }


template <class VTime>
PVirtualTime<VTime>::PVirtualTime
    (
    const ThisProperty & property
    ) : Super( property )
    {
    }


template <class VTime>
PVirtualTime<VTime> & PVirtualTime<VTime>::operator=
    (
    const VTime & time
    )
    {
    Super::operator=( time );
    return *this;
    }


template <class VTime>
PVirtualTime<VTime> & PVirtualTime<VTime>::operator=
    (
    const ThisProperty & property
    )
    {
    Super::operator=( property.read() );
    return *this;
    }


template <class VTime>
PVirtualTime<VTime> & PVirtualTime<VTime>::operator+=
    (
    const VTime & time
    )
    {
    VTime t( read() );
    t+= time;
    write( t );
    return *this;
    }


template <class VTime>
PVirtualTime<VTime> & PVirtualTime<VTime>::operator-=
    (
    const VTime & time
    )
    {
    VTime t( read() );
    t-= time;
    write( t );
    return *this;
    }


template <class VTime>
void PVirtualTime<VTime>::advance()
    {
    VTime t( read() );
    t.advance();
    write( t );
    }


template <class VTime>
void PVirtualTime<VTime>::invalidate()
    {
    VTime t;
    t.invalidate();
    write( t );
    }


template <class VTime>
bool PVirtualTime<VTime>::is_equal_to
    (
    const VTime & time
    ) const
    {
    const VTime & t = read();
    return t.is_equal_to( time );
    }


template <class VTime>
bool PVirtualTime<VTime>::is_older_than
    (
    const VTime & time
    ) const
    {
    const VTime & t = read();
    return t.is_older_than( time );
    }


template <class VTime>
void PVirtualTime<VTime>::bitstr_write
    (
    BitOStream &        bos,
    const Description & desc
    ) const
    {
    read().bitstr_write( bos, desc );
    }


template <class VTime>
void PVirtualTime<VTime>::bitstr_read
    (
    BitIStream &        bis,
    const Description & desc
    )
    {
    VTime t;
    t.bitstr_read( bis, desc );
    write( t );
    }


template <class VTime>
void PVirtualTime<VTime>::text_write
    (
    TextWriter &        tw,
    const Description & desc
    ) const
    {
    read().text_write( tw, desc );
    }


template <class VTime>
void PVirtualTime<VTime>::text_read
    (
    TextReader &        tr,
    const Description & desc
    )
    {
    VTime t;
    t.text_read( tr, desc );
    write( t );
    }


} // namespace Massiv::Core
} // namespace Massiv
