/** @file replica_group_set.cpp
 * Replication group enumaraion (implementation).
 */

/*
 * begin        : Dec 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "replication/replica_group_set.h"
#include "global/global.h"
#include "object/object_manager.h"


namespace Massiv {
namespace Core {


ReplicationGroupSet::ReplicationGroupSet
    (
    int the_repflags_yes,
    int the_repflags_no
    ) : repflags_yes( the_repflags_yes ),
        repflags_no( the_repflags_no )
    {
    }


ReplicationGroupSet::~ReplicationGroupSet()
    {
    }


void ReplicationGroupSet::operator()
    (
    ObjectProperty * object
    )
    {
    hope( object );
    object_ids.insert( object->get_object_id() );
    }


void ReplicationGroupSet::add_replication_group
    (
    ObjectProperty * object
    )
    {
    hope( object );
    if( object_ids.find( object->get_object_id() ) == object_ids.end() )
        {
        Global::object_manager().gc.enumerate_replication_group
            ( object, repflags_yes, repflags_no, *this );
        }
    }


void ReplicationGroupSet::add_referenced_objects
    (
    ObjectProperty * object
    )
    {
    hope( object );
    if( object_ids.find( object->get_object_id() ) == object_ids.end() )
        {
        Global::object_manager().gc.enumerate_replication_group
            ( object, repflags_yes, repflags_no, *this, GarbageCollector::POINTER_FORWARD );
        }
    }


void ReplicationGroupSet::add_referencee_objects
    (
    ObjectProperty * object
    )
    {
    hope( object );
    if( object_ids.find( object->get_object_id() ) == object_ids.end() )
        {
        Global::object_manager().gc.enumerate_replication_group
            ( object, repflags_yes, repflags_no, *this, GarbageCollector::POINTER_BACKWARD );
        }
    }


void ReplicationGroupSet::add_registered_replicas()
    {
    Global::object_manager().gc.enumerate_replicas( *this );
    }


} // namespace Massiv::Core
} // namespace Massiv


#ifndef ENABLE_INLINE
#include "replica_group_set_inline.h"
#endif
