/** @file replica_group_set.h
 * Replication group enumaraion.
 */

/*
 * begin        : Dec 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_REPLICA_GROUP_SET_H
#define MASSIV_REPLICA_GROUP_SET_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_REPLICATION_FWD_H
#include "replication/replication_fwd.h"
#endif

#ifndef MASSIV_GARBAGE_COLLECTOR_H
#include "pointer/garbage_collector.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup replication
 * @{
 */

/**
 * Object operation used to gather set of objects to replicate.
 */
class ReplicationGroupSet : public GarbageCollector::ObjectOperation
    {
public:

    /** Constructor. */
    ReplicationGroupSet
        (
        int the_repflags_yes,
        int the_repflags_no
        );

    /** Destructor. */
    virtual ~ReplicationGroupSet();

protected:

    /** Operation. */
    virtual void operator()
        (
        ObjectProperty * object
        );

public:

    /**
     * Add replication group of given @a object to the set.
     */
    void add_replication_group
        (
        ObjectProperty * object
        );

    /**
     * Add objects reachable via pointers matching given repflags from @a object to the set.
     */
    void add_referenced_objects
        (
        ObjectProperty * object
        );

    /**
     * Add objects reachable via backward pointers matching given repflags from @a object
     * to the set (add each object that can reach @a object via matching pointers).
     */
    void add_referencee_objects
        (
        ObjectProperty * object
        );

    /**
     * Add all replicas registered on local node to the set.
     * Replication flags are ignored in this case.
     */
    void add_registered_replicas();

public:

    /** Set of object pointers. */
    typedef ObjectShared::ObjectIdSet ObjectIdSet;

    ObjectIdSet     object_ids;
        /**< Set of ids of gathered objects. */

protected:

    /** Replication flags. */
    //@{
    int             repflags_yes;
    int             repflags_no;
    //@}

    }; // class ReplicationGroupSet

/** @} */ // addtogroup replication


} // namespace Massiv::Core
} // namespace Massiv


#ifdef ENABLE_INLINE
#include "replica_group_set_inline.h"
#endif


#endif // MASSIV_REPLICA_GROUP_SET_H
