/** @file replica_request.h
 * Information about object replicas requested by single node.
 */

/*
 * begin        : Fri Jan 24 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_REPLICA_REQUEST_H
#define MASSIV_REPLICA_REQUEST_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_HASH_H
#include "utility/hash.h"
#endif

#ifndef MASSIV_REPLICATION_FWD_H
#include "replication/replication_fwd.h"
#endif

#ifndef MASSIV_SERIALIZER_INT_H
#include "serializer/serializer_int.h" // must be included soon to prevent MSVC6 internal error
#endif

#ifndef MASSIV_STYPE_TIME_H
#include "serializer/stype_time.h"
#endif

#ifndef MASSIV_OBJECTID_H
#include "pointer/objectid.h"
#endif




namespace Massiv {
namespace Core {


/** @addtogroup replication
 * @{
 */

/**
 * Information about each object replica request.
 */
class ReplicaRequest
    {
public:

    /**
     * Hash map from ObjectId to ReplicaRequest.
     */
    typedef ext::hash_map< ObjectId, ReplicaRequest,
        ObjectId::Hash, ObjectId::EqualKey > HashMap;

public:

    /** Default constructor. */
    INLINE ReplicaRequest();

    /** Constructor. */
    INLINE ReplicaRequest
        (
        const STime & the_timeout
        );

public:

    STime           timeout;
        /**< Destroy replica when simulation time reaches #timeout. */

    }; // class ReplicaRequest


/**
 * ReplicaRequests contains information about all replica requests for
 * single node.
 */
class ReplicaRequests : public ReplicaRequest::HashMap
    {
public:

    /** Default constructor. */
    INLINE ReplicaRequests();

    }; // class ReplicaRequests

/** @} */ // addtogroup replication


} // namespace Massiv::Core
} // namespace Massiv


#ifdef ENABLE_INLINE
#include "replica_request_inline.h"
#endif


#endif // MASSIV_REPLICA_REQUEST_H
