/** @file replica_server_stats.cpp
 * ReplicaServer statistics (implementation).
 */

/*
 * begin        : Dec 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "replication/replica_server_stats.h"
#include <sstream>


namespace Massiv {
namespace Core {


namespace {


/**
 * Get statistics name from node id.
 */
std::string statistics_name
    (
    const NodeId & node_id
    )
    {
    std::ostringstream ss;
    ss << "Replication/Server/" << node_id.to_identifier();
    return ss.str();
    }


} // anonymous namespace


// class ReplicaServerNodeStatistics

ReplicaServerNodeStatistics::ReplicaServerNodeStatistics
    (
    const NodeId & node_id
    ) : Statistics
            (
            statistics_name( node_id ).c_str(),
            Status::FACILITY_REPLICATION,
            Statistics::OPEN_OVERWRITE
            ),
        packet_loss         ( create_float( "packet_loss",      0.0f ) ),
        packet_loss_max     ( create_float( "packet_loss_max",  0.0f ) ),
        ping                ( create_float( "ping",             0.0f ) ),
        ping_max            ( create_float( "ping_max",         0.0f ) ),
        wait_limit          ( create_float( "wait_limit",       0.0f ) ),
        update_period       ( create_float( "update_period",    0.0f ) ),
        updates_sent        ( create_integer( "updates_sent",       0 ) ),
        acks_received       ( create_integer( "acks_received",      0 ) ),
        acks_ignored        ( create_integer( "acks_ignored",       0 ) ),
        backlog_length      ( create_integer( "backlog_length",     0 ) ),
        backlog_length_max  ( create_integer( "backlog_length_max", 0 ) ),
        backlog_overflows   ( create_integer( "backlog_overflows",  0 ) ),
        request_count       ( create_integer( "request_count",      0 ) ),
        request_count_max   ( create_integer( "request_count_max",  0 ) ),
        replica_count       ( create_integer( "replica_count",      0 ) ),
        replica_count_max   ( create_integer( "replica_count_max",  0 ) ),
        updates_sent_tcp    ( create_integer( "updates_sent_tcp",   0 ) )
    {
    }


ReplicaServerNodeStatistics::~ReplicaServerNodeStatistics()
    {
    }


// class ReplicaServerStatistics

ReplicaServerStatistics::ReplicaServerStatistics()
      : Statistics
            (
            "Replication/Server",
            Status::FACILITY_REPLICATION,
            Statistics::OPEN_NORMAL
            ),
        node_count          ( create_integer( "node_count",         0 ) ),
        node_count_max      ( create_integer( "node_count_max",     0 ) ),
        request_count       ( create_integer( "request_count",      0 ) ),
        request_count_max   ( create_integer( "request_count_max",  0 ) ),
        replica_count       ( create_integer( "replica_count",      0 ) ),
        replica_count_max   ( create_integer( "replica_count_max",  0 ) ),
        updates_sent        ( create_integer( "updates_sent",       0 ) ),
        updates_sent_tcp    ( create_integer( "updates_sent_tcp",   0 ) ),
        acks_received       ( create_integer( "acks_received",      0 ) ),
        acks_ignored        ( create_integer( "acks_ignored",       0 ) )
    {
    }


ReplicaServerStatistics::~ReplicaServerStatistics()
    {
    }


} // namespace Massiv::Core
} // namespace Massiv
