/** @file replica_server_stats.h
 * ReplicaServer statistics.
 */

/*
 * begin        : Dec 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_REPLICA_SERVER_STATS_H
#define MASSIV_REPLICA_SERVER_STATS_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_REPLICATION_FWD_H
#include "replication/replication_fwd.h"
#endif

#ifndef MASSIV_NODEID_H
#include "node/nodeid.h"
#endif

#ifndef MASSIV_STATISTICS_H
#include "registry/statistics.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup replication
 * @{
 */

/**
 * Information about updates to single node.
 */
class ReplicaServerNodeStatistics : public Statistics
    {
public:

    /**
     * Constructor.
     */
    ReplicaServerNodeStatistics
        (
        const NodeId & node_id
        );

    /**
     * Destructor.
     */
    ~ReplicaServerNodeStatistics();

public:

    /** Information provided/used by ReplicaUpdateBacklog. */
    //@{

    Float &     packet_loss;
        /**< Packet loss: 0 = no packets lost, 1 = all packets lost. */

    Float &     packet_loss_max;
        /**< Maximal packet loss. */

    Float &     ping;
        /**< Average update-ack ping. */

    Float &     ping_max;
        /**< Maximal update-ack ping. */

    Float &     wait_limit;
        /**< Something weird happened if waitinig for ack for wait_limit seconds. */

    Float &     update_period;
        /**< Current update period. */

    Integer &   updates_sent;
        /**< Number of update packets sent. */

    Integer &   acks_received;
        /**< Number of acks received, both valid and invalid. */

    Integer &   acks_ignored;
        /**< Number of invalid acks received. */

    Integer &   backlog_length;
        /**< Current backlog length. */

    Integer &   backlog_length_max;
        /**< Maximal backlog length. */

    Integer &   backlog_overflows;
        /**< Number of backlog overflows. */

    //@}

public:

    /** Information provided per-node by ReplicaServer. */
    //@{

    Integer &   request_count;
        /**< Number of replication requests to the node. */

    Integer &   request_count_max;
        /**< Maximal number of replication requests to the node. */

    Integer &   replica_count;
        /**< Estimated number of living replicas created by this ReplicaServer on the node. */

    Integer &   replica_count_max;
        /**< Maximal number of living replicas created by this ReplicaServer on the node. */

    Integer &   updates_sent_tcp;
        /**< How many updates, of #updates_sent, have been sent via TCP instead of UDP. */

    //@}

    }; // class ReplicaServerStatistics


/**
 * Global ReplicaServer statistics.
 */
class ReplicaServerStatistics : public Statistics
    {
public:

    /**
     * Constructor.
     */
    ReplicaServerStatistics();

    /**
     * Destructor.
     */
    ~ReplicaServerStatistics();

public:

    Integer &   node_count;
        /**< Number of nodes requesting or having a replica. */

    Integer &   node_count_max;
        /**< Maximal number of nodes requesting or having a replica. */

    Integer &   request_count;
        /**< Number of replication requests.. */

    Integer &   request_count_max;
        /**< Maximal number of replication requests.. */

    Integer &   replica_count;
        /**< Estimated number of living replicas created by this ReplicaServer. */

    Integer &   replica_count_max;
        /**< Maximal number of living replicas created by this ReplicaServer. */

    Integer &   updates_sent;
        /**< Total number of updates sent. */

    Integer &   updates_sent_tcp;
        /**< How many updates, of #updates_sent, have been sent via TCP instead of UDP. */

    Integer &   acks_received;
        /**< Total number of acks received. */

    Integer &   acks_ignored;
        /**< How many acks, of #acks_received, have been ignored. */

    }; // class ReplicaServerStatistics

/** @} */ // addtogroup replication


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_SERVER_STATS_H
