/** @file replica_time.h
 * Replication timestamp.
 */

/*
 * begin        : Mar 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_REPLICA_TIME_H
#define MASSIV_REPLICA_TIME_H


#ifndef MASSIV_VIRTUAL_TIME_H
#include "utility/virtual_time.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup replication
 * @{
 */

/**
 * Replication timestamp.
 *
 * Used to mark object properties as dirty. It increases every time
 * replication updates are sent from replica server.
 */
class MASSIV_EXPORTED ReplicaTime : public VirtualTime<UInt32>
    {
public:

    typedef VirtualTime<UInt32>     Super;
    typedef UInt32                  TimeType;
    typedef Super::Less             Less;
    typedef Super::EqualTo          EqualTo;

public:

    /** Default constructor. */
    INLINE ReplicaTime();

    /** Constructor. */
    INLINE ReplicaTime
        (
        TimeType t
        );

    /** Copy constructor. */
    INLINE ReplicaTime
        (
        const ReplicaTime & t
        );

    /** Operator=. */
    INLINE ReplicaTime & operator=
        (
        const ReplicaTime & t
        );
    
    /**
     * Serialize to bitstream.
     */
    INLINE void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description &
        ) const;

    /**
     * Read from bitstream.
     */
    INLINE void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description &
        );

    /**
     * Serialize to text stream.
     * Writes TimeType using integer serializer.
     */
    INLINE void text_write
        (
        TextWriter &                    tw,
        const Serializer::Description &
        ) const;

    /**
     * Read from text stream.
     */
    INLINE void text_read
        (
        TextReader &                    tr,
        const Serializer::Description &
        );
    
    };

/** @} */ // addtogroup replication


} // namespace Massiv::Core
} // namespace Massiv

#ifdef ENABLE_INLINE
#include "replica_time_inline.h"
#endif

#endif // MASSIV_REPLICA_TIME_H
