/** @file replica_version_inline.h
 * Structure remembering versions of replicas on a single node.
 */

/*
 * begin        : Fri Jan 31 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Massiv {
namespace Core {


// class ReplicaVersion

ReplicaVersion::ReplicaVersion()
      : state( STATE_INVALID ),
        major_version( 0 )
    {
    }

    
ReplicaVersion::ReplicaVersion
    (
    const ReplicaTime & time,
    ObjectVersion       version
    ) : state( STATE_CREATING ),
        major_version( version ),
        create_time( time )
    {
    }


bool ReplicaVersion::is_state_living() const
    {
    return state == STATE_CREATING || state == STATE_REPLICATING;
    }


bool ReplicaVersion::is_state_destroying() const
    {
    return state == STATE_DESTROYING;
    }


void ReplicaVersion::set_state_creating
    (
    const ReplicaTime & time,
    ObjectVersion       version
    )
    {
    state = STATE_CREATING;
    minor_version = ReplicaTime();
    major_version = version;
    create_time = time;
    }

    
void ReplicaVersion::set_state_destroying
    (
    const ReplicaTime & time
    )
    {
    hope( create_time != ReplicaTime::INVALID );
    state = STATE_DESTROYING;
    destroy_time = time;
    }


// class ReplicaVersions

ReplicaVersions::ReplicaVersions()
    {
    }


std::string ReplicaVersions::ack_name
    (
    AckType             type
    ) const
    {
    switch( type )
        {
    case ACK_UPDATE:
        return "update";
    case ACK_DESTROY:
        return "destroy";
        }
    return "invalid";
    }


} // namespace Massiv::Core
} // namespace Massiv

