/** @file replication_settings.h
 * Replication settings.
 */

/*
 * begin        : Feb 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_REPLICATION_SETTINGS_H
#define MASSIV_REPLICATION_SETTINGS_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_REPLICATION_FWD_H
#include "replication/replication_fwd.h"
#endif

#ifndef MASSIV_NODEID_H
#include "node/nodeid.h"
#endif

#ifndef MASSIV_STATISTICS_H
#include "registry/statistics.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup replication
 * @{
 */

/**
 * Replication settings.
 *
 * These settings are used by both ReplicaServer and ReplicaClient.
 *
 * @todo Replication masks for priviliged client.
 */
class ReplicationSettings : public Settings
    {
public:

    /** Constructor. */
    ReplicationSettings();

public:

    Boolean &   debug;
        /**< Enable debugging logs. */

    Integer &   server_repflags_yes;
        /**< Replication mask for server nodes. */

    Integer &   server_repflags_no;
        /**< Replication mask for server nodes. */

    Integer &   client_repflags_yes;
        /**< Replication mask for client nodes. */

    Integer &   client_repflags_no;
        /**< Replication mask for client nodes. */

    Float &     min_client_update_period;
        /**< Minimal period between two updates to client node, in seconds. */

    Float &     max_client_update_period;
        /**< Maximal period between two updates to client node, in seconds. */

    Float &     min_server_update_period;
        /**< Minimal period between two updates to server node, in seconds. */

    Float &     max_server_update_period;
        /**< Maximal period between two updates to server node, in seconds. */

    Integer &   max_udp_update_size;
        /**< Maximal size of UDP update message in bytes.
             Larger updates are sent via TCP. */

public:

    /**
     * Get replication flags for given node.
     */
    void get_repflags
        (
        int &           repflags_yes,
        int &           repflags_no,
        const NodeId &  node
        ) const;

    /**
     * Get update period range for given node.
     */
    void get_update_periods
        (
        Float32 &       min_period,
        Float32 &       max_period,
        const NodeId &  node
        ) const;

    }; // class ReplicationSettings

/** @} */ // addtogroup replication


} // namespace Massiv::Core
} // namespace Massiv


#ifdef ENABLE_INLINE
#include "replication_settings_inline.h"
#endif


#endif // MASSIV_REPLICATION_SETTINGS_H
