/** @file serializer_bool.h
 * Boolean serializers.
 */

/*
 * begin        : Sun Apr 21 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SERIALIZER_BOOL_H
#define MASSIV_SERIALIZER_BOOL_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif



namespace Massiv {
namespace Core {
namespace Serializer {


/** @addtogroup serialization
 * @{
 */

/** @struct BitStrBool
 * BitStream boolean serializers.
 */
struct MASSIV_EXPORTED BitStrBool
    {

    static SERIALIZER_INLINE void read
        (
        BitIStream &   bis,
        bool &         result
        );

    static SERIALIZER_INLINE void write
        (
        BitOStream &   bos,
        bool           value
        );

    }; // struct BitStrBool



/** @struct TextBool
 * Text boolean serializers.
 */
struct MASSIV_EXPORTED TextBool
    {

    static void read
        (
        TextReader &    tr,
        bool &          value
        );

    static SERIALIZER_INLINE void write
        (
        TextWriter &    tw,
        bool            value
        );

    }; // struct TextBool

/** @} */ // addtogroup serialization


} // namespace Massiv::Core::Serializer
} // namespace Massiv::Core
} // namespace Massiv


#ifdef EXPOSE_SERIALIZERS
#include "serializer/serializer_bool_inline.h"
#endif


#endif // MASSIV_SERIALIZER_BOOL_H
