/** @file serializer_float.h
 * Floating-point serializers.
 */

/*
 * begin        : Thu Feb 21 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SERIALIZER_FLOAT_H
#define MASSIV_SERIALIZER_FLOAT_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif



namespace Massiv {
namespace Core {
namespace Serializer {


/** @addtogroup serialization
 * @{
 */

/** @struct BitStrFloat
 * BitStream floating-point serializers.
 */
struct MASSIV_EXPORTED BitStrFloat
    {

    /** Read 32bit float in little endian byte order. */
    static SERIALIZER_INLINE void read
        (
        BitIStream &    bis,
        Float32 &       result
        );

    /** Read 64bit float in little endian byte order. */
    static SERIALIZER_INLINE void read
        (
        BitIStream &    bis,
        Float64 &       result
        );

    /** Write 32bit float in little endian byte order. */
    static SERIALIZER_INLINE void write
        (
        BitOStream &    bos,
        Float32         value
        );

    /** Write 64bit float in little endian byte order. */
    static SERIALIZER_INLINE void write
        (
        BitOStream &    bos,
        Float64         value
        );

    /** Read quantized float if @a desc.lossy is @c true, full 32bit float otherwise. */
    static void read
        (
        BitIStream &        bis,
        const Description & desc,
        Float32 &           result,
        Float32             min_value,
        Float32             max_value,
        int                 bits
        );
    
    /** Read quantized float if @a desc.lossy is @c true, full 64bit float otherwise. */
    static void read
        (
        BitIStream &        bis,
        const Description & desc,
        Float64 &           result,
        Float64             min_value,
        Float64             max_value,
        int                 bits
        );

    /** Write quantized float if @a desc.lossy is @c true, full 32bit float otherwise. */
    static void write
        (
        BitOStream &        bos,
        const Description & desc,
        Float32             value,
        Float32             min_value,
        Float32             max_value,
        int                 bits
        );
    
    /** Write quantized float if @a desc.lossy is @c true, full 64bit float otherwise. */
    static void write
        (
        BitOStream &        bos,
        const Description & desc,
        Float64             value,
        Float64             min_value,
        Float64             max_value,
        int                 bits
        );

    }; // struct BitStrFloat


/** @struct TextFloat
 * Text floating-point serializers.
 */
struct MASSIV_EXPORTED TextFloat
    {

    static void read
        (
        TextReader &    tr,
        Float32 &       value
        );

    static void read
        (
        TextReader &    tr,
        Float64 &       value
        );

    /**
     * Write float in standard iostream style.
     * @todo Write bits directly as UInt32.
     */
    static SERIALIZER_INLINE void write
        (
        TextWriter &    tw,
        Float32         value
        );

    /**
     * Write float in standard iostream style.
     * @todo Write bits directly as UInt64.
     */
    static SERIALIZER_INLINE void write
        (
        TextWriter &    tw,
        Float64         value
        );

    }; // struct TextFloat

/** @} */ // addtogroup serialization


} // namespace Massiv::Core::Serializer
} // namespace Massiv::Core
} // namespace Massiv


#ifdef EXPOSE_SERIALIZERS
#include "serializer/serializer_float_inline.h"
#endif


#endif // MASSIV_SERIALIZER_FLOAT_H
