/** @file serializer_int.cpp
 * Integer serializers.
 */

/*
 * begin        : Wed Mar 19 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "global/common.h"
#include "utility/types.h"
#include "utility/text_reader.h"
#include "utility/text_writer.h"
#include "net/bitistream.h"
#include "net/bitostream.h"
#include "serializer/bitint_masks.h"
#include "serializer/serializer_int.h"

namespace Massiv {
namespace Core {
namespace Serializer {


template <class Type>
static void read
    (
    TextReader &    tr,
    Type &          value,
    int             bits
    )
    {
    std::istream &  is = tr.line();
    //hope( is );
    is >> value;
    if( is.fail() )
        {
        if( is.eof() )
            throw ExUnexpectedEol( tr.unexpected_eol() );
        else
            throw ExUnexpectedToken( tr.unexpected_token( "integer" ) );
        }
    }


template <class Type>
static void write
    (
    TextWriter &    tw,
    Type            value,
    int             bits
    )
    {
    tw.line() << static_cast<Type>( value & uint64_masks[ bits ] );
    }


void TextInt::read
    (
    TextReader &    tr,
    Int32 &         value,
    int             bits
    )
    {
    Serializer::read( tr, value, bits );
    }


void TextInt::read
    (
    TextReader &    tr,
    Int64 &         value,
    int             bits
    )
    {
    Serializer::read( tr, value, bits );
    }


void TextInt::read
    (
    TextReader &    tr,
    UInt32 &        value,
    int             bits
    )
    {
    Serializer::read( tr, value, bits );
    }


void TextInt::read
    (
    TextReader &    tr,
    UInt64 &        value,
    int             bits
    )
    {
    Serializer::read( tr, value, bits );
    }


void TextInt::write
    (
    TextWriter &    tw,
    Int32           value,
    int             bits
    )
    {
    Serializer::write( tw, value, bits );
    }


void TextInt::write
    (
    TextWriter &    tw,
    Int64           value,
    int             bits
    )
    {
    Serializer::write( tw, value, bits );
    }


void TextInt::write
    (
    TextWriter &    tw,
    UInt32          value,
    int             bits
    )
    {
    Serializer::write( tw, value, bits );
    }


void TextInt::write
    (
    TextWriter &    tw,
    UInt64          value,
    int             bits
    )
    {
    Serializer::write( tw, value, bits );
    }


} // namespace Massiv::Core::Serializer
} // namespace Massiv::Core
} // namespace Massiv


#ifndef EXPOSE_SERIALIZERS
#include "serializer/serializer_int_inline.h"
#endif
