/** @file serializer_int.h
 * Integral serializers.
 */

/*
 * begin        : Thu Feb 21 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SERIALIZER_INT_H
#define MASSIV_SERIALIZER_INT_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif



namespace Massiv {
namespace Core {
namespace Serializer {


/** @addtogroup serialization
 * @{
 */

/** @struct BitStrInt
 * BitStream integer serializers.
 */
struct MASSIV_EXPORTED BitStrInt
    {

    static SERIALIZER_INLINE void read
        (
        BitIStream &    bis,
        Int32 &         result,
        int             bits
        );

    static SERIALIZER_INLINE void read
        (
        BitIStream &    bis,
        UInt32 &        result,
        int             bits
        );

    static SERIALIZER_INLINE void read
        (
        BitIStream &    bis,
        Int64 &         result,
        int             bits
        );

    static SERIALIZER_INLINE void read
        (
        BitIStream &    bis,
        UInt64 &        result,
        int             bits
        );

    static SERIALIZER_INLINE void write
        (
        BitOStream &    bos,
        Int32           value,
        int             bits
        );

    static SERIALIZER_INLINE void write
        (
        BitOStream &    bos,
        UInt32          value,
        int             bits
        );

    static SERIALIZER_INLINE void write
        (
        BitOStream &    bos,
        Int64           value,
        int             bits
        );

    static SERIALIZER_INLINE void write
        (
        BitOStream &    bos,
        UInt64          value,
        int             bits
        );

    }; // struct BitStrInt


/** @struct TextInt
 * Text integer serializers.
 */
struct MASSIV_EXPORTED TextInt
    {

    static void read
        (
        TextReader &    tr,
        Int32 &         value,
        int             bits
        );
    
    static void read
        (
        TextReader &    tr,
        Int64 &         value,
        int             bits
        );
    
    static void read
        (
        TextReader &    tr,
        UInt32 &        value,
        int             bits
        );
    
    static void read
        (
        TextReader &    tr,
        UInt64 &        value,
        int             bits
        );

    static void write
        (
        TextWriter &    tw,
        Int32           value,
        int             bits
        );
    
    static void write
        (
        TextWriter &    tw,
        Int64           value,
        int             bits
        );
    
    static void write
        (
        TextWriter &    tw,
        UInt32          value,
        int             bits
        );
    
    static void write
        (
        TextWriter &    tw,
        UInt64          value,
        int             bits
        );

    }; // struct TextInt

/** @} */ // addtogroup serialization


} // namespace Massiv::Core::Serializer
} // namespace Massiv::Core
} // namespace Massiv

#ifdef EXPOSE_SERIALIZERS
#include "serializer/serializer_int_inline.h"
#endif

#endif // MASSIV_SERIALIZER_INT_H
