/** @file serializer_int_inline.h
 * Integer serializers.
 */

/*
 * begin        : Thu Feb 21 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_BITISTREAM_H
#include "net/bitistream.h"
#endif

#ifndef MASSIV_BITOSTREAM_H
#include "net/bitostream.h"
#endif


namespace Massiv {
namespace Core {
namespace Serializer {


void BitStrInt::read
    (
    BitIStream &    bis,
    Int32 &         result,
    int             bits
    )
    {
    hope( bits > 0 );
    hope( bits <= 32 );
    result = ( Int32 ) bis.read_int( bits );
    }


void BitStrInt::read
    (
    BitIStream &    bis,
    UInt32 &        result,
    int             bits
    )
    {
    hope( bits > 0 );
    hope( bits <= 32 );
    result = bis.read_int( bits );
    }


void BitStrInt::read
    (
    BitIStream &    bis,
    Int64 &         result,
    int             bits
    )
    {
    hope( bits > 0 );
    hope( bits <= 64 );
    result = ( Int64 ) bis.read_int64( bits );
    }


void BitStrInt::read
    (
    BitIStream &    bis,
    UInt64 &        result,
    int             bits
    )
    {
    hope( bits > 0 );
    hope( bits <= 64 );
    result = bis.read_int64( bits );
    }


void BitStrInt::write
    (
    BitOStream &    bos,
    Int32           value,
    int             bits
    )
    {
    hope( bits > 0 );
    hope( bits <= 32 );
    bos.write_int( ( UInt32 ) value, bits );
    }


void BitStrInt::write
    (
    BitOStream &    bos,
    UInt32          value,
    int             bits
    )
    {
    hope( bits > 0 );
    hope( bits <= 32 );
    bos.write_int( value, bits );
    }


void BitStrInt::write
    (
    BitOStream &    bos,
    Int64           value,
    int             bits
    )
    {
    hope( bits > 0 );
    hope( bits <= 64 );
    bos.write_int64( ( UInt64 ) value, bits );
    }


void BitStrInt::write
    (
    BitOStream &    bos,
    UInt64          value,
    int             bits
    )
    {
    hope( bits > 0 );
    hope( bits <= 64 );
    bos.write_int64( value, bits );
    }


} // namespace Massiv::Core::Serializer
} // namespace Massiv::Core
} // namespace Massiv
