/** @file serializer_string.h
 * String serializers.
 */

/*
 * begin        : Sun Oct 20 2002
 * copyright    : (C) 2002 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SERIALIZER_STRING_H
#define MASSIV_SERIALIZER_STRING_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif

#include <string>


namespace Massiv {
namespace Core {
namespace Serializer {


/** @addtogroup serialization
 * @{
 */

/** @struct BitStrString
 * BitStream string serializers.
 */
struct MASSIV_EXPORTED BitStrString
    {

    static SERIALIZER_INLINE void read
        (
        BitIStream &    bis,
        std::string &   result
        );

    static SERIALIZER_INLINE void write
        (
        BitOStream &            bos,
        const std::string &     value
        );

    }; // struct BitStrString


/** @struct TextString
 * Text string serializers.
 * String can contain new-line char.
 */
struct MASSIV_EXPORTED TextString
    {

    static void read
        (
        TextReader &    tr,
        std::string &   value
        );

    static SERIALIZER_INLINE void write
        (
        TextWriter &            tw,
        const std::string &     value
        );

    }; // struct TextString

/** @} */ // addtogroup serialization


} // namespace Massiv::Core::Serializer
} // namespace Massiv::Core
} // namespace Massiv


#ifdef EXPOSE_SERIALIZERS
#include "serializer/serializer_string_inline.h"
#endif


#endif // MASSIV_SERIALIZER_STRING_H
