/** @file serializer_vli.cpp
 * Variable length integer serializers.
 */

/*
 * begin        : Wed Mar 19 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "global/common.h"
#include "utility/types.h"
#include "utility/text_reader.h"
#include "utility/text_writer.h"
#include "serializer/serializer_vli.h"
#include "serializer/bitint_masks.h"
#include "net/bitistream.h"
#include "net/bitostream.h"
#include <sstream>


namespace Massiv {
namespace Core {
namespace Serializer {


void BitStrVli::read
    (
    BitIStream &    bis,
    Int32 &         result
    )
    {
#if 0
    UInt32 bits = 1;

    int sign = bis.read_int( 1 );
    while( bis.read_int( 1 ) != 1 )
        bits++;

    result = ( Int32 ) bis.read_int( bits );
    if( sign == 1 )
        result *= -1;
#else
    const int sign = bis.read_int( 1 );
    UInt32 value;
    read( bis, value );
    if( value & ( 1 << 31 ) )
        {
        throw ExSerializationError( "Unsigned part of 32bit signed integer has more than 31 bits." );
        }
    if( sign )
        {
        result = -static_cast< Int32 >( value );
        }
    else
        {
        result = static_cast< Int32 >( value );
        }
#endif
    }


void BitStrVli::read
    (
    BitIStream &    bis,
    UInt32 &        result
    )
    {
#if 0
    UInt32 bits = 1;
    while( bis.read_int( 1 ) != 1 )
        bits++;

    result = bis.read_int( bits );
#else
    const UInt32 bits = bis.read_unary() + 1;
    if( bits == 0 || bits > 32 )
        {
        std::ostringstream ss;
        ss << "Bad number of bits (" << bits << ") reading unary-coded 32bit integer.";
        throw ExSerializationError( ss.str() );
        }
    result = bis.read_int( bits );
#endif
    }


void BitStrVli::read
    (
    BitIStream &    bis,
    Int64 &         result
    )
    {
#if 0
    UInt32 bits = 1;

    int sign = bis.read_int( 1 );

    while( bis.read_int( 1 ) != 1 )
        bits++;

    result = ( Int64 ) bis.read_int64( bits );
    if( sign == 1 )
        result *= -1;
#else
    const int sign = bis.read_int( 1 );
    UInt64 value;
    read( bis, value );
    if( value & ( UINT64( 1 ) << 63 ) )
        {
        throw ExSerializationError( "Unsigned part of 64bit signed integer has more than 63 bits." );
        }
    if( sign )
        {
        result = -static_cast< Int64 >( value );
        }
    else
        {
        result = static_cast< Int64 >( value );
        }
#endif
    }


void BitStrVli::read
    (
    BitIStream &    bis,
    UInt64 &        result
    )
    {
#if 0
    UInt32 bits = 1;
    while( bis.read_int( 1 ) != 1 )
        bits++;

    result = bis.read_int64( bits );
#else
    const UInt32 bits = bis.read_unary() + 1;
    if( bits == 0 || bits > 64 )
        {
        std::ostringstream ss;
        ss << "Bad number of bits (" << bits << ") reading unary-coded 64bit integer.";
        throw ExSerializationError( ss.str() );
        }
    result = bis.read_int64( bits );
#endif
    }


void BitStrVli::write
    (
    BitOStream &    bos,
    Int32           value
    )
    {
#if 0
    int     bits = 1;
    int     sign = 0;
    UInt32  rest = 0;

    if( value < 0 )
        {
        sign = 1;
        value = ( -1 ) * value;
        }

    while( ( rest = ( ( UInt32 ) ( value >> bits ) ) ) > 1 )
        {
        bits++;
        }

    bits += rest;
    bos.write_int( sign, 1 );
    bos.write_unary( bits );
    bos.write_int( ( UInt32 ) value, bits );
#else
    if( value < 0 )
        {
        bos.write_int( 1, 1 );
        value = -value;
        }
    else
        {
        bos.write_int( 0, 1 );
        }
    hope( !( value & ( 1 << 31 ) ) );
    write( bos, static_cast< UInt32 >( value ) );
#endif
    }


void BitStrVli::write
    (
    BitOStream &    bos,
    UInt32          value
    )
    {
#if 0
    int     bits = 1;
    UInt32  rest = 0;

    while( ( rest = ( value >> bits ) ) > 1 )
        {
        bits++;
        }

    bits += rest;
    bos.write_unary( bits );
    bos.write_int( value, bits );
#else
    UInt32  mask = ~1;
    int     bits = 1; // always write at least single bit, even if value is zero
    while( value & mask )
        {
        bits++;
        mask <<= 1;
        }
    hope( bits >= 1 );
    hope( bits <= 32 );
    bos.write_unary( bits - 1 );
    bos.write_int( value, bits );
#endif
    }


void BitStrVli::write
    (
    BitOStream &    bos,
    Int64           value
    )
    {
#if 0
    int     bits = 1;
    int     sign = 0;
    UInt64  rest = 0;

    if( value < 0 )
        {
        sign = 1;
        value = ( -1 ) * value;
        }

    while( ( rest = ( ( ( UInt64 ) value ) >> bits ) )  > 1 )
        {
        bits++;
        }

    bits += ( int ) rest;
    bos.write_int( sign, 1 );
    bos.write_unary( bits );
    bos.write_int64( ( UInt64 ) value, bits );
#else
    if( value < 0 )
        {
        bos.write_int( 1, 1 );
        value = -value;
        }
    else
        {
        bos.write_int( 0, 1 );
        }
    hope( !( value & ( UINT64( 1 ) << 63 ) ) );
    write( bos, static_cast< UInt64 >( value ) );
#endif
    }


void BitStrVli::write
    (
    BitOStream &    bos,
    UInt64          value
    )
    {
#if 0
    int     bits = 1;
    UInt64  rest = 0;
    while( ( rest = ( value >> bits ) ) > 1 )
        bits++;

    bits += ( int ) rest;
    bos.write_unary( bits );
    bos.write_int64( value, bits );
#else
    UInt64  mask = ~UINT64( 1 );
    int     bits = 1; // always write at least single bit, even if value is zero
    while( value & mask )
        {
        bits++;
        mask <<= 1;
        }
    hope( bits >= 1 );
    hope( bits <= 64 );
    bos.write_unary( bits - 1 );
    bos.write_int64( value, bits );
#endif
    }


template <class Type>
static void read
    (
    TextReader &    tr,
    Type &          value
    )
    {
    std::istream &  is = tr.line();
    //hope( is );
    is >> value;
    if( is.fail() )
        {
        if( is.eof() )
            throw ExUnexpectedEol( tr.unexpected_eol() );
        else
            throw ExUnexpectedToken( tr.unexpected_token( "integer" ) );
        }
    }


template <class Type>
static void write
    (
    TextWriter &    tw,
    Type            value
    )
    {
    tw.line() << static_cast<Type>( value & uint64_masks[ sizeof( Type ) * 8 ] );
    }


void TextVli::read
    (
    TextReader &    tr,
    Int32 &         value
    )
    {
    Serializer::read( tr, value );
    }


void TextVli::read
    (
    TextReader &    tr,
    Int64 &         value
    )
    {
    Serializer::read( tr, value );
    }


void TextVli::read
    (
    TextReader &    tr,
    UInt32 &        value
    )
    {
    Serializer::read( tr, value );
    }


void TextVli::read
    (
    TextReader &    tr,
    UInt64 &        value
    )
    {
    Serializer::read( tr, value );
    }


void TextVli::write
    (
    TextWriter &    tw,
    Int32           value
    )
    {
    Serializer::write( tw, value );
    }


void TextVli::write
    (
    TextWriter &    tw,
    Int64           value
    )
    {
    Serializer::write( tw, value );
    }


void TextVli::write
    (
    TextWriter &    tw,
    UInt32          value
    )
    {
    Serializer::write( tw, value );
    }


void TextVli::write
    (
    TextWriter &    tw,
    UInt64          value
    )
    {
    Serializer::write( tw, value );
    }


} // namespace Massiv::Core::Serializer
} // namespace Massiv::Core
} // namespace Massiv


#ifndef EXPOSE_SERIALIZERS
#include "serializer/serializer_vli_inline.h"
#endif
