/** @file serializer_vli.h
 * Variable length integrer serializer.
 */

/*
 * begin        : Mon Mar 3 2002
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SERIALIZER_VLI_H
#define MASSIV_SERIALIZER_VLI_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif



namespace Massiv {
namespace Core {
namespace Serializer {


/** @addtogroup serialization
 * @{
 */

/** @struct BitStrVli
 * BitStream variable length integer serializers.
 */
struct MASSIV_EXPORTED BitStrVli
    {

    static void read
        (
        BitIStream &    bis,
        Int32 &         result
        );

    static void read
        (
        BitIStream &    bis,
        UInt32 &        result
        );

    static void read
        (
        BitIStream &    bis,
        Int64 &         result
        );

    static void read
        (
        BitIStream &    bis,
        UInt64 &        result
        );

    static void write
        (
        BitOStream &    bos,
        Int32           value
        );

    static void write
        (
        BitOStream &    bos,
        UInt32          value
        );

    static void write
        (
        BitOStream &    bos,
        Int64           value
        );

    static void write
        (
        BitOStream &    bos,
        UInt64          value
        );

    }; // struct BitStrVli


/** @struct TextVli
 * Text variable length integer serializers.
 */
struct MASSIV_EXPORTED TextVli
    {
    
    static void read
        (
        TextReader &    tr,
        Int32 &         value
        );
    
    static void read
        (
        TextReader &    tr,
        Int64 &         value
        );

    static void read
        (
        TextReader &    tr,
        UInt32 &        value
        );
    
    static void read
        (
        TextReader &    tr,
        UInt64 &        value
        );
    
    static void write
        (
        TextWriter &    tw,
        Int32           value
        );
    
    static void write
        (
        TextWriter &    tw,
        Int64           value
        );

    static void write
        (
        TextWriter &    tw,
        UInt32          value
        );

    static void write
        (
        TextWriter &    tw,
        UInt64          value
        );

    }; // struct TextVli

/** @} */ // addtogroup serialization


} // namespace Massiv::Core::Serializer
} // namespace Massiv::Core
} // namespace Massiv

#ifdef EXPOSE_SERIALIZERS
#include "serializer_vli_inline.h"
#endif

#endif // MASSIV_SERIALIZER_VLI_H
