/** @file stype_bool.h
 * Serializable lightweight booleans. Wrappers around bool native type.
 */

/*
 * begin        : Mon Apr 8 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_STYPE_BOOL_H
#define MASSIV_STYPE_BOOL_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif

#ifndef MASSIV_PROPERTY_FWD_H
#include "property/property_fwd.h"
#endif

#include <string>


namespace Massiv {
namespace Core {


/** @addtogroup stype
 * @{
 */

/** @class SBool
 * Serializable boolean.
 */
class MASSIV_EXPORTED SBool
    {
public:
    
    /// Corresponding native type.
    typedef bool    NativeType;
    /// Corresponding property type.
    typedef PBool   PropertyType;

public:

    /** Constructor. */
    INLINE SBool();

    /** Copy contructor. */
    INLINE SBool
        (
        const SBool & b
        );

    /** Constructor. */
    INLINE SBool
        (
        bool b
        );

public:

    /** Write value. */
    INLINE void write
        (
        bool b
        );

    /** Read value. */
    INLINE bool read() const;

    /** Operator=. */
    INLINE SBool & operator=
        (
        bool b
        );

    /** Read value. */
    INLINE operator bool() const;

public:

    /** Operator&=. */
    INLINE SBool & operator&=
        (
        bool b
        );

    /** Operator|=. */
    INLINE SBool & operator|=
        (
        bool b
        );

public:

    /**
     * Serialize to bitstream.
     */
    INLINE void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description &
        ) const;

    /**
     * Read from bitstream.
     */
    INLINE void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description &
        );

    /**
     * Serialize to text stream.
     */
    INLINE void text_write
        (
        TextWriter &                    tw,
        const Serializer::Description &
        ) const;

    /**
     * Read from text stream.
     */
    INLINE void text_read
        (
        TextReader &                    tr,
        const Serializer::Description &
        );

    /**
     * Print SType state (debug).
     */
    std::string to_string() const;

    /**
     * Return hash code.
     */
    INLINE int hash() const;

    /**
     * Compute memory size consumed by SType.
     */
    INLINE int memory_size() const;

    /**
     * Guess bitstream serialization size (in bits).
     */
    INLINE int bitstream_size() const;

private:

    bool value;
        /**< Value. */

    }; // class SBool

/** @} */ // addtogroup stype


} // namespace Massiv::Core
} // namespace Massiv


#ifdef ENABLE_INLINE
#include "stype_bool_inline.h"
#endif


#endif // MASSIV_STYPE_BOOL_H
