/** @file stype_bool_inline.h
 * Serializable lightweight booleans. Wrappers around bool native type.
 */

/*
 * begin        : Mon Apr 8 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SERIALIZER_BOOL_H
#include "serializer/serializer_bool.h"
#endif


namespace Massiv {
namespace Core {


SBool::SBool()
    : value( false )
    {
    }


SBool::SBool
    (
    const SBool & b
    ) : value( b.value )
    {
    }


SBool::SBool
    (
    bool b
    ) : value( b )
    {
    }


void SBool::write
    (
    bool b
    )
    {
    value = b;
    }


bool SBool::read() const
    {
    return value;
    }


SBool & SBool::operator=
    (
    bool b
    )
    {
    write( b );
    return *this;
    }


SBool::operator bool() const
    {
    return read();
    }


SBool & SBool::operator&=
    (
    bool b
    )
    {
    value &= b;
    return *this;
    }


SBool & SBool::operator|=
    (
    bool b
    )
    {
    value |= b;
    return *this;
    }


void SBool::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description &
    ) const
    {
    Serializer::BitStrBool::write( bos, read() );
    }


void SBool::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description &
    )
    {
    bool b;
    Serializer::BitStrBool::read( bis, b );
    write( b );
    }


void SBool::text_write
    (
    TextWriter &                    tw,
    const Serializer::Description &
    ) const
    {
    Serializer::TextBool::write( tw, read() );
    }


void SBool::text_read
    (
    TextReader &                    tr,
    const Serializer::Description &
    )
    {
    bool b;
    Serializer::TextBool::read( tr, b );
    write( b );
    }


int SBool::hash() const
    {
    return read();
    }


int SBool::memory_size() const
    {
    return sizeof( *this );
    }


int SBool::bitstream_size() const
    {
    return 1;
    }


} // namespace Massiv::Core
} // namespace Massiv
