/** @file stype_string.h
 * Serializable string.
 */

/*
 * begin        : Sun Oct 20 2002
 * copyright    : (C) 2002 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_STYPE_STRING_H
#define MASSIV_STYPE_STRING_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_ATTRIBUTE_H
#include "property/attribute.h"
#endif

#ifndef MASSIV_SERIALIZER_FWD_H
#include "serializer/serializer_fwd.h"
#endif

#ifndef MASSIV_PROPERTY_FWD_H
#include "property/property_fwd.h"
#endif

#include <string>


namespace Massiv {
namespace Core {


/** @addtogroup stype
 * @{
 */

/** @class SString
 * Serializable string.
 */
class MASSIV_EXPORTED SString : public StringAttribute<DefaultAttributeEventHandlers>
    {
private:

    typedef StringAttribute<DefaultAttributeEventHandlers> Super;

public:
    
    /// Corresponding native type.
    typedef std::string NativeType;
    /// Corresponding property type.
    typedef PString     PropertyType;

public:

    /** Default constructor. */
    INLINE SString();
    
    /** Constructor. */
    INLINE SString
        (
        const char * value
        );
    
    /** Constructor. */
    INLINE SString
        (
        const std::string & value
        );

    /** Operator=. */
    INLINE SString & operator=
        (
        const std::string & value
        );

public:

    /**
     * Serialize to bitstream.
     */
    INLINE void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description &
        ) const;

    /**
     * Read from bitstream.
     */
    INLINE void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description & desc
        );

    /**
     * Serialize to text stream.
     */
    INLINE void text_write
        (
        TextWriter &                    tw,
        const Serializer::Description & desc
        ) const;

    /**
     * Read from text stream.
     */
    INLINE void text_read
        (
        TextReader &                    tr,
        const Serializer::Description & desc
        );
    
    /**
     * Print SType state (debug).
     */
    INLINE std::string to_string() const;
    
    /**
     * Return hash code.
     */
    int hash() const;

    /**
     * Compute memory size consumed by SType.
     */
    INLINE int memory_size() const;

    /**
     * Guess bitstream serialization size (in bits).
     */
    INLINE int bitstream_size() const;

    }; // class SString

/** @} */ // addtogroup stype


} // namespace Massiv::Core
} // namespace Massiv


#ifdef ENABLE_INLINE
#include "serializer/stype_string_inline.h"
#endif


#endif // MASSIV_PROP_STRING_H
