/** @file stype_string_inline.h
 * Serializable string.
 */

/*
 * begin        : Sun Oct 20 2002
 * copyright    : (C) 2002 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SERIALIZER_STRING_H
#include "serializer/serializer_string.h"
#endif



namespace Massiv {
namespace Core {


SString::SString()
    {
    }


SString::SString
    (
    const char * value
    ) : Super( value )
    {
    }


SString::SString
    (
    const std::string & value
    ) : Super( value )
    {
    }


SString & SString::operator=
    (
    const std::string & value
    )
    {
    Super::operator=( value );
    return *this;
    }


void SString::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description &
    ) const
    {
    Serializer::BitStrString::write( bos, read() );
    }


void SString::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    std::string temp;
    Serializer::BitStrString::read( bis, temp );
    write( temp );
    }


void SString::text_write
    (
    TextWriter &                    tw,
    const Serializer::Description & desc
    ) const
    {
    Serializer::TextString::write( tw, read() );
    }


void SString::text_read
    (
    TextReader &                    tr,
    const Serializer::Description & desc
    )
    {
    std::string temp;
    Serializer::TextString::read( tr, temp );
    write( temp );
    }


std::string SString::to_string() const
    {
    return read();
    }


int SString::memory_size() const
    {
    return sizeof( *this ) + read().size();
    }


int SString::bitstream_size() const
    {
    return read().size() * 8;
    }


} // namespace Massiv::Core
} // namespace Massiv
