/** @file stype_time_inline.h
 * Serializable "simulation" ("game") time.
 */

/*
 * begin        : Fri Apr 5 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SERIALIZER_FLOAT_H
#include "serializer/serializer_float.h"
#endif

#include <sstream>

namespace Massiv {
namespace Core {


STime::STime()
    {
    }


STime::STime
    (
    TimeType time
    ) : Super( time )
    {
    }


STime::STime
    (
    const Super & time
    ) : Super( time )
    {
    }


STime::STime
    (
    const STime & t
    ) : Super( t )
    {
    }


STime & STime::operator=
    (
    const Super & t
    )
    {
    Super::operator=( t );
    return *this;
    }


bool STime::operator==
    (
    const Super & time
    ) const
    {
    return is_equal_to( time );
    }


bool STime::operator!=
    (
    const Super & time
    ) const
    {
    return !operator==( time );
    }


STime STime::asap()
    {
    return STime();
    }


STime STime::minute()
    {
    return STime( 60.0 );
    }


STime STime::hour()
    {
    return STime( 60.0 * 60.0 );
    }


STime STime::day()
    {
    return STime( 60.0 * 60.0 * 24.0 );
    }


STime STime::week()
    {
    return STime( 60.0 * 60.0 * 24.0 * 7.0 );
    }


void STime::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description &
    ) const
    {
    Serializer::BitStrFloat::write( bos, get_value() );
    }


void STime::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description &
    )
    {
    TimeType temp;
    Serializer::BitStrFloat::read( bis, temp );
    set_value( temp );
    }


void STime::text_write
    (
    TextWriter &                    tw,
    const Serializer::Description &
    ) const
    {
    Serializer::TextFloat::write( tw, get_value() );
    }


void STime::text_read
    (
    TextReader &                    tr,
    const Serializer::Description &
    )
    {
    TimeType temp;
    Serializer::TextFloat::read( tr, temp );
    set_value( temp );
    }


std::string STime::to_string() const
    {
    std::ostringstream ss;
    ss << get_value();
    return ss.str();
    }


int STime::hash() const
    {
    return static_cast<int>( get_value() );
    }


int STime::memory_size() const
    {
    return sizeof( *this );
    }


int STime::bitstream_size() const
    {
    return sizeof( TimeType ) * 8;
    }


} // namespace Massiv::Core
} // namespace Massiv
