/** @file stype_vli.h
 * Serializable lightweight variable-length-encoded integers.
 * Wrapper around *Int* native types.
 */

/*
 * begin        : Sat Mar 30 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_STYPE_VLI_H
#define MASSIV_STYPE_VLI_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_STYPE_FWD_H
#include "serializer/stype_fwd.h"
#endif

#ifndef MASSIV_PROPERTY_FWD_H
#include "property/property_fwd.h"
#endif

#ifndef MASSIV_STYPE_INT_H
#include "serializer/stype_int.h"
#endif

#ifndef MASSIV_SERIALIZER_VLI_H
#include "serializer/serializer_vli.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup stype
 * @{
 */

/** @class SVli
 * Serializable integer, variable-length encoding.
 * @a Type should be one of Int32, UInt32, Int64, UInt64.
 */
template <class Type>
class SVli : public SIntBase<Type>
    {
private:

    typedef SIntBase<Type>          Super;
    typedef Serializer::Description Description;    // Because of MSVC6.

public:

    /// Corresponding native type.
    typedef Type        NativeType;
    /// Corresponding property type.
    typedef PVli<Type>  PropertyType;

    using Super::read;
    using Super::write;

public:

    /** Default constructor. */
    INLINE SVli()
        {
        }

    /** Constructor. */
    INLINE SVli
        (
        Type v
        ) : SIntBase<Type>( v )
        {
        }

    /** Operator=. */
    INLINE SVli & operator=
        (
        Type v
        )
        {
        SIntBase<Type>::operator=( v );
        return *this;
        }

public:

    /**
     * Serialize to bitstream.
     */
    INLINE void bitstr_write
        (
        BitOStream &        bos,
        const Description &
        ) const
        {
        Serializer::BitStrVli::write( bos, read() );
        }

    /**
     * Read from bitstream.
     */
    INLINE void bitstr_read
        (
        BitIStream &        bis,
        const Description &
        )
        {
        Type temp;
        Serializer::BitStrVli::read( bis, temp );
        write( temp );
        }

    /**
     * Serialize to text stream.
     */
    INLINE void text_write
        (
        TextWriter &        tw,
        const Description &
        ) const
        {
        Serializer::TextVli::write( tw, read() );
        }

    /**
     * Read from text stream.
     */
    INLINE void text_read
        (
        TextReader &        tr,
        const Description &
        )
        {
        Type temp;
        Serializer::TextVli::read( tr, temp );
        write( temp );
        }

    }; // class SVli

/** @} */ // addtogroup stype


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_STYPE_VLI_H
