/** @file status.cpp
 * Status definition.
 * @see status.h
 */

/*
 * begin        : Tue Nov 6 2001
 * copyright    : (C) 2001 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "global/common.h"
#include "status/status.h"
#include <sstream>


namespace Massiv {
namespace Core {


std::string Status::type_string
    (
    Type type
    )
    {
    switch( type )
        {
        case TYPE_ANY:
            return "any";
        case TYPE_DEBUG:
            return "debug";
        case TYPE_INFO:
            return "info";
        case TYPE_WARNING:
            return "warning";
        case TYPE_ERROR:
            return "error";
        case TYPE_COUNT:
            break;
        }
    return "unknown_type";
    }


Status::Type Status::type_from_string
    (
    const std::string &     type_str
    )
    {
    if( type_str == "any" )         return TYPE_ANY;
    else
    if( type_str == "debug" )       return TYPE_DEBUG;
    else
    if( type_str == "info" )        return TYPE_INFO;
    else
    if( type_str == "warning" )     return TYPE_WARNING;
    else
    if( type_str == "error" )       return TYPE_ERROR;
    
    return TYPE_COUNT;
    }


std::string Status::priority_string
    (
    Priority priority
    )
    {
    switch( priority )
        {
        case PRIORITY_LOWEST:
            return "lowest";
        case PRIORITY_LOW:
            return "low";
        case PRIORITY_NORMAL:
            return "normal";
        case PRIORITY_HIGH:
            return "high";
        case PRIORITY_HIGHEST:
            return "highest";
        case PRIORITY_COUNT:
            break;
        }
    return "unknown_priority";
    }


Status::Priority Status::priority_from_string
    (
    const std::string &     pri_str
    )
    {
    if( pri_str == "lowest" )       return PRIORITY_LOWEST;
    else
    if( pri_str == "low" )          return PRIORITY_LOW;
    else
    if( pri_str == "normal" )       return PRIORITY_NORMAL;
    else
    if( pri_str == "high" )         return PRIORITY_HIGH;
    else
    if( pri_str == "highest" )      return PRIORITY_HIGHEST;
    
    return PRIORITY_COUNT;
    }


char Status::priority_char
    (
    Priority priority
    )
    {
    switch( priority )
        {
        case PRIORITY_LOWEST:
            return ' ';
        case PRIORITY_LOW:
            return ' ';
        case PRIORITY_NORMAL:
            return ' ';
        case PRIORITY_HIGH:
            return '!';
        case PRIORITY_HIGHEST:
            return '!';
        case PRIORITY_COUNT:
            break;
        }
    return '?';
    }


std::string Status::facility_string
    (
    Facility facility
    )
    {
    switch( facility )
        {
        case FACILITY_ANY:
            return "any";
        case FACILITY_GENERIC:
            return "generic";
        case FACILITY_SYSTEM:
            return "system";
        case FACILITY_IO:
            return "i/o";
        case FACILITY_LOGGER:
            return "logger";
        case FACILITY_NETWORK:
            return "network";
        case FACILITY_REGISTRY:
            return "registry";
        case FACILITY_OBJECT_MANAGEMENT:
            return "object_management";
        case FACILITY_OBJECT_LOCALIZATION:
            return "object_localization";
        case FACILITY_ARCHIVATION:
            return "archivation";
        case FACILITY_FILE_SYSTEM:
            return "file_system";
        case FACILITY_VOLUME_MANAGER:
            return "volume_manager";
        case FACILITY_LRU_CACHE:
            return "lru_cache";
        case FACILITY_SYNCHRONIZATION:
            return "synchronization";
        case FACILITY_THREAD:
            return "thread";
        case FACILITY_DATA_MANAGEMENT:
            return "data_management";
        case FACILITY_TIME_MANAGEMENT:
            return "time_management";
        case FACILITY_TEST:
            return "test";
        case FACILITY_NODE_DATABASE:
            return "node_database";
        case FACILITY_ARCHIVE_DATABASE:
            return "archive_database";
        case FACILITY_NODE_MANAGEMENT:
            return "node_management";
        case FACILITY_REPLICATION:
            return "replication";
        case FACILITY_RPC:
            return "rpc";
        case FACILITY_BALANCER:
            return "balancer";
        case FACILITY_LIB:
            return "lib";    
        case FACILITY_COUNT:
            break;
        }
    return "unknown_facility";
    }


Status::Facility Status::facility_from_string
    (
    const std::string & fac_str
    )
    {
    // Fixme - nacpat retezce do asociativniho pole? - ale asi je to 
    // zbytecne...

    if( fac_str == "any" )                      return FACILITY_ANY;
    else
    if( fac_str == "generic" )                  return FACILITY_GENERIC;
    else
    if( fac_str == "system" )                   return FACILITY_SYSTEM;
    else
    if( fac_str == "i/o" )                      return FACILITY_IO;
    else
    if( fac_str == "logger" )                   return FACILITY_LOGGER;
    else 
    if( fac_str == "network" )                  return FACILITY_NETWORK;
    else
    if( fac_str == "registry" )                 return FACILITY_REGISTRY;
    else
    if( fac_str == "object_management" )        return FACILITY_OBJECT_MANAGEMENT;
    else
    if( fac_str == "object_localization" )      return FACILITY_OBJECT_LOCALIZATION;
    else
    if( fac_str == "archivation" )              return FACILITY_ARCHIVATION;
    else
    if( fac_str == "file_system" )              return FACILITY_FILE_SYSTEM;
    else
    if( fac_str == "volume_manager" )           return FACILITY_VOLUME_MANAGER;
    else
    if( fac_str == "lru_cache" )                return FACILITY_LRU_CACHE;
    else
    if( fac_str == "synchronization" )          return FACILITY_SYNCHRONIZATION;
    else
    if( fac_str == "thread" )                   return FACILITY_THREAD;
    else
    if( fac_str == "data_management" )          return FACILITY_DATA_MANAGEMENT;
    else
    if( fac_str == "time_management" )          return FACILITY_TIME_MANAGEMENT;
    else
    if( fac_str == "test" )                     return FACILITY_TEST;
    else
    if( fac_str == "node_database" )            return FACILITY_NODE_DATABASE;
    else
    if( fac_str == "archive_database" )         return FACILITY_ARCHIVE_DATABASE;
    else
    if( fac_str == "node_management" )          return FACILITY_NODE_MANAGEMENT;
    else
    if( fac_str == "replication" )              return FACILITY_REPLICATION;
    else
    if( fac_str == "rpc" )                      return FACILITY_RPC;
    else
    if( fac_str == "balancer" )                 return FACILITY_BALANCER;
    else        
    if( fac_str == "lib" )                      return FACILITY_LIB;    

    return FACILITY_COUNT;
    }


Status::Status()
    {
    value.v = 0;
    value.c.facility = FACILITY_ANY;
    value.c.type = TYPE_ANY;
    value.c.priority = PRIORITY_LOWEST;
    }


Status::Status
    (
    Facility    facility,
    Type        type,
    Priority    priority
    )
    {
    hope( facility >= 0 );
    hope( facility < FACILITY_COUNT );
    hope( type >= 0 );
    hope( type < TYPE_COUNT );
    hope( priority >= 0 );
    hope( priority < PRIORITY_COUNT );
    value.v = 0;
    value.c.facility = facility;
    value.c.type = type;
    value.c.priority = priority;
    }


Status::Status
    (
    const Status &      s
    )
    {
    value = s.value;
    }


Status & Status::operator=
    (
    const Status &      s
    )
    {
    value.v = s.value.v;
    return *this;
    }


bool Status::operator==
    (
    const Status &      s
    ) const
    {
    return value.v == s.value.v;
    }


bool Status::operator!=
    (
    const Status &      s
    ) const
    {
    return value.v != s.value.v;
    }


std::string Status::get_description
    (
    const std::string & message
    ) const
    {
    std::ostringstream s;

    s   << priority_char( (Priority) value.c.priority )
        << facility_string( (Facility) value.c.facility )
        << " " << type_string( (Type) value.c.type )
        << ": " << message;

    return s.str();
    }


} // namespace Massiv::Core
} // namespace Massiv


#ifndef ENABLE_INLINE
#include "status_inline.h"
#endif
