/** @file account_msg.cpp
 * Account messages.
 */

/*
 * begin        : Fri Sep 6 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "sysevent/account_msg.h"
#include "net/bitistream.h"
#include "net/bitostream.h"
#include "global/global.h"
#include "database/node_database.h"
#include "data/account_manager.h"
#include <cstring>

namespace Massiv {
namespace Core {


// **** AccountRegisterRequestMessage start ****


AccountRegisterRequestMessage::AccountRegisterRequestMessage()
    {
    }


AccountRegisterRequestMessage::AccountRegisterRequestMessage
    (
    const AccountManager::AccountRequest &     _account_request
    ) :
    account_request( _account_request )
    {
    }


AccountRegisterRequestMessage::AccountRegisterRequestMessage
    (
    const AccountRegisterRequestMessage &      msg
    ) : 
    Super( msg ),
    account_request( msg.account_request )
    {
    }


AccountRegisterRequestMessage & AccountRegisterRequestMessage::operator=
    (
    const AccountRegisterRequestMessage &      msg
    )
    {
    Super::operator=( msg );
    account_request = msg.account_request;
    return *this;
    }


SystemMessage::Type AccountRegisterRequestMessage::get_type() const
    {
    return SystemMessage::ACCOUNT_REGISTER_REQUEST;
    }


SystemMessage::ExecuteResult AccountRegisterRequestMessage::execute()
    {
    AccountManager &      account_manager = Global::account_manager();
    account_manager.handle_account_register_request_message( *this );
    return SystemMessage::EXECUTE_OK;
    }


void AccountRegisterRequestMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );
    
    account_request.bitstr_write( bos, desc );
    }


void AccountRegisterRequestMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    account_request.bitstr_read( bis, desc );
    }


// **** AccountRegisterRequestMessage end ****
// **** AccountRegisterRespondMessage start ****


AccountRegisterRespondMessage::AccountRegisterRespondMessage()
    {
    }


AccountRegisterRespondMessage::AccountRegisterRespondMessage
    (
    const AccountManager::AccountRequest &     _account_request
    ) :
    account_request( _account_request )
    {
    }


AccountRegisterRespondMessage::AccountRegisterRespondMessage
    (
    const AccountRegisterRespondMessage &      msg
    ) : 
    Super( msg ),
    account_request( msg.account_request )
    {
    }


AccountRegisterRespondMessage & AccountRegisterRespondMessage::operator=
    (
    const AccountRegisterRespondMessage &      msg
    )
    {
    Super::operator=( msg );
    account_request = msg.account_request;
    return *this;
    }


SystemMessage::Type AccountRegisterRespondMessage::get_type() const
    {
    return SystemMessage::ACCOUNT_REGISTER_RESPOND;
    }


SystemMessage::ExecuteResult AccountRegisterRespondMessage::execute()
    {
    AccountManager &      account_manager = Global::account_manager();
    account_manager.handle_account_register_respond_message( *this );
    return SystemMessage::EXECUTE_OK;
    }


void AccountRegisterRespondMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );
    
    account_request.bitstr_write( bos, desc );
    }


void AccountRegisterRespondMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    account_request.bitstr_read( bis, desc );
    }

// **** AccountRegisterRespondMessage end ****
// **** AccountCreateRequestMessage start ****

AccountCreateRequestMessage::AccountCreateRequestMessage()
    {
    }

AccountCreateRequestMessage::AccountCreateRequestMessage
    (
    const AccountManager::AccountRequest &     _account_request
    ) : 
    account_request( _account_request )
    {
    }


AccountCreateRequestMessage::AccountCreateRequestMessage
    (
    const AccountCreateRequestMessage &      msg
    ) :
    Super( msg ),
    account_request( msg.account_request )
    {
    }


AccountCreateRequestMessage & AccountCreateRequestMessage::operator=
    (
    const AccountCreateRequestMessage &      msg
    )
    {
    Super::operator=( msg );
    account_request = msg.account_request;
    return *this;
    }


SystemMessage::Type AccountCreateRequestMessage::get_type() const
    {
    return SystemMessage::ACCOUNT_CREATE_REQUEST;
    }


SystemMessage::ExecuteResult AccountCreateRequestMessage::execute()
    {
    AccountManager &      account_manager = Global::account_manager();
    account_manager.handle_account_create_request_message( *this );
    return SystemMessage::EXECUTE_OK;
    }


void AccountCreateRequestMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );
    
    account_request.bitstr_write( bos, desc );
    }


void AccountCreateRequestMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    account_request.bitstr_read( bis, desc );
    }

// **** AccountCreateRequestMessage end ****
// **** AccountCreateRespondMessage start ****


AccountCreateRespondMessage::AccountCreateRespondMessage()
    {
    }


AccountCreateRespondMessage::AccountCreateRespondMessage
    (
    const AccountManager::AccountRequest &     _account_request
    ) :
    account_request( _account_request )
    {
    }


AccountCreateRespondMessage::AccountCreateRespondMessage
    (
    const AccountCreateRespondMessage &      msg
    ) : 
    Super( msg ),
    account_request( msg.account_request )
    {
    }


AccountCreateRespondMessage & AccountCreateRespondMessage::operator=
    (
    const AccountCreateRespondMessage &      msg
    )
    {
    Super::operator=( msg );
    account_request = msg.account_request;
    return *this;
    }


SystemMessage::Type AccountCreateRespondMessage::get_type() const
    {
    return SystemMessage::ACCOUNT_CREATE_RESPOND;
    }


SystemMessage::ExecuteResult AccountCreateRespondMessage::execute()
    {
    AccountManager &      account_manager = Global::account_manager();
    account_manager.handle_account_create_respond_message( *this );
    return SystemMessage::EXECUTE_OK;
    }


void AccountCreateRespondMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );
    
    account_request.bitstr_write( bos, desc );
    }


void AccountCreateRespondMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    account_request.bitstr_read( bis, desc );
    }

// **** AccountCreateRespondMessage end ****


} // namespace Massiv::Core
} // namespace Massiv
