/** @file account_msg.h
 * Account messages.
 */

/*
 * begin        : Fri Sep 6 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_ACCOUNT_MSG_H
#define MASSIV_ACCOUNT_MSG_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_SYSTEM_MSG_H
#include "sysevent/system_msg.h"
#endif

#ifndef MASSIV_BITIOSTREAM_FWD_H
#include "net/bitiostream_fwd.h"
#endif

#ifndef MASSIV_STYPE_STRING_H
#include "serializer/stype_string.h"
#endif

#ifndef MASSIV_ACCOUNT_MANAGER_H
#include "data/account_manager.h"
#endif

namespace Massiv {
namespace Core {


/** @addtogroup sysevent
 * @{
 */

/**
 * Account register request message.
 */
class AccountRegisterRequestMessage : public SystemMessage
    {
    typedef SystemMessage   Super;

public:

    /** 
     * Constructor. 
     */
    AccountRegisterRequestMessage();

    /** 
     * Constructor. 
     */
    AccountRegisterRequestMessage
        (
        const AccountManager::AccountRequest &     _account_request
        );

    /**
     * Copy constructor.
     */
    AccountRegisterRequestMessage
        (
        const AccountRegisterRequestMessage &      msg
        );

    /**
     * Operator=.
     */
    AccountRegisterRequestMessage & operator=
        (
        const AccountRegisterRequestMessage &       msg
        );

public:

    //
    // From SystemMessage.
    //

    /**
     *
     */
    virtual SystemMessage::Type get_type() const;

    /**
     *
     */
    virtual ExecuteResult execute();

public:

    /**
     * Serialize to bitstream.
     */
    virtual void bitstr_write
        (
        BitOStream &                        bos,
        const Serializer::Description &     desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual void bitstr_read
        (
        BitIStream &                        bis,
        const Serializer::Description &     desc
        );

public:

    AccountManager::AccountRequest      account_request;
        /**< Account request structure. */

    }; // AccountRegisterRequestMessage


/**
 * Account register respond message.
 */
class AccountRegisterRespondMessage : public SystemMessage
    {
    typedef SystemMessage   Super;

public:

    /** 
     * Constructor. 
     */
    AccountRegisterRespondMessage();

    /** 
     * Constructor. 
     */
    AccountRegisterRespondMessage
        (
        const AccountManager::AccountRequest &     _account_request
        );

    /**
     * Copy constructor.
     */
    AccountRegisterRespondMessage
        (
        const AccountRegisterRespondMessage &      msg
        );

    /**
     * Operator=.
     */
    AccountRegisterRespondMessage & operator=
        (
        const AccountRegisterRespondMessage &       msg
        );

public:

    //
    // From SystemMessage.
    //

    /**
     *
     */
    virtual SystemMessage::Type get_type() const;

    /**
     *
     */
    virtual ExecuteResult execute();

public:

    /**
     * Serialize to bitstream.
     */
    virtual void bitstr_write
        (
        BitOStream &                        bos,
        const Serializer::Description &     desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual void bitstr_read
        (
        BitIStream &                        bis,
        const Serializer::Description &     desc
        );

public:

    AccountManager::AccountRequest      account_request;
        /**< Account request structure. */
            
    }; // AccountRegisterRespondMessage


/**
 * Account create request message.
 */
class AccountCreateRequestMessage : public SystemMessage
    {
    typedef SystemMessage   Super;

public:

    /** 
     * Constructor. 
     */
    AccountCreateRequestMessage();

    /** 
     * Constructor. 
     */
    AccountCreateRequestMessage
        (
        const AccountManager::AccountRequest &     _account_request
        );

    /**
     * Copy constructor.
     */
    AccountCreateRequestMessage
        (
        const AccountCreateRequestMessage &      msg
        );

    /**
     * Operator=.
     */
    AccountCreateRequestMessage & operator=
        (
        const AccountCreateRequestMessage &       msg
        );

public:

    //
    // From SystemMessage.
    //

    /**
     *
     */
    virtual SystemMessage::Type get_type() const;

    /**
     *
     */
    virtual ExecuteResult execute();

public:

    /**
     * Serialize to bitstream.
     */
    virtual void bitstr_write
        (
        BitOStream &                        bos,
        const Serializer::Description &     desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual void bitstr_read
        (
        BitIStream &                        bis,
        const Serializer::Description &     desc
        );

public:

    AccountManager::AccountRequest      account_request;
        /**< Account request structure. */

    }; // AccountCreateRequestMessage


/**
 * Account create respond message.
 */
class AccountCreateRespondMessage : public SystemMessage
    {
    typedef SystemMessage   Super;

public:

    /** 
     * Constructor. 
     */
    AccountCreateRespondMessage();

    /** 
     * Constructor. 
     */
    AccountCreateRespondMessage
        (
        const AccountManager::AccountRequest &     _account_request
        );

    /**
     * Copy constructor.
     */
    AccountCreateRespondMessage
        (
        const AccountCreateRespondMessage &      msg
        );

    /**
     * Operator=.
     */
    AccountCreateRespondMessage & operator=
        (
        const AccountCreateRespondMessage &       msg
        );

public:

    //
    // From SystemMessage.
    //

    /**
     *
     */
    virtual SystemMessage::Type get_type() const;

    /**
     *
     */
    virtual ExecuteResult execute();

public:

    /**
     * Serialize to bitstream.
     */
    virtual void bitstr_write
        (
        BitOStream &                        bos,
        const Serializer::Description &     desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual void bitstr_read
        (
        BitIStream &                        bis,
        const Serializer::Description &     desc
        );

public:

    AccountManager::AccountRequest      account_request;
        /**< Account request structure. */
            
    }; // AccountCreateRespondMessage

/** @} */ // addtogroup node


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_ACCOUNT_MSG_H_
