/** @file archivation_msg.cpp
 * Archivation messages.
 */

/*
 * begin        : Sat Oct 19 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "sysevent/archivation_msg.h"
#include "serializer/serializer.h"
#include "serializer/stype_int.h"
#include "global/global.h"
#include "object/archive_manager.h"


namespace Massiv {
namespace Core {


//-----------------------------------------------------------------------------
// ArchivationMessage implementation.
//-----------------------------------------------------------------------------


ArchivationMessage::ArchivationMessage()
    {
    }


ArchivationMessage::ArchivationMessage
    (
    const ArchivationMessage & msg
    ) :
    Super( msg ),
    archivation_id( msg.archivation_id )
    {
    }


ArchivationMessage & ArchivationMessage::operator=
    (
    const ArchivationMessage & msg
    )
    {
    Super::operator=( msg );
    archivation_id = msg.archivation_id;
    return *this;
    }


void ArchivationMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    // Write super class.
    Super::bitstr_write( bos, desc );

    // Write itself.
    archivation_id.bitstr_write( bos, desc );
    }


void ArchivationMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    // Read super class.
    Super::bitstr_read( bis, desc );

    // Read itself.
    archivation_id.bitstr_read( bis, desc );
    }


//-----------------------------------------------------------------------------
// ArchivationStatusMessage implementation.
//-----------------------------------------------------------------------------


ArchivationStatusMessage::ArchivationStatusMessage() :
    status( ARCHIVATION_STARTED )
    {
    }


ArchivationStatusMessage::ArchivationStatusMessage
    (
    Status _status
    ) :
    status( _status )
    {
    }


ArchivationStatusMessage::ArchivationStatusMessage
    (
    const ArchivationStatusMessage & msg
    ) :
    Super( msg ),
    status( msg.status )
    {
    }


ArchivationStatusMessage & ArchivationStatusMessage::operator=
    (
    const ArchivationStatusMessage & msg
    )
    {
    Super::operator=( msg );
    status = msg.status;
    return *this;
    }


void ArchivationStatusMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    SInt<Int32, STATUS_BITS>    s( status );

    // Write super class.
    Super::bitstr_write( bos, desc );

    // Write itself.
    s.bitstr_write( bos, desc );
    }


void ArchivationStatusMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    SInt<Int32, STATUS_BITS>    s;

    // Read super class.
    Super::bitstr_read( bis, desc );

    // Read itself.
    s.bitstr_read( bis, desc );

    /** @todo fixme */
    hope( s == ARCHIVATION_STARTED || s == ARCHIVATION_FINISHED );

    status = static_cast<Status>( s.read() );
    }


SystemMessage::Type ArchivationStatusMessage::get_type() const
    {
    return SystemMessage::ARCHIVATION_STATUS;
    }


SystemMessage::ExecuteResult ArchivationStatusMessage::execute()
    {
    ArchiveManager & archive_manager = Global::archive_manager();
    return archive_manager.handle_archivation_status_message( *this );
    }


} // namespace Massiv::Core
} // namespace Massiv
