/** @file balancer_msg.cpp
 * Balancer messages.
 */

/*
 * begin        : Sat Oct 25 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "global/global.h"
#include "sysevent/balancer_msg.h"
#include "net/bitistream.h"
#include "net/bitostream.h"
#include "balance/balancer.h"


namespace Massiv {
namespace Core {


ResourceLoadMessage::ResourceLoadMessage()
    {
    }


ResourceLoadMessage::ResourceLoadMessage
    (
    int     num_objects,
    int     memory_load,
    float   cpu_load,
    float   rank
    ) : num_objects( num_objects ),
        memory_load( memory_load ),
        cpu_load( cpu_load ),
        rank( rank )
    {
    }


ResourceLoadMessage::ResourceLoadMessage
    (
    const ResourceLoadMessage & msg
    ) : Super( msg ),
        num_objects( msg.num_objects ),
        memory_load( msg.memory_load ),
        cpu_load( msg.cpu_load ),
        rank( msg.rank )
    {
    }


ResourceLoadMessage & ResourceLoadMessage::operator=
    (
    const ResourceLoadMessage & msg
    )
    {
    Super::operator=( msg );
    
    num_objects = msg.num_objects;
    memory_load = msg.memory_load;
    cpu_load = msg.cpu_load;
    rank = msg.rank;

    return *this;
    }    


int ResourceLoadMessage::get_num_objects() const
    {
    return num_objects;
    }


int ResourceLoadMessage::get_memory_load() const
    {
    return memory_load;
    }


float ResourceLoadMessage::get_cpu_load() const
    {
    return cpu_load;
    }


float ResourceLoadMessage::get_rank() const
    {
    return rank;
    }

    
SystemMessage::Type ResourceLoadMessage::get_type() const
    {
    return SystemMessage::RESOURCE_LOAD_MESSAGE;
    }


SystemMessage::ExecuteResult ResourceLoadMessage::execute()
    {
    Global::balancer().handle_resource_load_message( *this );
    return SystemMessage::EXECUTE_OK;
    }


void ResourceLoadMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );

    num_objects.bitstr_write( bos, desc );
    memory_load.bitstr_write( bos, desc );
    cpu_load.bitstr_write( bos, desc );
    rank.bitstr_write( bos, desc );
    }


void ResourceLoadMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );
    
    num_objects.bitstr_read( bis, desc );
    memory_load.bitstr_read( bis, desc );
    cpu_load.bitstr_read( bis, desc );
    rank.bitstr_read( bis, desc );
    }


} // namespace Massiv::Core
} // namespace Massiv
