/** @file balancer_msg.h
 * Balancer messages.
 */

/*
 * begin        : Sat Oct 25 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_BALANCER_MSG_H
#define MASSIV_BALANCER_MSG_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_SYSTEM_MSG_H
#include "sysevent/system_msg.h"
#endif

#ifndef MASSIV_BITIOSTREAM_FWD_H
#include "net/bitiostream_fwd.h"
#endif

#ifndef MASSIV_STYPE_INT_H
#include "serializer/stype_int.h"
#endif

#ifndef MASSIV_STYPE_FLOAT_H
#include "serializer/stype_float.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup sysevent
 * @{
 */

/**
 * ResourceLoadMessage.
 *
 * @ingroup balance
 */
class ResourceLoadMessage : public SystemMessage
    {
    typedef SystemMessage   Super;

public:

    /**
     * Default constructor.
     */
    ResourceLoadMessage();
    
    /**
     * Constructor.
     */
    ResourceLoadMessage
        (
        int                 num_objects,
        int                 memory_load,
        float               cpu_load,
        float               rank
        );

    /**
     * Copy constructor.
     */
    ResourceLoadMessage
        (
        const ResourceLoadMessage & msg
        );

    /**
     * Operator=.
     */
    ResourceLoadMessage & operator=
        (
        const ResourceLoadMessage & msg
        );

    /**
     * Get @c num_objects.
     */
    int get_num_objects() const;
    
    /**
     * Get @c memory_load.
     */
    int get_memory_load() const;

    /**
     * Get @c cpu_load.
     */
    float get_cpu_load() const;
    
    /**
     * Get @c rank.
     */
    float get_rank() const;

public:

    //
    // From SystemMessage.
    //

    /**
     */
    virtual SystemMessage::Type get_type() const;

    /**
     */
    virtual ExecuteResult execute();

public:

    /**
     * Serialize to bitstream.
     */
    virtual void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description & desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description & desc
        );

private:

    // Resource load attributes.
    // The meaning and interpretation is defined by Balancer.

    SInt32      num_objects;
        /**< Number of objects on the initiator node. */
    SInt32      memory_load;
        /**< Memory load on the initiator node. */
    SFloat32    cpu_load;
        /**< CPU load on the initiator node. */
    SFloat32    rank;
        /**< Resource rank. */

    }; // ResourceLoadMessage

/** @} */ // addtogroup sysevent


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_BALANCER_MSG_H
