/** @file client_connection_msg.h
 * Client connection messages.
 */

/*
 * begin        : Mon Feb 03 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "sysevent/client_connection_msg.h"
#include "net/bitistream.h"
#include "net/bitostream.h"
#include "global/global.h"
#include "node/node_manager.h"


namespace Massiv {
namespace Core {


// WelcomeMessage.
//~~~~~~~~~~~~~~~~

SystemMessage::Type WelcomeMessage::get_type() const
    {
    return WELCOME;
    }


SystemMessage::ExecuteResult WelcomeMessage::execute()
    {
    NodeManager &        node_manager = Global::node_manager();
    return node_manager.handle_welcome_message( *this );
    }


// ClientConnectionMessage.
//~~~~~~~~~~~~~~~~~~~~~~~~~

ClientConnectionMessage::ClientConnectionMessage()
    {
    }


ClientConnectionMessage::ClientConnectionMessage
    (
    const NodeId &  _client_node_id
    ) :
    client_node_id( _client_node_id )
    {
    }


ClientConnectionMessage::ClientConnectionMessage
    (
    const ClientConnectionMessage & msg
    ) :
    Super( msg ),
    client_node_id( msg.client_node_id )
    {
    }


ClientConnectionMessage & ClientConnectionMessage::operator=
    (
    const ClientConnectionMessage & msg
    )
    {
    Super::operator=( msg );
    client_node_id  = msg.client_node_id;
    return *this;
    }


void ClientConnectionMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );

    client_node_id.bitstr_write( bos, desc );
    }

void ClientConnectionMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    client_node_id.bitstr_read( bis, desc );
    }


// AcceptClientMessage.
//~~~~~~~~~~~~~~~~~~~~~

SystemMessage::Type AcceptClientMessage::get_type() const
    {
    return ACCEPT_CLIENT;
    }

    
SystemMessage::ExecuteResult AcceptClientMessage::execute()
    {
    NodeManager &        node_manager = Global::node_manager();
    return node_manager.handle_accept_client_message( *this );
    }


// ClientAcceptedMessage.
//~~~~~~~~~~~~~~~~~~~~~~~

SystemMessage::Type ClientAcceptedMessage::get_type() const
    {
    return CLIENT_ACCEPTED;
    }


SystemMessage::ExecuteResult ClientAcceptedMessage::execute()
    {
    NodeManager &        node_manager = Global::node_manager();
    return node_manager.handle_client_accepted_message( *this );
    }


// ClientReadyMessage.
//~~~~~~~~~~~~~~~~~~~~~

SystemMessage::Type ClientReadyMessage::get_type() const
    {
    return CLIENT_READY;
    }


SystemMessage::ExecuteResult ClientReadyMessage::execute()
    {
    NodeManager &        node_manager = Global::node_manager();
    return node_manager.handle_client_ready_message( *this );
    }

    
// ClientReadyReplyMessage.
//~~~~~~~~~~~~~~~~~~~~~~~~~

SystemMessage::Type ClientReadyReplyMessage::get_type() const
    {
    return CLIENT_READY_REPLY;
    }


SystemMessage::ExecuteResult ClientReadyReplyMessage::execute()
    {
    NodeManager &        node_manager = Global::node_manager();
    return node_manager.handle_client_ready_reply_message( *this );
    }


// KillClientMessage.
//~~~~~~~~~~~~~~~~~~~

SystemMessage::Type KillClientMessage::get_type() const
    {
    return KILL_CLIENT;
    }


SystemMessage::ExecuteResult KillClientMessage::execute()
    {
    NodeManager &        node_manager = Global::node_manager();
    return node_manager.handle_kill_client_message( *this );
    }
    
} // namespace Massiv::Core
} // namespace Massiv
