/** @file find_object_msg.cpp
 * Find object message.
 */

/*
 * begin        : Tue Apr 30 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "sysevent/find_object_msg.h"
#include "net/bitistream.h"
#include "net/bitostream.h"
#include "global/global.h"
#include "object/object_manager.h"


namespace Massiv {
namespace Core {


FindObjectMessage::FindObjectMessage() :
    hops( 0 ),
    provider_hops( 0 )
    {
    }


FindObjectMessage::FindObjectMessage
    (
    const ObjectId &    id,
    UInt8               hops_count,
    UInt8               provider_hops_count
    ) :
    destination( id ),
    hops( hops_count ),
    provider_hops( provider_hops_count )
    {
    hope( hops <= HOPS_MAX_VALUE );
    hope( provider_hops <= PROVIDER_HOPS_MAX_VALUE );
    }


FindObjectMessage::FindObjectMessage
    (
    const FindObjectMessage & msg
    ) :
    Super( msg ),
    destination( msg.destination ),
    hops( msg.hops ),
    provider_hops( msg.provider_hops )
    {
    hope( hops <= HOPS_MAX_VALUE );
    hope( provider_hops <= PROVIDER_HOPS_MAX_VALUE );
    }


FindObjectMessage & FindObjectMessage::operator=
    (
    const FindObjectMessage & msg
    )
    {
    Super::operator=( msg );
    destination   = msg.destination;
    hops          = msg.hops;
    provider_hops = msg.provider_hops;
    return *this;
    }


SystemMessage::Type FindObjectMessage::get_type() const
    {
    return SystemMessage::FIND_OBJECT;
    }


SystemMessage::ExecuteResult FindObjectMessage::execute()
    {
    ObjectManager &      object_manager = Global::object_manager();
    object_manager.handle_incoming_find_object_message( *this );
    
    return SystemMessage::EXECUTE_OK;
    }


void FindObjectMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );

    destination.bitstr_write( bos, desc );
    hops.bitstr_write( bos, desc );
    provider_hops.bitstr_write( bos, desc );
    }


void FindObjectMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    destination.bitstr_read( bis, desc );
    hops.bitstr_read( bis, desc );
    provider_hops.bitstr_read( bis, desc );

    /** @todo Test hops and provider_hops for invalid values? */
    }


} // namespace Massiv::Core
} // namespace Massiv
