/** @file find_object_msg.h
 * Find object message.
 */

/*
 * begin        : Tue Apr 30 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_FIND_OBJECT_MSG_H
#define MASSIV_FIND_OBJECT_MSG_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_SYSTEM_MSG_H
#include "sysevent/system_msg.h"
#endif

#ifndef MASSIV_BITIOSTREAM_FWD_H
#include "net/bitiostream_fwd.h"
#endif

#ifndef MASSIV_STYPE_INT_H
#include "serializer/stype_int.h"
#endif

#ifndef MASSIV_OBJECTID_H
#include "pointer/objectid.h"
#endif

#ifndef MASSIV_NODEID_H
#include "node/nodeid.h"
#endif



namespace Massiv {
namespace Core {


/** @addtogroup sysevent
 * @{
 */

/**
 * FindObject message packet.
 * Used for object localization or as common class for
 * messages that are delivered to some object.
 */
class FindObjectMessage : public SystemMessage
    {
    typedef SystemMessage   Super;

public:

    enum
        {
        HOPS_BITS_COUNT = 8
            /**< Number of bits required to store hops count. */
        };

    enum
        {
        HOPS_MAX_VALUE = ( 256 - 1 )
            /**< Max value that can be stored in hops count. */
        };

    enum 
        {
        PROVIDER_HOPS_BITS_COUNT = 2
            /**< Number of bits required to store provider hops count. */
        };

    enum
        {
        PROVIDER_HOPS_MAX_VALUE = ( 4 - 1 )
            /**< Max value that can be stored in provider hops count. */
        };

public:

    /**
     * Constructor.
     */
    FindObjectMessage();

    /**
     * Constructor.
     */
    FindObjectMessage
        (
        const ObjectId &    dest_object_id,
        UInt8               hops_count,
        UInt8               provider_hops_count
        );

    /**
     * Copy constructor.
     */
    FindObjectMessage
        (
        const FindObjectMessage & msg
        );

    /**
     * Operator=.
     */
    FindObjectMessage & operator=
        (
        const FindObjectMessage & msg
        );

public:

    //
    // From SystemMessage.
    //

    /**
     *
     */
    virtual SystemMessage::Type get_type() const;

    /**
     *
     */
    virtual ExecuteResult execute();

public:

    /**
     * Serialize to bitstream.
     */
    virtual void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description & desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description & desc
        );

public:

    ObjectId                                destination;
        /**< Object we are looking for. */

    SInt<UInt32, HOPS_BITS_COUNT>           hops;
        /**< How many times this packet was forwarded. */

    SInt<UInt32, PROVIDER_HOPS_BITS_COUNT>  provider_hops;
        /**< How many times this packet was forwarded through
             provider of the destination object. */

    }; // FindObjectMessage

/** @} */ // addtogroup sysevent


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_FIND_OBJECT_MSG_H
