/** @file global_search_msg.cpp
 * Messages used by global search protocol.
 */
 
/*
 * begin        : Wed Apr 9 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

 
#include "sysevent/global_search_msg.h"
#include "global/global.h"
#include "object/global_search_manager.h"


namespace Massiv {
namespace Core {


//------------------------------------------------------------------------------
// GlobalSearchMessage
//------------------------------------------------------------------------------


GlobalSearchMessage::GlobalSearchMessage() :
    session_id( 0 )
    {
    }


GlobalSearchMessage::GlobalSearchMessage
    (
    const ObjectId & _object,
    UInt32           _session_id
    ) :
    object( _object ),
    session_id( _session_id )
    {
    }


GlobalSearchMessage::GlobalSearchMessage
    (
    const GlobalSearchMessage & msg
    ) :
    Super( msg ),
    object( msg.object ),
    session_id( msg.session_id )
    {
    }


GlobalSearchMessage & GlobalSearchMessage::operator=
    (
    const GlobalSearchMessage & msg
    )
    {
    Super::operator=( msg );
    object     = msg.object;
    session_id = msg.session_id;
    return *this;
    }


SystemMessage::Type GlobalSearchMessage::get_type() const
    {
    return SystemMessage::GLOBAL_SEARCH;
    }
    
    
SystemMessage::ExecuteResult GlobalSearchMessage::execute()
    {
    Global::global_search_manager().handle_global_search_message( *this );
    return SystemMessage::EXECUTE_OK;
    }
    

void GlobalSearchMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );
    
    object.bitstr_write( bos, desc );
    session_id.bitstr_write( bos, desc );
    }


void GlobalSearchMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    object.bitstr_read( bis, desc );
    session_id.bitstr_read( bis, desc );
    }
    
    
//------------------------------------------------------------------------------
// GlobalSearchReplyMessage
//------------------------------------------------------------------------------


GlobalSearchReplyMessage::GlobalSearchReplyMessage() :
    session_id( 0 )
    {
    }
    
    
GlobalSearchReplyMessage::GlobalSearchReplyMessage
    (
    const ObjectId &        _object,
    const ObjectStatus &    _object_status,
    const NodeId &          _node_id,
    UInt32                  _session_id
    ) :
    Super( _object, _object_status, _node_id, false ),
    session_id( _session_id )
    {
    // Note: Super::trustworthy flags is always set to false for
    // GlobalSearchReplyMessage message.
    }
    
    
GlobalSearchReplyMessage::GlobalSearchReplyMessage
    (
    const GlobalSearchReplyMessage & msg
    ) :
    Super( msg ),
    session_id( msg.session_id )
    {
    }

    
GlobalSearchReplyMessage & GlobalSearchReplyMessage::operator=
    (
    const GlobalSearchReplyMessage & msg
    )
    {
    Super::operator=( msg );
    session_id  = msg.session_id;
    return *this;
    }
    
    
SystemMessage::Type GlobalSearchReplyMessage::get_type() const
    {
    return SystemMessage::GLOBAL_SEARCH_REPLY;
    }


SystemMessage::ExecuteResult GlobalSearchReplyMessage::execute()
    {
    Global::global_search_manager().handle_global_search_reply_message( *this );
    return SystemMessage::EXECUTE_OK;
    }


void GlobalSearchReplyMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );
    
    session_id.bitstr_write( bos, desc );
    }


void GlobalSearchReplyMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    session_id.bitstr_read( bis, desc );
    }


} // namespace Massiv::Core
} // namespace Massiv
