/** @file global_search_msg.h
 * Messages used by global search protocol.
 */
 
/*
 * begin        : Wed Apr 9 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_GLOBAL_SEARCH_MSG_H
#define MASSIV_GLOBAL_SEARCH_MSG_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_OBJECT_STATUS_MSG_H
#include "sysevent/object_status_msg.h"
#endif

#ifndef MASSIV_STYPE_INT_H
#include "serializer/stype_int.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup sysevent
 * @{
 */

/**
 * Global search messages used to find object in the distributed simulation.
 * It contains object id of the object this message is searching for.
 */
class GlobalSearchMessage : public SystemMessage
    {
    typedef SystemMessage   Super;
    
public:
    /** Constructor. */
    GlobalSearchMessage();

    /** Constructor. */
    GlobalSearchMessage
        (
        const ObjectId & object,
        UInt32           session_id
        );
        
    /** Copy constructor. */
    GlobalSearchMessage
        (
        const GlobalSearchMessage & msg
        );
        
    /** Operator=. */
    GlobalSearchMessage & operator=
        (
        const GlobalSearchMessage & msg
        );
        
public:

    //
    // From SystemMessage.
    //

    /** */
    virtual SystemMessage::Type get_type() const;

    /** */
    virtual SystemMessage::ExecuteResult execute();
    
public:

    /** Serialize to bitstream. */
    virtual void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description & desc
        ) const;

    /** Read from bitstream. */
    virtual void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description & desc
        );

public:
    ObjectId            object;
        /**< Object we are searching for. */
    
    SInt< UInt32, 32 >  session_id;
        /**< Search session identification.
         * Used to detect obsolete search messages etc.
         */

    }; // GlobalSearchMessage
    
    
/**
 * Global search message used to reply to GlobalSearchMessage.
 * The message contains information that the node
 * which sent this GlobalSearchReplyMessage has about the object.
 * 
 * @see GlobalSearchMessage
 */
class GlobalSearchReplyMessage : public ObjectStatusMessage
    {
    typedef ObjectStatusMessage Super;
    
public:
    /** Constructor. */
    GlobalSearchReplyMessage();
    
    /** Constructor. */
    GlobalSearchReplyMessage
        (
        const ObjectId &        object,
        const ObjectStatus &    object_status,
        const NodeId &          node,
        UInt32                  session_id
        );
        
    /** Copy constructor. */
    GlobalSearchReplyMessage
        (
        const GlobalSearchReplyMessage & msg
        );
        
    /** Operator=. */
    GlobalSearchReplyMessage & operator=
        (
        const GlobalSearchReplyMessage & msg
        );
        
public:

    //
    // From SystemMessage.
    //

    /** */
    virtual SystemMessage::Type get_type() const;

    /** */
    virtual SystemMessage::ExecuteResult execute();
    
public:

    /** Serialize to bitstream. */
    virtual void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description & desc
        ) const;

    /** Read from bitstream. */
    virtual void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description & desc
        );

public:
    SInt< UInt32, 32 >  session_id;
        /**< Search session identification.
         * @see GlobalSearchMessage
         */
    
    }; // GlobalSearchReplyMessage

/** @} */ // addtogroup sysevent


} // namespace Massiv::Core
} // namespace Massiv

#endif // MASSIV_GLOBAL_SEARCH_MSG_H
