/** @file node_action_msg.cpp
 * Various node action messages (implementation).
 */

/*
 * begin        : Sat Oct 4 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "sysevent/node_action_msg.h"
#include "global/global.h"
#include "node/node_action_manager.h"


namespace Massiv {
namespace Core {


// NodeActionMessage
//~~~~~~~~~~~~~~~~~~


NodeActionMessage::NodeActionMessage()
    {
    }


NodeActionMessage::NodeActionMessage( const NodeActionId & the_action_id ) :
    action_id( the_action_id )
    {
    }


NodeActionMessage::NodeActionMessage( const NodeActionMessage & other ) :
    Super( other ),
    action_id( other.action_id )
    {
    }

    
NodeActionMessage & NodeActionMessage::operator=( const NodeActionMessage & other )
    {
    Super::operator=( other );
    action_id = other.action_id;
    return *this;
    }


void NodeActionMessage::bitstr_write
    (
    BitOStream &                        bos,
    const Serializer::Description &     desc
    ) const
    {
    Super::bitstr_write( bos, desc );
    action_id.bitstr_write( bos, desc );
    }


void NodeActionMessage::bitstr_read
    (
    BitIStream &                        bis,
    const Serializer::Description &     desc
    )
    {
    Super::bitstr_read( bis, desc );
    action_id.bitstr_read( bis, desc );
    }


// NodeActionRequestMessage
//~~~~~~~~~~~~~~~~~~~~~~~~~


SystemMessage::ExecuteResult NodeActionRequestMessage::execute()
    {
    // All request messages are handled by same node action manager's method.
    NodeActionManager & node_action_manager = Global::node_action_manager();
    return node_action_manager.handle_node_action_request_message( *this );
    }


SystemMessage::Type NodeActionRequestMessage::get_type() const
    {
    return NODE_ACTION_REQUEST;
    }


void NodeActionRequestMessage::bitstr_write
    (
    BitOStream &                        bos,
    const Serializer::Description &     desc
    ) const
    {
    Super::bitstr_write( bos, desc );
    action_counter.bitstr_write( bos, desc );
    }


void NodeActionRequestMessage::bitstr_read
    (
    BitIStream &                        bis,
    const Serializer::Description &     desc
    )
    {
    Super::bitstr_read( bis, desc );
    action_counter.bitstr_read( bis, desc );
    }

    
// NodeActionRequestReplyMessage
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


SystemMessage::Type NodeActionRequestReplyMessage::get_type() const
    {
    return NODE_ACTION_REQUEST_REPLY;
    }

SystemMessage::ExecuteResult NodeActionRequestReplyMessage::execute()
    {
    NodeActionManager & node_action_manager = Global::node_action_manager();
    return node_action_manager.handle_node_action_request_reply_message( *this );
    }


void NodeActionRequestReplyMessage::bitstr_write
    (
    BitOStream &                        bos,
    const Serializer::Description &     desc
    ) const
    {
    Super::bitstr_write( bos, desc );
    action_counter.bitstr_write( bos, desc );
    }


void NodeActionRequestReplyMessage::bitstr_read
    (
    BitIStream &                        bis,
    const Serializer::Description &     desc
    )
    {
    Super::bitstr_read( bis, desc );
    action_counter.bitstr_read( bis, desc );
    }

    
// NodeActionCoordinatorSelectedMessage
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


SystemMessage::Type NodeActionCoordinatorSelectedMessage::get_type() const
    {
    return NODE_ACTION_COORDINATOR_SELECTED;
    }

SystemMessage::ExecuteResult NodeActionCoordinatorSelectedMessage::execute()
    {
    NodeActionManager & node_action_manager = Global::node_action_manager();
    return node_action_manager.handle_node_action_coordinator_selected_message( *this );
    }

    
// NodeActionPrepareFinishMessage
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


SystemMessage::Type NodeActionPrepareFinishMessage::get_type() const
    {
    return NODE_ACTION_PREPARE_FINISH;
    }

SystemMessage::ExecuteResult NodeActionPrepareFinishMessage::execute()
    {
    NodeActionManager & node_action_manager = Global::node_action_manager();
    return node_action_manager.handle_node_action_prepare_finish_message( *this );
    }


// NodeActionPrepareFinishReplyMessage
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


SystemMessage::Type NodeActionPrepareFinishReplyMessage::get_type() const
    {
    return NODE_ACTION_PREPARE_FINISH_REPLY;
    }

SystemMessage::ExecuteResult NodeActionPrepareFinishReplyMessage::execute()
    {
    NodeActionManager & node_action_manager = Global::node_action_manager();
    return node_action_manager.handle_node_action_prepare_finish_reply_message( *this );
    }

    
// NodeActionFinishMessage
//~~~~~~~~~~~~~~~~~~~~~~~~


SystemMessage::Type NodeActionFinishMessage::get_type() const
    {
    return NODE_ACTION_FINISH;
    }

SystemMessage::ExecuteResult NodeActionFinishMessage::execute()
    {
    NodeActionManager & node_action_manager = Global::node_action_manager();
    return node_action_manager.handle_node_action_finish_message( *this );
    }


} // namespace Massiv::Core
} // namespace Massiv
