/** @file object_status_msg.cpp
 * Object status message.
 */

/*
 * begin        : Tue Apr 30 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "sysevent/object_status_msg.h"
#include "net/bitistream.h"
#include "net/bitostream.h"
#include "global/global.h"
#include "object/object_manager.h"
#include <ostream>


namespace Massiv {
namespace Core {


ObjectStatusMessage::ObjectStatusMessage() :
    trustworthy( false )
    {
    }


ObjectStatusMessage::ObjectStatusMessage
    (
    const ObjectId &        id,
    const ObjectStatus &    status,
    const NodeId &          sid,
    bool                    trust
    ) :
    object( id ),
    object_status( status ),
    node( sid ),
    trustworthy( trust )
    {
    }


ObjectStatusMessage::ObjectStatusMessage
    (
    const ObjectStatusMessage & s
    ) :
    Super( s ),
    object( s.object ),
    object_status( s.object_status ),
    node( s.node ),
    trustworthy( s.trustworthy )
    {
    }


ObjectStatusMessage & ObjectStatusMessage::operator=
    (
    const ObjectStatusMessage & s
    )
    {
    Super::operator=( s );
    set( s.object, s.object_status, s.node, s.trustworthy );
    return *this;
    }


void ObjectStatusMessage::set
    (
    const ObjectId &        id,
    const ObjectStatus &    status,
    const NodeId &          sid,
    bool                    trust
    )
    {
    object          = id;
    object_status   = status;
    node            = sid;
    trustworthy     = trust;
    }


SystemMessage::Type ObjectStatusMessage::get_type() const
    {
    return SystemMessage::OBJECT_STATUS;
    }


SystemMessage::ExecuteResult ObjectStatusMessage::execute()
    {
    ObjectManager &      object_manager = Global::object_manager();
    object_manager.handle_incoming_object_status_message( *this );

    return SystemMessage::EXECUTE_OK;
    }


void ObjectStatusMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );

    object.bitstr_write( bos, desc );
    object_status.bitstr_write( bos, desc );
    node.bitstr_write( bos, desc );
    trustworthy.bitstr_write( bos, desc );
    }


void ObjectStatusMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    object.bitstr_read( bis, desc );
    object_status.bitstr_read( bis, desc );
    node.bitstr_read( bis, desc );
    trustworthy.bitstr_read( bis, desc );
    }


std::ostream & operator<<
    (
    std::ostream &              s,
    const ObjectStatusMessage & p
    )
    {
    s << "objectid: " << p.object
      << " status: " << p.object_status
      << " nodeid: " << p.node
      << " trustworthy: " << ( p.trustworthy ? "yes" : "no" );
    return s;
    }


}; // namespace Massiv::Core
}; // namespace Massiv
