/** @file object_status_msg.h
 * Object status message.
 */

/*
 * begin        : Tue Apr 30 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_OBJECT_STATUS_MSG_H
#define MASSIV_OBJECT_STATUS_MSG_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_SYSTEM_MSG_H
#include "sysevent/system_msg.h"
#endif

#ifndef MASSIV_BITIOSTREAM_FWD_H
#include "net/bitiostream_fwd.h"
#endif

#ifndef MASSIV_OBJECTID_H
#include "pointer/objectid.h"
#endif

#ifndef MASSIV_NODEID_H
#include "node/nodeid.h"
#endif

#ifndef MASSIV_STYPE_BOOL_H
#include "serializer/stype_bool.h"
#endif

#ifndef MASSIV_OBJECT_STATUS_H
#include "object/object_status.h"
#endif

#include <iosfwd>


namespace Massiv {
namespace Core {


/** @addtogroup sysevent
 * @{
 */

/**
 * ObjectStatus system message.
 * Used for notifying of object status.
 */
class ObjectStatusMessage : public SystemMessage
    {
    typedef SystemMessage   Super;

public:

    /** 
     * Constructor.
     */
    ObjectStatusMessage();

    /** 
     * Constructor.
     */
    ObjectStatusMessage
        (
        const ObjectId &        object,
        const ObjectStatus &    object_status,
        const NodeId &          node,
        bool                    trustwothy
        );

    /** 
     * Copy constructor.
     */
    ObjectStatusMessage
        (
        const ObjectStatusMessage & msg
        );

    /**
     * Operator=.
     */
     ObjectStatusMessage & operator=
        (
        const ObjectStatusMessage & msg
        );

public:

    /** 
     * Set content.
     */
    void set
        (
        const ObjectId &        object,
        const ObjectStatus &    object_status,
        const NodeId &          node,
        bool                    trustwothy
        );

public:

    //
    // From SystemMessage.
    //

    /**
     *
     */
    virtual SystemMessage::Type get_type() const;

    /**
     *
     */
    virtual ExecuteResult execute();

public:

    /**
     * Serialize to bitstream.
     */
    virtual void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description & desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description & desc
        );

public:

    ObjectId        object;
        /**< Object id of the object we are notifying of. */

    ObjectStatus    object_status;
        /**< Object status of the object. */

    NodeId          node;
        /**< Node id associated with the object status. */

    SBool           trustworthy;
        /**< Specifies whether this status is from some cache
         * or from object owner. Used by object providers
         * for tracked objects.
         */

    }; // ObjectStatusMessage


/**
 * Write ObjectStatus message content to output stream.
 */
std::ostream & operator<<
    (
    std::ostream &              s,
    const ObjectStatusMessage & status
    );

/** @} */ // addtogroup sysevent


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_OBJECT_STATUS_MSG_H
