/** @file replication_msg_inline.h
 * Replication-related messages (inline methods).
 */

/*
 * begin        : Feb 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Massiv {
namespace Core {


// class ReplicationUpdateMessage::ObjectInfo

ReplicationUpdateMessage::ObjectInfo::ObjectInfo()
      : major_version( 0 )
    {
    }


ReplicationUpdateMessage::ObjectInfo::ObjectInfo
    (
    const ObjectId &    the_object_id,
    ObjectVersion       the_major_version
    ) : object_id( the_object_id ),
        major_version( the_major_version )
    {
    }


void ReplicationUpdateMessage::ObjectInfo::bitstr_write
    (
    BitOStream &                        bos,
    const Serializer::Description &     desc
    ) const
    {
    object_id.bitstr_write( bos, desc );
    major_version.bitstr_write( bos, desc );
    }


void ReplicationUpdateMessage::ObjectInfo::bitstr_read
    (
    BitIStream &                        bis,
    const Serializer::Description &     desc
    )
    {
    object_id.bitstr_read( bis, desc );
    major_version.bitstr_read( bis, desc );
    }


// class ReplicationUpdateMessage

bool ReplicationUpdateMessage::is_empty() const
    {
    return objects_to_update.empty() && objects_to_destroy.empty();
    }


void ReplicationUpdateMessage::add_object_to_update
    (
    const ObjectId &    object_id,
    ObjectVersion       major_version
    )
    {
    hope( object_id.is_object_info_available() );
    objects_to_update.push_back( ObjectInfo( object_id, major_version ) );
    }


void ReplicationUpdateMessage::add_unchanged_object
    (
    const ObjectId &    object_id,
    ObjectVersion       major_version
    )
    {
    hope( object_id.is_object_info_available() );
    unchanged_objects.push_back( ObjectInfo( object_id, major_version ) );
    }


void ReplicationUpdateMessage::add_object_to_destroy
    (
    const ObjectId &    object_id,
    ObjectVersion       major_version
    )
    {
    objects_to_destroy.push_back( ObjectInfo( object_id, major_version ) );
    }


} // namespace Massiv::Core
} // namespace Massiv
