/** @file simulation_sync_msg.cpp
 * Simulation synchronization messages.
 */

/*
 * begin        : Sat Jan 18 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

 
#include "sysevent/simulation_sync_msg.h"
#include "net/bitistream.h"
#include "net/bitostream.h"
#include "global/global.h"
#include "serializer/stype_int.h"
#include "node/node_action_startup.h"
#include "serializer/stype_vli.h"


namespace Massiv {
namespace Core {


StartupMessage::StartupMessage() :
    id( ARCHIVATION_STATE_REQUEST )
    {
    }


StartupMessage::StartupMessage
    (
    Id the_id
    ) :
    id( the_id )
    {
    }


StartupMessage::StartupMessage
    (
    const StartupMessage & other
    ) :
    Super( other ),
    id( other.id ),
    archivation_id( other.archivation_id ),
    providers( other.providers )
    {
    }


StartupMessage & StartupMessage::operator=
    (
    const StartupMessage & other
    )
    {
    Super::operator=( other );
    id = other.id;
    archivation_id = other.archivation_id;
    providers = other.providers;
    return *this;
    }


SystemMessage::Type StartupMessage::get_type() const
    {
    return SystemMessage::STARTUP;
    }


SystemMessage::ExecuteResult StartupMessage::execute()
    {
    NodeActionStartup::handle_startup_message( *this );
    return EXECUTE_OK;
    }


void StartupMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );
    id.bitstr_write( bos, desc );
    archivation_id.bitstr_write( bos, desc );
    
    SVli< UInt32 > size = providers.size();
    size.bitstr_write( bos, desc );
    
    Providers::const_iterator it;
    for( it = providers.begin(); it != providers.end(); ++it )
        {
        const ProviderId & provider_id = *it;
        provider_id.bitstr_write( bos, desc );
        }
    }


void StartupMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );
    id.bitstr_read( bis, desc );
    archivation_id.bitstr_read( bis, desc );
    
    SVli< UInt32 > size;
    size.bitstr_read( bis, desc );
    
    for( ; size > 0; size-- )
        {
        ProviderId provider_id;
        provider_id.bitstr_read( bis, desc );
        providers.push_back( provider_id );
        }
    }


} // namespace Massiv::Core
} // namespace Massiv
