/** @file simulation_sync_msg.h
 * Simulation synchronization messages.
 */

/*
 * begin        : Sat Jan 18 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SIMULATION_SYNC_MSG_H
#define MASSIV_SIMULATION_SYNC_MSG_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_SYSTEM_MSG_H
#include "sysevent/system_msg.h"
#endif

#ifndef MASSIV_ARCHIVATION_ID_H
#include "object/archivation_id.h"
#endif

#ifndef MASSIV_PROVIDERID_H
#include "object/providerid.h"
#endif

#ifndef MASSIV_STYPE_ENUM_H
#include "serializer/stype_enum.h"
#endif

#include <vector>


namespace Massiv {
namespace Core {


/** @addtogroup sysevent
 * @{
 */

/**
 * Message used by server nodes during startup synchronization.
 * See #NodeActionStartup.
 */
class StartupMessage : public SystemMessage
    {
    typedef SystemMessage Super;
    
public:

    /** Message id. */
    enum Id
        {
        ARCHIVATION_STATE_REQUEST,
            /** Sent by coordinator to obtain valid archive files
             * located on each server node.
             */
        ARCHIVATION_STATE_REPLY,
            /**< Reply to ARCHIVATION_STATE_REQUEST message. */
        LOAD_SIMULATION_REQUEST,
            /**< Sent by coordinator to load simulation state from an archive file
             * to memory.
             */
        LOAD_SIMULATION_REPLY,
            /**< Reply to LOAD_SIMULATION_REQUEST. */
        UNPAUSE_SIMULATION_REQUEST
            /**< Sent by coordinator to unpause previously loaded simulation. */
        }; // Id

public:

    /** Constructor. */
    StartupMessage();
    
    /** Constructor. */
    StartupMessage
        (
        Id id
        );
    
    /** Copy constructor. */
    StartupMessage
        (
        const StartupMessage & other
        );
    
    /** Operator=. */
    StartupMessage & operator=
        (
        const StartupMessage & other
        );
        
public:

    //
    // From SystemMessage.
    //

    /** */
    virtual SystemMessage::Type get_type() const;

    /** */
    virtual ExecuteResult execute();

public:

    /**
     * Serialize to bitstream.
     */
    virtual void bitstr_write
        (
        BitOStream &                    bos,
        const Serializer::Description & desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual void bitstr_read
        (
        BitIStream &                    bis,
        const Serializer::Description & desc
        );

public:

    /** List of object provider ids. */
    typedef std::vector< ProviderId >   Providers;

    SEnum< Id >     id;
        /**< Startup message id. */

    ArchivationId   archivation_id;
        /**< For ARCHIVATION_STATE_REPLY contains last valid archivation id
         * of archive file which is available on the sender node.
         * For LOAD_SIMULATION_REQUEST contains archivation id of archive
         * file from which simulation should be loaded.
         */
    
    Providers       providers;
        /**< For LOAD_SIMULATION_REPLY conatins list of object providers
         * owned by the sender node.
         */
    
    }; // StartupMessage

/** @} */ // addtogroup sysevent


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_SIMULATION_SYNC_MSG_H
