/** @file system_event.h
 * System event common stuff.
 */

/*
 * begin        : Wed Apr 17 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_SYSTEM_EVENT_H
#define MASSIV_SYSTEM_EVENT_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_REFERENCE_H
#include "utility/reference.h"
#endif

#ifndef MASSIV_SYSTEM_EVENT_FWD_H
#include "sysevent/system_event_fwd.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup sysevent
 * @{
 */

/**
 * System event.
 */
class SystemEvent
    {
public:

    /**
     * System event type.
     */
    enum Type
        {
        SYSTEM_MESSAGE  = 0,
            /**< System message received from network. */        
        INVALID         = 1,
            /**< Invalid type. Do not use. Must be same as @c LAST. */
        LAST            = 1
            /**< Invalid type. Do not use. Must be same as @c INVALID. */
        };
   
public:

    /** Constructor. */
    SystemEvent();

    /** Copy constructor. */
    SystemEvent
        (
        const SystemEvent & event
        );

    /** Operator=. */
    SystemEvent & operator=
        (
        const SystemEvent & event
        );

public:

    /**
     * Execute the event.
     */
    void execute();

public:

    Type                        type;
        /**< The event type. */

    Reference< SystemMessage >  system_msg;
        /**< System message. Valid if event type is @c SYSTEM_MESSAGE. */

    }; // SystemEvent

/** @} */ // addtogroup sysevent

    
} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_SYSTEM_EVENT_H
