/** @file time_synch_msg.cpp
 * Time synchronization message.
 */

/*
 * begin        : Sun Jan 5 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "global/global.h"
#include "serializer/stype_float.h"
#include "sysevent/time_synch_msg.h"
#include "time/time_manager.h"
#include "net/bitistream.h"
#include "net/bitostream.h"


namespace Massiv {
namespace Core {


TimeSynchronizationMessage::TimeSynchronizationMessage()
    {
    }


TimeSynchronizationMessage::TimeSynchronizationMessage
    (
    bool                    _synchronized,
    const STime &           _massiv_time,
    const STime &           _sync_interval
    ) :
    synchronized( _synchronized ),
    massiv_time( _massiv_time ),
    sync_interval( _sync_interval )
    {
    }


TimeSynchronizationMessage::TimeSynchronizationMessage
    (
    const TimeSynchronizationMessage &      msg
    ) :
    Super( msg ),
    synchronized( msg.synchronized ),
    massiv_time( msg.massiv_time ),
    sync_interval( msg.sync_interval )
    {
    }


TimeSynchronizationMessage & TimeSynchronizationMessage::operator=
    (
    const TimeSynchronizationMessage &      msg
    )
    {
    Super::operator=( msg );
    synchronized = msg.synchronized;
    massiv_time = msg.massiv_time;
    sync_interval = msg.sync_interval;
    return *this;
    }


SystemMessage::Type TimeSynchronizationMessage::get_type() const
    {
    return SystemMessage::TIME_SYNCHRONIZATION;
    }


SystemMessage::ExecuteResult TimeSynchronizationMessage::execute()
    {
    TimeManager &      time_manager = Global::time_manager();
    time_manager.handle_time_synchronization_message( *this );

    return SystemMessage::EXECUTE_OK;
    }


void TimeSynchronizationMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );

    synchronized.bitstr_write( bos, desc );
    massiv_time.bitstr_write( bos, desc );
    sync_interval.bitstr_write( bos, desc );
    }


void TimeSynchronizationMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    synchronized.bitstr_read( bis, desc );
    massiv_time.bitstr_read( bis, desc );
    sync_interval.bitstr_read( bis, desc );
    }


// Time magic constant message implementation

TimeMagicConstantMessage::TimeMagicConstantMessage()
    {
    }


TimeMagicConstantMessage::TimeMagicConstantMessage
    (
    Float64         _magic_constant
    ) :
    magic_constant( _magic_constant )
    {
    }


TimeMagicConstantMessage::TimeMagicConstantMessage
    (
    const TimeMagicConstantMessage &      msg
    ) :
    Super( msg ),
    magic_constant( msg.magic_constant )
    {
    }


TimeMagicConstantMessage & TimeMagicConstantMessage::operator=
    (
    const TimeMagicConstantMessage &      msg
    )
    {
    Super::operator=( msg );
    magic_constant = msg.magic_constant;
    return *this;
    }


SystemMessage::Type TimeMagicConstantMessage::get_type() const
    {
    return SystemMessage::TIME_MAGIC_CONSTANT;
    }


SystemMessage::ExecuteResult TimeMagicConstantMessage::execute()
    {
    TimeManager &      time_manager = Global::time_manager();
    time_manager.handle_time_magic_constant_message( *this );

    return SystemMessage::EXECUTE_OK;
    }


void TimeMagicConstantMessage::bitstr_write
    (
    BitOStream &                    bos,
    const Serializer::Description & desc
    ) const
    {
    Super::bitstr_write( bos, desc );

    magic_constant.bitstr_write( bos, desc );
    }


void TimeMagicConstantMessage::bitstr_read
    (
    BitIStream &                    bis,
    const Serializer::Description & desc
    )
    {
    Super::bitstr_read( bis, desc );

    magic_constant.bitstr_read( bis, desc );
    }


} // namespace Massiv::Core
} // namespace Massiv
