/** @file time_synch_msg.h
 * Time synchronization message.
 */

/*
 * begin        : Sun Jan 5 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_TIME_SYNCH_MSG_H
#define MASSIV_TIME_SYNCH_MSG_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_SYSTEM_MSG_H
#include "sysevent/system_msg.h"
#endif

#ifndef MASSIV_BITIOSTREAM_FWD_H
#include "net/bitiostream_fwd.h"
#endif

#ifndef MASSIV_STYPE_TIME_H
#include "serializer/stype_time.h"
#endif

#ifndef MASSIV_STYPE_BOOL_H
#include "serializer/stype_bool.h"
#endif

#ifndef MASSIV_STYPE_FLOAT_H
#include "serializer/stype_float.h"
#endif

#ifndef MASSIV_STYPE_INT_H
#include "serializer/stype_int.h"
#endif



namespace Massiv {
namespace Core {


/** @addtogroup sysevent
 * @{
 */

/**
 * Time synchronization message.
 *
 * @ingroup time
 */
class TimeSynchronizationMessage : public SystemMessage
    {
    typedef SystemMessage   Super;

public:

    /** 
     * Constructor. 
     */
    TimeSynchronizationMessage();

    /**
     * Constructor.
     */
    TimeSynchronizationMessage
        (
        bool                    _synchronized,
        const STime &           _massiv_time,
        const STime &           _sync_interval
        );

    /**
     * Copy constructor.
     */
    TimeSynchronizationMessage
        (
        const TimeSynchronizationMessage &      msg
        );

    /**
     * Operator=.
     */
    TimeSynchronizationMessage & operator=
        (
        const TimeSynchronizationMessage &       msg
        );

public:

    //
    // From SystemMessage.
    //

    /**
     *
     */
    virtual SystemMessage::Type get_type() const;

    /**
     *
     */
    virtual ExecuteResult execute();

public:

    /**
     * Serialize to bitstream.
     */
    virtual void bitstr_write
        (
        BitOStream &                        bos,
        const Serializer::Description &     desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual void bitstr_read
        (
        BitIStream &                        bis,
        const Serializer::Description &     desc
        );

public:

    SBool               synchronized;
       /**< True if server was once synchronized. */

    STime               massiv_time;
        /**< Massiv time of a server. */
    
    STime               sync_interval;
        /**< Time interval of synchronization process. */

    }; // TimeSynchronizationMessage


/**
 * Time magic constant message.
 *
 * @ingroup time
 */
class TimeMagicConstantMessage : public SystemMessage
    {
    typedef SystemMessage   Super;

public:

    /** 
     * Constructor. 
     */
    TimeMagicConstantMessage();

    /**
     * Constructor.
     */
    TimeMagicConstantMessage
        (
        Float64         _magic_constant
        );

    /**
     * Copy constructor.
     */
    TimeMagicConstantMessage
        (
        const TimeMagicConstantMessage &      msg
        );

    /**
     * Operator=.
     */
    TimeMagicConstantMessage & operator=
        (
        const TimeMagicConstantMessage &       msg
        );

public:

    //
    // From SystemMessage.
    //

    /**
     *
     */
    virtual SystemMessage::Type get_type() const;

    /**
     *
     */
    virtual ExecuteResult execute();

public:

    /**
     * Serialize to bitstream.
     */
    virtual void bitstr_write
        (
        BitOStream &                        bos,
        const Serializer::Description &     desc
        ) const;

    /**
     * Read from bitstream.
     */
    virtual void bitstr_read
        (
        BitIStream &                        bis,
        const Serializer::Description &     desc
        );

public:

    SFloat64           magic_constant;
       /**< Magic constant to adjust massiv time. See time_manager.h
            for more information. */

    }; // TimeMagicConstantMessage

/** @} */ // addtogroup sysevent


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_TIME_SYNCH_MSG_H
