/** @file system_context.cpp
 * System context implementation.
 */

/*
 * begin        : Fri Dec 5 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "system/system_context.h"
#include "system/system.h"


namespace Massiv {


// Implementation of SystemContext
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
SystemContext::SystemContext
    (
    int flags
    ) : flags( flags )
    {
    }


// Implementation of SystemContext::SchedulerSettings
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
SystemContext::SchedulerSettings::SchedulerSettings() :
    Core::Settings( "System/Scheduler" ),
    tick_delta_low_limit( access_float( "tick_delta_low_limit", 0.005f ) ),
    tick_delta_high_limit( access_float( "tick_delta_high_limit", 0.02f ) ),
    min_sleep_time( access_float( "min_sleep_time", 0.005f ) ),
    max_sleep_time( access_float( "max_sleep_time", 0.05f ) ),
    sleep_time_increment_constant( access_float( "sleep_time_increment_constant", 0.005f ) ),
    sleep_time_decrement_constant( access_float( "sleep_time_decrement_constant", 0.01f ) )
    {
    }
    
    
// Implementation of SystemContext::SRPCSettings
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    
SystemContext::SRPCSettings::SRPCSettings() :
    Core::Settings( "System/SRPC" ),
    num_threads( access_integer( "num_threads", 4 ) ),
    reply_timeout( access_float( "reply_timeout", 30.0f ) ),
    optimize_local_calls( access_boolean( "optimize_local_calls", true ) ),
    min_stack_size( access_integer( "min_stack_size", 64 * 1024 ) ),
    statistics_period( access_float( "statistics_period", 5.0f ) )
    {
    }


// Implementation of SystemContext::SchedulerStatistics
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SystemContext::SchedulerStatistics::SchedulerStatistics() :
    Core::Statistics( "System/Scheduler", Core::Status::FACILITY_SYSTEM, 
        Core::Statistics::OPEN_NORMAL ),
    sleep_time( create_float( "sleep_time", 0.0f ) ),
    tick_boosted( create_boolean( "tick_boosted", false ) ),
    total_sleep_time( create_float( "total_sleep_time", 0.0f ) ),
    total_core_time( create_float( "total_core_time", 0.0f ) ),
    total_client_time( create_float( "total_client_time", 0.0f ) )
    {
    }


// Implementation of SystemContext::SRPCStatistics
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
SystemContext::SRPCStatistics::SRPCStatistics() :
    Core::Statistics( "System/SRPC", Core::Status::FACILITY_RPC, 
        Core::Statistics::OPEN_NORMAL ),
    total_calls( create_large_integer( "total_calls", 0 ) ),
    total_wait_time( create_float( "total_wait_time", 0.0f ) ),
    time_mark( create_float( "time_mark", 0.0f ) ),
    num_recent_calls( create_integer( "num_recent_calls", 0 ) ),
    recent_wait_time( create_float( "recent_wait_time", 0 ) ),
    average_calls_per_second( create_float( "average_calls_per_second", 0 ) ),
    average_response_time( create_float( "average_response_time", 0 ) )
    {
    }


} // namespace Massiv
