/** @file tick_lock.h
 * System tick lock.
 */

/*
 * begin        : Mon Jun 20 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TICK_LOCK_H
#define MASSIV_TICK_LOCK_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif


namespace Massiv {
namespace Core {
    

/**
 * Current system tick lock.
 * Standard constructor-destructor lock semantics.
 *
 * @ingroup unclassified
 */
class TickLock
    {
private:

    NO_COPY_CONSTRUCTOR( TickLock );
    NO_ASSIGNMENT_OPERATOR( TickLock );

public:
        
    /** Locks current system tick. */
    TickLock();

    /** Unlocks current system tick. */
    ~TickLock();

    }; // TickLock


/**
 * Use this to lock current system tick and prevent client from reentering
 * main core loop (calling @c System::next_tick()).
 * The tick is automatically unlocked when @c LOCK_SYSTEM() goes
 * out of scope.
 */
#define LOCK_SYSTEM()   TickLock system_lock


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_TICK_LOCK_H
