/** @file data_test.cpp
 * Testing data management.
 */

/*
 * begin        : Sun Feb 09 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "system/system.h"
#include "test/data_test/data_test.h"
#include "thread/thread.h"
#include "data/data_manager.h"
#include "net/net.h"
#include "time/system_time.h"

using namespace Massiv::Core;
using namespace Massiv::Core::Test;


DataTest::DataTest()
    {
    }


void DataTest::run()
    {
    System::initialize( System::INIT_COUT_LOGGER_DESTINATION );

    System::connect();

    while( System::get_phase() != System::PHASE_RUNNING )
        {
        System::next_tick();
        Thread::sleep( 1 );
        }
    
    static DataManager &    data_manager = Global::data_manager();
    while( !data_manager.is_ready_for_use() )
        {
        System::next_tick();
        Thread::sleep( 1 );
        }
    static Network & network = Global::network();
    NodeId local_node_id = network.get_local_node_id();
    NodeId node_id_1 = NodeId::create_from_type_and_relative_id( 0, 1 );
    NodeId node_id_2 = NodeId::create_from_type_and_relative_id( 0, 2 );
 
    int server_wait_time = 5;
    int client_wait_time = 13;
    int num = 10;

    SystemTime start_time = SystemTime::current_time();
    DataObject * data_object;
    while( 1 )
        {
        if( local_node_id == node_id_1 )
            {
            // Master data server.
            SystemTime current_time = SystemTime::current_time();
            hope( current_time >= start_time );
            if( ( current_time - start_time > server_wait_time ) && ( num ) )
                {

/*              start_time = current_time;
                num--;
                // Update server.
                std::string     data_node_name;
                std::string     data_file_name;
                std::string     data_full_file_name;
                std::string     data_object_name;
                std::string     file_content;
                DataNode *      data_node;

                std::ostringstream ss;
                ss << "test_" << ( int )current_time;
                std::string     base_name = ss.str();
                data_node_name = "/" + "test";
                data_file_name = base_name + ".do";
                data_full_file_name = data_manager.get_data_directory() + "/" + data_file_name;
                data_object_name = "test";
                data_node = new DataNode( data_node_name, data_file_name, DataManager::BLANK );
                

                // Create empty data file.
                std::ofstream ofs( data_full_file_name.c_str() );
                if( !ofs )
                    {
                    throw ExCommonError( "Cannot create file \"" + data_full_file_name + "\"." );
                    }
                ofs.close();
                data_manager.switch_to_update_phase();
                data_manager.add_data( data_node, data_object_name );
                data_manager.finish_update_phase();*/

                start_time = current_time;
                num--;
                // Update server.
                data_manager.switch_to_update_phase();
                try
                    {
                    data_manager.update_data_file( "test.do" );
                    data_manager.finish_update_phase();
                    }
                catch( ... )
                    {
                    data_manager.finish_update_phase();
                    // Data file doesnt exist. Create it.
                    std::string     data_node_name;
                    std::string     data_file_name;
                    std::string     data_full_file_name;
                    std::string     data_object_name;
                    std::string     file_content;
                    DataNode *      data_node;

                    data_node_name = "/test";
                    data_file_name = "test.do";
                    data_full_file_name = data_manager.get_data_directory() + "/" + data_file_name;
                    data_object_name = "test";
                    data_node = new DataNode( data_node_name, data_file_name, DataManager::BLANK );
                
                    // Create empty data file.
                    std::ofstream ofs( data_full_file_name.c_str() );
                    if( !ofs )
                        {
                        throw ExCommonError( "Cannot create file \"" + data_full_file_name + "\"." );
                        }
                    ofs << "Test file";
                    ofs.close();
                    data_manager.switch_to_update_phase();
                    data_manager.add_data( data_node, data_object_name );
                    data_manager.finish_update_phase();
                    }
                }
            }
        if( local_node_id == node_id_2 )
            {
            // Data client.
            SystemTime current_time = SystemTime::current_time();
            hope( current_time >= start_time );
            if( current_time - start_time > client_wait_time )
                {
                start_time = current_time;
                static int num2 = 0;
                if( num2 == 0 )
                    {
                    try
                        {
                        data_object = data_manager.get_data_object( "test", true );
                        num2++;
                        }
                    catch( ... )
                        {
                        // Data object test doesnt exist yet.
                        }
                    }
                else
                    {
                    // Data object "test" already created.
                    if( data_object->is_data_updated() )
                        {
                        data_object->refresh( true );
                        }
                    }
                }
            }
        System::next_tick();
        Thread::sleep( 1 );
        }
    }
