/** @file factgen_objects.h
 * Simple game object hierarchy used to test factory generator.
 *
 * Class hieararchy (FIXME: draw this using something else):
 * @verbatim
                     Base
                 /   /  \   \
               /    /  v \  v \
             A     B      X     Y
              \   |        |   /
               \  |        |  /
                 C          Z
                   \      /
                    \    /
                     Test
@endverbatim
 */

/*
 * begin        : Mon Oct 14 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_FACTGEN_OBJECTS_H
#define MASSIV_FACTGEN_OBJECTS_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_PROP_POINTER_H
#include "property/prop_pointer.h"
#endif

#ifndef MASSIV_PROP_INT_H
#include "property/prop_int.h"
#endif

#ifndef MASSIV_PROP_BOOL_H
#include "property/prop_bool.h"
#endif

#ifndef MASSIV_PROP_FLOAT_H
#include "property/prop_float.h"
#endif

#ifndef MASSIV_OBJECT_H
#include "object/object.h"
#endif



using namespace Massiv::Core;


namespace Massiv {
namespace Core {
namespace Test {

#if 0

/**
 * Parent of our stupid object hiearachy.
 */
class ObjectBase : public virtual Object
    {
    /*
    IDL_BEGIN
    class Massiv::Core::Test::ObjectBase
        parent virtual Massiv::Core::Object
        property base
    endclass
    IDL_END
    */

public:

    /**
     * Default constructor.
     */
    ObjectBase();

    /**
     * Destructor.
     */
    virtual ~ObjectBase();

    /**
     * Initialize object.
     */
    void initialize();

public:

    PInt< Int32, 32 > base;
        /**< Nice property. */

    }; // class ObjectBase


/**
 * Object A.
 */
class ObjectA : public ObjectBase
    {
    /*
    IDL_BEGIN
    class Massiv::Core::Test::ObjectA
        parent Massiv::Core::Test::ObjectBase
        property a
    endclass
    IDL_END
    */

public:

    /**
     * Default constructor.
     */
    ObjectA();

    /**
     * Destructor.
     */
    virtual ~ObjectA();

    /**
     * Initialize object.
     */
    void initialize();

public:

    PInt< UInt32, 24 > a;
        /**< Property. */

    }; // class ObjectA


/**
 * Object B.
 */
class ObjectB : public ObjectBase
    {
    /*
    IDL_BEGIN
    class Massiv::Core::Test::ObjectB
        parent Massiv::Core::Test::ObjectBase
        property b
    endclass
    IDL_END
    */

public:

    /**
     * Default constructor.
     */
    ObjectB();

    /**
     * Destructor.
     */
    virtual ~ObjectB();

    /**
     * Initialize object.
     */
    void initialize();

public:

    PInt< UInt64, 9 > b;
        /**< Property. */

    }; // class ObjectB


/**
 * Object C.
 */
class ObjectC : public ObjectA, public ObjectB
    {
    /*
    IDL_BEGIN
    class Massiv::Core::Test::ObjectC
        parent Massiv::Core::Test::ObjectA, Massiv::Core::Test::ObjectB
        property c
        method print
            arg separator : const std::string & -> Massiv::Core::PString
            arg indent :    int -> PInt32
        endmethod
    endclass
    IDL_END
    */

public:

    /**
     * Default constructor.
     */
    ObjectC();

    /**
     * Destructor.
     */
    virtual ~ObjectC();

    /**
     * Initialize object.
     */
    void initialize();

public:

    /**
     * Print object contents.
     */
    void print
        (
        const std::string & separator,
        int                 indent
        ) const;

public:

    PInt< Int64, 7 > c;
        /**< Property. */

    }; // class ObjectC


/**
 * Object X.
 */
class ObjectX : public virtual ObjectBase
    {
    /*
    IDL_BEGIN
    class Massiv::Core::Test::ObjectX
        parent virtual Massiv::Core::Test::ObjectBase
        property x
    endclass
    IDL_END
    */

public:

    /**
     * Default constructor.
     */
    ObjectX();

    /**
     * Destructor.
     */
    virtual ~ObjectX();

    /**
     * Initialize object.
     */
    void initialize();

public:

    PFloat< Float32 > x;
        /**< Property. */

    }; // class ObjectX


/**
 * Object Y.
 */
class ObjectY : public virtual ObjectBase
    {
    /*
    IDL_BEGIN
    class Massiv::Core::Test::ObjectY
        parent Massiv::Core::Test::ObjectBase
        property y
    endclass
    IDL_END
    */

public:

    /**
     * Default constructor.
     */
    ObjectY();

    /**
     * Destructor.
     */
    virtual ~ObjectY();

    /**
     * Initialize object.
     */
    void initialize();

public:

    struct FloatQTraits
        {
        static const Float64 min_value;
        static const Float64 max_value;
        enum { bits = 8 };
        };

    PFloatQ< Float64, FloatQTraits > y;
        /**< Property. */

    }; // class ObjectY


/**
 * Object Z.
 */
class ObjectZ : public ObjectX, public ObjectY
    {
    /*
    IDL_BEGIN
    class Massiv::Core::Test::ObjectZ
        parent Massiv::Core::Test::ObjectX, Massiv::Core::Test::ObjectY
        property z
    endclass
    IDL_END
    */

public:

    /**
     * Default constructor.
     */
    ObjectZ();

    /**
     * Destructor.
     */
    virtual ~ObjectZ();

    /**
     * Initialize object.
     */
    void initialize();

public:

    PBool z;
        /**< Property. */

    }; // class ObjectZ


/**
 * The last object in this silly hieararchy.
 */
class ObjectTest : public ObjectC, public ObjectZ
    {
    /*
    IDL_BEGIN
    class Massiv::Core::Test::ObjectTest
        parent Massiv::Core::Test::ObjectC, Massiv::Core::Test::ObjectZ
        property test : repflags = MIGRATE | BIT1 : ptr_repflags = MIGRATE
    endclass
    IDL_END
    */

public:

    /**
     * Default constructor.
     */
    ObjectTest();

    /**
     * Destructor.
     */
    virtual ~ObjectTest();

    /**
     * Initialize object.
     */
    void initialize();

public:

    PPointer<ObjectTest> test;
        /**< Property. */

    }; // class ObjectTest


//@{
/**
 * Objects used to cast from pointer to Object to pointer to various components
 * of ObjectTest.
 *
 * As you can see, this is pretty trick and really annoying to write. So do not
 * use stupid hierarchies similar to this one. For example, RPC will never work
 * even if you write all the cast objects.
 */
class BaseVirtualCast : public CastObject
    {
public:

    /// Perform the cast.
    virtual VariantPointer operator()
        (
        ObjectProperty * object
        ) const;

    /// Get type.
    virtual const std::type_info & get_target_type_info() const;

    }; // class BaseVirtualCast

class BaseViaACast : public CastObject
    {
public:

    /// Perform the cast.
    virtual VariantPointer operator()
        (
        ObjectProperty * object
        ) const;

    /// Get type.
    virtual const std::type_info & get_target_type_info() const;

    }; // class BaseViaACast

class BaseViaBCast : public CastObject
    {
public:

    /// Perform the cast.
    virtual VariantPointer operator()
        (
        ObjectProperty * object
        ) const;

    /// Get type.
    virtual const std::type_info & get_target_type_info() const;

    }; // class BaseViaBCast
//@}

#endif

} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv


#endif // _FACTGEN_OBJECTS_H_
