/** @file test.cpp
 * Base testing class.
 * @todo Send results to logger instead of cout unless testing logger.
 */

/*
 * begin        : Tue Nov 27 2001
 * copyright    : (C) 2001 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "test/test.h"
#include <typeinfo>
#include <iostream>


namespace Massiv {
namespace Core {
namespace Test {


Test::Test
    (
    const std::string & name
    ) : name( name ),
        success_count( 0 ),
        fail_count( 0 ),
        fail_exception( false )
    {
    std::cout << "\n*--------\n" <<
        "Testing " << name << "..." << std::endl;
    }


Test::~Test()
    {
    //std::cout << "Test " << name << " finished: ";
    std::cout << "Finished: ";
    if( fail_exception )
        {
        std::cout << "Unhandled exception after " << ( success_count + fail_count ) <<
                    " test(s), " << fail_count << " of them failed.";
        }
    else
        {
        if( fail_count )
            {
            std::cout << fail_count << " of " <<
                        ( success_count + fail_count ) << " test(s) failed.";
            }
        else
            {
            std::cout << "All " << success_count << " test(s) succeeded.";
            }
        }
    std::cout << "\n--------*" << std::endl;
    }


void Test::exception_failed()
    {
    fail_exception = true;
    }


void Test::test_condition
    (
    bool                result,
    const std::string & message,
    const char *        file,
    int                 line
    )
    {
    if( result )
        {
        test_success();
        }
    else
        {
        test_fail( message, file, line );
        }
    }


void Test::test_fail
    (
    const std::string & message,
    const char *        file,
    int                 line
    )
    {
    std::cout << file << ":" << line << ": " << message << std::endl;
    fail_count++;
    }


void Test::test_success()
    {
    success_count++;
    }


}; // namespace Massiv::Core::Test
}; // namespace Massiv::Core
}; // namespace Massiv
