/** @file test_bitiostream_speed.h
 * Bitstream speed testing.
 */

/*
 * begin        : Tue Feb 18 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TEST_BITIOSTREAM_SPEED_H
#define MASSIV_TEST_BITIOSTREAM_SPEED_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TEST_H
#include "test/test.h"
#endif

#ifndef MASSIV_SYSTEM_TIME_H
#include "time/system_time.h"
#endif



namespace Massiv {
namespace Core {
namespace Test {


class BitIOStreamSpeedTest : public Test
    {
    typedef Test    Super;
    
public:
  
    /**
     * Constructor.
     */
    BitIOStreamSpeedTest();

    /**
     * Destructor.
     */
    ~BitIOStreamSpeedTest();

public:

    /**
     * Run test
     */
    void test();
    
private:

    /**
     * For at least @c subtest_duration seconds writes
     * @a bits number of bits to @c BitOStream.
     */
    void test_uint32_bits_write
        (
        unsigned int bits,
        unsigned int chunk_in_bytes
        );
        
    /**
     * For at least @c subtest_duration seconds writes
     * random number of bits to @c BitOStream.
     */
    void test_uint32_random_bits_write
        (
        unsigned int chunk_in_bytes
        );

    /** Print subtest statistic. */
    void print_statistic();
    
private:
    SystemTime      start_time;
        /**< Subtest start time. */
        
    SystemTime      stop_time;
        /**< Subtest stop time. */
        
    UInt32          bits_written;
        /**< Number of bits written. */
        
private:
    SystemTime      subtest_duration;
        /**< Subtest duration. */

    }; // class BitIOStreamSpeedTest


} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_TEST_BITIOSTREAM_SPEED_H
