/** @file test_cache.cpp
 * Cache testing.
 */

/*
 * begin        : Thu Feb 21 2002
 * copyright    : (C) 2001 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "test/test_cache.h"
#include "utility/cache.h"
#include "utility/cache_impl.h"

using namespace Massiv::Core;

namespace Massiv {
namespace Core {
namespace Test {


typedef Cache< int, double, int > SimpleCache;


CacheTest::CacheTest() :
    Test( "cache" )
    {
    }


CacheTest::~CacheTest()
    {
    }


void CacheTest::test()
    {
    SimpleCache     c;
    int *           data;

    // Test inserting.
    test_condition_( c.is_empty() );
    c.update( 1, 1.0, -1 );
    test_condition_( c.find( 2, 0.5 ) == NULL );
    test_condition_( c.find( 1, 0.5 ) != NULL );

    // Test removing in find() if timeout is reached.
    test_condition_( c.find( 1, 1.5 ) == NULL );
    test_condition_( c.find( 1, 0.5 ) == NULL );

    // Test cache advance.
    test_condition_( c.is_empty() );
    c.update( 2, 1.0, -2 );
    c.advance( 0.5 );
    data = c.find( 2, 0.5 );
    test_condition_( data != NULL );
    test_condition_( (*data) == -2 );
    c.advance( 1.5 );
    test_condition_( c.find( 2, 1.5 ) == NULL );

    // Test updating content and timeout.
    c.update( 1, 1.0, -1 );
    c.update( 2, 1.0, -2 );
    c.update_content( 1, -3 );
    data = c.find( 1, 0 );
    test_condition_( data != NULL );
    test_condition_( (*data) == -3 );
    c.update_timeout( 2, 1.5 );
    c.advance( 1.3 );
    test_condition_( c.find( 1, 0 ) == NULL );
    test_condition_( c.find( 2, 0 ) != NULL );
    c.advance( 1.6 );
    test_condition_( c.is_empty() );

    // Test persistent cache elements.
    c.update( 1, 0, 100, true );
    test_condition_( c.find( 1, 1.0 ) != NULL );
    c.advance( 1.0 );
    test_condition_( c.find( 1, 1.0 ) != NULL );
    c.update( 1, 0.5, 200, false );
    data = c.find( 1, 0 );
    test_condition_( data != NULL );
    test_condition_( (*data) == 200 );
    test_condition_( c.find( 1, 1.0 ) == NULL );
    test_condition_( c.is_empty() );
    }
     

}; // namespace Massiv::Core::Test
}; // namespace Massiv::Core
}; // namespace Massiv
