/** @file test_dataobject.cpp
 * Data object test
 */

/*
 * begin        : Tue Dec 5 2002
 * copyright    : (C) 2002 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "global/global.h"
#include "test/test_dataobject.h"
#include "data/data_manager.h"
#include "net/net.h"
#include "sysevent/system_event_queue.h"
#include "filesys/file_system.h"
#include <iostream>

using namespace Massiv::Core::Test;
using namespace Massiv::Core;



namespace Massiv {
namespace Core {
namespace Test {

#define GRAPHIC_WARRIOR     "graphic_warrior"
#define GRAPHIC_ARCHER      "graphic_archer"
#define GRAPHIC_THIEF       "graphic_thief"

#define TEXTURE_WARRIOR         "texture_warrior"
#define TEXTURE_ARCHER          "texture_archer"
#define TEXTURE_THIEF           "texture_thief"

#define SOUND_WARRIOR_DEFAULT       "sound_warrior_default"
#define SOUND_ARCHER_DEFAULT        "sound_archer_default"
#define SOUND_THIEF_DEFAULT         "sound_thief_default"

#define SOUND_WARRIOR_VOICE         "sound_warrior_voice"
#define SOUND_ARCHER_VOICE          "sound_archer_voice"
#define SOUND_THIEF_VOICE           "sound_thief_voice"

#define SOUND_WARRIOR_YELL          "sound_warrior_yell"
#define SOUND_ARCHER_YELL           "sound_archer_yell"
#define SOUND_THIEF_YELL            "sound_thief_yell"


DataObjectTest::DataObjectTest() :
    Test( "data_object" )
    {
    }


DataObjectTest::~DataObjectTest()
    {
    }

#if 1

void DataObjectTest::test()
    {    
    // @todo: test init type, download type data manageru
    std::string     path( "./data_test_server" );
    DataManager & dom = Global::data_manager_test_server();
    
    dom._initialize
        (
        path,
        "",
        DataManager::MASTER_DATA_SERVICE,
        DataManager::NO_DOWNLOAD,
        true
        );

    std::string     data_node_name;
    std::string     data_file_name;
    std::string     data_object_name;
    std::string     file_content;
    DataManager::DataNodeTree::DataNode * data_node;

    std::string     data_object_path;
    DataObject *    data_object;

    std::string data_description_file_name = path + "/" + ".description";
    remove( data_description_file_name.c_str() ); // Remove old data description file.

    std::string data_file_list_file_name = path + "/" + ".filelist";
    remove( data_file_list_file_name.c_str() ); // Remove old data file list.

    // Create data description file for testing.

    while( !dom.is_ready_for_use() )
        {
        dom.next_tick();
        }

    dom.switch_to_update_phase( DataManager::MANUAL );

    // Now data object manager is initialized and can offer data objects, but 
    // there are no data objects available yet.

    // Now it is possible to add data nodes and data objects.
    
    // Graphics.

    data_node_name = "/graphic";
    data_file_name = "graphic/graphic.do";
    remove( ( path + "/" + data_file_name ).c_str() );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE, true );  // Deprecated.
    dom.add_data( data_node, "" );
    
    try
        {
        // New data object to create - should throw exception because data node is deprecated.
        dom.add_data_object( GRAPHIC_WARRIOR, data_node_name );
        dom.add_data_object( GRAPHIC_ARCHER, data_node_name );
        dom.add_data_object( GRAPHIC_THIEF, data_node_name );
        test_condition_( 0 );
        }
    catch( ... )
        {
        test_condition_( 1 );
        }

    data_node_name = "/graphic/figure";
    data_file_name = "graphic/figure.do";
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.add_data( data_node, "" );

    // New data object to create.
    dom.add_data_object( GRAPHIC_WARRIOR, data_node_name );
    dom.add_data_object( GRAPHIC_ARCHER, data_node_name );
    dom.add_data_object( GRAPHIC_THIEF, data_node_name );
    dom.finish_update_phase();
    test_data_object( dom, GRAPHIC_WARRIOR, file_content );
    test_data_object( dom, GRAPHIC_ARCHER, file_content );
    test_data_object( dom, GRAPHIC_THIEF, file_content );
    dom.switch_to_update_phase( DataManager::MANUAL );

    data_node_name = "/graphic/figure/man";
    data_file_name = "graphic/figure/man.do";
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.add_data( data_node, "" );

    // Data already objects exist - their path will be updated.
    // When creating data object, data file for "/graphic/figure" will be used instead.
    dom.add_data_object( GRAPHIC_WARRIOR, data_node_name );
    dom.add_data_object( GRAPHIC_ARCHER, data_node_name );
    dom.add_data_object( GRAPHIC_THIEF, data_node_name );
    dom.finish_update_phase();
    test_data_object( dom, GRAPHIC_WARRIOR, file_content );
    test_data_object( dom, GRAPHIC_ARCHER, file_content );
    test_data_object( dom, GRAPHIC_THIEF, file_content );
    dom.switch_to_update_phase( DataManager::MANUAL );

    data_node_name = "/graphic/figure/man/warrior";
    data_file_name = "graphic/figure/warrior.do";
    data_object_name = GRAPHIC_WARRIOR;
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.add_data( data_node, data_object_name ); // Data object path for warrior was updated.

    dom.finish_update_phase();
    test_data_object( dom, GRAPHIC_WARRIOR, file_content );
    dom.switch_to_update_phase( DataManager::MANUAL );

    data_node_name = "/graphic/figure/man/archer";
    data_file_name = "graphic/figure/archer.do";
    data_object_name = GRAPHIC_ARCHER;
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.add_data( data_node, data_object_name ); // Data object path for warrior was updated.

    dom.finish_update_phase();
    test_data_object( dom, GRAPHIC_ARCHER, file_content );

    // Data object path for thief still points to /graphic/figure/man
    test_data_object( dom, GRAPHIC_THIEF, "graphic/figure/man.do - version 1.0" );
    dom.switch_to_update_phase( DataManager::MANUAL );

    data_node_name = "/graphic/figure/woman";
    data_file_name = "graphic/figure/woman.do";
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.add_data( data_node, "" );
    // No data objects for woman created.
    

    // Textures.

    data_node_name = "/texture";
    data_file_name = "texture/texture.do";
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.add_data( data_node, "" );
  
    dom.add_data_object( TEXTURE_WARRIOR, data_node_name );
    dom.add_data_object( TEXTURE_ARCHER, data_node_name );
    dom.add_data_object( TEXTURE_THIEF, data_node_name );
    dom.finish_update_phase();
    test_data_object( dom, TEXTURE_WARRIOR, file_content );
    test_data_object( dom, TEXTURE_ARCHER, file_content );
    test_data_object( dom, TEXTURE_THIEF, file_content );
    dom.switch_to_update_phase( DataManager::MANUAL );

    data_node_name = "/texture/figure";
    data_file_name = "texture/texture_figure.do";
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.add_data( data_node, "" );
    
    dom.add_data_object( TEXTURE_WARRIOR, data_node_name );
    dom.add_data_object( TEXTURE_ARCHER, data_node_name );
    dom.add_data_object( TEXTURE_THIEF, data_node_name );
    dom.finish_update_phase();
    test_data_object( dom, TEXTURE_WARRIOR, file_content );
    test_data_object( dom, TEXTURE_ARCHER, file_content );
    test_data_object( dom, TEXTURE_THIEF, file_content );
    dom.switch_to_update_phase( DataManager::MANUAL );

    data_node_name = "/texture/figure/man";
    data_file_name = "texture/texture_man.do";
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.add_data( data_node, "" );

    dom.add_data_object( TEXTURE_WARRIOR, data_node_name );
    dom.add_data_object( TEXTURE_ARCHER, data_node_name );
    dom.add_data_object( TEXTURE_THIEF, data_node_name );
    dom.finish_update_phase();
    test_data_object( dom, TEXTURE_WARRIOR, file_content );
    test_data_object( dom, TEXTURE_ARCHER, file_content );
    test_data_object( dom, TEXTURE_THIEF, file_content );
    dom.switch_to_update_phase( DataManager::MANUAL );

    data_node_name = "/texture/figure/woman";
    data_file_name = "texture/texture_woman.do";
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.add_data( data_node, "" );

    // Sounds.
    
    data_node_name = "/sound";
    data_file_name = "sound/sound.do";
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.add_data( data_node, "" );

    dom.add_data_object( SOUND_WARRIOR_DEFAULT, data_node_name );
    dom.add_data_object( SOUND_ARCHER_DEFAULT, data_node_name );
    dom.add_data_object( SOUND_THIEF_DEFAULT, data_node_name );
    dom.finish_update_phase();
    test_data_object( dom, SOUND_WARRIOR_DEFAULT, file_content );
    test_data_object( dom, SOUND_ARCHER_DEFAULT, file_content );
    test_data_object( dom, SOUND_THIEF_DEFAULT, file_content );
    dom.switch_to_update_phase( DataManager::MANUAL );
    
    dom.add_data_object( SOUND_WARRIOR_VOICE, data_node_name );
    dom.add_data_object( SOUND_ARCHER_VOICE, data_node_name );
    dom.add_data_object( SOUND_THIEF_VOICE, data_node_name );
    dom.finish_update_phase();
    test_data_object( dom, SOUND_WARRIOR_VOICE, file_content );
    test_data_object( dom, SOUND_ARCHER_VOICE, file_content );
    test_data_object( dom, SOUND_THIEF_VOICE, file_content );
    dom.switch_to_update_phase( DataManager::MANUAL );
    
    dom.add_data_object( SOUND_WARRIOR_YELL, data_node_name );
    dom.add_data_object( SOUND_ARCHER_YELL, data_node_name );
    dom.add_data_object( SOUND_THIEF_YELL, data_node_name );
    dom.finish_update_phase();
    test_data_object( dom, SOUND_WARRIOR_YELL, file_content );
    test_data_object( dom, SOUND_ARCHER_YELL, file_content );
    test_data_object( dom, SOUND_THIEF_YELL, file_content );

    // Data description is already saved in file in data directory.
    dom.destroy();

    // Now it should read previously saved data description.
    dom._initialize
        (
        path,
        "",
        DataManager::MASTER_DATA_SERVICE,
        DataManager::NO_DOWNLOAD,
        true
        );

    while( !dom.is_ready_for_use() )
        {
        dom.next_tick();
        }
    
    // Now data object manager is initialized and can offer data objects.

    // Get data object.
    data_object = dom.get_data_object( GRAPHIC_THIEF, true );
    test_data_object_node_path( data_object, "graphic/figure/man.do - version 1.0" );

    // Update data object GRAPHIC_THIEF.
    data_node_name = "/graphic/figure/man/thief";
    data_file_name = "graphic/figure/thief.do";
    data_object_name = GRAPHIC_THIEF;
    file_content = data_file_name + " - version 1.0";
    create_data_file( path + "/" + data_file_name, file_content );
    data_node = new DataManager::DataNodeTree::DataNode( data_node_name, data_file_name, DataManager::CLIENT_DATA_FILE );
    dom.switch_to_update_phase( DataManager::MANUAL );
    dom.add_data( data_node, data_object_name ); // Data object path will be updated.
    dom.finish_update_phase();

    data_object_path = "";

    if( data_object->is_data_updated() )
        {
        data_object->refresh( true );
        }
    test_data_object_node_path( data_object, file_content );
    dom.next_tick(); // Idle phase.

    // Remove data object.
    dom.switch_to_update_phase( DataManager::MANUAL );
    dom.remove_data_object( GRAPHIC_THIEF );
    dom.finish_update_phase();

    try
        {
        data_object = dom.get_data_object( GRAPHIC_THIEF, true );
        test_condition_( 0 );
        }
    catch( ... )
        {
        test_condition_( 1 );
        }
  
    // Update data node's data file.
    data_node_name = "/graphic/figure/man/thief";
    data_file_name = "graphic/figure/thief.do";
    file_content = data_file_name + " - version 1.1";
    create_data_file( path + "/" + data_file_name, file_content );
    dom.switch_to_update_phase( DataManager::MANUAL );
    dom.update_data_file( data_file_name );
    dom.finish_update_phase();

    // Add data object again.
    dom.switch_to_update_phase( DataManager::MANUAL );
    dom.add_data_object( GRAPHIC_THIEF, data_node_name );
    dom.finish_update_phase();
    test_data_object( dom, GRAPHIC_THIEF, file_content );

    delete data_object;


    DataManager & dom_client = Global::data_manager_test_client();

    path = "./data_test_client";
    data_description_file_name = path + "/" + ".description";
    remove( data_description_file_name.c_str() ); // Remove old data description file.

    data_file_list_file_name = path + "/" + ".filelist";
    remove( data_file_list_file_name.c_str() ); // Remove old data file list.

    const NodeId & node_id = Global::network().get_local_node_id();
    //hope( node_id.is_type( NodeId::TYPE_ANONYMOUS ) );
    
    dom_client._initialize
        (
        path,
        "",
        DataManager::DATA_CLIENT,
        DataManager::FULL_DOWNLOAD,
        true,
        true,
        node_id
        );

    SystemEventQueue & system_event_queue  = Global::system_event_queue();

    // Cannot test download/upload in anonymous mode.
    /* 
    while( !dom_client.is_ready_for_use() )
        {
        dom.next_tick();
        system_event_queue.execute_all_events();
        dom_client.next_tick();
        system_event_queue.execute_all_events();
        }*/
    }

void DataObjectTest::create_data_file
    (
    const std::string &     file_name,
    const std::string &     text
    )
    {
    std::string::size_type  pos;
    if( ( pos = file_name.rfind( "/" ) ) != std::string::npos )
        {
        // We must create directory first.
        std::string     dir( file_name, 0, pos );
        Global::native_file_system().create_directory( dir );
        }
    std::ofstream   ofs( file_name.c_str() );
    if( !ofs )
        {
        throw;
        }
    ofs << text;
    ofs.close();
    }

void DataObjectTest::test_data_object
    (
    DataManager &               data_manager,
    const std::string &         data_object_name,
    const std::string &         file_content
    )
    {
    // Get data object.
    DataObject *    data_object = data_manager.get_data_object( data_object_name, true );
    test_data_object_node_path( data_object, file_content );
    delete data_object;
    }

void DataObjectTest::test_data_object_node_path
    (
    DataObject *            data_object,
    const std::string &     file_content
    )
    {
    hope( data_object != NULL );

    std::string     buffer;
    std::getline( data_object->data, buffer );
      
    test_condition_( buffer == file_content );
    }

#else

void DataObjectTest::test()
    {
    }

#endif

} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv
