/** @file test_factgen.cpp
 * Factory generator testing.
 */

/*
 * begin        : Mon Oct 14 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "test/test_factgen.h"
#include "test/factgen_objects.h"
#include "test/class_list.h"
#include "test/factgen_objects_generated.h"
#include "global/global.h"
#include "object/object_create.h"


namespace Massiv {
namespace Core {
namespace Test {


FactoryGeneratorTest::FactoryGeneratorTest()
      : Test( "factory generator" )
    {
    }


FactoryGeneratorTest::~FactoryGeneratorTest()
    {
    }


void FactoryGeneratorTest::test()
    {
#if 0

    ClassManager & class_manager = Global::class_manager();
    
    // Check if all our testing object are registered.

    const MetaObject * meta_object = class_manager.get_metaobject( typeid( ::Massiv::Core::Object ) );
    const MetaObject * meta_a = class_manager.get_metaobject( typeid( ObjectA ) );
    const MetaObject * meta_b = class_manager.get_metaobject( typeid( ObjectB ) );
    const MetaObject * meta_base = class_manager.get_metaobject( typeid( ObjectBase ) );
    const MetaObject * meta_c = class_manager.get_metaobject( typeid( ObjectC ) );
    const MetaObject * meta_test = class_manager.get_metaobject( typeid( ObjectTest ) );
    const MetaObject * meta_x = class_manager.get_metaobject( typeid( ObjectX ) );
    const MetaObject * meta_y = class_manager.get_metaobject( typeid( ObjectY ) );
    const MetaObject * meta_z = class_manager.get_metaobject( typeid( ObjectZ ) );

    // Ids are assigned to meta objects sorted by class name, including namespace.

    // Massiv::Core::Lib::AccountObjectInterface -> 0, 1
    // Massiv::Core::Lib::NodeObjectInterface -> 0, 2
    test_condition_( meta_object->get_class_type_id() == ClassTypeId( 0, 3 ) );
    // Massiv::Core::Lib::RPCObject -> 0, 4
    // Massiv::Core::Lib::ResultObject -> 0, 5
    test_condition_( meta_a->get_class_type_id() == ClassTypeId( 0, 6 ) );
    test_condition_( meta_b->get_class_type_id() == ClassTypeId( 0, 7 ) );
    test_condition_( meta_base != NULL );
    test_condition_( meta_base->get_class_type_id() == ClassTypeId( 0, 8 ) );
    test_condition_( meta_c->get_class_type_id() == ClassTypeId( 0, 9 ) );
    test_condition_( meta_test != NULL );
    test_condition_( meta_test->get_class_type_id() == ClassTypeId( 0, 10 ) );
    test_condition_( meta_x->get_class_type_id() == ClassTypeId( 0, 11 ) );
    test_condition_( meta_y->get_class_type_id() == ClassTypeId( 0, 12 ) );
    test_condition_( meta_z->get_class_type_id() == ClassTypeId( 0, 13 ) );

    // Test class names.

    test_condition_( meta_object->get_class_name() == "Massiv::Core::Object" );
    test_condition_( meta_test->get_class_name() == "Massiv::Core::Test::ObjectTest" );

    // Test inheritance introspection.
    // List of components is created using depth-first traversal of inheritance hierarchy.
    // Only the first occurance of virtual components is included.

    const MetaObject::ComponentInfos & components_object = meta_object->get_component_infos();
    test_condition_( components_object.size() == 1 );
    test_condition_( components_object[ 0 ].metaobject == meta_object );

    const MetaObject::ComponentInfos & components_test = meta_test->get_component_infos();
    test_condition_( components_test.size() == 11 );
    test_condition_( components_test[ 0 ].metaobject == meta_test );
    test_condition_( components_test[ 0 ].virtual_component == false );
    test_condition_( components_test[ 0 ].inherited_from == -1 );
    test_condition_( components_test[ 1 ].metaobject == meta_c );
    test_condition_( components_test[ 1 ].virtual_component == false );
    test_condition_( components_test[ 1 ].inherited_from == 0 );
    test_condition_( components_test[ 2 ].metaobject == meta_a );
    test_condition_( components_test[ 2 ].virtual_component == false );
    test_condition_( components_test[ 2 ].inherited_from == 1 );
    test_condition_( components_test[ 3 ].metaobject == meta_base );
    test_condition_( components_test[ 3 ].virtual_component == false );
    test_condition_( components_test[ 3 ].inherited_from == 2 );
    test_condition_( components_test[ 4 ].metaobject == meta_object );
    test_condition_( components_test[ 4 ].virtual_component == true );
    test_condition_( components_test[ 4 ].inherited_from == 3 );
    test_condition_( components_test[ 5 ].metaobject == meta_b );
    test_condition_( components_test[ 5 ].virtual_component == false );
    test_condition_( components_test[ 5 ].inherited_from == 1 );
    test_condition_( components_test[ 6 ].metaobject == meta_base );
    test_condition_( components_test[ 6 ].virtual_component == false );
    test_condition_( components_test[ 6 ].inherited_from == 5 );
    test_condition_( components_test[ 7 ].metaobject == meta_z );
    test_condition_( components_test[ 7 ].virtual_component == false );
    test_condition_( components_test[ 7 ].inherited_from == 0 );
    test_condition_( components_test[ 8 ].metaobject == meta_x );
    test_condition_( components_test[ 8 ].virtual_component == false );
    test_condition_( components_test[ 8 ].inherited_from == 7 );
    test_condition_( components_test[ 9 ].metaobject == meta_base );
    test_condition_( components_test[ 9 ].virtual_component == true );
    test_condition_( components_test[ 9 ].inherited_from == 8 );
    test_condition_( components_test[ 10 ].metaobject == meta_y );
    test_condition_( components_test[ 10 ].virtual_component == false );
    test_condition_( components_test[ 10 ].inherited_from == 7 );

    test_condition_( meta_test->inherits( typeid( ObjectTest ) ) );
    test_condition_( meta_test->inherits( typeid( ObjectA ) ) );
    test_condition_( meta_test->inherits( typeid( ObjectB ) ) );
    test_condition_( meta_test->inherits( typeid( ObjectC ) ) );
    test_condition_( meta_test->inherits( typeid( ObjectX ) ) );
    test_condition_( meta_test->inherits( typeid( ObjectY ) ) );
    test_condition_( meta_test->inherits( typeid( ObjectZ ) ) );
    test_condition_( meta_test->inherits( typeid( ObjectBase ) ) );
    test_condition_( meta_test->inherits( typeid( ::Massiv::Core::Object ) ) );

    test_condition_( !meta_c->inherits( typeid( ObjectTest ) ) );
    test_condition_( meta_c->inherits( typeid( ObjectA ) ) );
    test_condition_( meta_c->inherits( typeid( ObjectB ) ) );
    test_condition_( meta_c->inherits( typeid( ObjectC ) ) );
    test_condition_( !meta_c->inherits( typeid( ObjectX ) ) );
    test_condition_( !meta_c->inherits( typeid( ObjectY ) ) );
    test_condition_( !meta_c->inherits( typeid( ObjectZ ) ) );
    test_condition_( meta_c->inherits( typeid( ObjectBase ) ) );

    // Test property introspection.
    // Properties are sorted by components and then by the order they appear in IDL.

    const MetaObject::PropertyInfos & properties_object = meta_object->get_property_infos();
    test_condition_( properties_object.size() == 0 );
    
    const MetaObject::PropertyInfos & properties_base = meta_base->get_property_infos();
    test_condition_( properties_base.size() == 1 );
    test_condition_( properties_base[ 0 ].name == "base" );

    const MetaObject::PropertyInfos & properties_test = meta_test->get_property_infos();
    test_condition_( properties_test.size() == 10 );
    test_condition_( properties_test[ 0 ].name == "test" );
    test_condition_( properties_test[ 1 ].name == "c" );
    test_condition_( properties_test[ 2 ].name == "a" );
    test_condition_( properties_test[ 3 ].name == "base" );
    test_condition_( properties_test[ 4 ].name == "b" );
    test_condition_( properties_test[ 5 ].name == "base" );
    test_condition_( properties_test[ 6 ].name == "z" );
    test_condition_( properties_test[ 7 ].name == "x" );
    test_condition_( properties_test[ 8 ].name == "base" );
    test_condition_( properties_test[ 9 ].name == "y" );

    // Test RPC introspection.
    // Order from IDL is preserved. Inherited methods are not contained in the list (yet?).

    const MetaObject::MethodInfos & methods_object = meta_object->get_method_infos();
    test_condition_( methods_object.size() == 3 );
    test_condition_( methods_object[ 0 ].name == "migrate_to" );
    test_condition_( methods_object[ 1 ].name == "replicate_to" );
    test_condition_( methods_object[ 2 ].name == "cancel_replication" );

    const MetaObject::MethodInfos & methods_test = meta_test->get_method_infos();
    test_condition_( methods_test.size() == 0 );

    const MetaObject::MethodInfos & methods_c = meta_c->get_method_infos();
    test_condition_( methods_c.size() == 1 );
    test_condition_( methods_c[ 0 ].name == "print" );
    test_condition_( methods_c[ 0 ].arguments.size() == 2 );
    test_condition_( methods_c[ 0 ].arguments[ 0 ].name == "separator" );
    test_condition_( methods_c[ 0 ].arguments[ 1 ].name == "indent" );

    // Test pointers to ObjectTest.

    Pointer< ObjectTest > object_test = CreateObject< ObjectTest >();
    ObjectTest * const native_test = object_test.dereference();

    ObjectBase * const base_virtual = static_cast< ObjectZ * >( native_test );
    ObjectBase * const base_via_a = static_cast< ObjectA * >( native_test );
    ObjectBase * const base_via_b = static_cast< ObjectB * >( native_test );
    base_virtual->base = 1;
    base_via_a->base = 2;
    base_via_b->base = 3;

    bool exception_caught = false;
    try
        {
        Pointer< ObjectBase > object_base = object_test.convert();
        object_base->base = 0;
        }
    catch( Lib::IllegalPointerConversionException & )
        {
        exception_caught = true;
        }
    test_condition_( exception_caught == true );

    Pointer< ObjectZ > object_z = object_test;
    Pointer< ObjectA > object_a = object_test;
    Pointer< ObjectB > object_b = object_test;
    Pointer< ObjectBase, BaseVirtualCast > object_base_virtual = object_z;
    Pointer< ObjectBase, BaseViaACast > object_base_via_a = object_a;
    Pointer< ObjectBase, BaseViaBCast > object_base_via_b = object_b;
    test_condition_( object_base_virtual->base == 1 );
    test_condition_( object_base_via_a->base == 2 );
    test_condition_( object_base_via_b->base == 3 );

    // Test upcasting.

    VariantPointer variant_test = meta_test->downcast( native_test );
    test_condition_( !variant_test.is_null() );
    VariantPointer variant_c = meta_test->upcast( variant_test, "Massiv::Core::Test::ObjectC" ).first;
    VariantPointer variant_z = meta_test->upcast( variant_test, "Massiv::Core::Test::ObjectZ" ).first;
    VariantPointer variant_null = meta_test->upcast( variant_test, "Massiv::Core::Test::ObjectA" ).first;  // note: this may actually work in the future
    VariantPointer variant_base = meta_test->upcast( variant_test,
        "Massiv::Core::Test::ObjectC/Massiv::Core::Test::ObjectA/Massiv::Core::Test::ObjectBase" ).first;
    test_condition_( !variant_c.is_null() );
    test_condition_( !variant_z.is_null() );
    test_condition_( variant_null.is_null() );
    test_condition_( !variant_base.is_null() );

    // Test access to properties via introspection.

    Property * const prop_test = meta_test->get_property( variant_test, "test" );
    test_condition_( prop_test != NULL );
    test_condition_( prop_test->get_type() == Property::TYPE_POINTER );

    Property * const prop_null = meta_test->get_property( variant_test, "base" ); // ambiguous
    test_condition_( prop_null == NULL );

    MetaObject::PropertyPointers props_base;
    meta_test->find_properties( props_base, variant_test, "base" );
    test_condition_( props_base.size() == 3 );
    test_condition_( props_base[ 0 ]->to_string() == "2" ); // via a
    test_condition_( props_base[ 1 ]->to_string() == "3" ); // via b
    test_condition_( props_base[ 2 ]->to_string() == "1" ); // virtual

    Property * const prop_z = meta_z->get_property( variant_z, "z" );
    test_condition_( prop_z != NULL );
    test_condition_( prop_z->get_type() == Property::TYPE_VALUE_TYPE );

    Serializer::Description desc;
    std::ostringstream oss;
    TextWriter tw( oss );

    object_z->z = true;
    prop_z->text_write( tw, desc );
    test_condition_( oss.str() == "true" );
    tw.write_space();
    object_z->z = false;
    prop_z->text_write( tw, desc );
    test_condition_( oss.str() == "true false" );

    std::istringstream iss( oss.str() );
    TextReader tr( iss );
    tr.next_line();
    prop_z->text_read( tr, desc );
    test_condition_( object_z->z == true );
    prop_z->text_read( tr, desc );
    test_condition_( object_z->z == false );

    // TODO: test exception throwing - unexpected eol, eof, token, ...;
    //       preferably for each property type

    // Note: No RPC testing - we would have to run complete simulation.
    //       RPC tests belong to src/server/test.

#endif
    }


} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv
