/** @file test_filesys.h
 * Filesystem testing.
 */

/*
 * begin        : Fri Sep 13 2002
 * copyright    : (C) 2002 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TEST_FILESYS_H
#define MASSIV_TEST_FILESYS_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TEST_H
#include "test/test.h"
#endif

#ifndef MASSIV_COMPACT_VOLUME_H
#include "filesys/compact_volume.h"
#endif

#ifndef MASSIV_VOLUME_H
#include "filesys/volume.h"
#endif



namespace Massiv {
namespace Core {
namespace Test {


class FileSystemTest : public Test
    {
public:

    /**
     * Constructor.
     */
    FileSystemTest();

    /**
     * Destructor.
     */
    ~FileSystemTest();

public:

    /**
     * Runs test.
     */
    void test();

private:

    /**
     * Tests LRUCache.
     */
    void test_cache();

    /**
     * Tests BlockPool.
     */
    void test_block_pool();

    /**
     * Tests StreamPool.
     */
    void test_stream_pool();

    /**
     * Tests CompressedStream.
     */
    void test_compressed_stream();

    /**
     * Tests Volume.
     */
    void test_volume();

    /**
     * Tests CompactVolume.
     */
    void test_compact_volume();

private:

    /**
     */
    void copy_to_volume
        (
        BasicVolume &       volume,
        const std::string & file_name,
        int                 creation_flags
        );

    /**
     */
    void verify
        (
        BasicVolume &       volume,
        const std::string & file_name
        );

    /**
     */
    void FileSystemTest::copy_to_volume
        (
        BasicVolume &   volume,
        int             creation_flags
        );

    /**
     */
    void FileSystemTest::verify
        (
        BasicVolume &   volume
        );

    }; // FileSystemTest


}; // namespace Massiv::Core::Test
}; // namespace Massiv::Core
}; // namespace Massiv


#endif // _TEST_FILESYS_H_
