/** @file test_fs_enumeration.h
 * Test of interface for enumeration of files and directories in filesystem.
 */

/*
 * begin        : Sun Feb 02 2003
 * copyright    : (C) 2003 by Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TEST_FS_ENUMERATION_H
#define MASSIV_TEST_FS_ENUMERATION_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TEST_H
#include "test/test.h"
#endif

#ifndef MASSIV_FILE_SYSTEM_H
#include "filesys/file_system.h"
#endif

#include <string>

namespace Massiv {
namespace Core {
namespace Test {


/**
 *
 * Testing of interface for enumeration of files and directories in filesystem.
 */
class FsEnumerationTest : public Test
    {
public:

    /**
     * Constructor.
     */
    FsEnumerationTest();

    /**
     * Destructor.
     */
    ~FsEnumerationTest();

public:

    /**
     * Run test
     */
    void test();

private:

    /**
     * Initializes tests.
     */
    void init();

    /**
     * Deinitializes tests.
     */
    void deinit();

private:
    
    /**
     * Test of going through the directory. Calls 
     * @c perform_go_through_test() several times. The directory used for
     * the test is massiv/src/core/test/test_hierarchy.
     */
    void go_through_test();

    /**
     * Test of going through the directory. The directory used for the test 
     * is massiv/src/core/test/test_hierarchy.
     */
    void perform_go_through_test
        (
        FileIterator    it
        );

    /**
     * Method used by go_through_test(). Creates an iterator for the current
     * directory and returns it by value. Partially test reference counting 
     * of the architecture dependent iterator.
     * @param mask The mask for filtering relevant files.
     */
    FileIterator get_iterator_for_current
        (
        std::string     mask
        );

private:

    /**
     * Tests descent() and ascent() (and enter() for throwing exceptions
     * properly.
     */
    void descent_ascent_test();

    /**
     * Tests enter() and leave().
     */
    void enter_leave_test();

private:
    
    /**
     * Tests is_in_root().
     */
    void is_in_root_test();

private:
    
    /**
     * Tests creating and deleting directories.
     */
    void directory_manipulation_test();

private:

    FileSystem &    native_fs;
        /**< Native file system. */
    std::string     old_curr_dir;
        /**< The directory that was actual before launching this test. */
    std::string     test_dir;
        /**< Base directory for these tests. */

    };  // class FsEnumerationTest


} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_TEST_FS_ENUMERATION_H
