/** @file test_heap.cpp
 * Heap testing.
 */

/*
 * begin        : Sat Feb 23 2002
 * copyright    : (C) 2001 by Petr Tovarys
 */

 /***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "test/test_heap.h"
#include "utility/heap.h"
#include "utility/heap_impl.h"


using namespace Massiv::Core;

namespace Massiv {
namespace Core {
namespace Test {


struct Element
    {

    Element()
        {
        data        = 0;
        hope( !heap_index.is_valid() );
        }

    Element
        (
        int     element_data
        )
        {
        data        = element_data;
        hope( !heap_index.is_valid() );
        }

    HeapIndex   heap_index;
    int         data;

    }; // struct Element


/** @class IndexChanged
 * Monitors Element position changes in the heap.
 */
struct IndexChanged : public std::binary_function<Element, HeapIndex, void>
    {

    void operator()
        (
        Element &       e,
        HeapIndex       new_index
        ) const
        {
        e.heap_index = new_index;
        };

    }; // struct IndexChanged


typedef Heap<int, Element, std::less<int>, IndexChanged>  SimpleHeap;


HeapTest::HeapTest() :
    Test( "heap" )
    {
    }

HeapTest::~HeapTest()
    {
    }

void HeapTest::test()
    {
    SimpleHeap      h;
    int i;
    int last;
    int a[] = { 8, 100, 3, 60, 30, 2, 3, 4, 3, 100, 400, -100, 20, -24, 40, 6 };
    const int a_size = sizeof( a ) / sizeof( int );

    test_condition_( h.is_empty() );
    h.push( 1, Element( 100 ));
    test_condition_( !h.is_empty() );
    test_condition_( h[0].heap_index == 0 );
    test_condition_( h[0].data == 100 );
    h.pop();
    test_condition_( h.is_empty() );

    // Test sorting.
    last = 0;
    for( i = 0; i < a_size; i++ )
        {
        last = a [ i ] < last ? a [ i ] : last;
        h.push( a[ i ], Element( i ) );
        }

    test_condition_( h.get_size() == a_size );

    for( i = 0; i < h.get_size(); i++ )
        {
        test_condition_( h[ i ].heap_index == i );
        }

    while( !h.is_empty() )
        {
        Element e;
        test_condition_( h.get_key( 0 ) >= last );
        test_condition_( h[ 0 ].heap_index == 0 );
        test_condition_( h.get_key( 0 ) == a[ h[ 0 ].data ] );
        last = h.get_key( 0 );
        h.pop( 0, &e );
        test_condition_( !e.heap_index.is_valid() );
        }

    // @todo Pridat test na pop z prostredku haldy.
    }


}; // namespace Massiv::Core::Test
}; // namespace Massiv::Core
}; // namespace Massiv
