/** @file test_object_provider.cpp
 * Object provider related tests.
 */

/*
 * begin        : Tue Mar 11 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "test/test_object_provider.h"
#include "object/providerid.h"
#include "object/object_provider_state.h"
#include "sysevent/object_status_msg.h"
#include "object/object_provider.h"


namespace Massiv {
namespace Core {
namespace Test {


ObjectProviderTest::ObjectProviderTest() :
    Super( "object provider" )
    {
    }


void ObjectProviderTest::test()
    {
    #if 0
    test_provider_id();
    test_provider_state();
    test_provider_cache();
    #endif
    }
    
#if 0
    
void ObjectProviderTest::test_provider_id()
    {
    // Verify some bit counts.
        {
        test_condition_( ProviderId::PROVIDER_ID_BITS_COUNT > 1 && ProviderId::PROVIDER_ID_BITS_COUNT <= 32 );
        test_condition_( ProviderId::PROVIDER_RELATIVE_ID_BITS_COUNT > 1 );
        }
        
    // Verify creating virtual and dummy object ids.
        {
        ProviderId provider_id = ProviderId::create_virtual_provider_id();
        test_condition_( provider_id.is_virtual_provider_id() );
        test_condition_( !provider_id.is_dummy_provider_id() );
        }
        
        {
        ProviderId provider_id = ProviderId::create_dummy_provider_id();
        test_condition_( !provider_id.is_virtual_provider_id() );
        test_condition_( provider_id.is_dummy_provider_id() );
        }

    // Verify that create_from_number() is working properly.
        {
        UInt32 id = ProviderId::VIRTUAL_PROVIDER_ID;
        ProviderId provider_id = ProviderId::create_from_number( id );
        test_condition_( provider_id.is_virtual_provider_id() );
        }

        {
        UInt32 id = ProviderId::DUMMY_PROVIDER_ID;
        ProviderId provider_id = ProviderId::create_from_number( id );
        test_condition_( provider_id.is_dummy_provider_id() );
        }
    }


void ObjectProviderTest::test_provider_state()
    {
    // Verify that provider state created by default constructor
    // is not able to genearte any id.
        {
        ObjectProviderState pstate;
        
        test_condition_( pstate.is_provider_relative_object_id_pool_exhausted() );
        
        // Try to generate one id.
        test_condition_( pstate.generate_provider_relative_object_id() == UINT64( 0 ) );
        test_condition_( pstate.is_provider_relative_object_id_pool_exhausted() );
        
        // Try to generate another one.
        test_condition_( pstate.generate_provider_relative_object_id() == UINT64( 0 ) );
        test_condition_( pstate.is_provider_relative_object_id_pool_exhausted() );
        }
        
    // Try exhausting some state.
    // We will create provider state which should generate only one id.
        {
        ProviderId          provider_id = ProviderId::create_dummy_provider_id();
        UInt64              next_relative_id = ( ( UINT64( 1 ) << ProviderId::PROVIDER_RELATIVE_ID_BITS_COUNT ) - 1 );        
        ObjectProviderState pstate( provider_id, next_relative_id );
        
        hope( next_relative_id != 0 );
        test_condition_( !pstate.is_provider_relative_object_id_pool_exhausted() );
        
        // Try to generate one id. This will exhaust the id pool of the provider state.
        test_condition_( pstate.generate_provider_relative_object_id() == next_relative_id );
        test_condition_( pstate.is_provider_relative_object_id_pool_exhausted() );
        
        // Try to generate another one. Should fail.
        test_condition_( pstate.generate_provider_relative_object_id() == UINT64( 0 ) );
        test_condition_( pstate.is_provider_relative_object_id_pool_exhausted() );
        }
    }


void ObjectProviderTest::test_provider_cache()
    {
    // Lets create some provider.
    ProviderId          provider_id = ProviderId::create_dummy_provider_id();
    UInt64              next_relative_id = 1;
    ObjectProviderState pstate( provider_id, next_relative_id );
    ObjectProvider      provider( pstate );
    
    // Lets generate some object ids.
    ObjectId            normal_object_id;
    ObjectId            tracked_object_id;
    
    test_fail_( "Fixme." );
    return;
    //test_condition_( provider.generate_id( normal_object_id, false ) );
    //test_condition_( provider.generate_id( tracked_object_id, true ) );
    
    // Verify the generated ids.
    test_condition_( !normal_object_id.is_tracked_by_provider() );
    test_condition_( tracked_object_id.is_tracked_by_provider() );
    test_condition_( normal_object_id.get_provider_id() == provider_id );
    test_condition_( tracked_object_id.get_provider_id() == provider_id );
    test_condition_( !normal_object_id.is_proxy_node_object_id() );
    test_condition_( !tracked_object_id.is_proxy_node_object_id() );
    
    // The cache should be empty.
        {
        ObjectStatus        object_status;
        NodeId              node_id;
    
        provider.get_object_status( normal_object_id, object_status, node_id );
        test_condition_( object_status == ObjectStatus::UNKNOWN );
        test_condition_( node_id.is_null() );
        
        provider.get_object_status( tracked_object_id, object_status, node_id );
        test_condition_( object_status == ObjectStatus::UNKNOWN );
        test_condition_( node_id.is_null() );
        }
        
    // Refresh status of the generated object ids. The cache should stay empty
    // because the object ids are not in the cache.
        {
        ObjectStatus        object_status;
        NodeId              node_id;
    
        provider.refresh_object_status( normal_object_id );
        provider.refresh_object_status( tracked_object_id );
        
        provider.get_object_status( normal_object_id, object_status, node_id );
        test_condition_( object_status == ObjectStatus::UNKNOWN );
        test_condition_( node_id.is_null() );
        
        provider.get_object_status( tracked_object_id, object_status, node_id );
        test_condition_( object_status == ObjectStatus::UNKNOWN );
        test_condition_( node_id.is_null() );
        }
    
    // Lets cache some status about normal_object_id.
        {
        NodeId              src_node_id = NodeId::create_from_type_and_relative_id( NodeId::TYPE_SERVER, 1 );
        ObjectStatusMessage status_msg( normal_object_id, ObjectStatus::ALIVE, src_node_id, true );
        
        provider.notify_of_object_status( status_msg );
        
        // Verify the cached status.
            {
            ObjectStatus        object_status;
            NodeId              node_id;
        
            provider.get_object_status( normal_object_id, object_status, node_id );
            test_condition_( object_status == ObjectStatus::ALIVE );
            test_condition_( node_id == src_node_id );
            
            provider.get_object_status( tracked_object_id, object_status, node_id );
            test_condition_( object_status == ObjectStatus::UNKNOWN );
            test_condition_( node_id.is_null() );
            }
        }
    
    // Try cache untrustworthy alive status about tracked_object_id.
    // This should fail.
        {
        NodeId              src_node_id = NodeId::create_from_type_and_relative_id( NodeId::TYPE_SERVER, 1 );
        ObjectStatusMessage status_msg( tracked_object_id, ObjectStatus::ALIVE, src_node_id, false );
        
        provider.notify_of_object_status( status_msg );
        
        // Verify the cached status.
            {
            ObjectStatus        object_status;
            NodeId              node_id;
        
            provider.get_object_status( normal_object_id, object_status, node_id );
            test_condition_( object_status == ObjectStatus::ALIVE );
            test_condition_( node_id == src_node_id );
            
            provider.get_object_status( tracked_object_id, object_status, node_id );
            test_condition_( object_status == ObjectStatus::UNKNOWN );
            test_condition_( node_id.is_null() );
            }
        }
        
    // Try cache trustwothy alive status about tracked_object_id.
        {
        NodeId              src_node_id = NodeId::create_from_type_and_relative_id( NodeId::TYPE_SERVER, 1 );
        ObjectStatusMessage status_msg( tracked_object_id, ObjectStatus::ALIVE, src_node_id, true );
        
        provider.notify_of_object_status( status_msg );
        
        // Verify the cached status.
            {
            ObjectStatus        object_status;
            NodeId              node_id;
        
            provider.get_object_status( normal_object_id, object_status, node_id );
            test_condition_( object_status == ObjectStatus::ALIVE );
            test_condition_( node_id == src_node_id );
            
            provider.get_object_status( tracked_object_id, object_status, node_id );
            test_condition_( object_status == ObjectStatus::ALIVE );
            test_condition_( node_id == src_node_id );
            }
        }
    
    /// @todo Verify caching of NOT_ACCESSIBLE status?
        
    // Notify of unknown object status. This should clear the cache.
        {
        NodeId              src_node_id = NodeId::create_from_type_and_relative_id( NodeId::TYPE_SERVER, 1 );
        // The src_node_id should be ignored.
            {
            ObjectStatusMessage status_msg( normal_object_id, ObjectStatus::UNKNOWN, src_node_id, true );
            provider.notify_of_object_status( status_msg );
            }
            {
            ObjectStatusMessage status_msg( tracked_object_id, ObjectStatus::UNKNOWN, src_node_id, true );
            provider.notify_of_object_status( status_msg );
            }
        
        // Verify the cached status.
            {
            ObjectStatus        object_status;
            NodeId              node_id;
        
            provider.get_object_status( normal_object_id, object_status, node_id );
            test_condition_( object_status == ObjectStatus::UNKNOWN );
            test_condition_( node_id.is_null() );
            
            provider.get_object_status( tracked_object_id, object_status, node_id );
            test_condition_( object_status == ObjectStatus::UNKNOWN );
            test_condition_( node_id.is_null() );
            }
        }
    }

#endif
    
} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv
