/** @file test_objects.cpp
 * Object management testing.
 *
 * Stoupik's note:
 * This test is currently completely commented out. Because of recent changes
 * in metaobjects and factories, a lot of stuff must be changed to make this
 * test work again and I simply don't care. Testing of object management
 * functionality should be probably moved to test_factgen.
 */

/*
 * begin        : Thu Feb 7 2002
 * copyright    : (C) 2002 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "test/test_objects.h"
#include "utility/debug.h"
#include "object/object.h"
#include "pointer/object_pointer.h"
#include "object/object_factory.h"
#include "object/class_manager.h"
#include "object/object_manager.h"
#include "property/prop_pointer.h"
#include "property/prop_int.h"
#include "pointer/object_proxy.h"
#include <iostream>
#include <fstream>
#include <set>

using namespace Massiv::Core;


namespace Massiv {
namespace Core {
namespace Test {


// See file comment.
#if 0

class D;

}
}


// RPC stubs

namespace Generated { // Massiv::Generated

template <>
class RPCStubs< ::Massiv::Core::Test::D > : public ::Massiv::Core::RPCStubsBase
    {
public:

    void f()
        {
        }

    };

}
}

// end of RPC stubs


namespace Massiv {
namespace Core {
namespace Test {


// gcc-2.95 does not like DirectBases = NULL default parameter
const std::type_info * null_bases[] =
    {
    NULL
    };


template <class T, const std::type_info ** DirectBases = null_bases>
class SimpleFactory : public ObjectFactory
    {
public:

    SimpleFactory()
        {
        ClassManager & manager = Global::class_manager();

        manager.register_class( this, typeid( T ) );
        }

    T * create_ex
        (
        const T * source = NULL
        ) const
        {
        T * ptr = new T;
        initialize_object( *ptr );
        if( source )
            {
            *ptr = *source;
            }
        return ptr;
        }

    ObjectProperty * create() const
        {
        return create_ex();
        }

    ObjectProperty * create
        (
        const ObjectId & object_id
        ) const
        {
        panic( "Not implemented" );
        return NULL;
        }

    ObjectProperty * create_replica
        (
        const ObjectId & object_id
        ) const
        {
        panic( "Not implemented" );
        return NULL;
        }

    void initialize_property_attributes
        (
        ObjectProperty * object
        ) const
        {
        }

    ObjectProperty * clone_object
        (
        const ObjectProperty * object
        ) const
        {
        return create_ex( variant_cast< T * >( get_complete_object( object ) ) );
        }

    VariantPointer downcast
        (
        ObjectProperty * object
        ) const
        {
        return VariantPointer( dynamic_cast<T *>( object ) );
        }

    bool inherits
        (
        const std::type_info & another_type_info
        ) const
        {
        const std::type_info ** p = DirectBases;

        if( typeid( T ) == another_type_info )
            return true;

        if( typeid( Object ) == another_type_info )
            return true;

        if( !p )
            return false;

        while( *p )
            {
            ClassManager & manager = Global::class_manager();
            ObjectFactory * factory = manager.get_factory( manager.get_class_type_id( **p ) );

            if( factory->inherits( another_type_info ) )
                return true;

            p++;
            }

        return false;
        }

    void for_each_property_do
        (
        PropertyOperation &     operation,
        ObjectProperty *        object
        ) const
        {
        }

    virtual void bitstr_write( BitOStream &, const Serializer::Description &, const ObjectProperty * ) const { }
    virtual void bitstr_read( BitIStream &, const Serializer::Description &, ObjectProperty * ) const { }
    virtual void text_write( TextWriter &, const Serializer::Description &, const ObjectProperty * ) const { }
    virtual void text_read( TextReader &, const Serializer::Description &, ObjectProperty * ) const { }

    virtual bool get_property_info
        (
        const ObjectProperty *      object,
        const Property *            owned_property,
        Property::PropertyInfo &    info
        ) const
        {
        return false;
        }

    };


class A : public virtual Object
    {
public:

    A & operator=
        (
        const A & a
        )
        {
        integer = a.integer;
        pointer = a.pointer;
        pointer2 = a.pointer2;
        return *this;
        }

    PInt<Int32,32>  integer;
    PPointer<A>     pointer;
    PPointer<A>     pointer2;
    };


class B : public virtual Object
    {
public:

    B & operator=
        (
        const B &
        )
        {
        return *this;
        }
    };


class C : public A, public B
    {
public:

    C & operator=
        (
        const C & c
        )
        {
        A::operator=( *this );
        B::operator=( *this );
        a_member = c.a_member;
        return *this;
        }

    A   a_member;
    };


class D : public virtual GCRootObject
    {
public:

    D & operator=
        (
        const D & d
        )
        {
        pointer = d.pointer;
        return *this;
        }

    void f();

public:

    PPointer<A> pointer;
    };


void D::f()
    {
    }


const std::type_info * c_bases[] =
    {
    &typeid( A ),
    &typeid( B ),
    NULL
    };


void SimpleFactory<A>::for_each_property_do
    (
    PropertyOperation &     operation,
    ObjectProperty *        object
    ) const
    {
    A * a = dynamic_cast< A * >( object );

    a->integer.for_each_property_do( operation );
    a->pointer.for_each_property_do( operation );
    a->pointer2.for_each_property_do( operation );
    }


void SimpleFactory<A>::initialize_property_attributes
    (
    ObjectProperty * object
    ) const
    {
    A * a = dynamic_cast< A * >( object );
    Property::Attributes attributes;

    a->integer.initialize_attributes( attributes );
    attributes.ptr_repflags = REPFLAGS_MIGRATE;
    a->pointer.initialize_attributes( attributes );
    a->pointer2.initialize_attributes( attributes );
    }


bool SimpleFactory<A>::get_property_info
    (
    const ObjectProperty *      object,
    const Property *            owned_property,
    Property::PropertyInfo &    info
    ) const
    {
    const A * a = dynamic_cast< const A * >( object );

    if( owned_property == &a->integer )
        {
        info.property_name = "integer";
        return true;
        }
    else
    if( owned_property == &a->pointer )
        {
        info.property_name = "pointer";
        return true;
        }
    else
    if( owned_property == &a->pointer2 )
        {
        info.property_name = "pointer2";
        return true;
        }

    return false;
    }


bool SimpleFactory<C, c_bases>::get_property_info
    (
    const ObjectProperty *      object,
    const Property *            owned_property,
    Property::PropertyInfo &    info
    ) const
    {
    const C * c = dynamic_cast< const C * >( object );

    ClassManager & manager = Global::class_manager();
    ObjectFactory * a_factory = manager.get_factory( typeid( A ) );
    ObjectFactory * b_factory = manager.get_factory( typeid( B ) );

    if( owned_property == &c->a_member )
        {
        info.property_name = "a_member";
        return true;
        }
    else
    if( a_factory->get_property_info( c, owned_property, info ) )
        {
        info.property_name =
                std::string( a_factory->get_class_type_info().name() ) +
                std::string( "::" ) +
                info.property_name;

        return true;
        }
    else
    if( b_factory->get_property_info( c, owned_property, info ) )
        {
        info.property_name =
                std::string( b_factory->get_class_type_info().name() ) +
                std::string( "::" ) +
                info.property_name;

        return true;
        }

    return false;
    }


void SimpleFactory<C, c_bases>::for_each_property_do
    (
    PropertyOperation &     operation,
    ObjectProperty *        object
    ) const
    {
    C * c = dynamic_cast< C * >( object );

    // perform operation on this class

    c->a_member.for_each_property_do( operation );

    // perform operation on base classes

    ClassManager & manager = Global::class_manager();
    ObjectFactory * a_factory = manager.get_factory( typeid( A ) );
    ObjectFactory * b_factory = manager.get_factory( typeid( B ) );

    a_factory->for_each_property_do( operation, c );
    b_factory->for_each_property_do( operation, c );
    }


void SimpleFactory<C, c_bases>::initialize_property_attributes
    (
    ObjectProperty * object
    ) const
    {
    C * c = dynamic_cast< C * >( object );
    Property::Attributes attributes;

    c->integer.initialize_attributes( attributes );
    c->a_member.initialize_attributes( attributes );
    attributes.ptr_repflags = REPFLAGS_MIGRATE;
    c->pointer.initialize_attributes( attributes );
    c->pointer2.initialize_attributes( attributes );
    }


void SimpleFactory<D>::for_each_property_do
    (
    PropertyOperation &     operation,
    ObjectProperty *        object
    ) const
    {
    D * d = dynamic_cast< D * >( object );

    d->pointer.for_each_property_do( operation );
    }


bool SimpleFactory<D>::get_property_info
    (
    const ObjectProperty *      object,
    const Property *            owned_property,
    Property::PropertyInfo &    info
    ) const
    {
    const D * d = dynamic_cast< const D * >( object );

    if( owned_property == &d->pointer )
        {
        info.property_name = "pointer";
        }
    else
        {
        return false;
        }

    return true;
    }


void SimpleFactory<D>::initialize_property_attributes
    (
    ObjectProperty * object
    ) const
    {
    D * d = dynamic_cast< D * >( object );
    Property::Attributes attributes;
    attributes.ptr_repflags = REPFLAGS_MIGRATE;
    d->pointer.initialize_attributes( attributes );
    }


ObjectManagementTest::ObjectManagementTest() :
    Test( "object management" )
    {
    }


ObjectManagementTest::~ObjectManagementTest()
    {
    }


#ifdef OPTION_DEBUG
#define DEBUG_TRY try
#define DEBUG_CATCH( e ) catch( e )
#else // OPTION_DEBUG
#define DEBUG_TRY if( 0 )
#define DEBUG_CATCH( e )
#endif // OPTION_DEBUG


struct TestOperation : public GarbageCollector::ObjectOperation
    {
    virtual void operator()
        (
        ObjectProperty * object
        )
        {
        visited.insert( object );
        }

    void reset()
        {
        visited.clear();
        }

public:

    std::set<const ObjectProperty *> visited;
    };


static Pointer<A> generate_tree
    (
    SimpleFactory<A> &  a_factory,
    int                 depth
    )
    {
    Pointer<A> node;

    if( depth > 0 )
        {
        node = a_factory.create_ex();
        node->pointer = generate_tree( a_factory, depth - 1 );
        node->pointer2 = generate_tree( a_factory, depth - 4 );
        }

    return node;
    }


void ObjectManagementTest::test()
    {
    SimpleFactory<A>            a_factory;
    SimpleFactory<B>            b_factory;
    SimpleFactory<C, c_bases>   c_factory;
    SimpleFactory<D>            d_factory;
    reference_value( b_factory );

    bool                        null_id_test_passed = false;
    bool                        assign_test_passed = false;
    bool                        deref_test_passed = false;

    /* Misc. tests. */

        {
        Pointer<A>  a1, a2, a3;
        Pointer<B>  b1;
        Pointer<C>  c1;

        /* Pointer test. */

        a1 = a_factory.create_ex();
        a2 = a1;
        a2->integer = 3;

        DEBUG_TRY
            {
            a3->integer = 0; // Error: a3 is NULL_ID
            // Should segfault unless OPTION_DEBUG is defined.
            }
        DEBUG_CATCH( Lib::ObjectNotOnLocalSystemException & )
            {
            null_id_test_passed = true;
            }

        c1 = c_factory.create_ex();
        a2 = c1;

        DEBUG_TRY
            {
            //b1 = a1; // Error: classes A and B are unrelated, compile-time error
            b1 = a1.convert(); // Error: conversion fails
            }
        DEBUG_CATCH( Lib::IllegalPointerConversionException & )
            {
            assign_test_passed = true;
            };

        Global::garbage_collector().collect_object( c1 );

        DEBUG_TRY
            {
            a2->integer = 0; // Error: object pointed to by a2 does not exist any more
            // Should segfault unless OPTION_DEBUG is defined.
            }
        DEBUG_CATCH( Lib::ObjectNotOnLocalSystemException & )
            {
            deref_test_passed = true;
            };

        test_condition_( null_id_test_passed );
        test_condition_( assign_test_passed );
        test_condition_( deref_test_passed );

        /* Migration group test. */

        GarbageCollector &                  gc = Global::garbage_collector();
        TestOperation                       operation;
        std::set<const ObjectProperty *>    reachable;

        a1 = a_factory.create_ex();
        a2 = a_factory.create_ex();
        a3 = a_factory.create_ex();

        a1->pointer = a2;
        a3->pointer = a2;

        reachable.insert( a1.dereference().operator->() );
        reachable.insert( a2.dereference().operator->() );
        reachable.insert( a3.dereference().operator->() );

        operation.reset();

        gc.enumerate_migration_group( a3.dereference().operator->(), operation );
        test_condition_( reachable == operation.visited );

        operation.reset();

        gc.enumerate_migration_group( a2.dereference().operator->(), operation );
        test_condition_( reachable == operation.visited );

        operation.reset();

        gc.enumerate_migration_group( a1.dereference().operator->(), operation );
        test_condition_( reachable == operation.visited );

        /* Clone test. */

        a1 = a_factory.create_ex();

        a1->integer = 0x12345678;
        a2 = a1->clone().convert();
        test_condition_( a2->integer == a1->integer );
        test_condition_( a2->get_object_id() != a1->get_object_id() );
        }

    /* Garbage collector test. */

        {
        GarbageCollector & gc = Global::garbage_collector();
        Pointer<A>  a1;
        Pointer<D>  d1, d2;

        d1 = d_factory.create_ex();

        a1 = a_factory.create_ex();
        a1->pointer = a_factory.create_ex();
        a1->pointer->pointer = a_factory.create_ex();
        a1->pointer->pointer->pointer = a_factory.create_ex();

        gc.save_memory_snapshot( "test0.dot" );

        a1->pointer->pointer->pointer->pointer = a1->pointer->pointer;
        a1->pointer = a1;
        d1->pointer = a1;

        gc.save_memory_snapshot( "test1.dot" );

        d2 = d_factory.create_ex();
        d2->pointer = generate_tree( a_factory, 8 );

        gc.save_memory_snapshot( "test2.dot" );

        d2->pointer->pointer->pointer->pointer->pointer = d2->pointer->pointer;
        d2->pointer->pointer2 = d2->pointer;
        d2->pointer->pointer = d2->pointer;

        gc.save_memory_snapshot( "test3.dot" );

        do
            {
            gc.next_tick();
            }
        while( gc.is_running() );

        gc.save_memory_snapshot( "test4.dot" );
        }

    /* RPC test. */

        {
        Pointer<D>  ptr;
        Remote<D>   r;

        ptr = d_factory.create_ex();
        r = ptr;

        r->f();
        }
    }


#else // 0
// See file comment


ObjectManagementTest::ObjectManagementTest() :
    Test( "object management" )
    {
    }


ObjectManagementTest::~ObjectManagementTest()
    {
    }


void ObjectManagementTest::test()
    {
    // Make it fail to show something is not right, because no-one reads the sources.

    test_fail_( "This test no longer works, move object management tests to test_factgen." );
    }


#endif // 0
// See file comment


} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv
