/** @file test_registry.h
 * Registry tests.
 */

/*
 * begin        : Fri Apr 26 2002
 * copyright    : (C) 2002 by Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TEST_REGISTRY_H
#define MASSIV_TEST_REGISTRY_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TEST_H
#include "test/test.h"
#endif

#ifndef MASSIV_REGISTRY_H
#include "registry/registry.h"
#endif



namespace Massiv {
namespace Core {
namespace Test {


/**
 * 
 * Testing of registry.
 *
 * @todo Test weird usage of configuration file syntax?
 */
class RegistryTest : public Test
    {
public:

    /**
     * Constructor.
     */
    RegistryTest();

    /**
     * Destructor.
     */
    ~RegistryTest();

public:

    /**
     * Run test
     */
    void test();

protected:

    /**
     * Tests check_path_syntax() and check_name_syntax().
     */
    void test_syntax_checks();

    /**
     * Tests inserting variables into registry.
     */
    void test_inserts();

    /**
     * Tests inserting and resolving aliases.
     */
    void test_aliases();

    /**
     * Tests searching for variables withing one registry node.
     */
    void test_one_node_search();

    /**
     * Tests searching through the whole registry.
     */
    void test_registry_search();

    /**
     * Tests handling of wrong lines in the configuration files.
     */
    void test_wrong_strings();

    /**
     * Tests configuration file load and save.
     */
    void test_config_file();

protected:

    /**
     * Auxiliary routine for test_syntax_checks().
     * @a path should be syntactically correct registry path; the method
     * tests whether it will be really determined as correct.
     */
    void test_correct_path_syntax
        (
        std::string     path
        );

    /**
     * Auxiliary routine for test_syntax_checks().
     * @a path should be syntactically wrong registry path; the method
     * tests whether it will be really determined as wrong.
     */
    void test_wrong_path_syntax
        (
        std::string     path
        );

    /**
     * Auxiliary routine for test_syntax_checks.
     * @a name should be syntactically correct registry node, variable or
     * alias name; the method tests whether it will be really determined 
     * as correct.
     */
    void test_correct_name_syntax
        (
        std::string     name
        );

    /**
     * Auxiliary routine for test_syntax_checks.
     * @a name should be syntactically wrong registry node, variable or
     * alias name; the method tests whether it will be really determined
     * as wrong.
     */
    void test_wrong_name_syntax
        (
        std::string     name
        );

protected:

    Registry *  registry;

    };  // class RegistryTest


}; // namespace Massiv::Core::Test
}; // namespace Massiv::Core
}; // namespace Massiv


#endif // _TEST_REGISTRY_H_
