/** @file test_sockets.cpp
 * Sockets testing.
 *
 * @bug Does not compile after changes to network.
 */

/*
 * begin        : Tue Mar 8 2002
 * copyright    : (C) 2002 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "test/test_sockets.h"
#include "net/bitostream.h"
#include "net/massiv_checksum_impl.h"
#include "net/connection_info_impl.h"
#include "global/global.h"
#include "net/massiv_rsa.h"
#include "registry/registry.h"
#include "thread/thread.h"
#include "net/udpobuffer.h"
#include "sysevent/system_event_queue.h"
#include <math.h>
#include <iostream>


namespace Massiv {
namespace Core {
namespace Test {


NodeId              SocketsTest::nid_client( NodeId::create_from_type_and_relative_id( NodeId::TYPE_CLIENT, 2 ) );
NodeId              SocketsTest::nid_server( NodeId::create_from_type_and_relative_id( NodeId::TYPE_SERVER, 2 ) );
CriticalSection *   SocketsTest::cs;

SocketsTest::SocketsTest() :
    Test( "sockets" )
    {
    }


SocketsTest::~SocketsTest()
    {
    }


void SocketsTest::test()
    {
#if 0
    test_checksum();
    test_server_address();
    test_sockets();
#endif
    }


#if 0
void SocketsTest::test_sockets()
    {
    const std::string           ahoj = "ahooooojahooooj";
    Socket *                    tcpsocket;
    Socket *                    udpsocket;
    ConnectionInfo *            conn_info = new ConnectionInfo( true );

    Network & net = Global::network();

    conn_info->set_blowfish_key_even( "test" );
    conn_info->set_blowfish_key_odd( "test" );

    // We pretend, connection exists.
    conn_info->set_connection_status( ConnectionInfo::CONNECTED );

    net.assign_local_node_id( NodeId::create_from_type_and_relative_id( NodeId::TYPE_SERVER, 1 ) );

    net.node_table.insert
        (
        Network::NodeHash::value_type( nid_server, conn_info )
        ).first;

    tcpsocket = net.create_tcp_socket();
    tcpsocket->start_server( 12345, 10 );
    tcpsocket->close();
    udpsocket = net.create_udp_socket();
    udpsocket->start_server( 12345 );
    udpsocket->close();

    NetIBuffer          read;
    BitIOStreamBuffer   decrypted;

    std::string         key = "cipher123";
    std::string         crypted_string;
    std::string         signed_string;

    // Testing locking / dropping

    TCPOBuffer * buftcp;
    TCPOBuffer * buftcp1;
    TCPOBuffer * buftcp2;
    test_condition_( net.tcp_sendlist.size() == 0 );
    buftcp = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024 );

    test_condition_( net.tcp_sendlist.size() == 1 );
    test_condition_( net.have_i_something_for_server( nid_server ) == true ); no such function
    buftcp1 = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024 );
    test_condition_( net.tcp_sendlist.size() == 2 );
    buftcp2 = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024 );
    test_condition_( net.tcp_sendlist.size() == 3 );

    net.drop_all_to_server( nid_server );
    test_condition_( net.have_i_something_for_server( nid_server ) == false );
    test_condition_( net.tcp_sendlist.size() == 0 );

    buftcp = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024 );
    test_condition_( net.tcp_sendlist.size() == 1 );
    buftcp1 = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024 );
    test_condition_( net.tcp_sendlist.size() == 2 );
    buftcp2 = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024 );
    test_condition_( net.tcp_sendlist.size() == 3 );

    test_condition_( net.have_i_something_for_server( nid_server ) == true );

    try
        {
        net.drop( buftcp );
        test_condition_( true );
        test_condition_( net.tcp_sendlist.size() == 2 );
        net.drop( buftcp1 );
        test_condition_( true );
        test_condition_( net.tcp_sendlist.size() == 1 );
        net.drop( buftcp2 );
        test_condition_( true );
        test_condition_( net.tcp_sendlist.size() == 0 );
        test_condition_( net.have_i_something_for_server( nid_server ) == false );
        }
    catch(...)
        {
        test_condition_( false );
        }
    
    buftcp = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024 );
    test_condition_( net.tcp_sendlist.size() == 1 );
    buftcp1 = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024 );
    test_condition_( net.tcp_sendlist.size() == 2 );

    net.drop_all();
    test_condition_( net.have_i_something_for_server( nid_server ) == false );
    test_condition_( net.tcp_sendlist.size() == 0 );


    UDPOBuffer * bufudp;
    UDPOBuffer * bufudp1;
    UDPOBuffer * bufudp2;
    test_condition_( net.udp_sendlist.size() == 0 );
    bufudp = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 1 );
    bufudp1 = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 2 );
    bufudp2 = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 3 );
    test_condition_( net.have_i_something_for_server( nid_server ) == true );

    net.drop_all_to_server( nid_server );
    test_condition_( net.udp_sendlist.size() == 0 );
    test_condition_( net.have_i_something_for_server( nid_server ) == false );

    bufudp = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 1 );
    bufudp1 = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 2 );
    bufudp2 = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 3 );

    test_condition_( net.have_i_something_for_server( nid_server ) == true );
    try
        {
        net.drop( bufudp );
        test_condition_( true );
        test_condition_( net.udp_sendlist.size() == 2 );
        net.drop( bufudp1 );
        test_condition_( true );
        test_condition_( net.udp_sendlist.size() == 1 );
        net.drop( bufudp2 );
        test_condition_( true );
        test_condition_( net.udp_sendlist.size() == 0 );
        test_condition_( net.have_i_something_for_server( nid_server ) == false );
        }
    catch( ... )
        {
        test_condition_( false );
        }
    
    bufudp = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 1 );
    bufudp1 = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 2 );

    net.drop_all();
    test_condition_( net.udp_sendlist.size() == 0 );
    bufudp = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 1 );

    buftcp = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024 );
    test_condition_( net.tcp_sendlist.size() == 1 );

    net.drop_all();
    test_condition_( net.udp_sendlist.size() == 0 );

    try
        {
        buftcp = ( TCPOBuffer * ) net.lock_reliable( nid_client, 1024 );
        test_condition_( false );
        }
    catch( ... )
        {
        test_condition_( true );
        }

    test_condition_( net.can_lock_buffer_to_node( nid_client, NetworkCommon::STREAM_RELIABLE ) == false );

    net.drop_all();

    buftcp = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024, false );
    test_condition_( buftcp->byte_size() == 0 );

    buftcp->write_string( ahoj );
    
    test_condition_( ( UInt32 ) buftcp->byte_size() == ahoj.length() - 1 );

    net.drop_all();
    
    buftcp = ( TCPOBuffer * ) net.lock_reliable( nid_server, 1024 );
    buftcp->write_full_string( ahoj );
    test_condition_( ( UInt32 ) buftcp->byte_size() == ahoj.length() + 4 );
    
    test_condition_( buftcp->status == NetOBuffer::NET_BUFFER_LOCKED );
    net.unlock( buftcp );
    test_condition_( buftcp->status == NetOBuffer::NET_BUFFER_FLUSH_ALLOWED );

    net.drop_all();

    bufudp = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( bufudp->byte_size() == NetworkCommon::NET_BUFFER_HEAD_LEN_UDP );

    bufudp->write_string( ahoj );
    
    test_condition_( ( UInt32 ) bufudp->byte_size() == NetworkCommon::NET_BUFFER_HEAD_LEN_UDP + ceil( ( float ) ahoj.length() / 8 * 7 ) );

    net.drop_all();
    

    // UDP buffer reusing
    bufudp = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 1 );

    bufudp->write_full_string( ahoj );
    test_condition_( ( UInt32 ) bufudp->byte_size() == NetworkCommon::NET_BUFFER_HEAD_LEN_UDP + ahoj.length() + 4 );

    test_condition_( bufudp->status == NetOBuffer::NET_BUFFER_LOCKED );
    net.unlock_unreliable( bufudp, false );
    test_condition_( bufudp->status == NetOBuffer::NET_BUFFER_UNLOCKED );
    
    bufudp = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 1 );
    
    net.unlock_unreliable( bufudp, true );
    test_condition_( bufudp->status == NetOBuffer::NET_BUFFER_FLUSH_ALLOWED );

    bufudp = ( UDPOBuffer * ) net.lock_unreliable( nid_server, 1024 );
    test_condition_( net.udp_sendlist.size() == 1 );

    bufudp->align_fill( 0, 8150 );
    
    net.unlock_unreliable( bufudp, true );
    test_condition_( bufudp->status == NetOBuffer::NET_BUFFER_FLUSH_ALLOWED );
    test_condition_( net.have_i_something_for_server( nid_server ) == true );

    // There should be 1 message waiting for sending.
    test_condition_( net.udp_sendlist.size() == 1 );
    bufudp->flush();

    // There should be 2 messages waiting for sending.
    test_condition_( net.udp_sendlist.size() == 2 );
    test_condition_( bufudp->status == NetOBuffer::NET_BUFFER_CRYPTED );
    net.drop_all();

    

    test_condition_( net.have_i_something_for_server( nid_server ) == false );

    test_condition_( net.udp_sendlist.size() == 0 );
    } // SocketsTest::test_sockets()


void SocketsTest::test_server_address()
    {
    const UInt16    test_port   = 0x1234;
    const UInt16    test_port2  = 0x3456;

    ServerAddress   saddr( "localhost", test_port );

    const char *    test_dns_name   = "localhost";
    const UInt32    test_ip         = saddr.assemble_ip( 127, 0, 0, 1 );


    // Constructor
    test_condition_( saddr.port == htons( test_port ) );
    test_condition_( saddr.ip == ServerAddress::IP_UNDEFINED );
    test_condition_( saddr.caching == true );
    test_condition_( saddr.dns_name == test_dns_name );

    // Resolving existing address (localhost =~= 127.0.0.1)
    UInt32 ipaddr;
    ipaddr = saddr.get_ip();
    test_condition_( ipaddr == test_ip );

    UInt8 f1;
    UInt8 f2;
    UInt8 f3;
    UInt8 f4;

    saddr.get_ip_fields( f1, f2, f3, f4 );

    test_condition_( f1 == 127 );
    test_condition_( f2 == 0 );
    test_condition_( f3 == 0 );
    test_condition_( f4 == 1 );

    // Setters & getters.
    test_condition_( saddr.get_ip() == test_ip );
    test_condition_( saddr.get_dns_name() == test_dns_name );
    saddr.set_port( test_port2 );
    test_condition_( saddr.get_port() == test_port2 );


    // string_to_ip() & set_ip() & assemble_ip() tests
    ipaddr = ServerAddress::string_to_ip( "192.168.0.4" );
    test_condition_( saddr.assemble_ip( 192, 168, 0, 4 ) == ipaddr );
    saddr.set_ip( ipaddr );

    test_condition_( saddr.ip == ipaddr );
    test_condition_( saddr.dns_name == DNS_NAME_UNDEFINED );

    saddr.get_ip_fields( f1, f2, f3, f4 );
    test_condition_( f1 == 192 );
    test_condition_( f2 == 168 );
    test_condition_( f3 == 0 );
    test_condition_( f4 == 4 );


    // set_addr() for both DNS name and textual IP
    saddr.set_addr( "192.168.0.4" );
    test_condition_( saddr.ip == ipaddr );
    test_condition_( saddr.dns_name == "192.168.0.4" );

    saddr.set_addr( test_dns_name );
    test_condition_( saddr.ip == ServerAddress::IP_UNDEFINED );
    test_condition_( saddr.dns_name == test_dns_name );


    // Resolving non-existing address.
    const char *    foo_dns_name = "foohost.foo";
    ServerAddress   saddr2( foo_dns_name, test_port );

    try
        {
        ipaddr = saddr2.get_ip();
        test_condition_( false );
        }
    catch( ExInvalidDnsName & )
        {
        test_condition_( true );
        }

    test_condition_( saddr2.get_dns_name() == std::string( foo_dns_name ) );
    }  // SocketsTest::test_server_address()


void SocketsTest::test_checksum()
    {
    typedef InternetChecksum< BitIOStreamBuffer >   InternetChecksum32;
    typedef CRC32< BitIOStreamBuffer >              Crc32;

    BitOStream              stream;
    InternetChecksum32      ichksum( &stream.buffer );
    Crc32                   crc( &stream.buffer );
    UInt32                  checksum;

    // Testing of the InternetChecksum
    // Testing example from the RFC document
    stream.write_int( 0x03f20100, 32 );
    stream.write_int( 0xf7f6f5f4, 32 );

    checksum = ichksum.compute_checksum();
    test_condition_( checksum == ~0xf4f7e8fa );
    stream.buffer[ 1 ] = 0x02;
    test_condition_( checksum != ichksum.compute_checksum() );

    // Testing modified example from the RFC - number of bytes
    // in the buffer is not divisible by 4 ( == sizeof( ChecksumType ) )
    stream.reset();
    stream.write_int( 0x03f20100, 32 );
    stream.write_int( 0x00f6f5f4, 24 );
    test_condition_( ichksum.compute_checksum() == ~0xf4f7e803 );

    stream.reset();
    stream.write_int( 0x03f20100, 32 );
    stream.write_int( 0x0000f5f4, 16 );
    test_condition_(( checksum = ichksum.compute_checksum() ) == ~0xf4f6f203 );

    stream.reset();
    stream.write_int( 0x03f20100, 32 );
    stream.write_int( 0x000000f4, 8 );
    test_condition_(( checksum = ichksum.compute_checksum() ) == ~0xf401f203 );
    test_condition_( ichksum.validate_checksum( checksum ) );
    stream.align_fill( 0, 3 );
    stream.write_int( ( checksum & 0xff000000 ) >> 24, 8 );
    stream.write_int( ( checksum & 0x00ff0000 ) >> 16, 8 );
    stream.write_int( ( checksum & 0x0000ff00 ) >> 8, 8 );
    stream.write_int( ( checksum & 0x000000ff ), 8 );
    test_condition_( ichksum.validate_checksum() );

    stream.reset();
    stream.write_string( "nvaoejfoiajlakjfeioajfmjj3824jrfqjfj48j8j983jj9" );
    checksum = ichksum.compute_checksum();
    stream.reset();
    stream.write_string( "nvaoejfoiajlakjfejoajfmjj3824jrfqjfj48j8j983jj9" );
                                        // ^ this has changed
    test_condition_( checksum != ichksum.compute_checksum() );

    // Test the 1's complements arithmetics in the Internet checksum
    stream.reset();
    stream.write_int( 0x000000ff, 32 );
    stream.write_int( 0x000000ff, 32 );
    test_condition_(( checksum = ichksum.compute_checksum() ) == ~0xfe000001 );
    test_condition_( ichksum.validate_checksum( checksum ) );

    stream.reset();
    stream.write_int( 0xffffffff, 32 );
    stream.write_int( 0xffffffff, 32 );
    stream.write_int( 0xffffffff, 32 );
    stream.write_int( 0xffffffff, 32 );
    stream.write_int( 0xffffffff, 32 );
    stream.write_int( 0xffffffff, 32 );
    stream.write_int( 0x00000000, 24 );
    stream.write_int( 0x05, 8 );
    test_condition_(( checksum = ichksum.compute_checksum() ) == ~0x00000005 );

    // Testing Crc32 checksum
    stream.reset();
    stream.write_string( "nvaoejfoiajlakjfeioajfmjj3824jrfqjfj48j8j983jj9" );
    checksum = crc.compute_checksum();
    test_condition_( crc.validate_checksum( checksum ) );

    // Append the checksum to the buffer
    stream.write_int( checksum, 32 );

    // Validate the checksum written in the buffer
    test_condition_( crc.validate_checksum() );

    // Next Crc32 test
    stream.reset();
    stream.write_string( "nvaoejfoiajlakjfejoajemjj3824jrfqjfj48j8j983jj9" );
                                        // ^   ^ this has changed
    test_condition_( checksum != crc.compute_checksum() );
    test_condition_( !crc.validate_checksum( checksum ) );

    // Append the checksum to the buffer
    stream.write_int( checksum, 32 );

    // Validate the wrong checksum written in the buffer
    test_condition_( !crc.validate_checksum() );

    // Next Crc32 test
    stream.write_string( "nvaoejfoiajlakjfeioajfmjj3824jrfqjfj48j8j983jj8" );
    test_condition_( checksum != crc.compute_checksum() );


    // Testing the checksum support of UDPOBuffer and NetIBuffer
    UDPOBuffer      obuffer;
    NetIBuffer      ibuffer;

    obuffer.write_string( "nvaoejfoiajlakjfejoajemjj3824jrfqjfj48j8j983jj9" );
    obuffer.append_checksum();

    test_condition_( obuffer.engine_checksum.validate_checksum() );

    ibuffer.bind( obuffer.crypted_buffer, false, 0 );
    test_condition_( ibuffer.engine_checksum.validate_checksum() );

    obuffer.crypted_buffer[ 1 ] -= 7;
    obuffer.crypted_buffer[ 2 ] += 4;
    obuffer.crypted_buffer[ 3 ] += 11;
    test_condition_( !ibuffer.engine_checksum.validate_checksum() );

    }  // SocketsTest::test_checksum()
#endif


} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv
