/** @file test_sockets.h
 * Sockets testing.
 */

/*
 * begin        : Tue Nov 27 2001
 * copyright    : (C) 2001 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TEST_SOCKETS_H
#define MASSIV_TEST_SOCKETS_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TEST_H
#include "test/test.h"
#endif

#ifndef MASSIV_NET_H
#include "net/net.h"
#endif

#ifndef MASSIV_TCPOBUFFER_H
#include "net/tcpobuffer.h"
#endif

#ifndef MASSIV_NETIBUFFER_H
#include "net/netibuffer.h"
#endif

#ifndef MASSIV_SEMAPH_H
#include "thread/semaph.h"
#endif

#ifndef MASSIV_SYNCH_H
#include "thread/synch.h"
#endif

#include <stdio.h>

#ifdef SYSTEM_WIN32
    #include <winsock2.h>
#endif

#ifdef SYSTEM_UNIX
    #include <netinet/in.h>
#endif


namespace Massiv {
namespace Core {
namespace Test {


class SocketsTest : public Test
    {
public:
  
    /**
     * Constructor.
     */
    SocketsTest();

    /**
     * Destructor.
     */
    ~SocketsTest();

    /**
     * Initializes the registry node "Network settings".
     * @note Supposes that the "Network settings" registry node has
     *      not been created so far.
     */
    void init_registry_network_node();

public:

    /**
     * Run the main testing routine.
     */
    void test();

private:

    /**
     * Testing of sockets.
     */
    void test_sockets();

    /**
     * Testing of ServerAddress.
     */
    void test_server_address();

    /**
     * Testing InternetChecksum.
     */
    void test_checksum();

private:
    
    // Temporary debugging hack:
    static void try_send_tcp_server
        (
        Network &       net
        );

    static void try_send_tcp_client
        (
        Network &       net
        );

private:
    
    static NodeId               nid_server;
        /**< Node id of the server. */

    static NodeId               nid_client;
        /**< Node id of the client. */

    static CriticalSection *    cs;

    static Semaphore *          sem_server;
        /**< Semaphore to be used to block server during tests. */

    static Semaphore *          sem_client;
        /**< Semaphore to be used to block client during tests. */

    static Semaphore *          sem_rendezvous_server;
        /**< Semaphore reserved to be used in the rendezvous_server/client
             routines. */

    static Semaphore *          sem_rendezvous_client;
        /**< Semaphore reserved to be used in the rendezvous_server/client
             routines. */

    }; // class SocketsTest


}; // namespace Massiv::Core::Test
}; // namespace Massiv::Core
}; // namespace Massiv


#endif // _TEST_SOCKETS_H_
