/** @file test_thread.cpp
 * Thread test.
 */

/*
 * begin        : Tue Oct 15 2002
 * copyright    : (C) 2002 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "test/test_thread.h"
#include "thread/thread.h"
#include "thread/synch.h"
#include "thread/semaph.h"
#include "global/global.h"

using namespace Massiv::Core::Test;
using namespace Massiv::Core;


namespace Massiv {
namespace Core {
namespace Test {


int volatile        ThreadTest::sum;
CriticalSection *   ThreadTest::cs;
Semaphore       *   ThreadTest::empty;
Semaphore       *   ThreadTest::full;


#define THREAD_TEST_BUF_SIZE 2

ThreadTest::ThreadTest() :
    Test( "thread" )
    {
    sum = 0;
    CriticalSectionManager & csm = Global::critical_section_manager();
    cs = csm.create();

    SemaphoreManager & sm = Global::semaphore_manager();
    empty = sm.create( THREAD_TEST_BUF_SIZE );  
    full = sm.create( 0 );
    }


ThreadTest::~ThreadTest()
    {
    delete cs;
    delete empty;
    delete full;
    }


int ThreadTest::cs_test
    (
    VariantPointer arg
    )
    {
    int number = *variant_cast<int *>( arg );
    cs->enter();    
    sum += number;    
    cs->leave();
    return number;    
    }


int ThreadTest::producer
    (
    VariantPointer arg
    )
    {
    int number = *variant_cast<int *>( arg );  // how many times produce item
    while( number-- )
        { 
        // produce_item( &item )
        empty->down();
    
        cs->enter();
        // enter_item( item );
        cs->leave();
    
        full->up();
        }
    return number;    
    }


int ThreadTest::consumer
    (
    VariantPointer arg
    )
    {
    int number = *variant_cast<int *>( arg );  // how many times consume item
    while( number-- )
        { 
        full->down();

        cs->enter();
        // remove_item( item );
        cs->leave();
    
        empty->up();    
        // consume_item( item );
        }
    return number;    
    }


void ThreadTest::test_thread()
    {
    int t1_num = 10;
    int t2_num = 20;
    int t3_num = -5;
    int t4_num = -10;

    // thread and critical section test
    ThreadManager & tm = Global::thread_manager();
    Reference<Thread> thread1 = 
        tm.create( ThreadTest::cs_test, VariantPointer( &t1_num ) );
    Reference<Thread> thread2 = 
        tm.create( ThreadTest::cs_test, VariantPointer( &t2_num ) );
    Reference<Thread> thread3 = 
        tm.create( ThreadTest::cs_test, VariantPointer( &t3_num ) );
    Reference<Thread> thread4 = 
        tm.create( ThreadTest::cs_test, VariantPointer( &t4_num ) );
    
    test_condition_( thread1->join() == t1_num );
    test_condition_( thread2->join() == t2_num );
    test_condition_( thread3->join() == t3_num );
    test_condition_( thread4->join() == t4_num );

    test_condition_( sum == 15 ); // 10+20-5-10 = 15
    }


void ThreadTest::test_semaphore()
    {
    // semaphore test - producer & consumer
    int suma_sumarum = THREAD_TEST_BUF_SIZE * 4;
    int p1 = suma_sumarum / 2;
    int p2 = suma_sumarum / 2;
    int c1 = suma_sumarum / 4;
    int c2 = suma_sumarum / 4;
    int c3 = suma_sumarum / 4;
    int c4 = suma_sumarum / 4;
    
    ThreadManager & tm = Global::thread_manager();

    Reference<Thread> consumer1 = 
        tm.create( ThreadTest::consumer, VariantPointer( &c1 ) );
    Reference<Thread> consumer2 = 
        tm.create( ThreadTest::consumer, VariantPointer( &c2 ) );
    Reference<Thread> consumer3 = 
        tm.create( ThreadTest::consumer, VariantPointer( &c3 ) );
    Reference<Thread> consumer4 = 
        tm.create( ThreadTest::consumer, VariantPointer( &c4 ) );

    Reference<Thread> producer1 = 
        tm.create( ThreadTest::producer, VariantPointer( &p1 ) );
    Reference<Thread> producer2 = 
        tm.create( ThreadTest::producer, VariantPointer( &p2 ) );

    consumer1->join();
    consumer2->join();
    consumer3->join();
    consumer4->join();

    producer1->join();
    producer2->join();

    test_condition_( 1 ); // produced items == consumed items (it runs till here -> no deadlock)
    }


void ThreadTest::test()
    {
    test_thread();
    test_semaphore();
    }


} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv
