/** @file test_thread.h
 * Thread test.
 */

/*
 * begin        : Tue Oct 15 2002
 * copyright    : (C) 2002 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TEST_THREAD_H
#define MASSIV_TEST_THREAD_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TEST_H
#include "test/test.h"
#endif

#ifndef MASSIV_VARIANT_POINTER_H
#include "utility/variant_pointer.h"
#endif


namespace Massiv {
namespace Core {

class CriticalSection;
class Semaphore;

namespace Test {


class ThreadTest : public Test
    {
public:

    /**
     * Constructor.
     */
    ThreadTest();

    /**
     * Destructor.
     */
    ~ThreadTest();


public:

    /**
     * Thread function - critical section test.
     */
    static int cs_test
        (
        VariantPointer arg
        );

    /**
     * Thread function - producer - semaphore test.
     */
    static int producer
        (
        VariantPointer arg
        );

    /**
     * Thread function - consumer - semaphore test.
     */
    static int consumer
        (
        VariantPointer arg
        );

    /**
     * Follows test functions.
     */
    void test_thread();
    void test_semaphore();
        
    /**
     * Run test.
     */
    void test();

private:
    
    static int volatile         sum;
        /**< Threads manipulate with this variable. */
    
    static CriticalSection *    cs; 
        /**< Critical section testing. */
    
    static Semaphore *          empty;  
        /**< Number of free items in buffer. */
    
    static Semaphore *          full;
        /**< Number of used items in buffer. */            
            
    }; // class ThreadTest


} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_TEST_THREAD_H
