/** @file test_time.h
 * Time synchronization test.
 */

/*
 * begin        : Tue Feb 4 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TEST_TIME_H
#define MASSIV_TEST_TIME_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_TEST_H
#include "test/test.h"
#endif

#ifndef MASSIV_VARIANT_POINTER_H
#include "utility/variant_pointer.h"
#endif

#include <list>

namespace Massiv {
namespace Core {

class TimeManager;
class CriticalSection;

namespace Test {

class TimeTest : public Test
    {
public:

    typedef std::list<TimeManager *>             TimeManagerArray;
    typedef TimeManagerArray::iterator           iter_tm;

    /**
     * Structure passed as an argument to a time synchronization
     * thread function.
     */
    struct SynchronizeThreadData;

    /**
     * Constructor.
     */
    TimeTest();

    /**
     * Destructor.
     */
    ~TimeTest();

public:

    /**
     * Thread function - timer synchronization simulation.
     */
    static int synchronize_thread
        (
        VariantPointer arg
        );

    /**
     * Thread function - writes debug information to standard output.
     */
    static int synchronize_thread_screen_report
        (
        VariantPointer arg
        );

    /**
     * Test functions follows.
     */
    void test_system_time();
    void test_time_synchronization_algorithm();
    void test_time_manager();

    /**
     * Add next time manager to array.
     * Returns zero.
     */
    static int add_time_manager_to_array
        (
        TimeManagerArray &  tm_array,
        TimeManager *       time_manager
        );

    /**
     * Delete time manager from array.
     * Returns zero.
     */
    static int delete_time_manager_from_array
        (
        TimeManagerArray &  tm_array,
        TimeManager *       time_manager
        );

    /**
     * Send synchronization information from @param time_manager to other
     * TimeManagers from array.
     * Returns zero.
     */
    static int send_synchronization_info
        (
        TimeManagerArray &  tm_array,
        TimeManager *       time_manager
        );


    /**
     * Return unique nodeid.
     */
    static UInt32 get_node_id();

        
    /**
     * Run test.
     */
    void test();

private:
    
    TimeManagerArray            time_manager_array;
        /**< List of time managers which synchronize together. */

    static CriticalSection *    tma_cs;
        /**< CS for time_manager_array. */
            
    static CriticalSection *    cout_cs;
        /**< CS for time_manager_array. */

    }; // class TimeTest


} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_TEST_TIME_H
