/** @file test_union_find.cpp
 * Union-find test.
 */

/*
 * begin        : Sat Mar 17 2002
 * copyright    : (C) 2002 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "test/test_union_find.h"
#include "other/union_find.h"
#include <set>

using namespace Massiv::Core;

namespace Massiv {
namespace Core {
namespace Test {


struct UFItem
    {
    UFItem()
        {
        }

    UFItem
        (
        const char * str
        ) : str( str )
        {
        }

    std::string str;
    };

template class Massiv::Core::UnionFind<UFItem>;


UnionFindTest::UnionFindTest() :
    Test( "union-find" )
    {
    }


UnionFindTest::~UnionFindTest()
    {
    }


void UnionFindTest::test()
    {
    UnionFind<UFItem>            uf;
    UnionFind<UFItem>::Element   a( uf, "a" );
    UnionFind<UFItem>::Element   b( uf, "b" );
    UnionFind<UFItem>::Element   c( uf, "c" );
    UnionFind<UFItem>::Element   d( uf, "d" );
    UnionFind<UFItem>::Element   e( uf, "e" );
    UnionFind<UFItem>::Element   f( uf, "f" );

    // Test merging.

    test_condition_( !uf.same_groups( a, b ) );

    uf.merge( a, b );
    test_condition_( uf.same_groups( a, b ) );
    test_condition_( !uf.same_groups( a, c ) );

    uf.merge( b, c );
    test_condition_( uf.same_groups( a, c ) );
    test_condition_( uf.same_groups( a, b ) );
    test_condition_( uf.same_groups( a, c ) );

    uf.merge( d, e );
    uf.merge( d, f );
    test_condition_( uf.same_groups( d, e ) );
    test_condition_( uf.same_groups( e, f ) );
    test_condition_( uf.same_groups( d, f ) );

    uf.merge( b, e );
    test_condition_( uf.same_groups( a, b ) );
    test_condition_( uf.same_groups( b, c ) );
    test_condition_( uf.same_groups( c, d ) );
    test_condition_( uf.same_groups( d, e ) );
    test_condition_( uf.same_groups( e, f ) );

    // Test unlinking.

    uf.unlink( c );
    test_condition_( uf.same_groups( c, c ) );
    test_condition_( !uf.same_groups( a, c ) );
    test_condition_( !uf.same_groups( b, c ) );
    test_condition_( !uf.same_groups( d, c ) );
    test_condition_( !uf.same_groups( e, c ) );
    test_condition_( !uf.same_groups( f, c ) );

    test_condition_( uf.count( a ) == 5 );
    test_condition_( uf.count( c ) == 1 );

    // Enumerate group containing 'a'.

    UnionFind<UFItem>::Iterator it;
    std::set<std::string>       collected;
    std::set<std::string>       group;

    it = uf.begin( a );
    collected.clear();

    while( it != uf.end( a ) )
        {
        collected.insert( it->str );
        it++;
        }

    group.clear();
    group.insert( "a" );
    group.insert( "b" );
    group.insert( "d" );
    group.insert( "e" );
    group.insert( "f" );

    test_condition_( group == collected );

    // Enumerate group containing 'f' (= group containing 'a').

    it = uf.begin( a );
    collected.clear();

    while( it != uf.end( a ) )
        {
        collected.insert( it->str );
        it++;
        }

    test_condition_( group == collected );

    // Enumerate group containing 'c'.

    it = uf.begin( c );
    collected.clear();

    while( it != uf.end( c ) )
        {
        collected.insert( it->str );
        it++;
        }

    group.clear();
    group.insert( "c" );

    test_condition_( group == collected );
    }


} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv
