/** @file test_volume_enumeration.h
 * Test of interface for enumeration of files and directories in 
 * Massiv::Core::BasicVolume filesystem. These tests are written
 * separately from normal filesystem enumerator tests, although
 * both of them are quite similar. Otherwise the tests would be
 * very opaque.
 */

/*
 * begin        : Wed Feb 26 2003
 * copyright    : (C) 2003 by Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TEST_VOLUME_ENUMERATION_H
#define MASSIV_TEST_VOLUME_ENUMERATION_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TEST_H
#include "test/test.h"
#endif

#ifndef MASSIV_VOLUME_H
#include "filesys/volume.h"
#endif

#include <string>

namespace Massiv {
namespace Core {
namespace Test {


/**
 * Testing of interface for enumeration of files and directories in 
 * Massiv::Core::BasicVolume filesystem.
 */
class VolumeEnumerationTest : public Test
    {
public:

    /**
     * Constructor.
     */
    VolumeEnumerationTest();

    /**
     * Destructor.
     */
    ~VolumeEnumerationTest();

public:

    /**
     * Initializes this test class.
     * @return true if the test could be initialized successfuly, false 
     *      otherwise.
     */
    bool init();

    /**
     * Run test
     */
    void test();

private:
    
    /**
     * Test of going through the directory. Calls 
     * @c perform_go_through_test() several times. The directory used for
     * the test is massiv/src/core/test/test_hierarchy.
     */
    void go_through_test();

    /**
     * Test of going through the directory. The directory used for the test 
     * is massiv/src/core/test/test_hierarchy.
     */
    void perform_go_through_test
        (
        FileIterator    it
        );

private:

    /**
     * Tests descent() and ascent() (and enter() for throwing exceptions
     * properly.
     */
    void descent_ascent_test();

    /**
     * Tests enter() and leave().
     */
    void enter_leave_test();

private:
    
    /**
     * Tests is_in_root().
     */
    void is_in_root_test();

    /**
     * Miscelanous tests.
     */
    void misc_tests();

    /**
     * Tests pack/unpack features of BasicVolumePacking.
     */
    void pack_unpack_test();

private:
    
    /**
     * Copies a file from os filesystem to @c volume.
     */
    void copy_to_volume
        (
        BasicVolume &           volume,
        const std::string &     file_name
        );

private:
    
    FileSystem &    native_fs;
        /**< Native file system. */
    std::string     old_current_directory;
        /**< Directory that was current when this test was launched. */
    Volume *        volume;
        /**< Volume for testing iterators on it. */
    bool            inited_successfuly;
        /**< true if the test class has been successfuly initialized,
             false otherwise. */

    };  // class VolumeEnumerationTest


} // namespace Massiv::Core::Test
} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_TEST_VOLUME_ENUMERATION_H
