/** @file semaph_unix.cpp
 * Unix-specific semaphore implementation.
 * @see semaph_unix.h
 */

/* 
 * begin        : Tue Nov 1 2002
 * copyright    : (C) 2002 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "status/exception.h"
#include "thread/arch/unix/semaph_unix.h"
#include <sstream>
#include <errno.h>

namespace Massiv {
namespace Core {


// Implementation of SemaphoreManager
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Semaphore * SemaphoreManager::create
    (
    int init_count
    )
    {
    return new SemaphoreUnix( init_count );
    }


// Implementation of SemaphoreUnix
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SemaphoreUnix::SemaphoreUnix
    (
    int init_count
    )
    {
    if( sem_init( &sem, 0, init_count ) != 0 )
        {
        std::ostringstream   ss;
        ss << "Cannot create semaphore; errno: " << errno;
        throw ExSemaphoreCanNotCreate( ss.str() );
        }
    }


SemaphoreUnix::~SemaphoreUnix()
    {
    if( sem_destroy( &sem ) != 0 )
        {
        std::ostringstream   ss;
        ss << "Cannot destroy semaphore; errno: " << errno;
        throw ExSemaphoreCanNotDestroy( ss.str() );
        }      
    }


void SemaphoreUnix::down()
    {
    sem_wait( &sem );
    }


void SemaphoreUnix::up()
    {
    if( sem_post( &sem ) != 0 )
        {   
        std::ostringstream   ss;
        ss << "Cannot increase semaphore; errno: " << errno;
        throw ExSemaphoreCanNotIncrease( ss.str() );
        }
    }


} // namespace Massiv::Core
} // namespace Massiv
