/** @file semaph_unix.h
 * Unix-specific semaphore interface.
 */

/* 
 * begin        : Tue Nov 1 2002
 * copyright    : (C) 2002 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SEMAPH_UNIX_H
#define MASSIV_SEMAPH_UNIX_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_SEMAPH_H
#include "thread/semaph.h"
#endif

#include <semaphore.h>


namespace Massiv {
namespace Core {


/** @addtogroup thread
 * @{
 */

/** 
 * Unix-specific semaphore implementation.
 */
class SemaphoreUnix : public Semaphore
    {
    /// Semaphore can create SemaphoreUnix.
    friend class Semaphore;
    friend class SemaphoreManager;

private:

    /**
     * Constructor - initialization of Unix semaphore.
     * @param init_count Init semaphore count.
     */
    SemaphoreUnix
        (
        int init_count
        );

    /**
     * Destructor.
     */
    ~SemaphoreUnix();

    NO_DEFAULT_CONSTRUCTOR( SemaphoreUnix );
    NO_COPY_CONSTRUCTOR( SemaphoreUnix );
    NO_ASSIGNMENT_OPERATOR( SemaphoreUnix );

public:

    /**
     */
    virtual void down();

    /**
     */
    virtual void up();
    
private:

    sem_t   sem;
        /**< Unix semaphore id. */
    
    }; // SemaphoreUnix

/** @} */ // addtogroup thread


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_SEMAPH_UNIX_H
