/** @file shmem_unix.cpp
 * Unix-specific shared memory implementation.
 * @see shmem.h
 */

/* 
 * begin        : Thu Mar 10 2003
 * copyright    : (C) 2003 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "thread/shmem.h"
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <cerrno>
#include <string>
#include <sstream>

namespace Massiv {
namespace Core {

SharedMemory::SharedMemory
    (
    UInt32                  size,
    const std::string &     name
    ) :
    memory( NULL ),
    owner( true )
    {
    key_t   key = 0;
    std::string::const_iterator pc;
    for( pc = name.begin(); pc != name.end() ; ++pc )  key += *pc;
    if( key == IPC_PRIVATE )  key++;
    
    shmid = shmget( key, size, 0666 | IPC_CREAT | IPC_EXCL );
    if( shmid == -1 )
        {
        if( errno == EEXIST )
            {
            shmid = shmget( key, size, 0666 );
            owner = false;
            }
        }
    if( shmid == -1 )
        {
        std::ostringstream  ss;
        ss << "Cannot create shared memory with name \"" << name << "\". errno = " << errno;
        throw ExCommonError( ss.str() );
        }
    memory = shmat( shmid, NULL, 0 );
    if( !memory )
        {
        std::ostringstream  ss;
        ss << "Cannot create shared memory with name \"" << name << "\". errno = " << errno;
        throw ExCommonError( ss.str() );
        }
    
    _size = size;
    map_name = name;
    }
    

SharedMemory::~SharedMemory()
    {
    if( shmdt( memory ) == -1 )
        {
        std::ostringstream  ss;
        ss << "Cannot unmap shared memory with name \"" << map_name << "\". errno = " << errno;
        throw ExCommonError( ss.str() );        
        }
    if( owner )
        {
        if( shmctl( shmid, IPC_RMID, NULL ) == -1 )
            {
            std::ostringstream  ss;
            ss << "Cannot mark shared memory as destroyed. errno = " << errno;
            throw ExCommonError( ss.str() );
            }
        }
    }


void * SharedMemory::get_memory() const
    {
    return memory;
    }

        
} // namespace Massiv::Core
} // namespace Massiv
