/** @file synch_unix.h
 * Unix-specific critical section class.
 */

/*
 * begin        : Tue May 15 2002
 * copyright    : (C) 2002 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SYNCH_UNIX_H
#define MASSIV_SYNCH_UNIX_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_SYNCH_H
#include "thread/synch.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup thread
 * @{
 */

/**
 * Unix-specific implementation of the critical section using semaphores
 * (for even interprocess synchronization) or mutex (for synchronization
 * only withing a single process)..
 */
class CriticalSectionUnix : public CriticalSection
    {
    friend class CriticalSection;
    friend class CriticalSectionManager;

private:

    /**
     * Constructor.
     * CriticalSectionManager::create() delegates the call here.
     */
    CriticalSectionUnix
        (
        bool            owner,
        const char *    name
        );

    /**
     * Destructor.
     */
    ~CriticalSectionUnix();

    NO_DEFAULT_CONSTRUCTOR( CriticalSectionUnix );
    NO_COPY_CONSTRUCTOR( CriticalSectionUnix );
    NO_ASSIGNMENT_OPERATOR( CriticalSectionUnix );

public:

    /**
     */
    virtual void enter();

    /**
     */
    virtual bool try_enter();

    /**
     */
    virtual void leave();

private:

    pthread_mutex_t     mutex;
        /**< Unix mutex. */

    pthread_mutexattr_t attr;
        /**< Unix mutex attributes. */

    int                 semaphore_id;
        /**< Id of semaphore that will be used for interprocess critical
             section implementation. */

    bool                named;
        /**< true if the %CriticalSection is named.
             Named critical section can be used for
             interprocess synchronization (and is implemented using semaphore).
             If false, the %CriticalSection is local in the current process
             (and is implemented more efficiently using a pthread mutex). */

    bool                owner;
        /**< Indicates whether this instance of %CriticalSectionUnix is
             responsible for destroying the low-level synchronization
             primitives. This attribute is considered only in case of
             the named version of critical section. */

    }; // CriticalSectionUnix

/** @} */ // addtogroup thread


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_SYNCH_UNIX_H
