/** @file thread_unix.h
 * %Interface for Unix threads.
 */

/*
 * begin        : Tue May 16 2002
 * copyright    : (C) 2002 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_THREAD_UNIX_H
#define MASSIV_THREAD_UNIX_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_THREAD_H
#include "thread/thread.h"
#endif

#include <pthread.h>


namespace Massiv {
namespace Core {


/** 
 * Unix-specific thread implementation.
 */
class ThreadUnix : public Thread
    {
private:

    friend class Thread;
    friend class ThreadManager;

    /**
     * Constructor.
     *
     * Creates the thread object and launches the thread routine.
     * 
     * @param thread_routine The thread routine that will be started in
     *      the constructor.
     * @param arg Context that will be passed to the thread routine.
     */
    ThreadUnix
        (
        ThreadRoutine   thread_routine,
        VariantPointer  arg
        );

    /**
     * Destructor.
     */
    ~ThreadUnix();

    NO_DEFAULT_CONSTRUCTOR( ThreadUnix );
    NO_COPY_CONSTRUCTOR( ThreadUnix );
    NO_ASSIGNMENT_OPERATOR( ThreadUnix );

    /**
     * Suspends execution of the calling thread until the thread
     * identified by this instance of class %ThreadUnix exits.
     * 
     * @return The exit code of the thread the method is waiting for
     *      (see @link Thread::exit() exit() @endlink).
     */
    virtual int join();

private:

    /**
     * %Thread routine launcher.
     *
     * This is the routine that is actually called when a new thread
     * is created. Its purpose is to hide calling convention requirements,
     * parameters, etc.
     *
     * It is called from %ThreadUnix constructor; it calls @c run() to
     * actually launch the thread routine.
     *
     * @param context The context of the thread - pointer to the 
     *      relevant %ThreadUnix object.
     */
    static void * thread_launcher
        (
        void * context
        );

private:

    pthread_t       thread_id;
        /**< Unix thread id. */

    }; // ThreadUnix

/** @} */ // addtogroup thread


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_THREAD_UNIX_H
