/** @file semaph_win32.cpp
 * Win32-specific semaphore implementation.
 * @see semaph_win32.h
 */

/* 
 * begin        : Tue Nov 1 2002
 * copyright    : (C) 2002 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "status/exception.h"
#include "thread/arch/win32/semaph_win32.h"
#include <sstream>


/**
 * Maximum value the semaphore can hold.
 */
#define SEMAPHORE_MAX_COUNT 32768

namespace Massiv {
namespace Core {

    
// Implementation of SemaphoreManager
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Semaphore * SemaphoreManager::create
    (
    int init_count
    )
    {
    return new SemaphoreWin32( init_count );
    }


// Implementation of SemaphoreWin32
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SemaphoreWin32::SemaphoreWin32
    (
    int init_count
    )
    {
    sem = ::CreateSemaphore( NULL, init_count, SEMAPHORE_MAX_COUNT, NULL );
    if( sem == NULL )
        {
        std::ostringstream   ss;
        ss << "Cannot create semaphore: " << ::GetLastError();
        throw ExSemaphoreCanNotCreate( ss.str() );
        }
    }


SemaphoreWin32::~SemaphoreWin32()
    {
    if( ::CloseHandle( sem ) == 0 )
        {
        std::ostringstream   ss;
        ss << "Cannot destroy semaphore: " << ::GetLastError();
        throw ExSemaphoreCanNotDestroy( ss.str() );
        }      
    }


void SemaphoreWin32::down()
    {
    if( ::WaitForSingleObject( sem, INFINITE ) == WAIT_FAILED )
        {
        std::ostringstream   ss;
        ss << "Cannot decrease semaphore: " << ::GetLastError();
        throw ExSemaphoreCanNotDecrease( ss.str() );
        }
    }


void SemaphoreWin32::up()
    {
    if( ::ReleaseSemaphore( sem, 1, NULL ) == 0 )
        {   
        std::ostringstream   ss;
        ss << "Cannot increase semaphore: " << ::GetLastError();
        throw ExSemaphoreCanNotIncrease( ss.str() );
        }
    }


} // namespace Massiv::Core
} // namespace Massiv
