/** @file semaph_win32.h
 * Win32-specific semaphore interface.
 */

/* 
 * begin        : Tue Nov 1 2002
 * copyright    : (C) 2002 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_SEMAPH_WIN32_H
#define MASSIV_SEMAPH_WIN32_H

#ifndef MASSIV_SYNCH_H
#include "thread/synch.h"
#endif

#ifndef MASSIV_SEMAPH_H
#include "thread/semaph.h"
#endif

#include <windows.h>


namespace Massiv {
namespace Core {


/** @addtogroup thread
 * @{
 */

/** 
 * Win32-specific semaphore implementation.
 */
class SemaphoreWin32 : public Semaphore
    {
    friend class Semaphore;
    friend class SemaphoreManager;

private:

    /**
     * Constructor.
     * @param init_count Initial value of the semaphore counter.
     */
    SemaphoreWin32
        (
        int init_count
        );

    ~SemaphoreWin32();

    NO_DEFAULT_CONSTRUCTOR( SemaphoreWin32 );
    NO_COPY_CONSTRUCTOR( SemaphoreWin32 );
    NO_ASSIGNMENT_OPERATOR( SemaphoreWin32 );

public:

    /**
     */
    virtual void down();

    /**
     */
    virtual void up();
    
private:

    HANDLE  sem;
        /**< Win32 semaphore handle. */
    
    }; // SemaphoreWin32

/** @} */ // addtogroup thread


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_SEMAPH_WIN32_H
