/** @file shmem_win32.cpp
 * Win32-specific shared memory implementation.
 * @see shmem.h
 */

/* 
 * begin        : Thu Mar 10 2003
 * copyright    : (C) 2003 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "thread/shmem.h"
#include <windows.h>
#include <string>
#include <sstream>

namespace Massiv {
namespace Core {

SharedMemory::SharedMemory
    (
    UInt32                  size,
    const std::string &     name
    ) :
    memory( NULL )
    {
    HANDLE h = CreateFileMapping( NULL, NULL, PAGE_READWRITE, 0, size, name.c_str() );
    if( !h )
        {
        std::ostringstream  ss;
        ss << "Cannot create file mapping with name \"" << name << "\". GetLastError() = " << GetLastError();
        throw ExCommonError( ss.str() );
        }
    memory = MapViewOfFile( h, FILE_MAP_WRITE, 0, 0, size );
    if( !memory )
        {
        std::ostringstream  ss;
        ss << "Cannot map view of file with name \"" << name << "\". GetLastError() = " << GetLastError();
        throw ExCommonError( ss.str() );
        }
    map_name = name;
    }
    

SharedMemory::~SharedMemory()
    {
    if( UnmapViewOfFile( memory ) == 0 )
        {
        std::ostringstream  ss;
        ss << "Cannot unmap view of file with name \"" << map_name << "\". GetLastError() = " << GetLastError();
        throw ExCommonError( ss.str() );        
        }
    }


void * SharedMemory::get_memory() const
    {
    return memory;
    }

        
}; // namespace Massiv::Core
}; // namespace Massiv
