/** @file synch_win32.h
 * Win32-specific critical section class.
 */

/* 
 * begin        : Tue May 15 2002
 * copyright    : (C) 2002 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SYNCH_WIN32_H
#define MASSIV_SYNCH_WIN32_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_SYNCH_H
#include "thread/synch.h"
#endif

#include <windows.h>


namespace Massiv {
namespace Core {


/** @addtogroup thread
 * @{
 */

/** 
 * Win32-specific implementation of the critical section using the Win32
 * mutex.
 */
class CriticalSectionWin32 : public CriticalSection
    {

    friend class CriticalSection;
    friend class CriticalSectionManager;

private:

    /**
     * Constructor.
     * CriticalSectionManager::create() delegates the call here.
     */
    CriticalSectionWin32
        (
        bool            owner,
        const char *    name
        );

    /**
     * Destructor.
     */
    ~CriticalSectionWin32();

    NO_DEFAULT_CONSTRUCTOR( CriticalSectionWin32 );
    NO_COPY_CONSTRUCTOR( CriticalSectionWin32 );
    NO_ASSIGNMENT_OPERATOR( CriticalSectionWin32 );

public:

    /**
     */
    virtual void enter();

    /**
     */
    virtual bool try_enter();

    /**
     */
    virtual void leave();

private:

    HANDLE mutex;
        /**< Win32 mutex handle. */
    
    }; // CriticalSectionWin32

/** @} */ // addtogroup thread


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_SYNCH_WIN32_H
