/** @file thread_win32.h
 * %Interface for Win32 threads.
 */

/*
 * begin        : Tue May 16 2002
 * copyright    : (C) 2002 by Martin Havlista
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_THREAD_WIN32_H
#define MASSIV_THREAD_WIN32_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_THREAD_H
#include "thread/thread.h"
#endif

#include <windows.h>


namespace Massiv {
namespace Core {


/** @addtogroup thread
 * @{
 */

/** 
 * Win32-specific thread implementation.
 */
class ThreadWin32 : public Thread
    {

    friend class Thread;
    friend class ThreadManager;

private:

    /**
     * Constructor.
     *
     * Creates the thread object and launches the thread routine.
     * 
     * @param thread_routine The thread routine that will be started in
     *      the constructor.
     * @param arg Context that will be passed to the thread routine.
     */
    ThreadWin32
        (
        ThreadRoutine   thread_routine,
        VariantPointer  arg
        );

    /**
     * Destructor.
     */
    ~ThreadWin32();

    NO_DEFAULT_CONSTRUCTOR( ThreadWin32 );
    NO_COPY_CONSTRUCTOR( ThreadWin32 );
    NO_ASSIGNMENT_OPERATOR( ThreadWin32 );

    /**
     * Suspends execution of the calling thread until the thread
     * identified by this instance of class %ThreadWin32 exits.
     * 
     * @return The exit code of the thread the method is waiting for
     *      (see @link Thread::exit() exit() @endlink).
     */
    virtual int join();

private:

    /**
     * %Thread routine launcher.
     *
     * This is the routine that is actually called when a new thread
     * is created. Its purpose is to hide calling convention requirements,
     * parameters, etc.
     *
     * It is called from %ThreadWin32 constructor; it calls @c run() to
     * actually launch the thread routine.
     *
     * @param context The context of the thread - pointer to the 
     *      relevant %ThreadWin32 object.
     */
    static unsigned __stdcall thread_launcher
        (
        void * context
        );

private:

    HANDLE          thread_handle;
        /**< Win32 thread handle. */
    unsigned int    thread_id;
        /**< Win32 thread id. */

    }; // ThreadWin32

/** @} */ // addtogroup thread


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_THREAD_WIN32_H
