/** @file atomic.h
 * Atomic increment/decrement.
 */

/*
 * begin        : Thu Aug 28 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_ATOMIC_H
#define MASSIV_ATOMIC_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup thread
 * @{
 */

/** 
 * %Atomic increment/decrement.
 */
class Atomic
    {
public:

    /**
     * %Atomic increment.
     *
     * @param value The value to be incremented.
     * @return A negative value if the incremented @c value is 
     *      <tt>&lt; 0</tt>, @c 0 if the result is <tt>== 0</tt>, 
     *      a positive value if the result is <tt>&gt; 0</tt>.
     */
    static Int32 increment
        (
        Int32 & value
        );
        
    /**
     * %Atomic decrement.
     *
     * @param value The value to be decremented.
     * @return A negative value if the decremented @c value is 
     *      <tt>&lt; 0</tt>, @c 0 if the result is <tt>== 0</tt>, 
     *      a positive value if the result <tt>&gt; 0</tt>.
     */
    static Int32 decrement
        (
        Int32 & value
        );
        
    }; // class Atomic

/** @} */ // addtogroup thread


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_ATOMIC_H
