/** @file stack_tracer.h
 * Stack tracer enables to dump the call-stack of a thread any time.
 */

/*
 * begin        : Sat Dec 12 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_STACK_TRACER_H
#define MASSIV_STACK_TRACER_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_REFERENCE_H
#include "utility/reference.h"
#endif

#ifndef MASSIV_SEMAPH_H
#include "thread/semaph.h"
#endif

#ifndef MASSIV_SYNCH_H
#include "thread/synch.h"
#endif

#include <iosfwd>

namespace Massiv {
namespace Core {


/** @addtogroup thread
 * @{
 */

/** 
 * %Stack tracer enables to dump the call-stack of a %thread any time.
 */
class StackTracer
    {
public:
        
    /** Destructor. */
    virtual ~StackTracer();

    /** 
     * Dumps call stack of the calling thread to @c stream. 
     * @param stream The target stream.
     */
    virtual void dump_stack
        (
        std::ostream & stream
        ) = 0;

    /**
     * Callback to install an exception handler for the current thread.
     * If a Win32 SEH exception or a unix signal is raised, our handler
     * will be called and a nice stack trace will be printed.
     */
    virtual void install_parachute() = 0;

public:

    /**
     * Instantiates stack tracer instance.
     *
     * To be used by System when it starts up.
     *
     * @return NULL if the target architecture does not support such a feature.
     */
    static StackTracer * create();
    
    }; // StackTracer

/** @} */ // addtogroup thread


} // namespace Massiv::Core
} // namespace Massiv

#endif // MASSIV_STACK_TRACER_H
