/** @file system_time_unix.cpp
 * Unix system time abstraction.
 */

/* 
 * begin        : Tue Nov 19 2002
 * copyright    : (C) 2002 by Martin Havlista, Marek Vondrak
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "time/system_time.h"
#include <sys/time.h>

namespace Massiv {
namespace Core {


static struct ::timeval get_time()
    {
    struct ::timeval current_time;
    
    ::gettimeofday( &current_time, NULL );
    
    return current_time;
    }


SystemTime SystemTime::current_time()
    {
    static struct ::timeval initial_time = get_time();
    struct ::timeval        current_time = get_time();
    static Float64          previous_tick_count = 0;
    static Float64          tick_delta = 0;
    Float64                 tick_count;

    tick_count =
        ( current_time.tv_sec - initial_time.tv_sec ) * 1000 +
        ( current_time.tv_usec - initial_time.tv_usec ) / 1000 +
        tick_delta;

    if( previous_tick_count > tick_count )
        {
        initial_time = current_time;
        tick_delta = previous_tick_count;
        previous_tick_count = 0;
        }

    previous_tick_count = tick_count;
    return tick_count / 1000.0;
    }


} // namespace Massiv::Core
} // namespace Massiv
