/** @file system_time_win32.cpp
 * Win32 system time wrapper.
 */

/*
 * begin        : Tue Nov 19 2002
 * copyright    : (C) 2002 by Martin Havlista, Marek Vondrak
 */


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "time/system_time.h"
#include <windows.h>


namespace Massiv {
namespace Core {

static UInt32 get_tick_increment()
    {
    // The way how to get tick increment is correct even if the value 
    // of GetTickCount() has wrapped around to zero.
    static UInt32   previous_tick_count = ::GetTickCount();
    UInt32          tick_count = ::GetTickCount();
    UInt32          increment = tick_count - previous_tick_count;
    
    previous_tick_count = tick_count;
    return increment;
    }


SystemTime SystemTime::current_time()
    {
    static Int64    total_tick_count = 0;
    
    total_tick_count += get_tick_increment();
    return total_tick_count / 1000.0;
    }

} // namespace Massiv::Core
} // namespace Massiv
