/** @file system_time.h
 * Defines SystemTime, a local system time abstraction.
 */

/*
 * begin        : Fri Dec 06 2002
 * copyright    : (C) 2002 by Marek Svantner, Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_SYSTEM_TIME_H
#define MASSIV_SYSTEM_TIME_H

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#ifndef MASSIV_VIRTUAL_TIME_H
#include "utility/virtual_time.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup time
 * @{
 */

/**
 * %System (simulation) time implementation.
 * 
 * Measures time elapsed since the system start (unlike its ancestor 
 * VirtualTime, it has certain methods that make use of the real time on
 * the relevant computer).
 */
class MASSIV_EXPORTED SystemTime : public VirtualTime<Float64>
    {
public:

    typedef VirtualTime<Float64>    Super;
    typedef Float64                 TimeType;
    typedef Super::Less             Less;
    typedef Super::EqualTo          EqualTo;

public:

    /** Default constructor. */
    INLINE SystemTime();

    /** Constructor. */
    INLINE SystemTime
        (
        TimeType time
        );

    /** Constructor. */
    INLINE SystemTime
        (
        const Super & time
        );

    /** Copy constructor. */
    INLINE SystemTime
        (
        const SystemTime & t
        );

    /** Operator=. */
    INLINE SystemTime & operator=
        (
        const Super & t
        );
    
    /** Operator=. */
    INLINE SystemTime & operator=
        (
        const TimeType & t
        );
    
    /** 
     * Gets current time. 
     * 
     * @return Instance of %SystemTime that is set to the current time
     *      that is set on the relevant computer system.
     */
    static SystemTime current_time();

    }; // class SystemTime

/** @} */ // addtogroup time


} // namespace Massiv::Core
} // namespace Massiv

#ifdef ENABLE_INLINE
#include "time/system_time_inline.h"
#endif

#endif // MASSIV_SYSTEM_TIME_H
