/** @file code_string.cpp
 * String encoding/decoding.
 */

/*
 * begin        : Wed Nov 13 2002
 * copyright    : (C) 2002 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "utility/code_string.h"
#include "status/exception.h"
#include <sstream>


namespace Massiv {
namespace Core {


static unsigned char encode_digit
    (
    unsigned char c
    )
    {
    static unsigned char table[] = "0123456789abcdef";

    hope( c <= 15 );
    return table[ c ];
    }


static unsigned char decode_digit
    (
    unsigned char c
    )
    {
    if( c >= 'a' && c <= 'f' )
        return c - 'a' + 10;
    if( c >= 'A' && c <= 'F' )
        return c - 'A' + 10;
    if( c >= '0' && c <= '9' )
        return c - '0';

    throw ExCommonError( "Hex digit expected." );
    }


std::string CodeString::encode
    (
    const std::string & str
    )
    {
    std::string::const_iterator it;
    std::string                 output;

    it = str.begin();

    while( it != str.end() )
        {
        unsigned char c = static_cast<unsigned char>( *it );

        // Alphabet characters are encoded using standard ASCII codes.
        // The others are encoded by the "=XX" sequence,
        // where XX is hex ASCII code of @c.

        if( ( ( c >= '0' && c <= '9' ) ||
              ( c >= 'a' && c <= 'z' ) ||
              ( c >= 'A' && c <= 'Z' ) ) &&
              ( c != '=' ) )
            {
            output += c;
            }
        else
            {
            output += '=';
            output += encode_digit( ( c >> 4 ) & 0xf );
            output += encode_digit( c & 0xf );
            }

        it++;
        }

    return output;
    }


std::string CodeString::decode
    (
    const std::string & str
    )
    {
    std::string::const_iterator it;
    std::string                 output;

    it = str.begin();

    while( it != str.end() )
        {
        unsigned char c = static_cast<unsigned char>( *it );

        if( c == '=' )
            {
            if( str.end() - it < 2 )
                {
                throw ExCommonError( "Expecting hex ASCII code." );
                }
            else
                {
                unsigned char c1 = static_cast<unsigned char>( *++it );
                unsigned char c2 = static_cast<unsigned char>( *++it );

                c1 = decode_digit( c1 );
                c2 = decode_digit( c2 );

                c = ( c1 << 4 ) | c2;
                output += c;
                }
            }
        else
            {
            output += c;
            }

        it++;
        }

    return output;
    }


}; // namespace Massiv::Core
}; // namespace Massiv
