/** @file hash_function.h
 * Useful hash functions.
 *
 * @todo Currently in ext namespace. Move to Massiv or Massiv::Core?
 */

/*
 * begin        : 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef MASSIV_HASH_FUNCTION_H
#define MASSIV_HASH_FUNCTION_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#ifndef MASSIV_TYPES_H
#include "utility/types.h"
#endif

#include <cstdlib>


// Merge std and ext when using STLPort. Pretty lame...

#ifdef OPTION_STLPORT

    #ifndef ext
    #define ext  std
    #endif

#endif // OPTION_STLPORT


namespace ext {


using Massiv::Core::Int32;
using Massiv::Core::UInt32;
using Massiv::Core::Int64;
using Massiv::Core::UInt64;


/** @addtogroup utility
 * @{
 */

/**
 * Hash function for 32bit integers.
 *
 * It uses Robert Jenkins' 32bit mix function.
 *
 * @note This will not replace @c std::hash<int>, you must use it explicitly.
 * @see http://www.concentric.net/~Ttwang/tech/inthash.htm
 */
//@{
MASSIV_EXPORTED INLINE size_t hash_int
    (
    UInt32          key
    ) throw();
    
MASSIV_EXPORTED INLINE size_t hash_int
    (
    Int32           key
    ) throw();
//@}

    
/**
 * Hash function for 64bit integers.
 *
 * @note It uses #hash_uint32, because I was unable to find nice 64bit hash function.
 */
//@{
MASSIV_EXPORTED INLINE size_t hash_int
    (
    UInt64          key
    ) throw();

MASSIV_EXPORTED INLINE size_t hash_int
    (
    Int64           key
    ) throw();
//@}


/**
 * Hash function for null-terminated strings.
 */
MASSIV_EXPORTED INLINE size_t hash_string
    (
    const char *    key
    ) throw();

/** @} */ // addtogroup utility


} // namespace ext


#ifdef ENABLE_INLINE
#include "utility/hash_function_inline.h"
#endif

#endif // MASSIV_HASH_FUNCTION_H
