/** @file hash_function_inline.h
 * Useful hash functions (inline methods).
 */

/*
 * begin        : 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


namespace ext {


using Massiv::Core::Int32;
using Massiv::Core::UInt32;
using Massiv::Core::Int64;
using Massiv::Core::UInt64;


size_t hash_int
    (
    UInt32          key
    ) throw()
    {
    size_t result = key;

    result += ( result << 12 );
    result ^= ( result >> 22 );
    result += ( result << 4 );
    result ^= ( result >> 9 );
    result += ( result << 10 );
    result ^= ( result >> 2 );
    result += ( result << 7 );
    result ^= ( result >> 12 );

    return result;
    }
    
    
size_t hash_int
    (
    Int32           key
    ) throw()
    {
    return hash_int( static_cast< UInt32 >( key ) );
    }


size_t hash_int
    (
    UInt64          key
    ) throw()
    {
    UInt32 value = static_cast< UInt32 >( key );
    value ^= static_cast< UInt32 >( key >> 32 );
    return hash_int( value );
    }


size_t hash_int
    (
    Int64           key
    ) throw()
    {
    return hash_int( static_cast< UInt64 >( key ) );
    }


size_t hash_string
    (
    const char *    key
    ) throw()
    {
    // This implementation is "stolen" from libstdc++5, originally probably
    // from SGI STL implementation. Using CRC32 could be probably a better
    // solution, but API of our implementation is really hard to use for
    // standard C++ arrays...
    
    hope( key );
    size_t result = 0;
    while( *key )
        {
        result = 5 * result + *key;
        key++;
        }
    return result;
    }


} // namespace ext
