/** @file math_inline.h
 * Generic math stuff (inline methods).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Massiv {
namespace Core {


template< typename T >
typename FloatTraits< T >::FloatType FloatTraits< T >::degrees_to_radians
    (
    FloatType degrees
    )
    {
    return degrees * static_cast< FloatType >( MATH_PI ) / static_cast< FloatType >( 180 );
    }


template< typename T >
typename FloatTraits< T >::FloatType FloatTraits< T >::radians_to_degrees
    (
    FloatType radians
    )
    {
    return radians * static_cast< FloatType >( 180 ) / static_cast< FloatType >( MATH_PI );
    }


template< typename T >
typename FloatTraits< T >::FloatType FloatTraits< T >::safe_acos
    (
    FloatType x
    )
    {
    hope( x > static_cast< FloatType >( -1 ) - epsilon() );
    hope( x < static_cast< FloatType >( 1 ) + epsilon() );

    if( x < static_cast< FloatType >( -1 ) )
        {
        return acos( static_cast< FloatType >( -1 ) );
        }
    if( x > 1.0f )
        {
        return acos( static_cast< FloatType >( 1 ) );
        }
    return acos( x );
    }


template< typename T >
typename FloatTraits< T >::FloatType FloatTraits< T >::safe_asin
    (
    FloatType x
    )
    {
    hope( x > static_cast< FloatType >( -1 ) - epsilon() );
    hope( x < static_cast< FloatType >( 1 ) + epsilon() );

    if( x < static_cast< FloatType >( -1 ) )
        {
        return asin( static_cast< FloatType >( -1 ) );
        }
    if( x > 1.0f )
        {
        return asin( static_cast< FloatType >( 1 ) );
        }
    return asin( x );
    }


template< typename T >
typename FloatTraits< T >::FloatType FloatTraits< T >::clamp_periodic
    (
    FloatType value,
    FloatType minimum,
    FloatType maximum
    )
    {
    const FloatType delta = maximum - minimum;
    hope( delta > epsilon() );

    const FloatType period_count = floor( ( value - minimum ) / delta );
    const FloatType result = value - period_count * delta;

    // Because of floating-point precision problems, result might be slightly
    // below minimum or above maximum.

    if( ( result < minimum ) || ( result >= maximum ) )
        {
        return minimum;
        }
    return result;
    }


} // namespace Massiv::Core
} // namespace Massiv
