/** @file text_writer.cpp
 * Simple line-oriented text file writer.
 */

/*
 * begin        : Fri Apr 12 2002
 * copyright    : (C) 2002 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "utility/text_writer.h"
#include <iterator>
#include <algorithm>


namespace Massiv {
namespace Core {


TextWriter::TextWriter
    (
    std::ostream & stream
    ) : stream( stream ),
        current_line_number( 1 ),
        indentation( 0 ),
        indented( false )
    {
    }


std::ostream & TextWriter::line()
    {
    if( !indented )
        {
        write_space( indentation );
        indented = true;
        }
    return stream;
    }


void TextWriter::end_line()
    {
    stream << '\n';
    indented = false;
    current_line_number++;
    }


int TextWriter::line_number()
    {
    return current_line_number;
    }


bool TextWriter::is_line_empty()
    {
    return !indented;
    }


int TextWriter::change_indent
    (
    int delta
    )
    {
    indentation += delta;
    if( indentation < 0 )
        indentation = 0;
    return indentation;
    }


void TextWriter::set_indent
    (
    int indent
    )
    {
    hope( indent >= 0 );
    indentation = indent;
    }


void TextWriter::write_space
    (
    int count
    )
    {
    hope( count >= 0 );
    if( count )
        {
        std::ostream_iterator<char> ii( stream );
        std::fill_n( ii, count, ' ' );
        }
    }


} // namespace Massiv::Core
} // namespace Massiv
