/** @file types.h
 * Fixed-size types.
 * @todo Define constants MAX_INT8, etc.
 */

/*
 * begin        : Tue Dec 11 2001
 * copyright    : (C) 2001 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_TYPES_H
#define MASSIV_TYPES_H


#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif


namespace Massiv {
namespace Core {


/** @addtogroup bit_types
 * @{
 */

/** @def INT64(c)
 * Create 64bit signed int constant from @a c.
 */

/** @def UINT64(c)
 * Create 64bit unsigned int constant from @a c.
 */

/** @def PRINTF_INT64
 * Printf-like size prefix for format type specifier used to print Int64
 * and UInt64, not including the '%' character, type specifier, etc.
 *
 * Example:
 * @code
 * printf( "%" PRINTF_INT64 "d nebo treba 0x%04" PRINTF_INT64 "x\n", INT64( 10 ), UINT64( 20 ) );
 * @endcode
 *
 * However note that you should consider using standard c++ streams over
 * printf-like functions.
 */

#if defined( ARCH_X86 )
typedef signed char         Int8;
typedef signed short        Int16;
typedef signed int          Int32;
typedef unsigned char       UInt8;
typedef unsigned short      UInt16;
typedef unsigned int        UInt32;
typedef float               Float32;
typedef double              Float64;

#if defined( SYSTEM_UNIX )
typedef signed long long    Int64;
typedef unsigned long long  UInt64;
#define INT64( c ) c ## LL
#define UINT64( c ) c ## ULL
#define PRINTF_INT64 "ll"

#elif defined( SYSTEM_WIN32 )

// Note: MinGW is WIN32 and GCC.
#if defined( COMPILER_GCC )
typedef signed long long    Int64;
typedef unsigned long long  UInt64;
#define INT64( c ) c ## LL
#define UINT64( c ) c ## ULL
#define PRINTF_INT64 "ll"

#else // COMPILER_GCC
typedef signed __int64      Int64;
typedef unsigned __int64    UInt64;
#define INT64( c ) c ## i64
#define UINT64( c ) c ## Ui64
#define PRINTF_INT64 "I64"
#endif // COMPILER_GCC

#else //os
#error "Unknown x86 OS."
#endif // os

#else // architecture
#error "Unknown architecture."

#endif // architecture

/** @} */ // addtogroup bit_types


} // namespace Massiv::Core
} // namespace Massiv


#endif // MASSIV_TYPES_H
