/** @file wildcards.cpp
 * Wildcards implementation.
 */

/* 
 * begin        : Sat Feb 01 2003
 * copyright    : (C) 2002 by Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "utility/wildcards.h"

namespace Massiv {
namespace Core {


bool Wildcards::match
    (
    const std::string::value_type * pattern,
    const std::string::value_type * string
    )
    {
    hope( pattern );
    hope( string );
    
    std::string::value_type p = *pattern;
    std::string::value_type s = *string;

    while( ( p != '\0' ) && ( s != '\0' ) )
        {
        if( p == '*' )
            {
            // '*' matches arbitrary number of elements (including zero).

            pattern++;

            if( *pattern == '\0' )
                {
                // Optimize trailing '*' - this always matches.

                return true;
                }

            // Match iff any tail of string matches pattern.

            do  {
                if( match( pattern, string ) )
                    {
                    return true;
                    }
                } while( *string++ != '\0' );

            // No match.
            
            return false;
            }

        if( ( p != s ) && ( p != '?' ) )
            {
            // Characters are different and pattern is not '?' -> no match.

            return false;
            }

        // Characters at current position match.

        p = *++pattern;
        s = *++string;
        }

    // Reached end of pattern or string.

    if( ( p == '\0' ) && ( s == '\0' ) )
        {
        // Reached end of both pattern and string - match.

        return true;
        }

    return false;
    }


bool Wildcards::match
    (
    const std::string & pattern,
    const std::string & string
    )
    {
    return match( pattern.c_str(), string.c_str() );
    }


} // namespace Core
} // namespace Massiv
