/** @file wildcards.h
 * Simple class with static methods that test whether given string
 * matches a wildcard pattern.
 */

/* 
 * begin        : Sat Feb 01 2003
 * copyright    : (C) 2002 by Marek Svantner
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef MASSIV_WILDCARDS_H
#define MASSIV_WILDCARDS_H

#ifndef MASSIV_COMMON_H
#include "global/common.h"
#endif

#include <string>

namespace Massiv {
namespace Core {


/** @addtogroup utility
 * @{
 */

/**
 * Class for matching text with a wildcards pattern.
 */
class MASSIV_EXPORTED Wildcards
    {
    NO_DEFAULT_CONSTRUCTOR( Wildcards );
    NO_COPY_CONSTRUCTOR( Wildcards );
    NO_ASSIGNMENT_OPERATOR( Wildcards );
    
public:

    /**
     * @return true if the @a string matches the @a pattern, false otherwise.
     */
    static bool match
        (
        const std::string::value_type * pattern,
        const std::string::value_type * string
        );

    /**
     * @return true if the @a string matches the @a pattern, false otherwise.
     * @note Nor @c string nor @c pattern can contain '\0'.
     */
    static bool match
        (
        const std::string & pattern,
        const std::string & string
        );

    }; // class Wildcards

/** @} */ // addtogroup utility


} // namespace Core
} // namespace Massiv

#endif // MASSIV_WILDCARDS_H
