/** @file editor_add_entity_custm_panel.cpp
 * Editor add entity custom panel (implementation).
 */

/*
 * begin        : Fri Feb 20 2004
 * copyright    : (C) 2004 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "client_global.h"
#include "editor/editor_add_entity_custom_panel.h"
#include "gui/widget_layout.h"
#include "gui/editor.h"
#include "renderer/renderer.h"
#include <sstream>


namespace Demo {
namespace Client {


const Float32 BORDER_WIDTH = 16.0f;
const Float32 BORDER_HEIGHT = 16.0f;
const Float32 PADDING_X = 8.0f;
const Float32 PADDING_Y = 8.0f;


EditorAddEntityCustomPanel::EditorAddEntityCustomPanel()
    {
    // Init widget.
    set_border_size( BORDER_WIDTH, BORDER_HEIGHT );
    set_focusable( WINDOW_FOCUSABLE );
    set_dragable( true );
    set_auto_bringable_to_front( true );
    }


bool EditorAddEntityCustomPanel::key_down
    (
    SDLKey  key,
    SDLMod  mod
    )
    {
    if( key == SDLK_ESCAPE )
        {
        // Hide editor.

        disable_focus();
        Global::get_editor().set_visibility( false );
        return true;
        }
    return false;
    }


EditorAddEntityColorGeneratorCustomPanel::EditorAddEntityColorGeneratorCustomPanel()
    {
    Float32 width = 140.0f;
    Float32 height = 20.0f;

    // Tick period.
    tick_period_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    tick_period_label->set_parent_overlay_object( this );

    tick_period_edit_box.reset( new EditBox() );
    tick_period_edit_box->set_parent_widget( this );
    tick_period_edit_box->set_base_size( SVector2( width, height ) ); // Fixme, ugly.
    tick_period_edit_box->do_layout();
    tick_period_edit_box->set_max_text_length_from_size();

    // Tick period.
    period_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    period_label->set_parent_overlay_object( this );

    period_edit_box.reset( new EditBox() );
    period_edit_box->set_parent_widget( this );
    period_edit_box->set_base_size( SVector2( width, height ) ); // Fixme, ugly.
    period_edit_box->do_layout();
    period_edit_box->set_max_text_length_from_size();

    // Enable looping.
    enable_looping_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    enable_looping_label->set_parent_overlay_object( this );

    enable_looping_list_box.reset( new ListBox() );
    enable_looping_list_box->set_parent_widget( this );

    // Wave type.
    wave_type_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    wave_type_label->set_parent_overlay_object( this );

    wave_type_list_box.reset( new ListBox() );
    wave_type_list_box->set_parent_widget( this );

    // Color.
    color_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    color_label->set_parent_overlay_object( this );

    color_edit_box.reset( new EditBox() );
    color_edit_box->set_parent_widget( this );
    color_edit_box->set_base_size( SVector2( width, height ) ); // Fixme, ugly.
    color_edit_box->do_layout();
    color_edit_box->set_max_text_length_from_size();


    enable_looping_list_box->set_message_receiver( this );
    wave_type_list_box->set_message_receiver( this );

    // Texts.
    tick_period_label->print_line( "Tick period(sec):" );
    tick_period_edit_box->set_text( "0.1" );

    period_label->print_line( "Period(sec):     " );
    period_edit_box->set_text( "6" );

    enable_looping_label->print_line( "Enable looping:  " );
    enable_looping_list_box->push( ListBox::Item( 0, "true" ) );
    enable_looping_list_box->push( ListBox::Item( 0, "false" ) );
    enable_looping_list_box->set_base_size( enable_looping_list_box->get_ideal_size() );
    enable_looping_list_box->do_layout();

    wave_type_label->print_line( "Wave type:       " );
    wave_type_list_box->push( ListBox::Item( 0, "Sine Wave" ) );
    wave_type_list_box->push( ListBox::Item( 0, "Square Wave" ) );
    wave_type_list_box->push( ListBox::Item( 0, "Saw Wave" ) );
    wave_type_list_box->push( ListBox::Item( 0, "Random" ) );
    wave_type_list_box->set_base_size( wave_type_list_box->get_ideal_size() );
    wave_type_list_box->do_layout();

    color_label->print_line( "Color(RGBA):     " );
    color_edit_box->set_text( "1.0 1.0 1.0 1.0" );
    }


void EditorAddEntityColorGeneratorCustomPanel::do_layout()
    {
    // Do layout.

    // Make list box same width as edit box and label.
        {
        Float32 lbw = enable_looping_list_box->get_current_size().x();
        if( lbw < 200.0f )
            {
            enable_looping_list_box->set_base_size( SVector2( 200.0f,
                enable_looping_list_box->get_current_size().y() ) );
            enable_looping_list_box->do_layout();
            }
        lbw = wave_type_list_box->get_current_size().x();
        if( lbw < 200.0f )
            {
            wave_type_list_box->set_base_size( SVector2( 200.0f,
                wave_type_list_box->get_current_size().y() ) );
            wave_type_list_box->do_layout();
            }
        }

    WidgetTableLayout layout( BORDER_WIDTH, BORDER_HEIGHT, PADDING_X, PADDING_Y );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( tick_period_label.get() );
    layout.put_object( tick_period_edit_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( period_label.get() );
    layout.put_object( period_edit_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( enable_looping_label.get() );
    layout.put_object( enable_looping_list_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( wave_type_label.get() );
    layout.put_object( wave_type_list_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( color_label.get() );
    layout.put_object( color_edit_box.get() );

    const SVector2 new_size = layout.do_layout();
    set_base_size( new_size );
    }


void EditorAddEntityColorGeneratorCustomPanel::list_box_item_selected
    (
    ListBox *   list_box
    )
    {
    if( list_box == enable_looping_list_box.get() )
        {
        if( list_box->is_item_selected() )
            {
            }
        return;
        }
    if( list_box == wave_type_list_box.get() )
        {
        if( list_box->is_item_selected() )
            {
            }
        return;
        }
    }


std::string EditorAddEntityColorGeneratorCustomPanel::get_add_entity_custom_options_string()
    {
    std::ostringstream  ss;
    ss << tick_period_edit_box->get_text() << " "
       << period_edit_box->get_text() << " "
       << enable_looping_list_box->get_selected_item().name << " "
       << Massiv::Core::quote_const_string( wave_type_list_box->get_selected_item().name ) << " "
       << color_edit_box->get_text();
    //ss << "1.5 1 true WAVE_SINE 255 255 255 255";
    return ss.str();
    }


} // namespace Demo::Client
} // namespace Demo
