/** @file editor_add_entity_custom_panel.h
 * Editor add entity custom panel.
 */

/*
 * begin        : Fri Feb 20 2004
 * copyright    : (C) 2004 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_EDITOR_ADD_ENTITY_CUSTOM_PANEL_H
#define CLIENT_EDITOR_ADD_ENTITY_CUSTOM_PANEL_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef DEMO_CLIENT_SECTOR_H
#include "lib/shared/client_sector.h"
#endif

#ifndef CLIENT_PANEL_BORDER_H
#include "gui/panel_border.h"
#endif

#ifndef CLIENT_LIST_BOX_H
#include "gui/list_box.h"
#endif

#ifndef CLIENT_EDIT_BOX_H
#include "gui/edit_box.h"
#endif

#ifndef CLIENT_BUTTON_H
#include "gui/button.h"
#endif


namespace Demo {
namespace Client {


/**
 * Editor add entity custom panel.
 * @note EditorAddEntityCustomPanel automatically sets its size in @c do_layout().
 */
class EditorAddEntityCustomPanel : public PanelBorder, public ListBox::ListBoxMessageReceiver
    {
    typedef PanelBorder Super;

    NO_COPY_CONSTRUCTOR( EditorAddEntityCustomPanel );
    NO_ASSIGNMENT_OPERATOR( EditorAddEntityCustomPanel );
    
public:

    /** Constructor. */
    EditorAddEntityCustomPanel();
    
    // From Widget.
    virtual void do_layout() = 0;
    
    /**
     * Get custom parameter string. There are serialized custom options
     * in this string.
     */
    virtual std::string get_add_entity_custom_options_string() = 0;
     
protected:

    virtual bool key_down
        (
        SDLKey  key,
        SDLMod  mod
        );
        
public:

    // From ListBoxMessageReceiver.
    virtual void list_box_item_selected
        (
        ListBox *   list_box
        ) = 0;

    }; // EditorAddEntityCustomPanel

    
/**
 * Custom settings for ColorGenerator.
 */
class EditorAddEntityColorGeneratorCustomPanel : public EditorAddEntityCustomPanel
    {
    NO_COPY_CONSTRUCTOR( EditorAddEntityColorGeneratorCustomPanel );
    NO_ASSIGNMENT_OPERATOR( EditorAddEntityColorGeneratorCustomPanel );
    
public:

    /** Constructor. */
    EditorAddEntityColorGeneratorCustomPanel();
    
    // From Widget.
    virtual void do_layout();
    
    /**
     */
    virtual std::string get_add_entity_custom_options_string();

public:

    // From ListBoxMessageReceiver.
    virtual void list_box_item_selected
        (
        ListBox *   list_box
        );
    
private:

    std::auto_ptr< Renderer::TextRectangle >    tick_period_label;
    std::auto_ptr< EditBox >                    tick_period_edit_box;
        
    std::auto_ptr< Renderer::TextRectangle >    period_label;
    std::auto_ptr< EditBox >                    period_edit_box;

    std::auto_ptr< Renderer::TextRectangle >    enable_looping_label;
    std::auto_ptr< ListBox >                    enable_looping_list_box;

    std::auto_ptr< Renderer::TextRectangle >    wave_type_label;
    std::auto_ptr< ListBox >                    wave_type_list_box;
    
    std::auto_ptr< Renderer::TextRectangle >    color_label;
    std::auto_ptr< EditBox >                    color_edit_box;
    
    }; // EditorAddEntityColorGeneratorCustomPanel

} // namespace Demo::Client
} // namespace Demo


#endif // CLIENT_EDITOR_ADD_ENTITY_CUSTOM_PANEL_H
