/** @file editor_add_entity_panel.cpp
 * Editor add entity panel (implementation).
 */

/*
 * begin        : Sat Dec 6 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "client_global.h"
#include "editor/editor_add_entity_panel.h"
#include "gui/widget_layout.h"
#include "gui/editor.h"
#include "renderer/renderer.h"
#include <sstream>


namespace Demo {
namespace Client {


const Float32 BORDER_WIDTH = 16.0f;
const Float32 BORDER_HEIGHT = 16.0f;
const Float32 PADDING_X = 8.0f;
const Float32 PADDING_Y = 8.0f;


EditorAddEntityPanel::EditorAddEntityPanel()
    {
    // Init widget.
    set_border_size( BORDER_WIDTH, BORDER_HEIGHT );
    set_focusable( WINDOW_FOCUSABLE );
    set_dragable( true );
    set_auto_bringable_to_front( true );

    Float32 width = 140.0f;
    Float32 height = 20.0f;

    // Create widgets.

    add_entity_list_box.reset( new ListBox() );
    add_entity_list_box->set_parent_widget( this );
    // Size of add_entity_list_box is set in setup_list_boxes().

    orientation_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    orientation_label->set_parent_overlay_object( this );

    orientation_edit_box.reset( new EditBox() );
    orientation_edit_box->set_parent_widget( this );
    orientation_edit_box->set_base_size( SVector2( width, height ) ); // Fixme, ugly.
    orientation_edit_box->do_layout();
    orientation_edit_box->set_max_text_length_from_size();

    solid_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    solid_label->set_parent_overlay_object( this );

    solid_list_box.reset( new ListBox() );
    solid_list_box->set_parent_widget( this );
    solid_list_box->push( ListBox::Item( 0, "false" ) );
    solid_list_box->push( ListBox::Item( 0, "true" ) );
    solid_list_box->set_base_size( solid_list_box->get_ideal_size() );
    solid_list_box->do_layout();

    bound_x_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    bound_x_label->set_parent_overlay_object( this );

    bound_x_edit_box.reset( new EditBox() );
    bound_x_edit_box->set_parent_widget( this );
    bound_x_edit_box->set_base_size( SVector2( width, height ) ); // Fixme, ugly.
    bound_x_edit_box->do_layout();
    bound_x_edit_box->set_max_text_length_from_size();

    bound_y_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    bound_y_label->set_parent_overlay_object( this );

    bound_y_edit_box.reset( new EditBox() );
    bound_y_edit_box->set_parent_widget( this );
    bound_y_edit_box->set_base_size( SVector2( width, height ) ); // Fixme, ugly.
    bound_y_edit_box->do_layout();
    bound_y_edit_box->set_max_text_length_from_size();

    model_tag_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    model_tag_label->set_parent_overlay_object( this );

    model_tag_edit_box.reset( new EditBox() );
    model_tag_edit_box->set_parent_widget( this );
    model_tag_edit_box->set_base_size( SVector2( width, height ) ); // Fixme, ugly.
    model_tag_edit_box->do_layout();
    model_tag_edit_box->set_max_text_length_from_size();

    axis_align_bbox_button.reset( new Button( Button::TYPE_SWITCH ) );
    axis_align_bbox_button->set_parent_widget( this );
    axis_align_bbox_button->set_text( "Axis Aligned Box" );
    axis_align_bbox_button->set_size_from_text();
    axis_align_bbox_button->push();
    axis_align_bbox_button->do_layout();

    // Fill list boxes.
    setup_list_boxes();

    add_entity_list_box->set_message_receiver( this );
    }


void EditorAddEntityPanel::do_layout()
    {
    // Do layout.

    // Make list box same width as edit box and label.
        {
        const Float32 lbw = add_entity_list_box->get_current_size().x();
        if( lbw < 200.0f )
            {
            add_entity_list_box->set_base_size( SVector2( 200.0f,
                add_entity_list_box->get_current_size().y() ) );
            add_entity_list_box->do_layout();
            }
        }
        /*{
        const Float32 rlw = orientation_label->get_current_size().x();
        const Float32 rebw = orientation_edit_box->get_current_size().x();
        const Float32 mlbw = add_entity_list_box->get_current_size().x();
        if( rlw + PADDING_X + rebw > mlbw )
            {
            add_entity_list_box->set_base_size( SVector2( rlw + PADDING_X + rebw,
                add_entity_list_box->get_current_size().y() ) );
            add_entity_list_box->do_layout();
            }
        else
            {
            orientation_edit_box->set_base_size( SVector2( mlbw - PADDING_X - rlw,
                orientation_edit_box->get_current_size().y() ) );
            orientation_edit_box->do_layout();
            }
        }*/

    WidgetTableLayout layout( BORDER_WIDTH, BORDER_HEIGHT, PADDING_X, PADDING_Y );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_CENTER );
    layout.put_object( add_entity_list_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( orientation_label.get() );
    layout.put_object( orientation_edit_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( solid_label.get() );
    layout.put_object( solid_list_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( bound_x_label.get() );
    layout.put_object( bound_x_edit_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( bound_y_label.get() );
    layout.put_object( bound_y_edit_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( model_tag_label.get() );
    layout.put_object( model_tag_edit_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( axis_align_bbox_button.get() );

    const SVector2 new_size = layout.do_layout();
    set_base_size( new_size );
    }


void EditorAddEntityPanel::list_box_item_selected
    (
    ListBox *   list_box
    )
    {
    if( list_box == add_entity_list_box.get() )
        {
        if( list_box->is_item_selected() )
            {
            // Set custom label texts.
            std::string item_name = add_entity_list_box->get_selected_item().name;
            setup_add_entity_text_labels( item_name );

            Widget * parent_widget = &Global::get_widget_input_dispatcher();
            hope( parent_widget );

            // Set new custom panel.
            if( item_name == "ColorGenerator" )
                {
                if( !editor_add_entity_custom_panel.get() )
                    {
                    Float32 x = get_top_left_corner_on_screen().x() + get_current_size().x();
                    Float32 y = get_top_left_corner_on_screen().y();
                    editor_add_entity_custom_panel.reset( new EditorAddEntityColorGeneratorCustomPanel() );
                    editor_add_entity_custom_panel->set_base_offset( SVector2( x, y ) );
                    editor_add_entity_custom_panel->set_parent_widget( parent_widget );
                    editor_add_entity_custom_panel->do_layout();
                    }
                }
            else
                {
                // Delete existing custom panel.
                editor_add_entity_custom_panel.reset();
                }
            }
        }
    }


AddEntityOptions EditorAddEntityPanel::get_add_entity_options()
    {
    AddEntityOptions opt;

    if( !add_entity_list_box->is_item_selected() )
        {
        return opt;
        }

    opt.entity_class = add_entity_list_box->get_selected_item().name;

        {
        std::istringstream ss( orientation_edit_box->get_text() );
        ss >> opt.orientation;
        }

    opt.solid = false;
    if( solid_list_box->is_item_selected() )
        {
        if( solid_list_box->get_selected_item().name == "true" )
            {
            opt.solid = true;
            }
        }

        {
        std::istringstream ss( bound_x_edit_box->get_text() );
        ss >> opt.bound_x;
        }
        {
        std::istringstream ss( bound_y_edit_box->get_text() );
        ss >> opt.bound_y;
        }
        {
        opt.model_tags = model_tag_edit_box->get_text();
        }

    opt.axis_aligned = axis_align_bbox_button->is_down();

    return opt;
    }


bool EditorAddEntityPanel::key_down
    (
    SDLKey  key,
    SDLMod  mod
    )
    {
    if( key == SDLK_ESCAPE )
        {
        // Hide editor.

        disable_focus();
        Global::get_editor().set_visibility( false );
        return true;
        }
    return false;
    }


void EditorAddEntityPanel::setup_add_entity_list_box()
    {
    typedef ListBox::Item       Item;
    typedef Lib::ClientSector   Sector;
    int         id = 0;
    std::string name;

    name = "None";
    add_entity_list_box->push( Item( id, name ) );

    name = "Robot";
    add_entity_list_box->push( Item( id, name ) );

    name = "Player";
    add_entity_list_box->push( Item( id, name ) );

    name = "Entity";
    add_entity_list_box->push( Item( id, name ) );
    
    name = "Lamp";
    add_entity_list_box->push( Item( id, name ) );

    name = "ColorGenerator";
    add_entity_list_box->push( Item( id, name ) );

    add_entity_list_box->set_base_size( add_entity_list_box->get_ideal_size() );
    add_entity_list_box->do_layout();
    }


void EditorAddEntityPanel::setup_list_boxes()
    {
    setup_add_entity_list_box();
    setup_add_entity_text_labels( "" );
    }


void EditorAddEntityPanel::setup_add_entity_text_labels
    (
    const std::string &     entity_name
    )
    {
    orientation_label->print_line( "Orientation:   " );
    orientation_edit_box->set_text( "0" );

    solid_label->print_line( "Solid:         " );

    bound_x_label->print_line( "Width:         " );
    bound_x_edit_box->set_text( "1.0" );

    bound_y_label->print_line( "Depth:         " );
    bound_y_edit_box->set_text( "1.0" );

    model_tag_label->print_line( "Model tag:     " );
    model_tag_edit_box->set_text( "" );
    }

void EditorAddEntityPanel::set_visibility
    (
    bool visible
    )
    {
    Super::set_visibility( visible );
    if( editor_add_entity_custom_panel.get() )
        {
        editor_add_entity_custom_panel->set_visibility( visible );
        }
    }


std::string EditorAddEntityPanel::get_add_entity_custom_options_string()
    {
    if( editor_add_entity_custom_panel.get() )
        {
        return editor_add_entity_custom_panel->get_add_entity_custom_options_string();
        }
    return "";
    }


} // namespace Demo::Client
} // namespace Demo
