/** @file editor_add_entity_panel.h
 * Editor add entity panel.
 */

/*
 * begin        : Sat Dec 6 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_EDITOR_ADD_ENTITY_PANEL_H
#define CLIENT_EDITOR_ADD_ENTITY_PANEL_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef DEMO_CLIENT_SECTOR_H
#include "lib/shared/client_sector.h"
#endif

#ifndef CLIENT_PANEL_BORDER_H
#include "gui/panel_border.h"
#endif

#ifndef CLIENT_LIST_BOX_H
#include "gui/list_box.h"
#endif

#ifndef CLIENT_EDIT_BOX_H
#include "gui/edit_box.h"
#endif

#ifndef CLIENT_BUTTON_H
#include "gui/button.h"
#endif

#ifndef CLIENT_EDITOR_ADD_ENTITY_CUSTOM_PANEL_H
#include "editor/editor_add_entity_custom_panel.h"
#endif

namespace Demo {
namespace Client {


/**
 * Add entity options.
 */
class AddEntityOptions
    {
public:

    std::string     entity_class;
    Float32         orientation;
    bool            solid;
    Float32         bound_x;
    Float32         bound_y;
    std::string     model_tags;
    bool            axis_aligned;

    }; // AddEntityOptions


/**
 * Editor add entity panel.
 * @note EditorAddEntityPanel automatically sets its size in @c do_layout().
 */
class EditorAddEntityPanel : public PanelBorder, public ListBox::ListBoxMessageReceiver
    {
    typedef PanelBorder Super;

    NO_COPY_CONSTRUCTOR( EditorAddEntityPanel );
    NO_ASSIGNMENT_OPERATOR( EditorAddEntityPanel );

public:

    /** Constructor. */
    EditorAddEntityPanel();

    // From Widget.
    virtual void do_layout();

    /** Get current options. */
    AddEntityOptions get_add_entity_options();

    std::string get_add_entity_custom_options_string();

protected:

    virtual bool key_down
        (
        SDLKey  key,
        SDLMod  mod
        );

public:

    // From ListBoxMessageReceiver.
    virtual void list_box_item_selected
        (
        ListBox *   list_box
        );

    /** FIXME */
    void set_visibility
        (
        bool visible
        );

private:

    /** Enumerate entities to add entity list box. */
    void setup_add_entity_list_box();

    /** Setup content of list boxes. */
    void setup_list_boxes();

    /** Setup text labels for selected entity. */
    void setup_add_entity_text_labels
        (
        const std::string &     entity_name
        );

private:

    std::auto_ptr< ListBox >                    add_entity_list_box;

    std::auto_ptr< Renderer::TextRectangle >    orientation_label;
    std::auto_ptr< EditBox >                    orientation_edit_box;

    std::auto_ptr< Renderer::TextRectangle >    solid_label;
    std::auto_ptr< ListBox >                    solid_list_box;

    std::auto_ptr< Renderer::TextRectangle >    bound_x_label;
    std::auto_ptr< EditBox >                    bound_x_edit_box;

    std::auto_ptr< Renderer::TextRectangle >    bound_y_label;
    std::auto_ptr< EditBox >                    bound_y_edit_box;

    std::auto_ptr< Renderer::TextRectangle >    model_tag_label;
    std::auto_ptr< EditBox >                    model_tag_edit_box;

    std::auto_ptr< Button >                     axis_align_bbox_button;

    SystemTime                                  last_time_add_entity_refreshed;
        /**< Last time add entity list box was refreshed. */

    std::auto_ptr< EditorAddEntityCustomPanel > editor_add_entity_custom_panel;

    }; // EditorAddEntityPanel


} // namespace Demo::Client
} // namespace Demo


#endif // CLIENT_EDITOR_ADD_ENTITY_PANEL_H
