/** @file editor_delete_entity_panel.cpp
 * Editor delete entity panel (implementation).
 */

/*
 * begin        : Sat Dec 21 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "client_global.h"
#include "editor/editor_delete_entity_panel.h"
#include "gui/widget_layout.h"
#include "gui/editor.h"
#include "renderer/renderer.h"
#include "lib/shared/client_entity.h"
#include "gui/client_full_console.h"
#include "game/game_context.h"
#include "lib/shared/account_object_rpc.h"
#include <sstream>


namespace Demo {
namespace Client {


const Float32 BORDER_WIDTH = 16.0f;
const Float32 BORDER_HEIGHT = 16.0f;
const Float32 PADDING_X = 8.0f;
const Float32 PADDING_Y = 8.0f;

EditorDeleteEntityPanel::EditorDeleteEntityPanel()
    {
    // Init widget.
    set_border_size( BORDER_WIDTH, BORDER_HEIGHT );
    set_focusable( WINDOW_FOCUSABLE );
    set_dragable( true );
    set_auto_bringable_to_front( true );

    Float32 width = 150.0f;
    Float32 height = 20.0f;

    // Create widgets.

    hint_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    hint_label->set_parent_overlay_object( this );

    entity_name_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    entity_name_label->set_parent_overlay_object( this );

    entity_name_edit_box.reset( new EditBox() );
    entity_name_edit_box->set_parent_widget( this );
    entity_name_edit_box->set_base_size( SVector2( width, height ) ); // Fixme, ugly.
    entity_name_edit_box->do_layout();
    entity_name_edit_box->set_max_text_length_from_size();

    delete_button.reset( new Button( Button::TYPE_ONE_SHOT ) );
    delete_button->set_parent_widget( this );
    delete_button->set_text( "Delete" );
    delete_button->set_size_from_text();
    delete_button->do_layout();
    delete_button->set_message_receiver( this ); // Receive delete click.
    delete_button->disable(); // default disabled.

    setup_delete_entity_text_labels( "" );
    }


void EditorDeleteEntityPanel::do_layout()
    {
    // Do layout.

    WidgetTableLayout layout( BORDER_WIDTH, BORDER_HEIGHT, PADDING_X, PADDING_Y );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_CENTER );
    layout.put_object( hint_label.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( entity_name_label.get() );
    layout.put_object( entity_name_edit_box.get() );

    layout.next_row();
    layout.next_cell( 1, WidgetTableLayout::HALIGN_LEFT );
    layout.put_object( delete_button.get() );

    const SVector2 new_size = layout.do_layout();
    set_base_size( new_size );
    }


void EditorDeleteEntityPanel::button_clicked
    (
    Button * button
    )
    {
    if( button == delete_button.get() )
        {
        DeleteEntityOptions opt = get_delete_entity_options();

        // Send delete entity command to account.
        Remote< Lib::AccountObject > account = Global::get_game_context().get_account();
        if( account != null )
            {
            account->async_delete_entity_command
                (
                opt.entity_name
                );
            }
        }
    }


DeleteEntityOptions EditorDeleteEntityPanel::get_delete_entity_options()
    {
    DeleteEntityOptions opt;

    opt.entity_name = entity_name_edit_box->get_text();

    return opt;
    }


void EditorDeleteEntityPanel::set_entity_intersection
    (
    const EntityManager::RayIntersection & entity_intersection
    )
    {
    EntityManager::EntityPointer client_entity = entity_intersection.entity;
    actual_client_entity = client_entity;
    const Massiv::Core::ObjectId entity_object_id = client_entity->entity.get_object_id();

        {
        std::ostringstream  ss;
        ss << entity_object_id; //entity.get_object_id();
        entity_name_edit_box->set_text( ss.str() );
        }
    delete_button->enable();
    }


bool EditorDeleteEntityPanel::key_down
    (
    SDLKey  key,
    SDLMod  mod
    )
    {
    if( key == SDLK_ESCAPE )
        {
        // Hide editor.

        disable_focus();
        Global::get_editor().set_visibility( false );
        return true;
        }
    return false;
    }


void EditorDeleteEntityPanel::setup_delete_entity_text_labels
    (
    const std::string &     entity_name
    )
    {
    // Common part.

    hint_label->print_line( "Click entity with right button" );

    entity_name_label->print_line( "Entity name:    " );
    entity_name_edit_box->set_text( "" );
    }

} // namespace Demo::Client
} // namespace Demo
