/** @file editor_delete_entity_panel.h
 * Editor delete entity panel.
 */

/*
 * begin        : Sat Dec 21 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_EDITOR_DELETE_ENTITY_PANEL_H
#define CLIENT_EDITOR_DELETE_ENTITY_PANEL_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef DEMO_CLIENT_SECTOR_H
#include "lib/shared/client_sector.h"
#endif

#ifndef CLIENT_PANEL_BORDER_H
#include "gui/panel_border.h"
#endif

#ifndef CLIENT_LIST_BOX_H
#include "gui/list_box.h"
#endif

#ifndef CLIENT_EDIT_BOX_H
#include "gui/edit_box.h"
#endif

#ifndef CLIENT_BUTTON_H
#include "gui/button.h"
#endif

#ifndef CLIENT_ENTITY_MANAGER_H
#include "game/entity_manager.h"
#endif

#ifndef DEMO_CLIENT_ENTITY_H
#include "lib/shared/client_entity.h"
#endif

namespace Demo {
namespace Client {


/**
 * Edit entity options.
 */
class DeleteEntityOptions
    {
public:

    std::string     entity_name;

    }; // DeleteEntityOptions


/**
 * Editor delete entity panel.
 * @note EditorDeleteEntityPanel automatically sets its size in @c do_layout().
 */
class EditorDeleteEntityPanel : public PanelBorder, public Button::ButtonMessageReceiver
    {
    typedef PanelBorder Super;

    NO_COPY_CONSTRUCTOR( EditorDeleteEntityPanel );
    NO_ASSIGNMENT_OPERATOR( EditorDeleteEntityPanel );
    
public:

    /** Constructor. */
    EditorDeleteEntityPanel();
    
    // From Widget.
    virtual void do_layout();
    
    /** Get current options. */
    DeleteEntityOptions get_delete_entity_options();
        
    /**
     * Set entity intersection.
     */
    void set_entity_intersection
        (
        const EntityManager::RayIntersection & entity_intersection
        );

protected:

    virtual bool key_down
        (
        SDLKey  key,
        SDLMod  mod
        );
        
public:

    // From ButtonMessageReceiver.
    virtual void button_clicked
        (
        Button * button
        );

private:
    
    /** Setup text labels for selected entity. */
    void setup_delete_entity_text_labels
        (
        const std::string &     entity_name
        );

private:

    std::auto_ptr< Renderer::TextRectangle >    hint_label;

    std::auto_ptr< Renderer::TextRectangle >    entity_name_label;
    std::auto_ptr< EditBox >                    entity_name_edit_box;

    std::auto_ptr< Button >                     delete_button;
    
    Massiv::Core::WeakPointer< Lib::ClientEntity > actual_client_entity;
        
    }; // EditorEditEntityPanel

    
} // namespace Demo::Client
} // namespace Demo


#endif // CLIENT_EDITOR_DELETE_ENTITY_PANEL_H
