/** @file editor_edit_entity_panel.h
 * Editor edit entity panel.
 */

/*
 * begin        : Sat Dec 16 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_EDITOR_EDIT_ENTITY_PANEL_H
#define CLIENT_EDITOR_EDIT_ENTITY_PANEL_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef DEMO_CLIENT_SECTOR_H
#include "lib/shared/client_sector.h"
#endif

#ifndef CLIENT_PANEL_BORDER_H
#include "gui/panel_border.h"
#endif

#ifndef CLIENT_LIST_BOX_H
#include "gui/list_box.h"
#endif

#ifndef CLIENT_EDIT_BOX_H
#include "gui/edit_box.h"
#endif

#ifndef CLIENT_BUTTON_H
#include "gui/button.h"
#endif

#ifndef CLIENT_ENTITY_MANAGER_H
#include "game/entity_manager.h"
#endif

#ifndef DEMO_CLIENT_ENTITY_H
#include "lib/shared/client_entity.h"
#endif

namespace Demo {
namespace Client {


/**
 * Edit entity options.
 */
class EditEntityOptions
    {
public:

    std::string     entity_name;
    std::string     value;

    }; // EditEntityOptions


/**
 * Editor edit entity panel.
 * @note EditorEditEntityPanel automatically sets its size in @c do_layout().
 */
class EditorEditEntityPanel : public PanelBorder, public Button::ButtonMessageReceiver, public ListBox::ListBoxMessageReceiver
    {
    typedef PanelBorder Super;

    NO_COPY_CONSTRUCTOR( EditorEditEntityPanel );
    NO_ASSIGNMENT_OPERATOR( EditorEditEntityPanel );
    
public:

    /** Constructor. */
    EditorEditEntityPanel();
    
    // From Widget.
    virtual void do_layout();
    
    /** Get current options. */
    EditEntityOptions get_edit_entity_options();

    /**
     * Set position coordinates.
     */
    void set_position_coordinates
        (
        Float32     x,
        Float32     y
        );
        
    /**
     * Set entity intersection.
     */
    void set_entity_intersection
        (
        const EntityManager::RayIntersection & entity_intersection
        );

protected:

    virtual bool key_down
        (
        SDLKey  key,
        SDLMod  mod
        );
        
public:

    // From ButtonMessageReceiver.
    virtual void button_clicked
        (
        Button * button
        );

    // From ListBoxMessageReceiver.
    virtual void list_box_item_selected
        (
        ListBox *   list_box
        );

private:
    
    /** 
     * Set value of selected property. 
     */
    void set_property_value();

    /** 
     * Add properties for selected entity to the list box.
     * @param type entity type
     * @param init true when called during initialization 
     */
    void setup_entity_properties_list_box
        (
        const Demo::Lib::ClientEntity::Type     type,
        bool                                    init = true
        );

    /** Setup text labels for selected entity. */
    void setup_edit_entity_text_labels
        (
        const std::string &     entity_name
        );

private:

    std::auto_ptr< Renderer::TextRectangle >    hint_label;

    std::auto_ptr< Renderer::TextRectangle >    entity_name_label;
    std::auto_ptr< EditBox >                    entity_name_edit_box;

    std::auto_ptr< Renderer::TextRectangle >    entity_properties_label;
    std::auto_ptr< ListBox >                    entity_properties_list_box;

    std::auto_ptr< Renderer::TextRectangle >    value_label;
    std::auto_ptr< EditBox >                    value_edit_box;

    std::auto_ptr< Button >                     apply_button;
    std::auto_ptr< Renderer::TextRectangle >    hint_value_label;
    
    Massiv::Core::WeakPointer< Lib::ClientEntity > actual_client_entity;
        
    }; // EditorEditEntityPanel

    
} // namespace Demo::Client
} // namespace Demo


#endif // CLIENT_EDITOR_EDIT_ENTITY_PANEL_H
