/** @file editor_elevation_panel.h
 * Modify elevation panel.
 */

/*
 * begin        : Oct 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_EDITOR_ELEVATION_PANEL_H
#define CLIENT_EDITOR_ELEVATION_PANEL_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef CLIENT_PANEL_BORDER_H
#include "gui/panel_border.h"
#endif

#ifndef CLIENT_LIST_BOX_H
#include "gui/list_box.h"
#endif

#ifndef DEMO_CLIENT_SECTOR_H
#include "lib/shared/client_sector.h"
#endif

#ifndef CLIENT_EDIT_BOX_H
#include "gui/edit_box.h"
#endif

#ifndef CLIENT_BUTTON_H
#include "gui/button.h"
#endif


namespace Demo {
namespace Client {


/**
 * Modify elevation options.
 */
class ModifyElevationOptions
    {
public:
    typedef Lib::ClientSector::ElevationMapKind         MapKind;
    typedef Lib::ClientSector::ElevationMapOperation    MapOperation;
    typedef Lib::ClientSector::ElevationMapFunction     MapFunction;

public:

    MapKind         map;
    MapOperation    operation;
    MapFunction     function;
    Float32         radius;
    Float32         strength;
    bool            align_to_grid;

    }; // ModifyElevationOptions


/**
 * Modify elevation panel.
 */
class ModifyElevationPanel : public PanelBorder
    {
    NO_COPY_CONSTRUCTOR( ModifyElevationPanel );
    NO_ASSIGNMENT_OPERATOR( ModifyElevationPanel );
    
public:

    /** Constructor. */
    ModifyElevationPanel();
    
    /** Get current options. */
    ModifyElevationOptions get_modify_elevation_options();

protected:

    virtual bool key_down
        (
        SDLKey  key,
        SDLMod  mod
        );
    
private:

    /** Setup content of list boxes. */
    void setup_list_boxes();
    
private:

    std::auto_ptr< ListBox >    map_kind_list_box;
    std::auto_ptr< ListBox >    operation_list_box;
    std::auto_ptr< ListBox >    function_list_box;
    std::auto_ptr< Renderer::TextRectangle > radius_label;
    std::auto_ptr< Renderer::TextRectangle > strength_label;
    std::auto_ptr< EditBox >    radius_edit_box;
    std::auto_ptr< EditBox >    strength_edit_box;
    std::auto_ptr< Button >     align_to_grid_button;
    std::auto_ptr< Button >     inverse_strength_button;
    
    }; // ModifyElevationPanel

    
} // namespace Demo::Client
} // namespace Demo


#endif // CLIENT_EDITOR_ELEVATION_PANEL_H
