/** @file editor_material_panel.h
 * Editor material panel.
 */

/*
 * begin        : Nov 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_EDITOR_MATERIAL_PANEL_H
#define CLIENT_EDITOR_MATERIAL_PANEL_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef DEMO_CLIENT_SECTOR_H
#include "lib/shared/client_sector.h"
#endif

#ifndef CLIENT_PANEL_BORDER_H
#include "gui/panel_border.h"
#endif

#ifndef CLIENT_LIST_BOX_H
#include "gui/list_box.h"
#endif

#ifndef CLIENT_EDIT_BOX_H
#include "gui/edit_box.h"
#endif

#ifndef CLIENT_BUTTON_H
#include "gui/button.h"
#endif


namespace Demo {
namespace Client {


/**
 * Modify material options.
 */
class ModifyMaterialOptions
    {
public:
    typedef Lib::ClientSector::MaterialId       MaterialId;

public:

    MaterialId      material_id;
    Float32         radius;
    bool            align_to_grid;

    }; // ModifyMaterialOptions


/**
 * Editor material panel.
 * @note EditorMaterialPanel automatically sets its size in @c do_layout().
 */
class EditorMaterialPanel : public PanelBorder
    {
    typedef PanelBorder Super;

    NO_COPY_CONSTRUCTOR( EditorMaterialPanel );
    NO_ASSIGNMENT_OPERATOR( EditorMaterialPanel );
    
public:

    /** Constructor. */
    EditorMaterialPanel();
    
    // From Widget.
    virtual void do_layout();
    
    /** Get current options. */
    ModifyMaterialOptions get_modify_material_options();
    
    /**
     * Refresh list box containing materials.
     * If material database was not updated from last refresh(),
     * nothing is done.
     */
    void refresh
        (
        bool force = false
        );

protected:

    virtual bool key_down
        (
        SDLKey  key,
        SDLMod  mod
        );
    
private:

    /** Enumerate materials to material list box. */
    void setup_material_list_box();

    /** Setup content of list boxes. */
    void setup_list_boxes();
    
private:

    std::auto_ptr< ListBox >    material_list_box;
    std::auto_ptr< Renderer::TextRectangle > radius_label;
    std::auto_ptr< EditBox >    radius_edit_box;
    std::auto_ptr< Button >     align_to_grid_button;
    
    SystemTime                  last_time_materials_refreshed;
        /**< Last time material list box was refreshed. */
    
    }; // EditorMaterialPanel

    
} // namespace Demo::Client
} // namespace Demo


#endif // CLIENT_EDITOR_MATERIAL_PANEL_H
