/** @file editor_mode_panel.cpp
 * Panel to select editor mode (implementation).
 */

/*
 * begin        : Nov 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "editor/editor_mode_panel.h"
#include "gui/editor.h"
#include "client_global.h"


namespace Demo {
namespace Client {


const Float32 BORDER_WIDTH = 4.0f;
const Float32 BORDER_HEIGHT = 4.0f;


namespace {


void update_max_y
    (
    Float32 &                 max_y,
    Renderer::OverlayObject * object
    )
    {
    const Float32 bottom_y = object->get_current_offset().y() + object->get_current_size().y();
    max_y = ( max_y < bottom_y ? bottom_y : max_y );
    }


} // anonymous namespace


EditorModePanel::EditorModePanel()
    {
    // Init widget.
    set_border_size( BORDER_WIDTH, BORDER_HEIGHT );
    set_dragable( true );
    set_auto_bringable_to_front( true );

    // Create widgets.
    mode_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    mode_label->set_parent_overlay_object( this );
    mode_label->print_line( "Editor mode:" );
    mode_label->set_size_from_font();

    mode_list_box.reset( new ListBox() );
    mode_list_box->set_parent_widget( this );

    // Fill list boxes.
    setup_list_boxes();

    mode_list_box->set_message_receiver( this );
    }

void EditorModePanel::do_layout()
    {
    Super::do_layout();

    const Float32 px = 8.0f; // Padding x.
    const Float32 py = 8.0f; // Padding y.
    Float32 x;
    Float32 y;
    Float32 max_y; // Maximum bottom y coordinate of child widgets.

    x = BORDER_WIDTH + px;
    y = BORDER_HEIGHT + py;
    max_y = y;

    mode_label->set_base_offset( SVector2( x, y ) );
    // Size of mode_label is set in EditorModePanel constructor.
    update_max_y( max_y, mode_label.get() );

    x += mode_label->get_current_size().x() + px;
    mode_list_box->set_base_offset( SVector2( x, y ) );
    // Size of mode_list_box is set in setup_list_boxes().
    update_max_y( max_y, mode_list_box.get() );

    x += mode_list_box->get_current_size().x() + px + BORDER_WIDTH;
    set_base_size( SVector2( x, max_y + py + BORDER_HEIGHT ) );
    }


void EditorModePanel::list_box_item_selected
    (
    ListBox *   list_box
    )
    {
    if( list_box == mode_list_box.get() )
        {
        if( list_box->is_item_selected() )
            {
            const ListBox::Item & item = list_box->get_selected_item();
            const Editor::Mode editor_mode = static_cast< Editor::Mode >( item.id );
            Global::get_editor().set_mode( editor_mode );
            }
        }
    }


void EditorModePanel::setup_list_boxes()
    {
    typedef ListBox::Item Item;
    int id;
    std::string name;

    // Editor modes.

    id = Editor::MODE_NONE;
    name = "None";
    mode_list_box->push( Item( id, name ) );

    id = Editor::MODE_MODIFY_ELEVATION;
    name = "Modify Elevation";
    mode_list_box->push( Item( id, name ) );

    id = Editor::MODE_MODIFY_MATERIAL;
    name = "Modify Material";
    mode_list_box->push( Item( id, name ) );

    id = Editor::MODE_ADD_ENTITY;
    name = "Add Entity";
    mode_list_box->push( Item( id, name ) );

    id = Editor::MODE_EDIT_ENTITY;
    name = "Edit Entity";
    mode_list_box->push( Item( id, name ) );

    id = Editor::MODE_DELETE_ENTITY;
    name = "Delete Entity";
    mode_list_box->push( Item( id, name ) );

    mode_list_box->set_base_size( mode_list_box->get_ideal_size() );
    mode_list_box->do_layout();
    }


} // namespace Demo::Client
} // namespace Demo
