/** @file editor_mode_panel.h
 * Panel to select editor mode.
 */

/*
 * begin        : Nov 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_EDITOR_MODE_PANEL_H
#define CLIENT_EDITOR_MODE_PANEL_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef CLIENT_PANEL_BORDER_H
#include "gui/panel_border.h"
#endif

#ifndef CLIENT_LIST_BOX_H
#include "gui/list_box.h"
#endif

#ifndef CLIENT_BUTTON_H
#include "gui/button.h"
#endif


namespace Demo {
namespace Client {


/**
 * Panel to select editor mode.
 * @note EditorModePanel automatically sets its size in @c do_layout().
 */
class EditorModePanel : public PanelBorder, public ListBox::ListBoxMessageReceiver
    {
    typedef PanelBorder Super;

    NO_COPY_CONSTRUCTOR( EditorModePanel );
    NO_ASSIGNMENT_OPERATOR( EditorModePanel );
    
public:

    /** Constructor. */
    EditorModePanel();
    
    // From Widget.
    virtual void do_layout();
    
public:

    // From ListBoxMessageReceiver.
    virtual void list_box_item_selected
        (
        ListBox *   list_box
        );

private:

    /** Setup content of panel's list boxes. */
    void setup_list_boxes();

private:

    std::auto_ptr< Renderer::TextRectangle >    mode_label;
    std::auto_ptr< ListBox >                    mode_list_box;

    }; // ModifyElevationPanel

    
} // namespace Demo::Client
} // namespace Demo


#endif // CLIENT_EDITOR_MODE_PANEL_H
