/** @file game_context.cpp
 * Game context (implementation).
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "game/game_context.h"
#include "lib/shared/account_object.h"
#include "lib/shared/client_entity.h"


namespace Demo {
namespace Client {


GameContext::GameContext()
    {
    }


bool GameContext::is_linked_with_account() const
    {
    return account != null;
    }


void GameContext::link_with_account
    (
    WeakPointer< Lib::AccountObject > the_account
    )
    {
    account = the_account;
    }


Remote< Lib::AccountObject > GameContext::require_account() const
    {
    if( !is_linked_with_account() )
        {
        panic( "Can't return account object because the client "
            "is not yet linked with its accont object." );
        }
    
    return account;
    }


Remote< Lib::AccountObject > GameContext::get_account() const
    {
    return account;
    }


bool GameContext::is_player_selected() const
    {
    return selected_player != null;
    }
    

void GameContext::player_selected
    (
    WeakPointer< Lib::ClientEntity > the_player
    )
    {
    selected_player = the_player;
    }


WeakPointer< Lib::ClientEntity > GameContext::get_selected_player() const
    {
    return selected_player;
    }


} // namespace Client
} // namespace Demo
