/** @file game_input.h
 * Handler of keyboard input not eaten by GUI.
 */

/*
 * begin        : Dec 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_GAME_INPUT_H
#define CLIENT_GAME_INPUT_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef CLIENT_INPUT_OBJECT_H
#include "input/input_object.h"
#endif

#ifndef CLIENT_MESSAGE_BOX_H
#include "gui/message_box.h"
#endif


namespace Demo {
namespace Client {


/**
 * Handler of keyboard input not eaten by GUI - game input.
 */
class GameInput :
    public InputObject,
    public MessageBox::MessageBoxReceiver
    {
public:

    /**
     * Constructor.
     */
    GameInput();

    /**
     * Destructor.
     */
    virtual ~GameInput();

public:

    /**
     * Did user answer "yes" to quit requester?
     */
    bool quit_requested() const;

protected:

    virtual bool key_up
        (
        SDLKey  key,
        SDLMod  mod
        );

    virtual bool key_down
        (
        SDLKey  key,
        SDLMod  mod
        );

    virtual void message_box_action
        (
        MessageBox * message_box
        );

private:

    /**
     * Create and initialize #help_message.
     */
    void create_help_message();

    /**
     * Create and initialize #credits_message.
     */
    void create_credits_message();

    /**
     * Create and initialize #quit_requester.
     */
    void create_quit_requester();

    /**
     * Create and initialize #disable_rendering_requester.
     */
    void create_disable_rendering_requester();

private:

    std::auto_ptr< MessageBox > help_message;
        /**< Message box with help. */

    std::auto_ptr< MessageBox > credits_message;
        /**< Message box with credits. */

    std::auto_ptr< MessageBox > quit_requester;
        /**< Modal "do you want to quit?" requester. */

    std::auto_ptr< MessageBox > disable_rendering_requester;
        /**< Modal "do you want to disable screen updates?" requester. */

    bool                        should_quit;
        /**< Did user answer "yes" to quit requester? */

    }; // class GameInput


} // namespace Client
} // namespace Demo

#endif // CLIENT_GAME_INPUT_H
