/** @file item_mirror.cpp
 * Mirror of Item object (implementation).
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "game/item_mirror.h"
#include "lib/shared/client_item.h"


namespace Demo {
namespace Client {


ItemMirror::ItemMirror() :
    equipped( false )
    {
    }


bool ItemMirror::is_empty() const
    {
    return ( item_replica == null );
    }


std::string ItemMirror::get_icon_name() const
    {
    return icon_name;
    }


std::string ItemMirror::get_description() const
    {
    if( !item_replica.is_local() )
        {
        return "unknown item";
        }

    std::string desc = item_replica->get_name();
    if( item_replica->is_equipped() )
        {
        desc += " (equipped)";
        }
    else if( item_replica->is_equipable() )
        {
        desc += " (equippable)";
        }
    return desc;
    }


void ItemMirror::update
    (
    WeakPointer< Lib::ClientItem > replica
    )
    {
    if( replica == null || !replica.is_local() )
        {
        clear();
        }
    else
        {
        hope( replica.is_local() );
        item_replica = replica;
        update_icon();
        if( replica->is_equipable() && replica->is_equipped() )
            {
            equipped = true;
            }
        else
            {
            equipped = false;
            }
        }
    }


void ItemMirror::clear()
    {
    item_replica = null;
    icon_name.clear();
    equipped = false;
    }


void ItemMirror::update_icon()
    {
    hope( item_replica.is_local() );
    Lib::ItemType   item_type = item_replica->item_type;
    int             item_variant = item_replica->item_variant;
    bool icon_assigned = false;

    switch( item_type )
        {
        case Lib::ITEM_POTION:
            {
            switch( item_variant )
                {
                case Lib::POTION_HEALTH:
                    icon_name = "item/icon_red_potion";
                    icon_assigned = true;
                    break;

                case Lib::POTION_SPEED:
                    icon_name = "item/icon_green_potion";
                    icon_assigned = true;
                    break;

                case Lib::POTION_MEAT:
                    icon_name = "item/icon_meat";
                    icon_assigned = true;
                    break;
                }
            break;
            }

        case Lib::ITEM_GARBAGE:
            {
            switch( item_variant )
                {
                case Lib::GARBAGE_FLASK:
                    icon_name = "item/icon_potion_flask";
                    icon_assigned = true;
                    break;

                case Lib::GARBAGE_BONE:
                    icon_name = "item/icon_bone";
                    icon_assigned = true;
                    break;
                }
            break;
            }

        case Lib::ITEM_BOOTS:
            {
            icon_name = "item/icon_magic_boots";
            icon_assigned = true;
            break;
            }

        case Lib::ITEM_WEAPON:
            {
            switch( item_variant )
                {
                case Lib::WEAPON_AXE:
                    icon_name = "item/icon_weapon_axe";
                    icon_assigned = true;
                    break;
                }
            break;
            }

        case Lib::ITEM_UNKNOWN:
            break;
        }

    if( !icon_assigned )
        {
        icon_name = "item/icon_unknown";
        }
    }


} // namespace Demo::Client
} // namespace Demo
