/** @file item_mirror.h
 * Mirror of Item object.
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_ITEM_MIRROR_H
#define CLIENT_ITEM_MIRROR_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SHARED_FWD_H
#include "lib/shared/shared_fwd.h"
#endif


namespace Demo {
namespace Client {

INJECT_CORE_SYMBOLS();


/**
 * Mirror of Item object.
 */
class ItemMirror
    {
public:

    /** Constructor. */
    ItemMirror();
    
public:

    bool is_empty() const;

public:

    /**
     * Get icon template name of the item.
     */
    std::string get_icon_name() const;
    
    /**
     * Get simple description of the item.
     */
    std::string get_description() const;

public:

    /** Update the mirror by @a replica. */
    void update
        (
        WeakPointer< Lib::ClientItem > replica
        );
    
    /** Clear content of the mirror. */
    void clear();
    
private:

    /** Update @a icon_name from @a item_replica. */
    void update_icon();

public:
    
    WeakPointer< Lib::ClientItem >  item_replica;
    std::string                     icon_name;
    bool                            equipped;

    }; // ItemMirror


} // namespace Demo::Client
} // namespace Demo

#endif // CLIENT_ITEM_MIRROR_H
