/** @file item_slot_container_mirror.cpp
 * Mirror of ItemSlotContainer object (implementation).
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "game/item_slot_container_mirror.h"
#include "lib/shared/client_item_slot_container.h"

namespace Demo {
namespace Client {


ItemSlotContainerMirror::ItemSlotContainerMirror() :
    slot_columns( 0 ),
    slot_rows( 0 )
    {
    }


void ItemSlotContainerMirror::update
    (
    WeakPointer< Lib::ClientItemSlotContainer > replica
    )
    {
    if( !replica.is_local() )
        {
        clear();
        return;
        }
    
    if( !replica->is_consistent() )
        {
        std::ostringstream oss;
        oss << "Received inconsistent replica of player's inventory.";
        Massiv::Core::Global::log_info( Status::FACILITY_GENERIC, Status::PRIORITY_NORMAL, oss.str() );
        
        // Leav previous state.
        return;
        }
    
    // Verify the replica.
    if( !verify_replica( replica ) )
        {
        std::ostringstream oss;
        oss << "Verification of player's inventory replica failed. Clearing inventory mirror.";
        Massiv::Core::Global::log_warning( Status::FACILITY_GENERIC, Status::PRIORITY_HIGH, oss.str() );
        
        clear();
        }
    else
        {
        // Update the container.
        do_update( replica );
        }
    }


void ItemSlotContainerMirror::clear()
    {
    slot_columns = 0;
    slot_rows = 0;
    slots.clear();
    }


bool ItemSlotContainerMirror::is_valid_slot_index
    (
    int slot_index
    ) const
    {
    unsigned int size = static_cast< unsigned int >( slot_columns * slot_rows );
    hope( slots.size() == size );
    return ( slot_index >= 0 && ( slot_index < ( slot_columns * slot_rows ) ) );
    }


void ItemSlotContainerMirror::do_update
    (
    WeakPointer< Lib::ClientItemSlotContainer > replica
    )
    {
    try 
        {
        int i;
        slot_columns = replica->slot_columns;
        slot_rows    = replica->slot_rows;
        slots.resize( slot_columns * slot_rows );
        hope( slots.size() == replica->slots.size() );
        
        for( i = 0; i < ( slot_columns * slot_rows ); ++i )
            {
            slots[ i ].update( replica->slots[ i ] );
            }
        }
    catch( ... )
        {
        clear();
        throw;
        }
    }


bool ItemSlotContainerMirror::verify_replica
    (
    WeakPointer< Lib::ClientItemSlotContainer > replica
    )
    {
    if( replica->slot_columns < 0 || replica->slot_rows < 0 )
        {
        return false;
        }
    
    if( replica->slot_columns == 0 && replica->slot_rows != 0 )
        {
        return false;
        }
    
    if( replica->slot_columns != 0 && replica->slot_rows == 0 )
        {
        return false;
        }
    
    unsigned int size = static_cast< unsigned int >( replica->slot_columns * replica->slot_rows );
    if( replica->slots.size() != size )
        {
        return false;
        }
    
    return true;
    }


} // namespace Demo::Client
} // namespace Demo
