/** @file item_slot_container_mirror.h
 * Mirror of ItemSlotContainer object.
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_ITEM_SLOT_CONTAINER_MIRROR_H
#define CLIENT_ITEM_SLOT_CONTAINER_MIRROR_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef CLIENT_ITEM_MIRROR_H
#include "game/item_mirror.h"
#endif

#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SHARED_FWD_H
#include "lib/shared/shared_fwd.h"
#endif

#include <vector>


namespace Demo {
namespace Client {

INJECT_CORE_SYMBOLS();


/**
 * Mirror of ItemSlotContainer object.
 */
class ItemSlotContainerMirror
    {
public:
    typedef std::vector< ItemMirror > Slots;
    
public:
    
    /** Constructor. */
    ItemSlotContainerMirror();
    
public:

    /** Update the mirror by @a replica. */
    void update
        (
        WeakPointer< Lib::ClientItemSlotContainer > replica
        );
    
    /** Clear content of the mirror. */
    void clear();

public:

    /**
     * Check whether @a slot_index is valid slot index
     * into @a slots array.
     */
    bool is_valid_slot_index
        (
        int slot_index
        ) const;

private:

    /** Update the mirror by verified @a replica. */
    void do_update
        (
        WeakPointer< Lib::ClientItemSlotContainer > replica
        );
    
    /** Verify content of the @a replica. */
    bool verify_replica
        (
        WeakPointer< Lib::ClientItemSlotContainer > replica
        );

public:
    int     slot_columns;
        /**< Number of slot columns. Zero if there are no slots. */

    int     slot_rows;
        /**< Number of slot rows. Zero if there are no slots. */

    Slots   slots;
        /**< Slots containing mirrors of Item objects. Size is always
         * equal to @a slot_columns * @a slot_rows.
         */

    }; // ItemSlotContainerMirror


} // namespace Demo::Client
} // namespace Demo

#endif // CLIENT_ITEM_SLOT_CONTAINER_MIRROR_H
