/** @file sector_manager_inline.h
 * Manager of sector replicas and terrains (inline methods).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "utility/hash_function.h"

namespace Demo {
namespace Client {


// class SectorManager::SectorAddress

std::size_t SectorManager::SectorAddress::Hash::operator()
    (
    const SectorAddress & address
    ) const throw()
    {
    return ext::hash_int( address.x ) ^ ext::hash_int( address.y * 1021 ) ^
        ObjectId::Hash()( address.layer_id );
    }


SectorManager::SectorAddress::SectorAddress()
      : x( 0 ),
        y( 0 )
    {
    }


SectorManager::SectorAddress::SectorAddress
    (
    const ObjectId &    the_layer_id,
    const int           the_x,
    const int           the_y
    ) : layer_id( the_layer_id ),
        x( the_x ),
        y( the_y )
    {
    }


SectorManager::SectorAddress::~SectorAddress()
    {
    }


bool SectorManager::SectorAddress::operator==
    (
    const SectorAddress & other
    ) const
    {
    return ( layer_id == other.layer_id ) && ( x == other.x ) && ( y == other.y );
    }


// class SectorManager::SectorInfo

SectorManager::SectorInfo::SectorInfo()
    {
    }


SectorManager::SectorInfo::~SectorInfo()
    {
    terrain.reset();
    heightmap.reset();
    }


SectorManager::SectorInfo::SectorInfo
    (
    const SectorInfo & other
    ) : client_sector( other.client_sector ),
        address( other.address ),
        heightmap( other.heightmap ),
        terrain( other.terrain )
    {
    }


// class SectorManager

const SectorManager::ObjectId & SectorManager::get_visible_layer() const
    {
    return visible_layer;
    }


} // namespace Demo::Client
} // namespace Demo
