/** @file blocker_widget.cpp
 * Click-blocking widget used to implement modal dialog hacks (implementation).
 */

/*
 * begin        : Dec 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "client_global.h"
#include "gui/blocker_widget.h"
#include "renderer/renderer.h"


namespace Demo {
namespace Client {


BlockerWidget::BlockerWidget
    (
    Float32 the_opacity
    ) : opacity( the_opacity )
    {
    set_dispatcher_as_parent();
    }


BlockerWidget::~BlockerWidget()
    {
    }


void BlockerWidget::set_opacity
    (
    Float32 new_opacity
    )
    {
    opacity = new_opacity;
    }


void BlockerWidget::render()
    {
    // Get standard overlay pipeline state and enable texturing.

    Renderer::PipelineState & state = get_pipeline_state();
    state.color = get_current_color();
    state.color.set_alpha( state.color.alpha() * opacity );
    Renderer::Global::apply_state( state );

    // Render the icon.

    OpenGL::Vector2 top_left = screenspace_translation();
    OpenGL::Vector2 bottom_right( top_left );
    bottom_right.v[ 0 ] += get_current_size().x();
    bottom_right.v[ 1 ] += get_current_size().y();

    glBegin( GL_QUADS );
    glVertex2f( top_left.x(), top_left.y() );
    glVertex2f( top_left.x(), bottom_right.y() );
    glVertex2f( bottom_right.x(), bottom_right.y() );
    glVertex2f( bottom_right.x(), top_left.y() );
    glEnd();

    // Revert global state.

    state.color = OpenGL::Color( 1.0f, 1.0f, 1.0f, 1.0f );
    }

    
} // namespace Demo::Client
} // namespace Demo
