/** @file blocker_widget.h
 * Click-blocking widget used to implement modal dialog hacks.
 */

/*
 * begin        : Dec 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_BLOCKER_WIDGET_H
#define CLIENT_BLOCKER_WIDGET_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef CLIENT_WIDGET_H
#include "gui/widget.h"
#endif


namespace Demo {
namespace Client {


/**
 * Simple alpha blended rectangle that eats all clicks.
 *
 * Use #set_base_color() method from #OverlayObject to modify blocker object,
 * but do not set alpha to zero - clicks would pass through the blocker
 * in that case. Use #set_opacity() instead of color alpha.
 */
class BlockerWidget : public Widget
    {
    NO_COPY_CONSTRUCTOR( BlockerWidget );
    NO_ASSIGNMENT_OPERATOR( BlockerWidget );

    /// Superclass.
    typedef Widget Super;

public:

    /**
     * Constructor.
     *
     * Automatically registers global input dispatcher as parent.
     *
     * @param opacity Blocker opacity. Even if it's zero, blocker will still
     *                eat all clicks (unlike zero alpha from OverlayObject::color).
     */
    explicit BlockerWidget
        (
        Float32 opacity = 0.0f
        );

    /**
     * Destructor.
     */
    virtual ~BlockerWidget();

public:

    /**
     * Set blocker opacity.
     */
    void set_opacity
        (
        Float32 opacity
        );

public:

    /**
     * Render blocker.
     */
    virtual void render();

private:

    Float32 opacity;
        /**< Alpha multiplier. */

    }; // class BlockerWidget


} // namespace Demo::Client
} // namespace Demo


#endif // CLIENT_BLOCKER_WIDGET_H
