/** @file camera_control.h
 * Camera control.
 */

/*
 * begin        : Fri Nov 14 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_CAMERA_CONTROL_H
#define CLIENT_CAMERA_CONTROL_H

#include "client_common.h"
#include "input/input_manager.h"
#include "renderer/camera.h"

namespace Demo {
namespace Client {


/**
 * Circling camera control.
 */
class CameraControl
    {
public:

    /** Constructor. */
    CameraControl();
    
public:

    /**
     * Updates camera state. Must be called whenever arrow key that controls
     * the camera is pressed/released. The implementation accesses
     * InputManager directly to get control input.
     */
    void next_tick();

    /**
     * Applies current control state to a given @c camera.
     * Actual camera longitude will be a sum of @c longitude and @c longitude
     * offset.
     */
    void apply_control
        (
        Renderer::CirclingCamera &  camera,
        Float32                     longitude_offset = 0.0f
        ) const;

private:

    /**
     * Determine how long the key described by given @c state was pressed in
     * the current tick.
     */
    Float32 get_key_press_duration
        (
        const InputManager::KeyState &  state,
        const SystemTime &              current_time
        );

private:
        
    Float32     longitude;
        /**< Longitude. */
    Float32     longitude_speed;
        /**< Longitude change speed. */
    Float32     longitude_acceleration;
        /**< Longitude change acceleration. */
    Float32     distance;
        /**< Distance. */
    Float32     distance_speed;
        /**< Distance change speed. */
    Float32     distance_acceleration;
        /**< Distance change acceleration. */
    Float32     latitude;
        /**< Latitude. */
    SystemTime  last_tick;
        /**< When the last tick was finished. */

    }; // CameraControl


} // namespace Client
} // namespace Demo

#endif // CLIENT_CAMERA_CONTROL_H
