/** @file client_chat_console.cpp
 * Chat console implementation.
 */


/*
 * begin        : Thu Mar 13 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "client_global.h"
#include "gui/client_chat_console.h"
#include "sdl/client_sdl.h"
#include "renderer/layers.h"
#include "gui/widget_input_dispatcher.h"
#include "input/input_manager.h"
#include "renderer/font.h"
#include <cstring>
#include <sstream>

namespace Demo {
namespace Client {


// Implementation of ChatConsole
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
ChatConsole::ChatConsole()
      : Console()
    {
    loose_focus_on_submit = true;
    set_focusable( CLICK_FOCUSABLE );
    chat_console_height = Global::chat_console_height;
    }


void ChatConsole::do_layout()
    {
    redraw_display = true;
    redraw_command = true;
    std::string command = prompt + current_line;
 
    const int rows =
            (
            command.size() ?
                ( command.size() + num_columns ) / num_columns
                :
                1
            );
    
    hope( rows > 0 );
    
    if( chat_console_height < 3 + rows - 1 )
        {
        chat_console_height = 3 + rows - 1;
        }
        
    num_lines = chat_console_height + 1; // Plus command line.
    
    set_base_size
        (
        SVector2
            ( 
            static_cast< Float32 >( SDL::get_screen_width() ),
            static_cast< Float32 >( ( chat_console_height + 1 ) * cell_height )
            )
        );

    set_base_offset
        (
        SVector2
            (
            0.0f, 
            static_cast< Float32 >( SDL::get_screen_height() - ( chat_console_height + 1 ) * cell_height )
            ) 
        );
        
    panel_border->set_base_size
        ( 
        get_base_size()
        );

    sb->set_base_size
        (
        SVector2
            ( 
            static_cast< Float32 > ( 2 * cell_width ),
            static_cast< Float32 > ( panel_border->get_base_size().y() - 2 * cell_height )
            )
        );
        
    sb->do_layout();
    
    const SColor color_speed( 1.0f, 1.0f, 1.0f, 1.0f );
    const SColor initial_factor( 1.0f, 1.0f, 1.0f, 0.0f );

    Renderer::Font & font = Global::get_console_font();

    display_text.reset( new Renderer::TextRectangle( &font, num_columns, chat_console_height ) );    

    display_text->set_base_offset
        ( 
        SVector2( 0.0f, static_cast< Float32 >( cell_height ) ) 
        );
        
    display_text->add_animation_object( new Renderer::OverlayOffsetAnimation
        (
        SVector2( 1.0f, static_cast< Float32 >( SDL::get_screen_height() ) ),
        display_text->get_base_offset()
        ) );
    display_text->add_animation_object( new Renderer::OverlayColorFactorAnimation
        ( color_speed, initial_factor ) );
    display_text->set_parent_overlay_object( this );
            
    command_text->set_base_offset
        ( SVector2( 0.0f, static_cast< Float32 >( ( chat_console_height - rows ) * cell_height ) ) );
        
    }


void ChatConsole::initialize()
    {
    hope( Global::chat_console_height >= 4 );
    Renderer::Font & font = Global::get_console_font();

    // Compute sizes.
    
    prompt = "Enter message : ";

    font.get_texture(); // initializes font info as a side effect
    cell_width = font.get_character_width();
    cell_height = font.get_character_height();
    num_columns = ( SDL::get_screen_width() - 2 * cell_width ) / cell_width;
    
    
    UInt16 max_height = static_cast< UInt16 >( SDL::get_screen_height() / cell_height );
    
    if( Global::chat_console_height > max_height )
        {
        Global::chat_console_height = max_height;
        }
        
    num_lines = Global::chat_console_height + 1; // Plus command line.

    set_base_size
        (
        SVector2
            ( 
            static_cast< Float32 >( SDL::get_screen_width() ),
            static_cast< Float32 >( ( Global::chat_console_height + 1 ) * cell_height )
            )
        );
        
    set_base_offset
        (
        SVector2
            (
            0.0f, 
            static_cast< Float32 >( SDL::get_screen_height() - ( Global::chat_console_height + 1 ) * cell_height )
            ) 
        );
    
        
    // Create overlay objects.

    const SColor color_speed( 1.0f, 1.0f, 1.0f, 1.0f );
    const SColor initial_factor( 1.0f, 1.0f, 1.0f, 0.0f );

    add_animation_object( new Renderer::OverlayOffsetAnimation
        (
        SVector2( 1.0f, static_cast< Float32 >( SDL::get_screen_height() ) ),
        SVector2( 0.0f, 0.0f )
        ) );
    add_animation_object( new Renderer::OverlayColorFactorAnimation
        ( color_speed, initial_factor ) );
    
    // Create panels.

    panel_border.reset( new PanelBorder
        (
        "gui/panel_corner",
        "gui/panel_hborder",
        "gui/panel_vborder",
        "gui/panel_background",
        0.0f,
        48.0f
        ) );
        
    panel_border->set_base_size
        ( 
        get_base_size()
        );
    panel_border->set_parent_widget( this );

    panel_border->set_base_offset
        ( 
        SVector2( 0.0f, 0.0f )
        );
                
    panel_border->add_animation_object( new Renderer::OverlayColorFactorAnimation
        ( color_speed, initial_factor ) );
        
    panel_border->set_type( Widget::TRANSPARENT_WIDGET );
    
    //scroll bar
    sb.reset( new ScrollBar( ScrollBar::VERTICAL ) );
    sb->set_base_offset
        ( 
        SVector2
            ( 
            static_cast< Float32 > ( SDL::get_screen_width() - 2 * cell_width ),
            static_cast< Float32 > ( cell_height ) 
            ) 
        );
    sb->set_base_size
        (
        SVector2
            ( 
            static_cast< Float32 > ( 2 * cell_width ), 
            static_cast< Float32 > ( panel_border->get_base_size().y() - 2 * cell_height )
            )
        );
    sb->do_layout();
    sb->set_scroll_range( ScrollBar::ScrollRange( 0, 100 ) );
    sb->set_scroll_position( 0 );
    sb->set_scroll_line_size( 1 );
    sb->set_scroll_page_size( 10 );
    sb->set_parent_widget( this );
    sb->set_message_receiver( this );
    
    // Create rectangles.

    display_text.reset( new Renderer::TextRectangle( &font, num_columns, Global::chat_console_height ) );

    display_text->set_base_offset
        ( 
        SVector2( 0.0f, static_cast< Float32 >( cell_height ) ) 
        );
        
    display_text->add_animation_object( new Renderer::OverlayOffsetAnimation
        (
        SVector2( 1.0f, static_cast< Float32 >( SDL::get_screen_height() ) ),
        display_text->get_base_offset()
        ) );
    display_text->add_animation_object( new Renderer::OverlayColorFactorAnimation
        ( color_speed, initial_factor ) );
    display_text->set_parent_overlay_object( this );
    
    if( Global::chat_console_height < 3 )
        { 
        Global::chat_console_height = 3;
        }
        
    command_text.reset( new Renderer::TextRectangle( &font, num_columns ) );
    command_text->set_base_offset
        ( SVector2( 0.0f, static_cast< Float32 >( ( Global::chat_console_height - 1 ) * cell_height ) ) );

    command_text->add_animation_object( new Renderer::OverlayColorFactorAnimation
        ( color_speed, initial_factor ) );
    command_text->set_parent_overlay_object( this );

    cursor_text.reset( new Renderer::TextRectangle( &font ) );
    cursor_text->set_base_offset
        ( SVector2( 0.0f, static_cast< Float32 >( ( Global::chat_console_height - 1 ) * cell_height ) ) );
    cursor_text->add_animation_object( new Renderer::OverlayOffsetAnimation
        (
        SVector2
            ( 
            static_cast< Float32 >( SDL::get_screen_width() ),
            static_cast< Float32 >( SDL::get_screen_height() ) 
            ),
        cursor_text->get_base_offset()
        ) );
    cursor_text->add_animation_object( new Renderer::OverlayColorFactorAnimation
        ( color_speed, initial_factor ) );
    cursor_text->print( 0, 0, "_" );
    cursor_text->set_parent_overlay_object( this );

    clear_display();
    
    redraw_display = true;
    redraw_command = true;
    }


void ChatConsole::redraw_animations
    (
    int rows
    )
    {
    // Animate alpha.

    const SColor target_color_factor( 1.0f, 1.0f, 1.0f, ( should_be_visible ) ? 1.0f : 0.0f );
    
    require_color_factor_animation().set_target_factor( target_color_factor );
    
    display_text->require_color_factor_animation().set_target_factor( target_color_factor );
    panel_border->require_color_factor_animation().set_target_factor( target_color_factor );
        
    command_text->require_color_factor_animation().set_target_factor( target_color_factor );
    cursor_text->require_color_factor_animation().set_target_factor( target_color_factor );

    // Animate positions.

    Float32 command_y = get_base_size().y();
    Float32 chat_y = 0.0f;
    Float32 position = 0.0f;
    
    if( should_be_visible )
        {
        command_y = get_base_size().y() - 2 * cell_height - 2;
        
        chat_y = static_cast< Float32 >
            ( 
            -cell_height * ( command_text->get_rows() - 1 ) + cell_height
            );
        
        position = static_cast< Float32 >
            ( 
            SDL::get_screen_height() - ( chat_console_height + 1 ) * cell_height 
            );
        }
    else
        {
        position = static_cast< Float32 >( SDL::get_screen_height() );
        }
    
    require_offset_animation().set_target_offset( SVector2( 0.0f, position ) );
    display_text->require_offset_animation().set_target_offset( SVector2( 0.0f, chat_y ) );
    
    // Cursor position.
    
    int cursor_offset;
    
    cursor_offset = cursor_position + prompt.size();
    
    const int printable_rows = ( rows > num_lines - 2 ) ? num_lines - 2: rows;

    const int cursor_row = cursor_offset / command_text->get_columns() - ( rows - printable_rows );
    const int cursor_column = cursor_offset % command_text->get_columns();

    if( has_focus() )
        {
        cursor_text->require_offset_animation().set_target_offset
            (
            SVector2
                (
                static_cast< Float32 >( cell_width * cursor_column ),
                static_cast< Float32 >( command_text->get_current_offset().y() + cursor_row * cell_height )
                )
            );
        }
    else
        {
        cursor_text->require_offset_animation().set_target_offset
            (
            SVector2
                (
                static_cast< Float32 >( cell_width * cursor_column ),
                static_cast< Float32 >( get_current_size().y() )
                )
            );
        }
    }


void ChatConsole::print_welcome()
    {
    ColorString color_str;
    
    color_str.set_color( SColor( 1.0f, 0.8f, 0.2f ) );
    color_str = "Welcome to Massiv! Press ";
    color_str.set_color( SColor( 0.2f, 1.0f, 0.4f ) );
    color_str += "t";
    color_str.set_color( SColor( 1.0f, 0.8f, 0.2f ) );
    color_str += " to enter chat console, ";
    color_str.set_color( SColor( 0.2f, 1.0f, 0.4f ) );
    color_str += "h";
    color_str.set_color( SColor( 1.0f, 0.8f, 0.2f ) );
    color_str += " for help.";
    print( color_str );
    }


bool ChatConsole::top_border_click
    (
    )
    {
    SVector2 position = Global::get_input_manager().get_mouse_position(); 
    if( ( position.y() >= get_base_offset().y() ) && 
        ( position.y() <= get_base_offset().y() + cell_height ) 
        )
        {
        return true;
        }
    return false;
    };
    
    
bool ChatConsole::mouse_move
    (
    int     dx,
    int     dy,
    SDLMod  mod
    )
    {
    if( resizing )
        {
        SVector2 position = Global::get_input_manager().get_mouse_position();
        chat_console_height = static_cast< Int32>
            ( 
            Massiv::Core::FloatTraits< Float32 >::round
                (  
                ( SDL::get_screen_height() - cell_height - position.y() ) / cell_height 
                ) 
            );
        do_layout();
        return true;
        }
    return false;
    }

} // namespace Client
} // namespace Demo
