/** @file color_string.h
 * Color string implementation.
 */

/*
 * begin        : Wed Sep 03 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef COLOR_STRING_H
#define COLOR_STRING_H


#ifndef MASSIV_COMMON_H
#include "client_common.h"
#endif

#include <string>
#include <map>


namespace Demo {
namespace Client {

/**
 * Used in console to print colored lines.
 * The idea is to have a normal text to write( std::string ) and
 * a map of colors to specified positions in text.
 * The work is similar to work with std::string, some methods are
 * only implemented.
 */
class ColorString
    {
public:

    /**
     * Default constructor.
     */
    ColorString();

    /**
     * Constructor.
     */
    ColorString
        (
        const std::string & str
        );
    
    /**
     * Copy constructor.
     */
    INLINE ColorString
        (
        const ColorString & other
        );
        
    /**
     * Default destructor.
     */
    ~ColorString();

public:

    typedef std::map
        <
        unsigned int, 
        SColor, 
        std::less<unsigned int> 
        > ColorMap;
    /**< Map for storing colors and positions. */ 
        
public:
    /**
     * Assignment operator.
     */
    INLINE ColorString & operator=
        (
        const ColorString & other
        );
        
    /**
     * Assignment operator.
     */
    INLINE ColorString & operator=
        (
        const std::string & str
        );

    /**
     * Operator+=.
     */
    INLINE ColorString & operator+=
        (
        const ColorString & other
        );
        
    /**
     * Operator+=.
     */
    INLINE ColorString & operator+=
        (
        const std::string & str
        );
        
    /**
     * Operator+.
     */
    INLINE ColorString operator+
        (
        const ColorString & other
        );
        
    /**
     * Operator+.
     */
    INLINE ColorString operator+
        (
        const std::string & other
        );

public:
    /**
     * Return size of the color string.
     */
    INLINE unsigned int size();
    
    /**
     * Substring similar to std::string's substring.
     */
    INLINE ColorString substr
        (
        unsigned int startpos,
        int          length = -1
        );
    
    /**
     * Getter for color map.
     */
    INLINE const ColorMap & get_color_map() const;
    
    /**
     * Getter for included string.
     */
    INLINE const std::string & to_string() const;
    
    /**
     * Sets new color for specified position.
     */
    INLINE void set_color
        (
        SColor       color,
        int          position = -1
        );
    
private:
    std::string string;
        /**< String to store text to. */ 
    
    ColorMap color_map;
        /**< Map of color positions. */ 
        
    }; // class ColorString


} // namespace Client
} // namespace Demo


#ifdef ENABLE_INLINE
#include "color_string_inline.h"
#endif


#endif // COLOR_STRING_H
