/** @file color_string_inline.h
 * Color string inline implementation.
 */

/*
 * begin        : Wed Sep 03 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo{
namespace Client{


INLINE ColorString::ColorString
    (
    const ColorString & other
    )
    {
    string = other.string;
    color_map.clear();
    
    ColorMap::const_iterator it;
    it = other.color_map.begin();
    while( it != other.color_map.end() )
        {
        color_map.insert( ColorMap::value_type( it->first, it->second ) ); 
        ++it;
        }
    }


INLINE ColorString & ColorString::operator=
    (
    const ColorString & other
    )
    {
    string = other.string;
    
    color_map.clear();
    ColorMap::const_iterator it = other.color_map.begin();
    while( it != other.color_map.end() )
        {
        color_map.insert( ColorMap::value_type( it->first, it->second ) ); 
        ++it;
        }
    return *this;
    }

        
INLINE ColorString & ColorString::operator=
    (
    const std::string & str
    )
    {
    string = str;
    
    if( !color_map.size() )
        color_map.insert( ColorMap::value_type( 0 , SColor( 1.0f, 1.0f, 0.5f ) ) ); 
    return *this;
    }
        

INLINE ColorString & ColorString::operator+=
    (
    const ColorString & other
    )
    {
    int old_size = string.size();
    
    string += other.string;
    
    ColorMap::const_iterator it = other.color_map.begin();
    while( it != other.color_map.end() )
        {
        color_map.insert( ColorMap::value_type( it->first + old_size, it->second ) ); 
        ++it;
        }
    return *this;
    }


INLINE ColorString & ColorString::operator+=
    (
    const std::string & str
    )
    {
    string += str;
    return *this;
    }
 

INLINE ColorString ColorString::operator+
    (
    const ColorString & other
    )
    {
    ColorString str( *this );
    str += other;
    return str;
    }
         

INLINE ColorString ColorString::operator+
    (
    const std::string & other
    )
    {
    ColorString str( *this );
    str += other;
    return str;
    }
 

INLINE unsigned int ColorString::size()
    {
    return string.size();
    }

    
INLINE const std::string & ColorString::to_string() const
    {
    return string;
    }
    

INLINE void ColorString::set_color
    (
    SColor      color,
    int         position
    )
    {
    if( position < 0 )
        {
        position = string.size();
        }
        
    ColorMap::const_iterator it = color_map.find( position );
    if( it != color_map.end() )
        {
        color_map.erase( position );
        } 
    color_map.insert( ColorMap::value_type( position, color ) ); 
    }
    
 
INLINE const ColorString::ColorMap & ColorString::get_color_map() const
    {
    return color_map;
    }    
    

INLINE ColorString ColorString::substr
    (
    unsigned int startpos,
    int          length
    )
    {
    hope( color_map.size() );
    
    ColorString str( *this );
    SColor last_color;
    
    // Check input params.
    if( startpos >= string.size() )
        {
        str.string = "";
        str.color_map.clear();
        color_map.insert( ColorMap::value_type( 0 , SColor( 1.0f, 1.0f, 0.5f ) ) );
        return str; 
        }
    
    // Should I use the rest of the string?
    if( length < 0 || startpos + length > str.string.size() )
        {
        length = str.string.size() - startpos;
        }
    
    str.string = str.string.substr( startpos, length );
    ColorMap::const_iterator it = str.color_map.begin();
    ColorMap::const_iterator pom;
    
    // Remove mapping to non-existing string parts.
    while( it != str.color_map.end() )
        {
        pom = it;
        if( it->first < startpos )
            {
            last_color = it->second;
            }
        
        unsigned int old_pos = it->first;
        SColor color = it->second;
        unsigned int new_pos = old_pos - startpos;
        it++;
        if( ( new_pos != old_pos  ) || ( new_pos >= str.string.size() ) )
            {
            str.color_map.erase( pom->first );
            }
        if( new_pos < str.string.size() && new_pos != old_pos )
            {
            str.color_map.insert( ColorMap::value_type( new_pos, color ) );
            }

        }
    
    // Let's see, whether initial color exists.
    it = str.color_map.find( 0 );
    if( it == str.color_map.end() )
        {
        str.color_map.insert( ColorMap::value_type( 0, last_color ) );
        }
    return str;
    } 

    
} // namespace Demo
} // namespace Client
