/** @file icon_button.cpp
 * Icon button (implementation).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "gui/icon_button.h"
#include "client_global.h"
#include "media_repository.h"


namespace Demo {
namespace Client {



IconButton::IconButton
    (
    ButtonType          the_type,
    const std::string & the_button_up_icon,
    const std::string & the_button_down_icon,
    const std::string & the_button_up_mouse_on_icon,
    const std::string & the_button_down_mouse_on_icon
    ) :
    Super( the_type )
    {
    MediaRepository & media = Global::get_media_repository();
    
    // Note: We do not register the icons as child overlay objects
    // so we don't need to care which one should be visible or not
    // during rendering.
    up_icon.reset( media.create_icon( the_button_up_icon ) );
    down_icon.reset( media.create_icon( the_button_down_icon ) );
    up_mouse_on_icon.reset( media.create_icon( the_button_up_mouse_on_icon ) );
    down_mouse_on_icon.reset( media.create_icon( the_button_down_mouse_on_icon ) );
    }


void IconButton::render()
    {
    Renderer::Icon *    icon = NULL;
    bool                mouse_on = is_mouse_on();
    
    if( is_down() )
        {
        icon = mouse_on ? down_mouse_on_icon.get() : down_icon.get();
        }
    else
        {
        icon = mouse_on ? up_mouse_on_icon.get() : up_icon.get();
        }
    
    if( icon )
        {
        // We can't set icon size and color only if 'size_dirty' or
        // 'color_dirty' is true, because the size and color are set only for
        // current icon. Same for base offset.
        icon->set_origin( OverlayObject::TOP_LEFT );
        icon->set_base_offset( get_top_left_corner_on_screen() );
        icon->set_base_size( get_current_size() );
        icon->set_base_color( get_current_color() );
        icon->render();
        }
    }
    
    
} // namespace Demo::Client
} // namespace Demo


#ifndef ENABLE_INLINE
#include "gui/icon_button_inline.h"
#endif
