/** @file icon_button.h
 * Icon button.
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_ICON_BUTTON_H
#define CLIENT_ICON_BUTTON_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef CLIENT_BUTTON_H
#include "gui/button.h"
#endif

#ifndef CLIENT_ICON_H
#include "renderer/icon.h"
#endif

#include <memory>


namespace Demo {
namespace Client {


/**
 * Simple icon button.
 * IconButton is #Button.
 * IconButton uses four icons for drawing - first when the button is up,
 * second when the button is down, third when the button is up and mouse
 * is above the button, fourth when the button is down and mouse is above
 * the button.
 * Default rendering of #Button is ignored by IconButton.
 *
 */
class IconButton : public Button
    {
    typedef Button  Super;
    
    NO_COPY_CONSTRUCTOR( IconButton );
    NO_ASSIGNMENT_OPERATOR( IconButton );

public:

    /**
     * Constructor.
     * Parameters are names of icon templates from media repository.
     */
    IconButton
        (
        ButtonType          type,
        const std::string & button_up_icon_name,
        const std::string & button_down_icon_name,
        const std::string & button_up_mouse_on_icon_name,
        const std::string & button_down_mouse_on_icon_name
        );

public:

    // Renderer::OverlayObject
    virtual void render();
    
private:

    std::auto_ptr< Renderer::Icon >     up_icon;
        /**< Icon to draw when the button is up. */

    std::auto_ptr< Renderer::Icon >     down_icon;
        /**< Icon to draw when the button is down. */

    std::auto_ptr< Renderer::Icon >     up_mouse_on_icon;
        /**< Icon to draw when the button is up and mouse is above the button. */
        
    std::auto_ptr< Renderer::Icon >     down_mouse_on_icon;
        /**< Icon to draw when the button is down and mouse is above the button. */

    }; // Button


} // namespace Demo::Client
} // namespace Demo


#ifdef ENABLE_INLINE
#include "gui/icon_button_inline.h"
#endif


#endif // CLIENT_ICON_BUTTON_H
