/** @file login_dialog.h
 * Login dialog.
 */

/*
 * begin        : Wed Jul 9 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "core.h"
#include "data/data_manager.h"
#include "data/path_manager.h"
#include "data/account_manager.h"
#include "gui/login_dialog.h"
#include "renderer/passes.h"
#include "renderer/layers.h"
#include "input/input_layers.h"
#include "client_global.h"
#include "gui/widget_input_dispatcher.h"
#include <sstream>
#include <fstream>


namespace Demo {
namespace Client {


namespace {

const Float32 BORDER_WIDTH = 32.0f;
const Float32 BORDER_HEIGHT = 32.0f;

} // anonymous namespace


LoginDialog::LoginDialog() :
    mode( MODE_NONE ),
    result( RESULT_CANCEL )
    {
    }


void LoginDialog::initialize
    (
    const std::string & the_login_name,
    const std::string & the_login_password,
    const std::string & the_error_text
    )
    {
    Float32     scr_width = static_cast< Float32 >( SDL::get_screen_width() );
    Float32     scr_height = static_cast< Float32 >( SDL::get_screen_height() );
    SVector2    offset( 0.0f, 0.0f );
    SVector2    size( scr_width, scr_height );
    Float32     content_width = 250.0f;
    SVector2    content_top_left( ( scr_width - content_width ) / 2.0f, 200.0f );
    Float32     content_vpadding = 20.0f;

    set_base_offset( offset );
    set_base_size( size );
    set_border_size( BORDER_WIDTH, BORDER_HEIGHT );
    set_focusable( WINDOW_FOCUSABLE );
    set_dispatcher_as_parent();

    //
    // Create all gui items.
    //

    picture.reset( new Renderer::Icon( "image/login_picture" ) );
    picture->set_parent_overlay_object( this );

    title.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    title->set_parent_overlay_object( this );

    login_name_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    login_name_label->set_parent_overlay_object( this );

    login_name_edit_box.reset( new EditBox() );
    login_name_edit_box->set_parent_widget( this );
    login_name_edit_box->set_message_receiver( this );

    login_password_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    login_password_label->set_parent_overlay_object( this );

    login_password_edit_box.reset( new EditBox( EditBox::TYPE_PASSWORD ) );
    login_password_edit_box->set_parent_widget( this );
    login_password_edit_box->set_message_receiver( this );

    login_password_again_label.reset( new Renderer::TextRectangle( &Global::get_console_font() ) );
    login_password_again_label->set_parent_overlay_object( this );

    login_password_again_edit_box.reset( new EditBox( EditBox::TYPE_PASSWORD ) );
    login_password_again_edit_box->set_parent_widget( this );
    login_password_again_edit_box->set_message_receiver( this );

    ok_button.reset( new Button( Button::TYPE_ONE_SHOT ) );
    ok_button->set_parent_widget( this );
    ok_button->set_message_receiver( this );

    cancel_button.reset( new Button( Button::TYPE_ONE_SHOT ) );
    cancel_button->set_parent_widget( this );
    cancel_button->set_message_receiver( this );

    create_account_button.reset( new Button( Button::TYPE_ONE_SHOT ) );
    create_account_button->set_parent_widget( this );
    create_account_button->set_message_receiver( this );

    error_text.reset( new Renderer::TextRectangle() );
    error_text->set_font( &Global::get_console_font() );
    error_text->set_parent_overlay_object( this );
    error_text->add_animation_object( new Renderer::OverlayColorFactorAnimation
        ( SColor( 1.0f / 10.0f, 1.0f / 10.0f, 1.0f / 10.0f, 1.0f / 10.0f ) ) ); // speed

    info_text.reset( new Renderer::TextRectangle() );
    info_text->set_font( &Global::get_console_font() );
    info_text->set_parent_overlay_object( this );

    //
    // Initialize gui items.
    //

    Float32 next_widget_x = content_top_left.x();
    Float32 next_widget_y = content_top_left.y();

    // Background picture.
    picture->set_origin( Renderer::OverlayObject::BOTTOM_RIGHT );
    picture->set_base_size( SVector2( 300.0f, 512.0f ) );
    picture->set_base_color( SColor( 1.0f, 1.0f, 1.0f, 0.4f ) );

    // Title.
    set_title( "MASSIV Demo" );

    // Login name label.
    login_name_label->set_base_offset( SVector2( next_widget_x, next_widget_y ) );
    login_name_label->set_base_color( SColor( 1.0f, 0.85f, 0.8f, 0.6f ) );
    next_widget_y += login_name_label->get_current_size().y() + content_vpadding / 2.0f;

    // Login name edit box.
    login_name_edit_box->set_base_offset( SVector2( next_widget_x, next_widget_y ) );
    login_name_edit_box->set_base_size( SVector2
        ( content_width, login_name_edit_box->get_character_height() + 4.0f ) );
    login_name_edit_box->set_max_text_length_from_size();
    next_widget_y += login_name_edit_box->get_current_size().y() + content_vpadding;

    // Login password label.
    login_password_label->set_base_offset( SVector2( next_widget_x, next_widget_y ) );
    login_password_label->set_base_color( SColor( 1.0f, 0.85f, 0.8f, 0.6f ) );
    next_widget_y += login_password_label->get_current_size().y() + content_vpadding / 2.0f;

    // Login password edit box.
    login_password_edit_box->set_base_offset( SVector2( next_widget_x, next_widget_y ) );
    login_password_edit_box->set_base_size( SVector2
        ( content_width, login_password_edit_box->get_character_height() + 4.0f ) );
    login_password_edit_box->set_max_text_length_from_size();
    next_widget_y += login_password_edit_box->get_current_size().y() + content_vpadding;

    // Second login password label.
    login_password_again_label->set_base_offset( SVector2( next_widget_x, next_widget_y ) );
    login_password_again_label->set_base_color( SColor( 1.0f, 0.85f, 0.8f, 0.6f ) );
    next_widget_y += login_password_again_label->get_current_size().y() + content_vpadding / 2.0f;

    // Second login password edit box.
    login_password_again_edit_box->set_base_offset( SVector2( next_widget_x, next_widget_y ) );
    login_password_again_edit_box->set_base_size( SVector2
        ( content_width, login_password_again_edit_box->get_character_height() + 4.0f ) );
    login_password_again_edit_box->set_max_text_length_from_size();
    next_widget_y += login_password_again_edit_box->get_current_size().y() + content_vpadding;

    // Ok button.
    ok_button->set_base_offset( SVector2( next_widget_x, next_widget_y ) );
    ok_button->set_text( "" );
    ok_button->set_size_from_text();

    // Cancel button.
    cancel_button->set_base_offset( SVector2( next_widget_x + 150.0f, next_widget_y ) );
    cancel_button->set_text( "" );
    cancel_button->set_size_from_text();

    next_widget_y += ok_button->get_current_size().y() + 2.0f * content_vpadding;

    // Create account button.
    create_account_button->set_base_offset( SVector2( next_widget_x, next_widget_y ) );
    next_widget_y += create_account_button->get_current_size().y() + content_vpadding;

    // Error text.
    if( the_error_text.empty() )
        {
        set_error_text( "Welcome to massiv demo!" );
        }
    else
        {
        set_error_text( the_error_text );
        }

    // Info text.
    info_text->set_base_color( SColor( 1.0f, 1.0f, 1.0f, 0.45f ) );
    set_info_text( "http://massiv.objectweb.org" );

    set_ask_login_mode( the_login_name, the_login_password );
    }


void LoginDialog::next_tick()
    {
    if( mode == MODE_WAITING_ACCOUNT_CREATION &&
        Massiv::System::get_local_node_id().is_type( Massiv::Core::NodeId::TYPE_ANONYMOUS ) )
        {
        // Check if account was created.
        Massiv::Core::AccountManager & account_manager = Massiv::Core::Global::account_manager();
        Massiv::Core::AccountManager::AccountRequestStatus status;

        status = account_manager.get_account_request_status();
        switch( status )
            {
            case Massiv::Core::AccountManager::ACCOUNT_REQUEST_STATUS_SUCCESS:
                {
                if( !account_manager.save_account( Massiv::Core::Global::path_manager().get_login_directory() + "/"
                    + login_name_edit_box->get_text(), login_password_edit_box->get_text() ) )
                    {
                    set_error_text( "Save account config file failed!" );
                    }
                else
                    {
                    set_error_text( "Account successfully created. You should be able to login in few minutes." );
                    }

                set_ask_login_mode( login_name_edit_box->get_text(), "" );
                break;
                }

            case Massiv::Core::AccountManager::ACCOUNT_REQUEST_STATUS_ERROR:
                {
                set_ask_login_mode( login_name_edit_box->get_text(), "" );
                set_error_text( "Account creation failed. Try again later." );
                break;
                }

            case Massiv::Core::AccountManager::ACCOUNT_REQUEST_STATUS_PENDING:
                // Wait.
                break;

            default:
                {
                set_ask_login_mode( login_name_edit_box->get_text(), "" );
                set_error_text( "Unexpected error." );
                break;
                }
            }
        }
    }


LoginDialog::Result LoginDialog::get_result() const
    {
    return result;
    }


std::string LoginDialog::get_login_name() const
    {
    return login_name;
    }


std::string LoginDialog::get_login_password() const
    {
    return login_password;
    }


void LoginDialog::button_clicked
    (
    Button * button
    )
    {
    if( button == ok_button.get() )
        {
        if( mode == MODE_ASK_LOGIN )
            {
            if( commit_input_data() )
                {
                result = RESULT_OK;
                Massiv::System::request_quit();
                }
            }
        else if( mode == MODE_ASK_NEW_ACCOUNT )
            {
            if( login_password_edit_box->get_text() != login_password_again_edit_box->get_text() )
                {
                set_error_text( "Second password differs from the first one." );
                login_password_edit_box->set_text( "" );
                login_password_again_edit_box->set_text( "" );
                login_password_edit_box->set_focus();
                }
            else
                {
                request_account_creation();
                }
            }
        }
    else if( button == cancel_button.get() )
        {
        if( mode == MODE_ASK_LOGIN )
            {
            result = RESULT_CANCEL;
            Massiv::System::request_quit();
            }
        else if( mode == MODE_ASK_NEW_ACCOUNT || mode == MODE_WAITING_ACCOUNT_CREATION )
            {
            set_ask_login_mode( login_name_edit_box->get_text(), "" );
            }
        }
    else if( button == create_account_button.get() )
        {
        if( mode == MODE_ASK_LOGIN )
            {
            Massiv::Core::AccountManager & account_manager = Massiv::Core::Global::account_manager();
            account_manager.set_account_request_idle_status();
            set_ask_new_account_mode();
            }
        }
    }


bool LoginDialog::key_down
    (
    SDLKey  key,
    SDLMod  mod
    )
    {
    // Handle the event by super class.
    if( Super::key_down( key, mod ) )
        {
        return true;
        }

    switch( key )
        {
        case SDLK_ESCAPE:
            {
            result = RESULT_CANCEL;
            Massiv::System::request_quit();
            return true;
            }

        case SDLK_RETURN:
        case SDLK_KP_ENTER:
            {
            if( mode == MODE_ASK_NEW_ACCOUNT )
                {
                button_clicked( ok_button.get() );
                return true;
                }
            if( commit_input_data() )
                {
                result = RESULT_OK;
                Massiv::System::request_quit();
                }
            return true;
            }

        default:
            break; // get rid of tons of warnings
        }

    return false;
    }


bool LoginDialog::commit_input_data()
    {
    std::string     n = login_name_edit_box->get_text();
    std::string     p = login_password_edit_box->get_text();

    std::string     file_name = Massiv::Core::Global::path_manager().get_login_directory() + "/" + n;
    std::ifstream   file( file_name.c_str() );


    login_name      = n;
    login_password  = p;
    return true;

    }


void LoginDialog::request_account_creation()
    {
    hope( mode == MODE_ASK_NEW_ACCOUNT );
    hope( Massiv::System::get_local_node_id().is_type( Massiv::Core::NodeId::TYPE_ANONYMOUS ) );

    Massiv::Core::AccountManager & account_manager = Massiv::Core::Global::account_manager();

    std::string n = login_name_edit_box->get_text();

    // Check if login name is empty.
    if( n.find_first_not_of( " \t" ) == std::string::npos )
        {
        std::ostringstream ss;
        ss  << "Invalid login name.";
        set_error_text( ss.str() );
        return;
        }

    // Check if login file already exists.
        {
        std::string     file_name = Massiv::Core::Global::path_manager().get_login_directory() + "/" + n;
        std::ifstream   file( file_name.c_str() );
        if( !file.fail() )
            {
            std::ostringstream ss;
            ss  << "Config file for this login name already exists. Select another name.";
            set_error_text( ss.str() );
            return;
            }
        }

    // Send request to create new account.
    if( account_manager.create_account() )
        {
        // Account create request message was sent to service.
        set_waiting_account_creation_mode();
        }
    else
        {
        // Previous request is not finished yet.
        std::ostringstream ss;
        ss  << "Previous request is not finished yet.";
        set_error_text( ss.str() );
        }
    }


void LoginDialog::set_ask_login_mode
    (
    const std::string & the_login_name,
    const std::string & the_login_password
    )
    {
    set_title( "MASSIV Demo" );

    mode = MODE_ASK_LOGIN;

    login_name_label->set_visibility( true );
    login_name_label->print_line( "Login name:" );

    login_name_edit_box->set_visibility( true );
    login_name_edit_box->set_text( the_login_name );

    login_password_label->set_visibility( true );
    login_password_label->print_line( "Password:" );

    login_password_edit_box->set_visibility( true );
    login_password_edit_box->set_text( the_login_password );

    login_password_again_label->set_visibility( false );

    login_password_again_edit_box->set_visibility( false );

    ok_button->set_visibility( true );
    ok_button->set_text( "Login" );
    ok_button->set_size_from_text();

    cancel_button->set_visibility( true );
    cancel_button->set_text( "Quit " );
    cancel_button->set_size_from_text();

    create_account_button->set_visibility( true );
    create_account_button->set_text( "Create New Account" );
    create_account_button->set_size_from_text();

    login_name_edit_box->set_focus();
    }


void LoginDialog::set_ask_new_account_mode()
    {
    set_title( "MASSIV Demo - Create new account" );

    mode = MODE_ASK_NEW_ACCOUNT;

    login_name_label->set_visibility( true );
    login_name_label->print_line( "Login name:" );

    login_name_edit_box->set_visibility( true );
    login_name_edit_box->set_text( "" );

    login_password_label->set_visibility( true );
    login_password_label->print_line( "Password:" );

    login_password_edit_box->set_visibility( true );
    login_password_edit_box->set_text( "" );

    login_password_again_label->set_visibility( true );
    login_password_again_label->print_line( "Password again:" );

    login_password_again_edit_box->set_visibility( true );
    login_password_again_edit_box->set_text( "" );

    ok_button->set_visibility( true );
    ok_button->set_text( "Create" );
    ok_button->set_size_from_text();

    cancel_button->set_visibility( true );
    cancel_button->set_text( "Cancel " );
    cancel_button->set_size_from_text();

    create_account_button->set_visibility( false );

    login_name_edit_box->set_focus();
    }

void LoginDialog::set_waiting_account_creation_mode()
    {
    set_title( "Creating account, please wait..." );

    mode = MODE_WAITING_ACCOUNT_CREATION;

    login_name_label->set_visibility( false );

    login_name_edit_box->set_visibility( false );

    login_password_label->set_visibility( false );

    login_password_edit_box->set_visibility( false );

    login_password_again_label->set_visibility( false );

    login_password_again_edit_box->set_visibility( false );

    ok_button->set_visibility( false );

    cancel_button->set_visibility( true );
    cancel_button->set_text( "Cancel " );
    cancel_button->set_size_from_text();

    create_account_button->set_visibility( false );
    }

void LoginDialog::set_error_text
    (
    const std::string & text
    )
    {
    error_text->print_line( text );

    SColor      target_color_factor( 1.0f, 0.3f, 0.4f, 0.0f );
    SVector2    dialog_size = get_current_size();
    SVector2    text_size = error_text->get_current_size();
    SVector2    pos( ( dialog_size.x() - text_size.x() ) / 2.0f, dialog_size.y() - 70.0f );

    error_text->set_base_offset( pos );
    error_text->set_base_color( SColor( 1.0f, 1.0f, 1.0f, 1.0f ) );
    error_text->require_color_factor_animation().set_factor( SColor( 1.0f, 1.0f, 1.0f, 1.0f ) );
    error_text->require_color_factor_animation().set_target_factor( target_color_factor );
    }


void LoginDialog::set_info_text
    (
    const std::string & the_info_text
    )
    {
    hope( info_text.get() );
    info_text->print_line( the_info_text );

    SVector2    dialog_size = get_current_size();
    SVector2    text_size = info_text->get_current_size();
    SVector2    pos( ( dialog_size.x() - text_size.x() ) / 2.0f, dialog_size.y() - 50.0f );
    info_text->set_base_offset( pos );
    }


void LoginDialog::set_title
    (
    const std::string & the_title
    )
    {
    hope( title.get() );
    title->print_line( the_title );
    title->set_base_size( title->get_current_size() * 2.0f ); // Scale 2x.
    title->set_base_offset( SVector2
        ( ( get_current_size().x() - title->get_current_size().x()) / 2.0f, 100.0f ) );
    }


} // namespace Demo::Client
} // namespace Demo
