/** @file panel_status_border.cpp
 * Decorative panel border containing text area at the bottom border (implementation).
 */

/*
 * begin        : Sep 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "gui/panel_status_border.h"
#include "client_global.h"
#include "client_console.h"

namespace Demo {
namespace Client {


PanelStatusBorder::PanelStatusBorder
    (
    const std::string & the_corner_icon,
    const std::string & the_hborder_icon,
    const std::string & the_vborder_icon,
    const std::string & the_background_icon,
    Float32             the_border_width,
    Float32             the_border_height,
    Renderer::Font    & the_font,
    Float32             the_status_line_height,
    Float32             the_status_left_offset,
    Float32             the_status_right_offset,
    Float32             the_status_y_offset,
    Float32             the_status_text_y_offset
    ) : PanelBorder
            (
            the_corner_icon,
            the_hborder_icon,
            the_vborder_icon,
            the_background_icon,
            the_border_width,
            the_border_height
            ),
        font( &the_font ),
        status_y_offset( the_status_y_offset ),
        status_text_y_offset( the_status_text_y_offset ),
        status_line_height( the_status_line_height ),
        status_left_offset( the_status_left_offset ),
        status_right_offset( the_status_right_offset )
    {
    status_panel.reset( new PanelBorder
        (
        "gui/panel_corner",
        "gui/panel_hborder",
        "gui/panel_vborder",
        "gui/panel_background",
        0.0f,
        0.0f
        ) );

    status_panel->set_parent_widget( this );

    status_text.reset( new Renderer::TextRectangle( font, 1, 1 ) );
    status_text->set_parent_overlay_object( status_panel.get() );
    }


void PanelStatusBorder::render()
    {
    PanelBorder::render();
    render_status_line();
    }

    
void PanelStatusBorder::render_status_line()
    {
    const int cell_width = font->get_character_width();

    status_panel->set_base_size
        ( 
        SVector2
            ( 
            static_cast< Float32 >( get_current_size().x() - status_left_offset - status_right_offset ),
            status_line_height
            )
        );
        
    status_panel->set_base_offset
        ( SVector2
            (
            static_cast< Float32 >( status_left_offset ), 
            static_cast< Float32 >( get_current_size().y() + status_y_offset )
            ) 
        );

    status_panel->render();

    int new_columns = ( int ) ( status_panel->get_base_size().x() - 2 * cell_width ) / cell_width;
    if( new_columns != status_text->get_columns() )
        {
        status_text.reset();
        status_text.reset( new Renderer::TextRectangle( font, new_columns, 1 ) );
        status_text->set_parent_overlay_object( status_panel.get() );
        }

    status_text->set_base_offset
        ( 
        SVector2
            (
            static_cast< Float32 >( cell_width ), 
            static_cast< Float32 >( status_text_y_offset )
            ) 
        );

    status_text->render();
    }


int PanelStatusBorder::get_columns() const
    {
    return status_text->get_columns();
    }


const std::auto_ptr< Renderer::TextRectangle >  & PanelStatusBorder::get_text_rectangle() const
    {
    return status_text;
    }

} // namespace Demo::Client
} // namespace Demo
