/** @file panel_stauts_border.h
 * Decorative panel border containing text area at the bottom border.
 */

/*
 * begin        : Sep 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_PANEL_STATUS_BORDER_H
#define CLIENT_PANEL_STATUS_BORDER_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef CLIENT_PANEL_BORDER_H
#include "gui/panel_border.h"
#endif

#ifndef CLIENT_FONT_H
#include "renderer/font.h"
#endif

#ifndef CLIENT_TEXT_RECTANGLE_H
#include "renderer/text_rectangle.h"
#endif

#include <memory>

namespace Demo {
namespace Client {

class Console;

/**
 * PanelBorder is simple widget object. When drawn,
 * it fills its screen rectangle with background texture
 * and decorative border. The bottom border contains 
 * text rectangle which can bear any text.
 */
class PanelStatusBorder : public PanelBorder
    {
    typedef PanelBorder  Super;
    
    NO_COPY_CONSTRUCTOR( PanelStatusBorder );
    NO_ASSIGNMENT_OPERATOR( PanelStatusBorder );

public:

    /**
     * Constructor.
     * Icon parameters are names of icon templates from media repository.
     * @param corner_icon Image to draw at corners.
     * @param hborder_icon Image containing horizontal border.
     *      The texture must be repeatable in horizontal direction.
     *      The icon must occupy entire texture.
     * @param hborder_icon Image containing vertical border.
     *      The texture must be repeatable in vertical direction.
     *      The icon must occupy entire texture.
     * @param background_icon Background texture.
     *      The texture must be repeatable in both directions.
     *      The icon must occupy entire texture.
     * @param the_font Font for the status line.
     * @param the_status_line_height Height of the status line panel.
     * @param the_status_left_offset Left x offset of the panel.
     * @param the_status_right_offset Right x offset of the panel.
     *        The_status_left_offset and the_status_right_offset
     *        define the length of status line.
     * @param the_status_y_offset Panel relative Y offset to bottom 
     *        line of PanelStatusBorder.
     * @note  the_status_y_offset should be < 0.
     * @param the_status_text_y_offset Text rectangle relative Y offset to
     *        bottom line of PanelStatusBorder.
     * @note  the_status_y_offset should be < 0.
     */
    PanelStatusBorder
        (
        const std::string & corner_icon,
        const std::string & hborder_icon,
        const std::string & vborder_icon,
        const std::string & background_icon,
        Float32             border_width,
        Float32             border_height,
        Renderer::Font    & the_font,
        Float32             the_status_line_height,
        Float32             the_status_left_offset,
        Float32             the_status_right_offset,
        Float32             the_status_y_offset,
        Float32             the_status_text_y_offset
        );
        
public:

    // Renderer::OverlayObject
    virtual void render();
    
    /**
     * Returns number of columns.
     */
    int get_columns() const;

    /**
     * Returns status text rectangle.
     */
    const std::auto_ptr< Renderer::TextRectangle >  & get_text_rectangle() const;

private:

    /**
     * Render status line.
     */
    void render_status_line();
    
private:

    std::auto_ptr< Renderer::TextRectangle >    status_text;
        /**< Status text overlay object. */

    std::auto_ptr< PanelBorder >                status_panel;
        /**< Status text background overlay object. */

    Renderer::Font *                            font;
        /**< Font used for printing. */

    Float32                                     status_y_offset;
        /**< Panel relative Y offset to bottom 
             line of PanelStatusBorder. */

    Float32                                     status_text_y_offset;
        /**< Text rectangle relative Y offset to
             bottom line of PanelStatusBorder. */

    Float32                                     status_line_height;
        /**< Height of the staus panel. */

    Float32                                     status_left_offset;
        /**< Left x offset of the panel. */

    Float32                                     status_right_offset;
        /**< Right x offset of the panel. */
        
    }; // PanelBorder


} // namespace Demo::Client
} // namespace Demo


#endif // CLIENT_PANEL_STATUS_BORDER_H
