/** @file slot_panel_inline.h
 * Slot panel (inline methods).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


namespace Demo {
namespace Client {


// SlotPanel::Slot
//~~~~~~~~~~~~~~~~


SlotPanel::Slot::Slot() :
    highlight( false )
    {
    }


SlotPanel::Slot::~Slot()
    {
    }


SlotPanel::Slot::Slot
    (
    const Slot & other
    ) :
    icon_name( other.icon_name ),
    highlight( other.highlight ),
    text( other.text ),
    title( other.title ),
    screen_top_left( other.screen_top_left ),
    screen_bottom_right( other.screen_bottom_right )
    {
    icon.reset( other.icon.release() );
    text_rectangle.reset( other.text_rectangle.release() );
    }
    

SlotPanel::Slot & SlotPanel::Slot::operator=
    (
    const Slot & other
    )
    {
    icon.reset( other.icon.release() );
    icon_name = other.icon_name;
    highlight = other.highlight;
    text_rectangle.reset( other.text_rectangle.release() );
    text = other.text;
    title = other.title;
    screen_top_left = other.screen_top_left;
    screen_bottom_right = other.screen_bottom_right;
    return *this;
    }


const std::string & SlotPanel::Slot::get_text() const
    {
    return text;
    }


const std::string & SlotPanel::Slot::get_title() const
    {
    return title;
    }


void SlotPanel::Slot::set_title
    (
    const std::string & the_title
    )
    {
    title = the_title;
    }


void SlotPanel::Slot::clear()
    {
    icon.reset();
    icon_name.clear();
    highlight = false;
    text_rectangle.reset();
    text.clear();
    title.clear();
    }


// SlotPanel
//~~~~~~~~~~


int SlotPanel::get_slot_columns() const
    {
    return cols;
    }

    
int SlotPanel::get_slot_rows() const
    {
    return rows;
    }


void SlotPanel::select_slot
    (
    int slot_index
    )
    {
    if( slot_index < 0 )
        {
        selected_slot_index = -1;
        }
    else
        {
        hope( is_valid_slot_index( slot_index ) );
        selected_slot_index = slot_index;
        }
    }
    
    
int SlotPanel::get_selected_slot_index() const
    {
    hope( selected_slot_index < 0 || is_valid_slot_index( selected_slot_index ) );
    return selected_slot_index;
    }


const SlotPanel::Slot & SlotPanel::get_slot
    (
    int slot_index
    ) const
    {
    hope( is_valid_slot_index( slot_index ) );
    return slots[ slot_index ];
    }


SlotPanel::Slot & SlotPanel::get_slot
    (
    int slot_index
    )
    {
    hope( is_valid_slot_index( slot_index ) );
    return slots[ slot_index ];
    }


const SlotPanel::Slot * SlotPanel::get_slot_ptr
    (
    int slot_index
    ) const
    {
    if( is_valid_slot_index( slot_index ) )
        {
        return &( get_slot( slot_index ) );
        }
    else
        {
        return NULL;
        }
    }


SlotPanel::Slot * SlotPanel::get_slot_ptr
    (
    int slot_index
    )
    {
    if( is_valid_slot_index( slot_index ) )
        {
        return &( get_slot( slot_index ) );
        }
    else
        {
        return NULL;
        }
    }


bool SlotPanel::is_valid_slot_index
    (
    int slot_index
    ) const
    {
    if( slot_index < 0 )
        {
        return false;
        }
    
    if( static_cast< Slots::size_type >( slot_index ) >= slots.size() )
        {
        return false;
        }
    
    return true;
    }

    
void SlotPanel::set_message_receiver
    (
    SlotPanelMessageReceiver * the_receiver
    )
    {
    message_receiver = the_receiver;
    }


} // namespace Demo::Client
} // namespace Demo
