/** @file terrain_window.cpp
 * Terrain window (implementation)
 */

/*
 * begin        : Oct 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "gui/terrain_window.h"
#include "gui/widget_input_dispatcher.h"
#include "game/player_manager.h"
#include "client_global.h"
#include "gui/editor.h"
#include "gui/game_interface.h"
#include "input/input_manager.h"
#include "renderer/camera.h"


namespace Demo {
namespace Client {


TerrainWindow::TerrainWindow()
    {
    }


void TerrainWindow::initialize()
    {
    set_focusable( CLICK_LOSE_FOCUS );
    set_dispatcher_as_parent();
    set_base_size_from_screen();
    do_layout();
    }

    
bool TerrainWindow::mouse_button_down
    (
    int     button,
    SDLMod  mod
    )
    {
    if( Super::mouse_button_down( button, mod ) )
        {
        return true;
        }
    
    if( !is_enabled() || !is_visible() )
        {
        return false;
        }
        
    switch( button )
        {
        case SDL_BUTTON_LEFT:
            left_mouse_button_down( mod );
            return true;
            
        case SDL_BUTTON_RIGHT:
            right_mouse_button_down( mod );
            return true;
        
        default:
            break;
        }
    
    return false;
    }


TerrainWindow::RayCastResult TerrainWindow::ray_cast
    (
    SectorManager::RayIntersection & sector_intersection,
    EntityManager::RayIntersection & entity_intersection
    )
    {
    sector_intersection.layer_id = entity_intersection.layer_id =
                Global::get_player_manager().get_visible_layer();
    Renderer::CirclingCamera & camera = Global::get_player_manager().get_camera();
    int mouse_x, mouse_y;
    Global::get_input_manager().get_mouse_position( mouse_x, mouse_y );
    camera.cursor_ray( sector_intersection.origin, sector_intersection.direction,
                mouse_x, mouse_y );
    entity_intersection.origin = sector_intersection.origin;
    entity_intersection.direction = sector_intersection.direction;
    const bool sector_found = Global::get_sector_manager().cast_ray( sector_intersection );
    const bool entity_found = Global::get_entity_manager().cast_ray( entity_intersection );

    if( sector_found || entity_found )
        {
        if( sector_found && ( !entity_found ||
            entity_intersection.distance > sector_intersection.distance ) )
            {
            return SECTOR_FOUND;
            }
        else
            {
            hope( entity_found );
            return ENTITY_FOUND;
            }
        }
    else
        {
        return MISSED;
        }
    }


void TerrainWindow::update_camera()
    {
    PlayerManager & player_manager = Global::get_player_manager();

    camera_control.next_tick();
    camera_control.apply_control( player_manager.get_camera() );
    }


void TerrainWindow::left_mouse_button_down
    (
    SDLMod mod
    )
    {
    SectorManager::RayIntersection sector_intersection;
    EntityManager::RayIntersection entity_intersection;
    RayCastResult ray_cast_result = ray_cast( sector_intersection, entity_intersection );
    GameInterface & game_interface = Global::get_game_interface();
    
    if( !game_interface.is_visible() )
        {
        return;
        }
    
    switch( ray_cast_result )
        {
        case SECTOR_FOUND:
            game_interface.handle_sector_click( mod, sector_intersection );
            break;
        
        case ENTITY_FOUND:
            game_interface.handle_entity_click( mod, entity_intersection );
            break;
        
        case MISSED:
            break;
        }
    }


void TerrainWindow::right_mouse_button_down
    (
    SDLMod mod
    )
    {
    SectorManager::RayIntersection sector_intersection;
    EntityManager::RayIntersection entity_intersection;
    RayCastResult ray_cast_result = ray_cast( sector_intersection, entity_intersection );
    Editor & editor = Global::get_editor();
    
    if( !editor.is_visible() )
        {
        return;
        }
    
    switch( ray_cast_result )
        {
        case SECTOR_FOUND:
            editor.handle_sector_click( mod, sector_intersection );
            break;
        
        case ENTITY_FOUND:
            editor.handle_entity_click( mod, entity_intersection );
            break;
        
        case MISSED:
            break;
        }
    }


} // namespace Demo::Client
} // namespace Demo
