/** @file terrain_window.h
 * Terrain window.
 */

/*
 * begin        : Oct 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_TERRAIN_WINDOW_H
#define CLIENT_TERRAIN_WINDOW_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef CLIENT_WIDGET_H
#include "gui/widget.h"
#endif

#ifndef CLIENT_CAMERA_CONTROL_H
#include "gui/camera_control.h"
#endif

#ifndef CLIENT_ENTITY_MANAGER_H
#include "game/entity_manager.h"
#endif

#ifndef CLIENT_SECTOR_MANAGER_H
#include "game/sector_manager.h"
#endif


namespace Demo {
namespace Client {


/**
 * TerrainWindow is invisible window that handles mouse clicks
 * and projects the clicks to terrain. TerrainWindow lies
 * at the bottom of widget objects stack so it handles all input events
 * that are not processed by other input objects (console, editor, ...).
 */
class TerrainWindow : public Widget
    {
    typedef Widget  Super;
    
    NO_COPY_CONSTRUCTOR( TerrainWindow );
    NO_ASSIGNMENT_OPERATOR( TerrainWindow );
    
public:

    /** Result of terrain raycast. */
    enum RayCastResult
        {
        SECTOR_FOUND,
        ENTITY_FOUND,
        MISSED
        }; // RayCastResult
    
public:

    /** Constructor. */
    TerrainWindow();
    
    /** Initialize. */
    void initialize();
    
public:

    /// From superclass.
    virtual bool mouse_button_down
        (
        int     button,
        SDLMod  mod
        );

public:

    /**
     * Cast ray from current camera to visible terrain.
     * @note @a sector_intersection and @a entity_intersection
     *     are modified even if intersection is not found.
     */
    RayCastResult ray_cast
        (
        SectorManager::RayIntersection & sector_intersection,
        EntityManager::RayIntersection & entity_intersection
        );

    /**
     * Update camera according to @c camera_control.
     */
    void update_camera();

private:

    /**< Handle left mouse click. */
    void left_mouse_button_down
        (
        SDLMod mod
        );

    /**< Handle right mouse click. */
    void right_mouse_button_down
        (
        SDLMod mod
        );

private:

    CameraControl camera_control;
        /**< Camera control. */
    
    }; // TerrainWindow


} // namespace Demo::Client
} // namespace Demo

#endif // CLIENT_TERRAIN_WINDOW_H
