/** @file media_repository.cpp
 * Media repository (implementation).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "media_repository.h"
#include "client_exception.h"
#include "renderer/icon.h"
#include <sstream>


namespace Demo {
namespace Client {


MediaRepository::MediaRepository()
    {
    }


void MediaRepository::initialize()
    {

    // UI buttons.

        {
        std::string texture_name = "gui/buttons";
        Float32     u1;
        Float32     u2;
        Float32     v1;
        Float32     v2;
        Float32     du = 32.0f / 128.0f;
        Float32     dv = 32.0f / 128.0f;

        // Inventory.

        u1 = 0.0f;
        v1 = 0.0f;
        u2 = 0.0f + du;
        v2 = 0.0f + dv;
        register_icon_template( "gui/inventory_button_up",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "gui/inventory_button_down",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "gui/inventory_button_up_mouse_on",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "gui/inventory_button_down_mouse_on",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        // Ability.

        u1 = 0.0f;
        v1 = 0.0f + dv + dv;
        u2 = 0.0f + du;
        v2 = v1 + dv;
        register_icon_template( "gui/ability_button_up",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "gui/ability_button_down",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "gui/ability_button_up_mouse_on",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "gui/ability_button_down_mouse_on",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        // Close button.

        u1 = 0.0f;
        v1 = 0.0f + dv + dv + dv;
        u2 = 0.0f + du;
        v2 = v1 + dv;
        register_icon_template( "gui/close_button_up",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "gui/close_button_down",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "gui/close_button_up_mouse_on",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "gui/close_button_down_mouse_on",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        }

    // UI Panel

        {
        register_icon_template( "gui/panel_corner",
            IconTemplate( "gui/panel_corner", 0.0f, 0.0f, 1.0f, 1.0f ) );

        register_icon_template( "gui/panel_hborder",
            IconTemplate( "gui/panel_hborder", 0.0f, 0.0f, 1.0f, 1.0f ) );

        register_icon_template( "gui/panel_vborder",
            IconTemplate( "gui/panel_vborder", 0.0f, 0.0f, 1.0f, 1.0f ) );

        register_icon_template( "gui/panel_background",
            IconTemplate( "texture/stone1", 0.0f, 0.0f, 1.0f, 1.0f ) );

        register_icon_template( "gui/slot_panel/slot_background",
            IconTemplate( "gui/slot_panel", 0.0f, 0.0f, 32.0f / 64.0f, 32.0f / 64.0f ) );

        register_icon_template( "gui/slot_panel/slot_highlight",
            IconTemplate( "gui/slot_panel", 32.0f / 64.0f, 0.0f, 64.0f / 64.0f, 32.0f / 64.0f ) );
        }

    // Portraits.
        {
        register_icon_template( "image/portrait1",
            IconTemplate( "image/portrait1", 0.0f, 0.0f, 1.0f, 1.0f ) );
        }

    // Items.
        {
        std::string texture_name = "item/items1";
        Float32     u1;
        Float32     u2;
        Float32     v1;
        Float32     v2;
        Float32     du = 32.0f / 128.0f;
        Float32     dv = 32.0f / 128.0f;

        u1 = 0.0f;
        v1 = 0.0f;
        u2 = u1 + du;
        v2 = v1 + dv;
        register_icon_template( "item/icon_potion_flask",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "item/icon_red_potion",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "item/icon_green_potion",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "item/icon_unknown",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 = 0.0f;
        v1 += dv;
        u2 = u1 + du;
        v2 = v1 + dv;
        register_icon_template( "item/icon_boots",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "item/icon_magic_boots",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "item/icon_weapon_axe",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "item/icon_money",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 = 0.0f;
        v1 += dv;
        u2 = u1 + du;
        v2 = v1 + dv;
        register_icon_template( "item/icon_meat",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        u1 += du;
        u2 += du;
        register_icon_template( "item/icon_bone",
            IconTemplate( texture_name, u1, v1, u2, v2 ) );

        }
    }


Renderer::Icon * MediaRepository::create_icon
    (
    const std::string & icon_template_name
    )
    {
    std::auto_ptr< Renderer::Icon > icon;
    IconTemplates::const_iterator icon_template_it = icon_templates.find( icon_template_name );
    if( icon_template_it != icon_templates.end() )
        {
        // Icon template found.
        const IconTemplate & icon_template = icon_template_it->second;

        // Create new icon object.
        icon.reset( new Renderer::Icon(
            icon_template.texture_name,
            icon_template.top_left_uv.x(), icon_template.top_left_uv.y(),
            icon_template.bottom_right_uv.x(), icon_template.bottom_right_uv.y() ) );
        }
    else
        {
        // Icon template not found.
        error_media_not_registered( "icon", icon_template_name );
        }

    return icon.release();
    }


void MediaRepository::register_icon_template
    (
    const std::string & icon_template_name,
    const IconTemplate & icon_template
    )
    {
    bool inserted = icon_templates.insert( IconTemplates::value_type( icon_template_name, icon_template ) ).second;
    if( !inserted )
        {
        error_media_already_registered( "icon", icon_template_name );
        }
    }


void MediaRepository::error_media_not_registered
    (
    const std::string & media_type,
    const std::string & media_name
    )
    {
    std::ostringstream oss;
    oss << "Specified " << media_type << " '" << media_name
        << "' was not registered in media repository.";
    throw ExMediaRepositoryError( oss.str() );
    }


void MediaRepository::error_media_already_registered
    (
    const std::string & media_type,
    const std::string & media_name
    )
    {
    std::ostringstream oss;
    oss << "Specified " << media_type << " '" << media_name
        << "' is already registered in media repository.";
    throw ExMediaRepositoryError( oss.str() );
    }


} // namespace Demo::Client
} // namespace Demo


#ifndef ENABLE_INLINE
#include "media_repository_inline.h"
#endif
