/** @file raycast.h
 * Generic raycasting.
 *
 * @todo Ray vs. axis-aligned box. Ray vs. sphere.
 *
 * @todo Move this to different directory; src/demo/shared?
 *       Server libs may use this in the future too.
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_RAYCAST_H
#define CLIENT_RAYCAST_H

#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif


namespace Demo {
namespace Client {
namespace Raycast {


/**
 * Compute intersection of ray and triangle.
 *
 * The ray is specified by @a origin and @a direction.
 * The triangle is specified by three vertices (@a v0, @a v1 and @a v2)
 * in counter-clockwise order.

 * If the ray intersects the triangle, #intersect_triangle() will return @c true,
 * @a distance will be set to distance from @a origin to intersection point in units
 * specified by length of @a direction
 * (ie. <tt>intersection = origin + direction * distance</tt>) and @a normal will
 * be set to triangle normal. @a Distance will be always positive. @c false will be
 * returned if the ray does not intersect the triangle or if triangle is degenerated
 * or backfacing.
 *
 * Use #intersect_triangle_with_normal() if you already know the normal for slightly
 * faster version.
 */
bool intersect_triangle
    (
    Float32 &           distance,
    SVector3 &          normal,
    const SVector3 &    origin,
    const SVector3 &    direction,
    const SVector3 &    v0,
    const SVector3 &    v1,
    const SVector3 &    v2
    );


/**
 * Compute intersection of ray and triangle.
 *
 * See #intersect_triangle() for more info. The only difference is that the
 * @a normal is specified as an argument. The @a normal should be zero vector
 * if the triangle is degenerated.
 */
bool intersect_triangle_with_normal
    (
    Float32 &           distance,
    const SVector3 &    origin,
    const SVector3 &    direction,
    const SVector3 &    v0,
    const SVector3 &    v1,
    const SVector3 &    v2,
    const SVector3 &    normal
    );


} // namespace Demo::Client::Raycast
} // namespace Demo::Client
} // namespace Demo


#ifdef ENABLE_INLINE
#include "raycast_inline.h"
#endif


#endif // CLIENT_RAYCAST_H
