/** @file camera.h
 * 3D camera (inline methods).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo {
namespace Client {
namespace Renderer {


// class Camera

Camera::Projection Camera::get_projection() const
    {
    return projection;
    }

    
Float32 Camera::get_fov() const
    {
    return fov;
    }


Float32 Camera::get_zoom() const
    {
    return zoom;
    }


void Camera::set_projection
    (
    Projection new_projection
    )
    {
    hope( ( new_projection == PERSPECTIVE ) || ( new_projection == PARALLEL ) );
    projection = new_projection;
    }


void Camera::set_fov
    (
    Float32 new_fov
    )
    {
    hope( new_fov > 0.0f );
    hope( new_fov < MATHF_PI );
    if( new_fov < 0.01f )
        {
        fov = 0.01f;
        }
    else if( new_fov > MATHF_PI - 0.01f )
        {
        fov = MATHF_PI - 0.01f;
        }
    else
        {
        fov = new_fov;
        }
    }


void Camera::set_zoom
    (
    Float32 new_zoom
    )
    {
    hope( new_zoom > 0.0f );
    zoom = new_zoom;
    }


// class CirclingCamera

const SVector3 & CirclingCamera::get_target() const
    {
    return target;
    }


Float32 CirclingCamera::get_latitude() const
    {
    return latitude;
    }


Float32 CirclingCamera::get_longitude() const
    {
    return longitude;
    }


Float32 CirclingCamera::get_distance() const
    {
    return distance;
    }


void CirclingCamera::set_target
    (
    const SVector3 & new_target
    )
    {
    target = new_target;
    }


void CirclingCamera::set_latitude
    (
    Float32 new_latitude
    )
    {
    latitude = new_latitude;
    }


void CirclingCamera::set_longitude
    (
    Float32 new_longitude
    )
    {
    longitude = new_longitude;
    }


void CirclingCamera::set_distance
    (
    Float32 new_distance
    )
    {
    distance = new_distance;
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo
