/** @file icon.cpp
 * Overlay object - icon (implementation).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/icon.h"
#include "renderer/renderer.h"
#include "renderer/texture_manager.h"
#include "renderer/pipeline_state.h"


namespace Demo {
namespace Client {
namespace Renderer {


namespace {

static const int TEXTURE_FLAGS = 0;

} // anonymous namespace


Icon::Icon
    (
    const std::string & texture_name,
    GLfloat             u1,
    GLfloat             v1,
    GLfloat             u2,
    GLfloat             v2
    ) :
    top_left_uv( u1, v1 ),
    bottom_right_uv( u2, v2 )
    {
    TextureManager & texture_manager = Global::get_texture_manager();
    texture_data_object = texture_manager.get_texture_data_object( texture_name );
    }


Icon::~Icon()
    {
    }


void Icon::set_pattern_uv()
    {
    hope( !texture_data_object.is_null() );
    Texture::Reference texture = texture_data_object->get_texture( TEXTURE_FLAGS );
    const Float32 texture_width = static_cast< Float32 >( texture->get_width() );
    const Float32 texture_height = static_cast< Float32 >( texture->get_height() );
    const Float32 width = get_current_size().x();
    const Float32 height = get_current_size().y();
    const Float32 uv_h_size = width / texture_width;
    const Float32 uv_v_size = height / texture_height;
    set_uv( 0.0f, 0.0f, uv_h_size, uv_v_size );
    }


void Icon::set_size_from_texture()
    {
    hope( !texture_data_object.is_null() );
    Texture::Reference texture = texture_data_object->get_texture( TEXTURE_FLAGS );
    set_base_size( SVector2
        (
        static_cast< Float32 >( texture->get_width() ),
        static_cast< Float32 >( texture->get_height() )
        ) );
    }


void Icon::render()
    {
    // Get standard overlay pipeline state and enable texturing.

    PipelineState & state = get_pipeline_state();
    hope( !texture_data_object.is_null() );
    state.texture[ 0 ].data = texture_data_object;
    state.texture[ 0 ].flags = TEXTURE_FLAGS;
    state.color = get_current_color();
    Global::apply_state( state );

    // Render the icon.

    OpenGL::Vector2 top_left = screenspace_translation();
    OpenGL::Vector2 bottom_right( top_left );
    bottom_right.v[ 0 ] += get_current_size().x();
    bottom_right.v[ 1 ] += get_current_size().y();

    glBegin( GL_QUADS );
    
    glTexCoord2f( top_left_uv.x(), top_left_uv.y() );
    glVertex2f( top_left.x(), top_left.y() );

    glTexCoord2f( top_left_uv.x(), bottom_right_uv.y() );
    glVertex2f( top_left.x(), bottom_right.y() );

    glTexCoord2f( bottom_right_uv.x(), bottom_right_uv.y() );
    glVertex2f( bottom_right.x(), bottom_right.y() );

    glTexCoord2f( bottom_right_uv.x(), top_left_uv.y() );
    glVertex2f( bottom_right.x(), top_left.y() );

    glEnd();

    // Revert global state.

    state.texture[ 0 ].data = NULL;
    state.texture[ 0 ].flags = 0;
    state.color = OpenGL::Color( 1.0f, 1.0f, 1.0f, 1.0f );
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo


#ifndef ENABLE_INLINE
#include "icon_inline.h"
#endif
