/** @file icon.h
 * Overlay object - icon.
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_ICON_H
#define CLIENT_ICON_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef CLIENT_OVERLAY_OBJECT_H
#include "renderer/overlay_object.h"
#endif

#ifndef CLIENT_TEXTURE_DATA_OBJECT_H
#include "renderer/texture_data_object.h"
#endif


namespace Demo {
namespace Client {
namespace Renderer {


/**
 * Icon - rectangular object with single alpha blended texture.
 *
 * @todo Vertex coloring.
 */
class Icon : public OverlayObject
    {
public:

    /**
     * Constructor.
     */
    Icon
        (
        const std::string & texture_name,
        GLfloat             u1 = 0.0f,
        GLfloat             v1 = 0.0f,
        GLfloat             u2 = 1.0f,
        GLfloat             v2 = 1.0f
        );

    /**
     * Destructor.
     */
    virtual ~Icon();
    
public:

    /**
     * Get top left texture uv coordinates.
     */
    INLINE OpenGL::Vector2 get_top_left_uv() const;
    
    /**
     * Get bottom right texture uv coordinates.
     */
    INLINE OpenGL::Vector2 get_bottom_right_uv() const;
    
    /**
     * Set texture uv coordinates.
     */
    INLINE void set_uv
        (
        GLfloat u1,
        GLfloat v1,
        GLfloat u2,
        GLfloat v2
        );
        
    /**
     * Set texture uv coordinates in such way that drawing
     * the icon with its current size will draw repeatable pattern
     * of the icon texture.
     * Width and height of the pattern is set from the texture size
     * (one texel will be drawn as one pixel).
     */
    void set_pattern_uv();

public:

    /**
     * Initialize object size from texture size.
     */
    void set_size_from_texture();

    /**
     * Render the icon.
     *
     * @todo Clamp instead of Repeat.
     */
    virtual void render();

private:

    TextureDataObject::Reference texture_data_object;
        /**< Texture to use. */
    
    OpenGL::Vector2         top_left_uv;
        /**< Top left texture coordinate. */

    OpenGL::Vector2         bottom_right_uv;
        /**< Bottom right texture coordinate. */

    }; // class Icon



} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo


#ifdef ENABLE_INLINE
#include "icon_inline.h"
#endif


#endif // CLIENT_ICON_H
