/** @file light.cpp
 * Light source (implementation).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/light.h"
#include "renderer/render_manager.h"
#include "renderer/pass_manager.h"
#include "renderer/renderer.h"
#include "renderer/render_entity.h"


namespace Demo {
namespace Client {
namespace Renderer {


Light::Light()
      : position( 0.0f, -1.0f, 0.0f ),
        infinite( true ),
        ambient_intensity( 0.0f, 0.0f, 0.0f ),
        diffuse_intensity( 1.0f, 1.0f, 1.0f ),
        specular_intensity( 1.0f, 1.0f, 1.0f ),
        attenuation( 1.0f, 0.0f, 0.0f ),
        manager( NULL )
    {
    }


Light::~Light()
    {
    unregister_from_manager();
    }


void Light::register_to_manager
    (
    RenderManager * new_manager
    )
    {
    if( manager )
        {
        manager->unregister_light( this );
        manager = NULL;
        }
    if( new_manager )
        {
        manager = new_manager;
        manager->register_light( this );
        }
    }


void Light::register_to_global_manager
    (
    int pass_index
    )
    {
    PassManager & pass_manager = Global::get_pass_manager();
    RenderManager & render_manager = pass_manager.get_render_manager( pass_index );
    register_to_manager( &render_manager );
    }


void Light::set_pipeline_light
    (
    PipelineState::Lighting::Light &    light,
    const RenderEntity *                entity
    ) const
    {
    light.enable = true;
    light.ambient = ambient_intensity;
    light.diffuse = diffuse_intensity;
    light.specular = specular_intensity;
    light.infinite = infinite;
    light.attenuation = attenuation;

    // Set position.

    if( entity )
        {
        // Use inverse of entity transformation to transform #position to entity-space.

        // Translate.

        SVector3 translated = position;
        if( !infinite )
            {
            translated -= entity->get_position();
            }

        // Rotate.

        SVector3 right, forward, up;
        entity->get_orientation().rotation_base( right, forward, up );
        const SVector3 rotated
            (
            right * translated,
            forward * translated,
            up * translated
            );

        // Scale.

        const SVector3 & scale = entity->get_scale();
        light.position = OpenGL::Vector3
            (
            rotated.x() / scale.x(),
            rotated.y() / scale.y(),
            rotated.z() / scale.z()
            );
        }
    else
        {
        // No transformation.

        light.position = position;
        }
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo


#ifndef ENABLE_INLINE
#include "light_inline.h"
#endif
