/** @file mesh_entity.cpp
 * Mesh-based render entity (implementation).
 */

/*
 * begin        : Oct 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/mesh_entity.h"


namespace Demo {
namespace Client {
namespace Renderer {


MeshEntity::MeshEntity()
      : meshes( NULL )
    {
    }


MeshEntity::~MeshEntity()
    {
    if( meshes )
        {
        delete [] meshes;
        }
    }


void MeshEntity::allocate
    (
    std::size_t object_count
    )
    {
    hope( object_count > 0 );
    hope( !render_objects );
    hope( render_object_count == 0 );
    Super::allocate( object_count );
    hope( render_object_count == object_count );
    meshes = new Mesh[ object_count ];
    }


void MeshEntity::initialize_render_objects()
    {
    hope( meshes );
    hope( render_objects );

    std::size_t i;
    for( i = 0; i < render_object_count; i++ )
        {
        // Create and initialize vertex array.

        meshes[ i ].create_array( render_objects[ i ].array );

        // Allocate single render job.

        render_objects[ i ].initialize_simple_jobs();

        // Pipeline state.

        PipelineState & state = render_objects[ i ].pipeline_states[ 0 ];
        state.depth_test.enable = true;
        state.buffer_mask.depth = true;

        // Geometry.

        meshes[ i ].initialize_index_array( render_objects[ i ].geometries[ 0 ] );
        }
    }


bool MeshEntity::cast_ray
    (
    Float32 &           distance,
    SVector3 &          normal,
    const SVector3 &    origin,
    const SVector3 &    direction,
    std::size_t         stream_index
    ) const
    {
    hope( meshes );

    bool best_found = false;
    Float32 current_distance;
    SVector3 current_normal;
    std::size_t i;
    for( i = 0; i < render_object_count; i++ )
        {
        bool found = meshes[ i ].cast_ray( current_distance, current_normal,
                    origin, direction, stream_index );
        if( found )
            {
            if( !best_found || current_distance < distance )
                {
                best_found = true;
                distance = current_distance;
                normal = current_normal;
                }
            }
        }

    return best_found;
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo


#ifndef ENABLE_INLINE
#include "simple_entity_inline.h"
#endif
