/** @file model_entity.cpp
 * Model-based render entity (implementation).
 */

/*
 * begin        : Oct 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/model_entity.h"
#include "renderer/model_manager.h"
#include "renderer/renderer.h"
#include "global/global.h"


namespace Demo {
namespace Client {
namespace Renderer {


ModelEntity::ModelEntity
    (
    const std::string & name
    )
    {
    ModelManager & model_manager = Global::get_model_manager();
    model_data_object = model_manager.get_model_data_object( name );
    if( model_data_object->is_dummy() )
        {
        using namespace Massiv::Core;
        std::ostringstream oss;
        oss << "No data for model '" << name << "' available - using dummy model.";
        Massiv::Core::Global::log( Status::warning( Status::FACILITY_GENERIC,
                    Status::PRIORITY_HIGH ), oss.str() );
        }
    }


ModelEntity::~ModelEntity()
    {
    render_object.clear();
    model = NULL;
    model_data_object = NULL;
    }


void ModelEntity::render
    (
    const SystemTime &
    )
    {
    refresh_model();
    hope( !model.is_null() );

    apply_transformation();
    if( manager )
        {
        render_object.set_pipeline_lighting( *manager, this );
        }
    render_object.render();
    revert_transformation();
    }


void ModelEntity::refresh_model()
    {
    hope( !model_data_object.is_null() );
    Model::Reference new_model = model_data_object->get_model();
    if( model.is_null() || new_model->version != model->version )
        {
        // Model updated.

        model = new_model;
        model->initialize_render_object( render_object );
        }
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo


#ifndef ENABLE_INLINE
#include "model_entity_inline.h"
#endif
