/** @file opengl_ext_functions.h
 * Header with macros describing OpenGL extension functions.
 *
 * Before including this header, define the following macros:
 * - GLFN_BEGIN_EXTENSION
 * - GLFN_END_EXTENSION
 * - GLFN
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


/** @def GLFN_BEGIN_EXTENSION
 * Macro called at the beginning of each extension block.
 *
 * @param name Extension name (for example @c ARB_multitexture).
 */

/** @def GLFN_END_EXTENSION
 * Macro called at the end of each extension block.
 *
 * @param name Extension name.
 */

/** @def GLFN
 * Macro called inside extension block for each function.
 *
 * @param result    Return type
 * @param name      Function name (for example ActiveTextureARB).
 * @param arguments Function arguments.
 */

#if !defined( GLFN ) || !defined( GLFN_BEGIN_EXTENSION ) || !defined( GLFN_END_EXTENSION )
#error "Required macro not defined in opengl_ext_functions.h"
#endif


// ARB_multitexture. Part of OpenGL 1.3.

GLFN_BEGIN_EXTENSION( ARB_multitexture )
GLFN( void, ActiveTextureARB, ( GLenum texture ) )
GLFN( void, ClientActiveTextureARB, ( GLenum texture ) )
GLFN( void, MultiTexCoord1dARB, ( GLenum target, GLdouble s ) )
GLFN( void, MultiTexCoord1dvARB, ( GLenum target, const GLdouble * v ) )
GLFN( void, MultiTexCoord1fARB, ( GLenum target, GLfloat s ) )
GLFN( void, MultiTexCoord1fvARB, ( GLenum target, const GLfloat * v ) )
GLFN( void, MultiTexCoord1iARB, ( GLenum target, GLint s ) )
GLFN( void, MultiTexCoord1ivARB, ( GLenum target, const GLint * v ) )
GLFN( void, MultiTexCoord1sARB, ( GLenum target, GLshort s ) )
GLFN( void, MultiTexCoord1svARB, ( GLenum target, const GLshort * v ) )
GLFN( void, MultiTexCoord2dARB, ( GLenum target, GLdouble s, GLdouble t ) )
GLFN( void, MultiTexCoord2dvARB, ( GLenum target, const GLdouble * v ) )
GLFN( void, MultiTexCoord2fARB, ( GLenum target, GLfloat s, GLfloat t ) )
GLFN( void, MultiTexCoord2fvARB, ( GLenum target, const GLfloat * v ) )
GLFN( void, MultiTexCoord2iARB, ( GLenum target, GLint s, GLint t ) )
GLFN( void, MultiTexCoord2ivARB, ( GLenum target, const GLint * v ) )
GLFN( void, MultiTexCoord2sARB, ( GLenum target, GLshort s, GLshort t ) )
GLFN( void, MultiTexCoord2svARB, ( GLenum target, const GLshort * v ) )
GLFN( void, MultiTexCoord3dARB, ( GLenum target, GLdouble s, GLdouble t, GLdouble r ) )
GLFN( void, MultiTexCoord3dvARB, ( GLenum target, const GLdouble * v ) )
GLFN( void, MultiTexCoord3fARB, ( GLenum target, GLfloat s, GLfloat t, GLfloat r ) )
GLFN( void, MultiTexCoord3fvARB, ( GLenum target, const GLfloat * v ) )
GLFN( void, MultiTexCoord3iARB, ( GLenum target, GLint s, GLint t, GLint r ) )
GLFN( void, MultiTexCoord3ivARB, ( GLenum target, const GLint * v ) )
GLFN( void, MultiTexCoord3sARB, ( GLenum target, GLshort s, GLshort t, GLshort r ) )
GLFN( void, MultiTexCoord3svARB, ( GLenum target, const GLshort * v ) )
GLFN( void, MultiTexCoord4dARB, ( GLenum target, GLdouble s, GLdouble t, GLdouble r, GLdouble q ) )
GLFN( void, MultiTexCoord4dvARB, ( GLenum target, const GLdouble * v ) )
GLFN( void, MultiTexCoord4fARB, ( GLenum target, GLfloat s, GLfloat t, GLfloat r, GLfloat q ) )
GLFN( void, MultiTexCoord4fvARB, ( GLenum target, const GLfloat * v ) )
GLFN( void, MultiTexCoord4iARB, ( GLenum target, GLint s, GLint t, GLint r, GLint q ) )
GLFN( void, MultiTexCoord4ivARB, ( GLenum target, const GLint * v ) )
GLFN( void, MultiTexCoord4sARB, ( GLenum target, GLshort s, GLshort t, GLshort r, GLshort q ) )
GLFN( void, MultiTexCoord4svARB, ( GLenum target, const GLshort * v ) )
GLFN_END_EXTENSION( ARB_multitexture )


// EXT_draw_range_elements. Part of OpenGL 1.2.

GLFN_BEGIN_EXTENSION( EXT_draw_range_elements )
GLFN( void, DrawRangeElementsEXT, ( GLenum mode, GLuint start, GLuint end,
            GLsizei count, GLenum type, const GLvoid * indices ) )
GLFN_END_EXTENSION( EXT_draw_range_elements )

// EXT_compiled_vertex_array.

GLFN_BEGIN_EXTENSION( EXT_compiled_vertex_array )
GLFN( void, LockArraysEXT, ( GLint first, GLsizei count ) )
GLFN( void, UnlockArraysEXT, ( void ) )
GLFN_END_EXTENSION( EXT_compiled_vertex_array )
