/** @file opengl_extensions.h
 * Various OpenGL extensions:
 * - ARB_multitexture
 * - EXT_draw_range_elements
 * - EXT_compiled_vertex_array
 *
 * For each extension, standard @c GL_* constants are defined, all extension
 * functions are declared (without @c gl prefix, in <tt>namespace GL</tt>)
 * and boolean variable @c <extension-name>_supported is declared in @c GL
 * namespace.
 *
 * This header should be included only from opengl.h.
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef CLIENT_OPENGL_EXTENSIONS_H
#define CLIENT_OPENGL_EXTENSIONS_H


#ifndef CLIENT_OPENGL_H
#error "opengl_extensions.h should be included from opengl.h"
#endif


namespace GL {


// Constants.

// ARB_multitexture

#ifndef GL_ARB_multitexture
#define GL_ARB_multitexture 1

#define GL_TEXTURE0_ARB                         0x84C0
#define GL_TEXTURE1_ARB                         0x84C1
#define GL_TEXTURE2_ARB                         0x84C2
#define GL_TEXTURE3_ARB                         0x84C3
#define GL_TEXTURE4_ARB                         0x84C4
#define GL_TEXTURE5_ARB                         0x84C5
#define GL_TEXTURE6_ARB                         0x84C6
#define GL_TEXTURE7_ARB                         0x84C7
#define GL_TEXTURE8_ARB                         0x84C8
#define GL_TEXTURE9_ARB                         0x84C9
#define GL_TEXTURE10_ARB                        0x84CA
#define GL_TEXTURE11_ARB                        0x84CB
#define GL_TEXTURE12_ARB                        0x84CC
#define GL_TEXTURE13_ARB                        0x84CD
#define GL_TEXTURE14_ARB                        0x84CE
#define GL_TEXTURE15_ARB                        0x84CF
#define GL_TEXTURE16_ARB                        0x84D0
#define GL_TEXTURE17_ARB                        0x84D1
#define GL_TEXTURE18_ARB                        0x84D2
#define GL_TEXTURE19_ARB                        0x84D3
#define GL_TEXTURE20_ARB                        0x84D4
#define GL_TEXTURE21_ARB                        0x84D5
#define GL_TEXTURE22_ARB                        0x84D6
#define GL_TEXTURE23_ARB                        0x84D7
#define GL_TEXTURE24_ARB                        0x84D8
#define GL_TEXTURE25_ARB                        0x84D9
#define GL_TEXTURE26_ARB                        0x84DA
#define GL_TEXTURE27_ARB                        0x84DB
#define GL_TEXTURE28_ARB                        0x84DC
#define GL_TEXTURE29_ARB                        0x84DD
#define GL_TEXTURE30_ARB                        0x84DE
#define GL_TEXTURE31_ARB                        0x84DF
#define GL_ACTIVE_TEXTURE_ARB                   0x84E0
#define GL_CLIENT_ACTIVE_TEXTURE_ARB            0x84E1
#define GL_MAX_TEXTURE_UNITS_ARB                0x84E2

#endif // GL_ARB_multitexture


// EXT_draw_range_elements

#ifndef GL_EXT_draw_range_elements
#define GL_EXT_draw_range_elements 1

#define GL_MAX_ELEMENTS_VERTICES_EXT            0x80E8
#define GL_MAX_ELEMENTS_INDICES_EXT             0x80E9

#endif // GL_EXT_draw_range_elements


// EXT_compiled_vertex_array

#ifndef GL_EXT_compiled_vertex_array
#define GL_EXT_compiled_vertex_array 1

#define GL_ARRAY_ELEMENT_LOCK_FIRST_EXT         0x81A8
#define GL_ARRAY_ELEMENT_LOCK_COUNT_EXT         0x81A9

#endif // GL_EXT_compiled_vertex_array


// Declarations.

#define GLFN_BEGIN_EXTENSION( n )   extern bool n ## _supported;
#define GLFN( r, n, a )             extern r ( GLAPIENTRY * n ) a;
#define GLFN_END_EXTENSION( n )

#include "opengl_ext_functions.h"

#undef GLFN_END_EXTENSION
#undef GLFN_BEGIN_EXTENSION
#undef GLFN


} // namespace GL


#endif // CLIENT_OPENGL_EXTENSIONS_H
