/** @file opengl_types.cpp
 * OpenGL types (implementation).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/opengl_types.h"
#include "renderer/renderer.h"
#include "renderer/texture_manager.h"


namespace Demo {
namespace Client {
namespace OpenGL {


// Check sizes of the types at compile-time. Just to be safe.

namespace {

typedef int Vector2SizeIsOk
    [
        (
            ( sizeof( Vector2 ) == 2 * sizeof( GLfloat ) ) &&
            ( sizeof( Vector2 ) % 4 == 0 )
        ) ? 1 : -1
    ];

typedef int Vector3SizeIsOk
    [
        (
            ( sizeof( Vector3 ) == 3 * sizeof( GLfloat ) ) &&
            ( sizeof( Vector3 ) % 4 == 0 )
        ) ? 1 : -1
    ];

// note: it think sizeof( GLubyte ) could be > 1 in some OpenGL implementations,
//       but we require it to be 1.
typedef int ColorSizeIsOk
    [
        (
            ( sizeof( Color ) == 4 * sizeof( GLubyte ) ) &&
            ( sizeof( Color ) == 4 )
        ) ? 1 : -1
    ];

} // anonymous namespace


// class Vector2

void Vector2::gl_tex_coord
    (
    GLenum texture
    ) const
    {
    if( GL::ARB_multitexture_supported )
        {
        Renderer::TextureManager & texture_manager = Renderer::Global::get_texture_manager();
        hope( static_cast< int >( texture ) - GL_TEXTURE0_ARB <
            texture_manager.get_texture_units() );
        GL::MultiTexCoord2fvARB( texture, v );
        }
    else
        {
        hope( texture == GL_TEXTURE0_ARB );
        gl_tex_coord();
        }
    }


} // namespace Demo::Client::OpenGL
} // namespace Demo::Client
} // namespace Demo


#ifndef ENABLE_INLINE
#include "opengl_types_inline.h"
#endif
