/** @file opengl_types_inline.h
 * OpenGL types (inline methods).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo {
namespace Client {
namespace OpenGL {


// class Vector2

Vector2::Vector2()
    {
    v[ 0 ] = v[ 1 ] = 0.0f;
    }


Vector2::Vector2
    (
    const Vector2 & other
    )
    {
    *this = other;
    }


Vector2::Vector2
    (
    const SVector2 & other
    )
    {
    *this = other;
    }


Vector2::Vector2
    (
    GLfloat x,
    GLfloat y
    )
    {
    v[ 0 ] = x;
    v[ 1 ] = y;
    }


Vector2 & Vector2::operator=
    (
    const Vector2 & other
    )
    {
    v[ 0 ] = other.v[ 0 ];
    v[ 1 ] = other.v[ 1 ];
    return *this;
    }


Vector2 & Vector2::operator=
    (
    const SVector2 & other
    )
    {
    v[ 0 ] = static_cast< GLfloat >( other[ 0 ] );
    v[ 1 ] = static_cast< GLfloat >( other[ 1 ] );
    return *this;
    }


bool Vector2::operator==
    (
    const Vector2 & other
    ) const
    {
    return ( ( v[ 0 ] == other.v[ 0 ] ) &&
             ( v[ 1 ] == other.v[ 1 ] ) );
    }


bool Vector2::operator!=
    (
    const Vector2 & other
    ) const
    {
    return !( *this == other );
    }


GLfloat Vector2::x() const
    {
    return v[ 0 ];
    }


GLfloat Vector2::y() const
    {
    return v[ 1 ];
    }


SVector2 Vector2::svector() const
    {
    return SVector2( x(), y() );
    }


void Vector2::gl_vertex() const
    {
    glVertex2fv( v );
    }


void Vector2::gl_tex_coord() const
    {
    glTexCoord2fv( v );
    }


// class Vector3

Vector3::Vector3()
    {
    v[ 0 ] = v[ 1 ] = v[ 2 ] = 0.0f;
    }


Vector3::Vector3
    (
    const Vector3 & other
    )
    {
    *this = other;
    }


Vector3::Vector3
    (
    const SVector3 & other
    )
    {
    *this = other;
    }


Vector3::Vector3
    (
    GLfloat x,
    GLfloat y,
    GLfloat z
    )
    {
    v[ 0 ] = x;
    v[ 1 ] = y;
    v[ 2 ] = z;
    }


Vector3 & Vector3::operator=
    (
    const Vector3 & other
    )
    {
    v[ 0 ] = other.v[ 0 ];
    v[ 1 ] = other.v[ 1 ];
    v[ 2 ] = other.v[ 2 ];
    return *this;
    }


bool Vector3::operator==
    (
    const Vector3 & other
    ) const
    {
    return ( ( v[ 0 ] == other.v[ 0 ] ) &&
             ( v[ 1 ] == other.v[ 1 ] ) &&
             ( v[ 2 ] == other.v[ 2 ] ) );
    }


bool Vector3::operator!=
    (
    const Vector3 & other
    ) const
    {
    return !( *this == other );
    }


Vector3 & Vector3::operator=
    (
    const SVector3 & other
    )
    {
    v[ 0 ] = static_cast< GLfloat >( other[ 0 ] );
    v[ 1 ] = static_cast< GLfloat >( other[ 1 ] );
    v[ 2 ] = static_cast< GLfloat >( other[ 2 ] );
    return *this;
    }


GLfloat Vector3::x() const
    {
    return v[ 0 ];
    }


GLfloat Vector3::y() const
    {
    return v[ 1 ];
    }


GLfloat Vector3::z() const
    {
    return v[ 2 ];
    }


SVector3 Vector3::svector() const
    {
    return SVector3( x(), y(), z() );
    }


void Vector3::gl_vertex() const
    {
    glVertex3fv( v );
    }


void Vector3::gl_normal() const
    {
    glNormal3fv( v );
    }


// class Color

Color::Color()
    {
    rgba[ 0 ] = rgba[ 1 ] = rgba[ 2 ] = 0;
    rgba[ 3 ] = 255;
    }


Color::Color
    (
    const Color & other
    )
    {
    *this = other;
    }


Color::Color
    (
    const SColor & other
    )
    {
    set_red( other.red() );
    set_green( other.green() );
    set_blue( other.blue() );
    set_alpha( other.alpha() );
    }


Color::Color
    (
    GLfloat red,
    GLfloat green,
    GLfloat blue,
    GLfloat alpha
    )
    {
    set_red( red );
    set_green( green );
    set_blue( blue );
    set_alpha( alpha );
    }


Color::Color
    (
    GLubyte red,
    GLubyte green,
    GLubyte blue,
    GLubyte alpha
    )
    {
    set_red( red );
    set_green( green );
    set_blue( blue );
    set_alpha( alpha );
    }


Color::Color
    (
    const Vector3 & rgb,
    GLfloat         alpha
    )
    {
    set_red( rgb.x() );
    set_green( rgb.y() );
    set_blue( rgb.z() );
    set_alpha( alpha );
    }


Color & Color::operator=
    (
    const Color & other
    )
    {
    rgba[ 0 ] = other.rgba[ 0 ];
    rgba[ 1 ] = other.rgba[ 1 ];
    rgba[ 2 ] = other.rgba[ 2 ];
    rgba[ 3 ] = other.rgba[ 3 ];
    return *this;
    }


Color & Color::operator=
    (
    const SColor & other
    )
    {
    set_red( other.red() );
    set_green( other.green() );
    set_blue( other.blue() );
    set_alpha( other.alpha() );
    return *this;
    }


bool Color::operator==
    (
    const Color & other
    ) const
    {
    return ( ( rgba[ 0 ] == other.rgba[ 0 ] ) &&
             ( rgba[ 1 ] == other.rgba[ 1 ] ) &&
             ( rgba[ 2 ] == other.rgba[ 2 ] ) &&
             ( rgba[ 3 ] == other.rgba[ 3 ] ) );
    }


bool Color::operator!=
    (
    const Color & other
    ) const
    {
    return !( *this == other );
    }


bool Color::operator<
    (
    const Color & other
    ) const
    {
    if( rgba[ 0 ] == other.rgba[ 0 ] )
        {
        if( rgba[ 1 ] == other.rgba[ 1 ] )
            {
            if( rgba[ 2 ] == other.rgba[ 2 ] )
                {
                return ( rgba[ 3 ] < other.rgba[ 3 ] );
                }
            else
                {
                return ( rgba[ 2 ] < other.rgba[ 2 ] );
                }
            }
        else
            {
            return ( rgba[ 1 ] < other.rgba[ 1 ] );
            }
        }
    else
        {
        return ( rgba[ 0 ] < other.rgba[ 0 ] );
        }
    }


GLfloat Color::red() const
    {
    return ubyte2float( rgba[ 0 ] );
    }


GLfloat Color::green() const
    {
    return ubyte2float( rgba[ 1 ] );
    }


GLfloat Color::blue() const
    {
    return ubyte2float( rgba[ 2 ] );
    }


GLfloat Color::alpha() const
    {
    return ubyte2float( rgba[ 3 ] );
    }
    

GLubyte Color::red_byte() const
    {
    return rgba[ 0 ];
    }


GLubyte Color::green_byte() const
    {
    return rgba[ 1 ];
    }


GLubyte Color::blue_byte() const
    {
    return rgba[ 2 ];
    }


GLubyte Color::alpha_byte() const
    {
    return rgba[ 3 ];
    }


Vector3 Color::rgb() const
    {
    return Vector3( red(), green(), blue() );
    }

SColor Color::scolor() const
    {
    return SColor( red(), green(), blue(), alpha() );
    }


void Color::set_red
    (
    GLfloat red
    )
    {
    rgba[ 0 ] = float2ubyte( red );
    }


void Color::set_green
    (
    GLfloat green
    )
    {
    rgba[ 1 ] = float2ubyte( green );
    }


void Color::set_blue
    (
    GLfloat blue
    )
    {
    rgba[ 2 ] = float2ubyte( blue );
    }


void Color::set_alpha
    (
    GLfloat alpha
    )
    {
    rgba[ 3 ] = float2ubyte( alpha );
    }


void Color::set_red
    (
    GLubyte red
    )
    {
    rgba[ 0 ] = red;
    }


void Color::set_green
    (
    GLubyte green
    )
    {
    rgba[ 1 ] = green;
    }


void Color::set_blue
    (
    GLubyte blue
    )
    {
    rgba[ 2 ] = blue;
    }


void Color::set_alpha
    (
    GLubyte alpha
    )
    {
    rgba[ 3 ] = alpha;
    }

void Color::set_rgb
    (
    const Vector3 & rgb
    )
    {
    set_red( rgb.x() );
    set_green( rgb.y() );
    set_blue( rgb.z() );
    }


void Color::gl_color() const
    {
    glColor4ubv( rgba );
    }


GLubyte Color::float2ubyte
    (
    GLfloat x
    )
    {
    const Float32 float_rounded = Massiv::Core::FloatTraits< Float32 >::round( x * 255.0f );
    const int rounded = static_cast< int >( float_rounded );
    return static_cast< GLubyte >( Massiv::Core::clamp( rounded, 0, 255 ) );
    }


GLfloat Color::ubyte2float
    (
    GLubyte x
    )
    {
    return static_cast< GLfloat >( x / 255.0 );
    }


// class FloatColor

FloatColor::FloatColor()
    {
    rgba[ 0 ] = rgba[ 1 ] = rgba[ 2 ] = 0.0f;
    rgba[ 3 ] = 1.0f;
    }


FloatColor::FloatColor
    (
    const FloatColor & other
    )
    {
    *this = other;
    }


FloatColor::FloatColor
    (
    const SColor & other
    )
    {
    set_red( other.red() );
    set_green( other.green() );
    set_blue( other.blue() );
    set_alpha( other.alpha() );
    }


FloatColor::FloatColor
    (
    GLfloat red,
    GLfloat green,
    GLfloat blue,
    GLfloat alpha
    )
    {
    set_red( red );
    set_green( green );
    set_blue( blue );
    set_alpha( alpha );
    }


FloatColor::FloatColor
    (
    const Vector3 & rgb,
    GLfloat         alpha
    )
    {
    set_red( rgb.x() );
    set_green( rgb.y() );
    set_blue( rgb.z() );
    set_alpha( alpha );
    }


FloatColor & FloatColor::operator=
    (
    const FloatColor & other
    )
    {
    rgba[ 0 ] = other.rgba[ 0 ];
    rgba[ 1 ] = other.rgba[ 1 ];
    rgba[ 2 ] = other.rgba[ 2 ];
    rgba[ 3 ] = other.rgba[ 3 ];
    return *this;
    }


FloatColor & FloatColor::operator=
    (
    const SColor & other
    )
    {
    set_red( other.red() );
    set_green( other.green() );
    set_blue( other.blue() );
    set_alpha( other.alpha() );
    return *this;
    }


bool FloatColor::operator==
    (
    const FloatColor & other
    ) const
    {
    return ( ( rgba[ 0 ] == other.rgba[ 0 ] ) &&
             ( rgba[ 1 ] == other.rgba[ 1 ] ) &&
             ( rgba[ 2 ] == other.rgba[ 2 ] ) &&
             ( rgba[ 3 ] == other.rgba[ 3 ] ) );
    }


bool FloatColor::operator!=
    (
    const FloatColor & other
    ) const
    {
    return !( *this == other );
    }

    
GLfloat FloatColor::red() const
    {
    return rgba[ 0 ];
    }


GLfloat FloatColor::green() const
    {
    return rgba[ 1 ];
    }


GLfloat FloatColor::blue() const
    {
    return rgba[ 2 ];
    }


GLfloat FloatColor::alpha() const
    {
    return rgba[ 3 ];
    }


Vector3 FloatColor::rgb() const
    {
    return Vector3( red(), green(), blue() );
    }

SColor FloatColor::scolor() const
    {
    return SColor( red(), green(), blue(), alpha() );
    }


void FloatColor::set_red
    (
    GLfloat red
    )
    {
    rgba[ 0 ] = red;
    }


void FloatColor::set_green
    (
    GLfloat green
    )
    {
    rgba[ 1 ] = green;
    }


void FloatColor::set_blue
    (
    GLfloat blue
    )
    {
    rgba[ 2 ] = blue;
    }


void FloatColor::set_alpha
    (
    GLfloat alpha
    )
    {
    rgba[ 3 ] = alpha;
    }


void FloatColor::set_rgb
    (
    const Vector3 & rgb
    )
    {
    set_red( rgb.x() );
    set_green( rgb.y() );
    set_blue( rgb.z() );
    }


void FloatColor::gl_color() const
    {
    glColor4fv( rgba );
    }


} // namespace Demo::Client::OpenGL
} // namespace Demo::Client
} // namespace Demo

