/** @file overlay_object_inline.h
 * Generic 2D object (inline methods).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo {
namespace Client {
namespace Renderer {


bool OverlayObject::is_visible() const
    {
    return visible && ( get_current_color().alpha() > 0.0f );
    }


OverlayObject::ScreenCorner OverlayObject::get_origin() const
    {
    return origin;
    }


const SVector2 & OverlayObject::get_base_offset() const
    {
    return offset;
    }


const SVector2 & OverlayObject::get_base_size() const
    {
    return size;
    }


const SColor & OverlayObject::get_base_color() const
    {
    return color;
    }


SVector2 OverlayObject::get_current_offset() const
    {
    return offset + offset_delta;
    }


SVector2 OverlayObject::get_current_size() const
    {
    return size + size_delta;
    }


SColor OverlayObject::get_current_color() const
    {
    return color * color_factor + color_delta;
    }


void OverlayObject::set_visibility
    (
    bool flag
    )
    {
    visible = flag;
    }


void OverlayObject::set_origin
    (
    ScreenCorner new_origin
    )
    {
    if( origin != new_origin )
        {
        origin = new_origin;
        position_dirty = true;
        }
    }


void OverlayObject::set_base_offset
    (
    const SVector2 & new_offset
    )
    {
    if( offset != new_offset )
        {
        offset = new_offset;
        position_dirty = true;
        }
    }


void OverlayObject::set_base_size
    (
    const SVector2 & new_size
    )
    {
    if( size != new_size )
        {
        size = new_size;
        size_dirty = true;
        }
    }


void OverlayObject::set_base_color
    (
    const SColor & new_color
    )
    {
    if( color != new_color )
        {
        color = new_color;
        color_dirty = true;
        }
    }


void OverlayObject::add_offset_delta
    (
    const SVector2 & delta
    )
    {
    if( delta != SVector2() )
        {
        offset_delta += delta;
        position_dirty = true;
        }
    }


void OverlayObject::add_size_delta
    (
    const SVector2 & delta
    )
    {
    if( delta != SVector2() )
        {
        size_delta += delta;
        size_dirty = true;
        }
    }


void OverlayObject::add_color_delta
    (
    const SColor & delta
    )
    {
    if( delta != SColor() )
        {
        color_delta += delta;
        color_dirty = true;
        }
    }

    
void OverlayObject::multiply_color_factor
    (
    const SColor & factor
    )
    {
    if( factor != SColor( 1.0f, 1.0f, 1.0f, 1.0f ) )
        {
        color_factor *= factor;
        color_dirty = true;
        }
    }

    
void OverlayObject::unregister_from_manager()
    {
    register_to_manager( NULL, 0 );
    }


OverlayObject * OverlayObject::get_parent_overlay_object() const
    {
    return parent;
    }


#define GET_ANIMATION( name, Type ) \
    Type * OverlayObject::get_ ## name \
        ( \
        int index \
        ) \
        { \
        return static_cast< Type * >( get_animation( typeid( Type ), index ) ); \
        } \
    \
    Type & OverlayObject::require_ ## name \
        ( \
        int index \
        ) \
        { \
        return *static_cast< Type * >( require_animation( typeid( Type ), index ) ); \
        }

GET_ANIMATION( offset_animation, OverlayOffsetAnimation )              
GET_ANIMATION( color_factor_animation, OverlayColorFactorAnimation )


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo
