/** @file pass_manager_inline.h
 * Manager of rendering passes (inline methods).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo {
namespace Client {
namespace Renderer {


// class PassManager::Pass

PassManager::Pass::Pass()
    {
    }


PassManager::Pass::~Pass()
    {
    }


PassManager::Pass::Pass
    (
    const Pass & other
    ) : overlay_manager( other.overlay_manager ),
        render_manager( other.render_manager )
    {
    }


PassManager::Pass & PassManager::Pass::operator=
    (
    const Pass & other
    )
    {
    overlay_manager = other.overlay_manager;
    render_manager = other.render_manager;
    return *this;
    }


OverlayManager & PassManager::Pass::create_overlay_manager()
    {
    if( !overlay_manager.get() )
        {
        overlay_manager.reset( new OverlayManager );
        }
    return *overlay_manager;
    }


RenderManager & PassManager::Pass::create_render_manager()
    {
    if( !render_manager.get() )
        {
        render_manager.reset( new RenderManager );
        }
    return *render_manager;
    }


bool PassManager::Pass::has_overlay_manager()
    {
    return overlay_manager.get() != NULL;
    }


bool PassManager::Pass::has_render_manager()
    {
    return render_manager.get() != NULL;
    }


OverlayManager & PassManager::Pass::get_overlay_manager()
    {
    hope( overlay_manager.get() );
    return *overlay_manager;
    }


RenderManager & PassManager::Pass::get_render_manager()
    {
    hope( render_manager.get() );
    return *render_manager;
    }


// class PassManager

PassManager::Pass & PassManager::create_pass
    (
    int pass_index
    )
    {
    return passes[ pass_index ];
    }


OverlayManager & PassManager::create_overlay_manager
    (
    int pass_index
    )
    {
    return passes[ pass_index ].create_overlay_manager();
    }


RenderManager & PassManager::create_render_manager
    (
    int pass_index
    )
    {
    return passes[ pass_index ].create_render_manager();
    }


bool PassManager::has_pass
    (
    int pass_index
    )
    {
    Pass::Map::iterator it = passes.find( pass_index );
    return it != passes.end();
    }


bool PassManager::has_overlay_manager
    (
    int pass_index
    )
    {
    if( has_pass( pass_index ) )
        {
        return get_pass( pass_index ).has_overlay_manager();
        }
    return false;
    }


bool PassManager::has_render_manager
    (
    int pass_index
    )
    {
    if( has_pass( pass_index ) )
        {
        return get_pass( pass_index ).has_render_manager();
        }
    return false;
    }


PassManager::Pass & PassManager::get_pass
    (
    int pass_index
    )
    {
    Pass::Map::iterator it = passes.find( pass_index );
    hope( it != passes.end() );
    return it->second;
    }


OverlayManager & PassManager::get_overlay_manager
    (
    int pass_index
    )
    {
    return get_pass( pass_index ).get_overlay_manager();
    }


RenderManager & PassManager::get_render_manager
    (
    int pass_index
    )
    {
    return get_pass( pass_index ).get_render_manager();
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo
