/** @file render_entity.cpp
 * Generic 3D renderable entity (implementation).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/render_entity.h"
#include "renderer/render_manager.h"
#include "renderer/pass_manager.h"
#include "renderer/renderer.h"


namespace Demo {
namespace Client {
namespace Renderer {


RenderEntity::RenderEntity()
      : visible( true ),
        scale( 1.0f, 1.0f, 1.0f ),
        manager( NULL )
    {
    }


RenderEntity::~RenderEntity()
    {
    unregister_from_manager();
    }


void RenderEntity::register_to_manager
    (
    RenderManager * new_manager
    )
    {
    if( manager )
        {
        manager->unregister_entity( this );
        manager = NULL;
        }
    if( new_manager )
        {
        manager = new_manager;
        manager->register_entity( this );
        }
    }


void RenderEntity::register_to_global_manager
    (
    int pass_index
    )
    {
    PassManager & pass_manager = Global::get_pass_manager();
    RenderManager & render_manager = pass_manager.get_render_manager( pass_index );
    register_to_manager( &render_manager );
    }


void RenderEntity::apply_transformation() const
    {
    using namespace ::Massiv::Core;
    
    glPushMatrix();

    // Note that the latter transformations will be applied sooner, so read
    // this code from bottom to top. The transformations will be applied in the
    // following order:
    // - Scale the object.
    // - Rotate the object. First apply bank, then pitch, then heading (this is
    //   reverse order from the one in #Orientation documentation because we rotate
    //   the object about world-space axes, not object-space axes).
    // - Translate the object.

    // Translate.

    glTranslatef( position.x(), position.y(), position.z() );

    // Rotate.

    glRotatef( FloatTraits< Float32 >::radians_to_degrees( orientation.heading() ),
                0.0f, 0.0f, 1.0f );
    glRotatef( FloatTraits< Float32 >::radians_to_degrees( orientation.pitch() ),
                1.0f, 0.0f, 0.0f );
    glRotatef( FloatTraits< Float32 >::radians_to_degrees( orientation.bank() ),
                0.0f, 1.0f, 0.0f );

    // Scale.

    glScalef( scale.x(), scale.y(), scale.z() );
    }


void RenderEntity::revert_transformation() const
    {
    glPopMatrix();
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo


#ifndef ENABLE_INLINE
#include "render_entity_inline.h"
#endif
