/** @file render_object_dump.cpp
 * Vertex-array based object - dumping.
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/render_object.h"
#include <iostream>


namespace Demo {
namespace Client {
namespace Renderer {


namespace {


/**
 * Convert render mode to string.
 */
std::string gl_mode
    (
    GLenum mode
    )
    {
    switch( mode )
        {
        case GL_TRIANGLES:
            return "triangles";
        case GL_TRIANGLE_STRIP:
            return "triangle strip";
        case GL_TRIANGLE_FAN:
            return "triangle fan";
        case GL_QUADS:
            return "quads";
        // FIXME: all gl begin-end modes
        }
    return "unknown";
    }


} // anonymous namespace


void RenderObject::dump
    (
    std::ostream &      stream,
    const std::string & indent
    )
    {
    // Dump array.

    array.dump( stream, indent );

    // Dump jobs.

    std::size_t i;
    for( i = 0; i < jobs.size(); i++ )
        {
        Job & job = jobs[ i ];

        // FIXME: dump pipeline state

        if( job.geometry )
            {
            Geometry & g = *job.geometry;

            // Mode.

            stream << "\n" << indent << "Job " << i << " geometry: " << gl_mode( g.get_mode() );

            // Indices.

            if( g.is_index_array() )
                {
                // Index array.

                g.lock();
                stream << "\n" << indent << "    index array, " << g.get_size() << " elements:";
                std::size_t j;
                for( j = 0; j < g.get_size(); j++ )
                    {
                    if( j % 8 == 0 )
                        {
                        stream << "\n" << indent << "        ";
                        }
                    else
                        {
                        stream << ", ";
                        }
                    stream << g.indices[ j ];
                    }
                g.unlock();
                }
            else
                {
                // Index range.

                stream << "\n" << indent << "    index range: " << g.get_first() <<
                            " to " << g.get_first() + g.get_size() - 1;
                }
            }
        }
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo
