/** @file render_object_inline.h
 * Vertex-array based object (inline methods).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo {
namespace Client {
namespace Renderer {


// class RenderObject::Geometry

RenderObject::Geometry::Geometry()
      : mode( GL_POINTS ),
        memory( NULL ),
        size( 0 ),
        first( 0 ),
        locked( false )
    {
    }


RenderObject::Geometry::Geometry
    (
    const Geometry & other
    ) : mode( other.mode ),
        memory( other.memory ),
        size( other.size ),
        first( other.first ),
        locked( false )
    {
    // It's error to copy locked geometry.

    hope( !other.locked );
    other.memory = NULL;
    }


RenderObject::Geometry & RenderObject::Geometry::operator=
    (
    const Geometry & other
    )
    {
    // It's error to assign locked geometry.

    hope( !other.locked );

    mode = other.mode;
    memory = other.memory;
    other.memory = NULL;
    size = other.size;
    first = other.first;
    locked = false;

    return *this;
    }


void RenderObject::Geometry::initialize_range
    (
    GLenum      the_mode,
    std::size_t first_element,
    std::size_t element_count
    )
    {
    hope( !memory );
    hope( size == 0 );
    hope( element_count > 0 );

    mode = the_mode;
    first = first_element;
    size = element_count;
    }

    
bool RenderObject::Geometry::is_range() const
    {
    return ( memory == NULL );
    }


bool RenderObject::Geometry::is_index_array() const
    {
    return !is_range();
    }


GLenum RenderObject::Geometry::get_mode() const
    {
    return mode;
    }


const GLushort * RenderObject::Geometry::get_memory() const
    {
    return memory;
    }


GLushort * RenderObject::Geometry::get_memory()
    {
    return memory;
    }


std::size_t RenderObject::Geometry::get_size() const
    {
    return size;
    }


std::size_t RenderObject::Geometry::get_first() const
    {
    return first;
    }


bool RenderObject::Geometry::is_locked() const
    {
    return locked;
    }


// class RenderObject::Job

RenderObject::Job::Job()
      : pipeline_state( NULL ),
        geometry( NULL )
    {
    }


RenderObject::Job::~Job()
    {
    }


// class RenderObject

RenderObject::RenderObject()
    {
    }


RenderObject::~RenderObject()
    {
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo

