/** @file simple_entity.cpp
 * Simple render entity (implementation).
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/simple_entity.h"


namespace Demo {
namespace Client {
namespace Renderer {


SimpleEntity::SimpleEntity()
      : render_objects( NULL ),
        render_object_count( 0 )
    {
    }


SimpleEntity::~SimpleEntity()
    {
    if( render_objects )
        {
        delete [] render_objects;
        }
    }


void SimpleEntity::allocate
    (
    std::size_t object_count
    )
    {
    hope( object_count > 0 );
    hope( !render_objects );
    hope( render_object_count == 0 );
    render_objects = new RenderObject[ object_count ];
    render_object_count = object_count;
    }


void SimpleEntity::render
    (
    const SystemTime &
    )
    {
    hope( render_objects );
    apply_transformation();

    std::size_t i;
    for( i = 0; i < render_object_count; i++ )
        {
        // Note: OpenGL lighting of SimpleEntity is not supported.

        render_objects[ i ].render();
        }

    revert_transformation();
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo


#ifndef ENABLE_INLINE
#include "simple_entity_inline.h"
#endif
