/** @file simple_entity.h
 * Simple render entity.
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_SIMPLE_ENTITY_H
#define CLIENT_SIMPLE_ENTITY_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef CLIENT_OPENGL_TYPES_H
#include "renderer/opengl_types.h"
#endif

#ifndef CLIENT_RENDERER_FWD_H
#include "renderer/renderer_fwd.h"
#endif

#ifndef CLIENT_RENDER_ENTITY_H
#include "renderer/render_entity.h"
#endif

#ifndef CLIENT_RENDER_OBJECT_H
#include "renderer/render_object.h"
#endif

#include <vector>


namespace Demo {
namespace Client {
namespace Renderer {


/**
 * Simple render entity.
 *
 * #SimpleEntity is just a simple list of #RenderObjects. If offers several
 * ways to initialize the list to basic shapes:
 * - Allocate and initialize #render_objects and #render_object_count
 *   manually.
 * - Allocate #render_objects manually and use @c initialize_*() or @c create_*()
 *   functions from namespace Demo::Client::Renderer::Objects to initialize the objects.
 */
class SimpleEntity : public RenderEntity
    {
    NO_COPY_CONSTRUCTOR( SimpleEntity );
    NO_ASSIGNMENT_OPERATOR( SimpleEntity );

public:

    /**
     * Constructor.
     */
    SimpleEntity();

    /**
     * Destructor.
     */
    virtual ~SimpleEntity();

public:

    /**
     * Allocate given number of render objects.
     *
     * @pre No render objects are allocated.
     */
    virtual void allocate
        (
        std::size_t object_count = 1
        );

public:

    /**
     * Render the entity.
     *
     * Simple entities are not lit by #RenderManager lights.
     */
    virtual void render
        (
        const SystemTime & time_delta
        );

public:

    RenderObject *  render_objects;
        /**< Array of render objects. */

    std::size_t     render_object_count;
        /**< Length of #render_objects array. */

    }; // class SimpleEntity


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo


#ifdef ENABLE_INLINE
#include "simple_entity_inline.h"
#endif


#endif // CLIENT_SIMPLE_ENTITY_H
