/** @file simple_font.cpp
 * Simple font.
 */

/*
 * begin        : Mon Mar 10 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/simple_font.h"
#include "renderer/renderer.h"
#include "renderer/texture_manager.h"
#include "renderer/pipeline_state.h"
#include "client_global.h"


namespace Demo {
namespace Client {


SimpleFont::SimpleFont() :
    texture_width( 256 ),
    texture_height( 128 ),
    texture_char_width( 16 ),
    texture_char_height( 16 ),
    char_width( 8 ),
    char_height( 16 ),
    first_char( 32 ),
    char_count( 128 )
    {
    }
    
    
SimpleFont::~SimpleFont()
    {
    }
    
    
void SimpleFont::initialize
    (
    const std::string & name
    )
    {
    hope( texture_data.is_null() );
    texture_data = Renderer::Global::get_texture_manager().get_texture_data_object( name );
    }
    
    
void SimpleFont::draw
    (
    float x,
    float y,
    const std::string & text
    )
    {
    // Change pipeline state.

    Renderer::PipelineState state;
    state.texture[ 0 ].data = texture_data;
    state.texture[ 0 ].flags = Renderer::Texture::EXTEND_IMAGE |
                               Renderer::TextureDataObject::INTENSITY_ALPHA;
    state.blend.enable = true;
    state.blend.source = GL_ONE;
    state.blend.destination = GL_ONE_MINUS_SRC_ALPHA;
    Renderer::Global::apply_state( state );
    
    std::string::const_iterator it;
    for( it = text.begin(); it != text.end(); ++it )
        {
        int c = ( *it );
        draw_char( x, y, c );
        x += char_width;
        }
    }
    
    
float SimpleFont::get_char_width()
    {
    return char_width;
    }


float SimpleFont::get_char_height()
    {
    return char_height;
    }


void SimpleFont::draw_char
    (
    float x,
    float y,
    int   c
    )
    {
    if( c >= first_char && ( c - first_char ) < char_count )
        {
        float region[ 8 ] = 
                {
                x, y,
                x + char_width, y,
                x + char_width, y + char_height,
                x, y + char_height
                };
        float uv[ 8 ];
        
        generate_char_uv( c, uv );
        draw_rectangle( region, uv );
        }
    }
    
    
void SimpleFont::generate_char_uv
    (
    int c,
    float uv[ 8 ]
    )
    {
    int k, l;
    
    c = c - first_char;

    k = c % ( int )( texture_width / texture_char_width );
    l = c / ( int )( texture_width / texture_char_width );

    uv[ 0 ] = ( k * texture_char_width ) / texture_width;
    uv[ 1 ] = ( l * texture_char_height ) / texture_height;
    k++; l++;
    uv[ 4 ] = ( k * texture_char_width ) / texture_width;
    uv[ 5 ] = ( l * texture_char_height ) / texture_height;
    
    uv[ 2 ] = uv[ 4 ];
    uv[ 3 ] = uv[ 1 ];
    
    uv[ 6 ] = uv[ 0 ];
    uv[ 7 ] = uv[ 5 ];
    }
    
    
void SimpleFont::draw_rectangle
    (
    const float s[ 8 ],
    const float uv[ 8 ]
    )
    {
        {
        glBegin( GL_TRIANGLE_STRIP );

        glTexCoord2fv( uv + 0 );
        glVertex2fv( s + 0 );

        glTexCoord2fv( uv + 2 );
        glVertex2fv( s + 2 );

        glTexCoord2fv( uv + 6 );
        glVertex2fv( s + 6 );

        glTexCoord2fv( uv + 4 );
        glVertex2fv( s + 4 );

        glEnd();
        }
    }
    
    
} // namespace Client
} // namespace Demo
