/** @file simple_font.h
 * Simple font.
 */

/*
 * begin        : Mon Mar 10 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

 
#ifndef CLIENT_SIMPLE_FONT_H
#define CLIENT_SIMPLE_FONT_H


#include "client_common.h"
#include "renderer/texture_data_object.h"


namespace Demo {
namespace Client {


/**
 * Simple (stupid and slow) font.
 * Quick temporary hack - to be romoved soon.
 */
class SimpleFont
    {
public:
    /** Constructor. */
    SimpleFont();
    
    /** Destructor. */
    ~SimpleFont();
    
    /** Initialize. */
    void initialize
        (
        const std::string & texture_name
        );
        
public:
    /**
     * Draw text.
     * @param x Coordinate on screen relative to left top corner.
     * @param y Coordinate on screen relative to left top corner.
     */
    void draw
        (
        float x,
        float y,
        const std::string & text
        );
    
public:
    /** Returns char width. */
    float get_char_width();

    /** Returns char height. */
    float get_char_height();

private:
    /** Draw one character. */
    void draw_char
        (
        float x,
        float y,
        int   c
        );
    
    /** Generate char's uv coordinates. */
    void generate_char_uv
        (
        int c,
        float uv[ 8 ]
        );
    
    /**
     * Draw rectangle.
     * Corners are drawn in top left, top right, bottom right and
     * bottom left order.
     */
    void draw_rectangle
        (
        const float region[ 8 ],
        const float uv[ 8 ]
        );
    
private:

    Renderer::TextureDataObject::Reference texture_data;
        /**< Font texture data object. */
        
    float   texture_width;
        /**< Texture width in pixels. */
    
    float   texture_height;
        /**< Texture height in pixels */
        
    float   texture_char_width;
        /**< Char width in texture. */
        
    float   texture_char_height;
        /**< Char height in texture. */
        
    float   char_width;
        /**< Char width in pixels. */
        
    float   char_height;
        /**< CHar height in pixels. */
        
    int     first_char;
        /**< Ascii code of first char in the texture. */
        
    int     char_count;
        /**< Number of chars in the texture. */

    }; // SimpleFont


} // namespace Client
} // namespace Demo


#endif // CLIENT_SIMPLE_FONT_H
