/** @file terrain_build_inline.h
 * Terrain building (inline methods).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo {
namespace Client {
namespace Renderer {


// struct TerrainBuild::PointerHash

std::size_t TerrainBuild::PointerHash::operator()
    (
    const TextureDataObject * ptr
    ) const
    {
    return reinterpret_cast< std::size_t >( ptr );
    }

    
// class TerrainBuild::VertexInfo

TerrainBuild::VertexInfo::VertexInfo()
      : height( 0.0f ),
        array_index( -1 )
    {
    weights[ 0 ] = 0.0f;
    weights[ 1 ] = 0.0f;
    }


bool TerrainBuild::VertexInfo::has_texture
    (
    TextureDataObject * texture
    ) const
    {
    hope( texture );
    if( textures[ 0 ].get() == texture )
        {
        return true;
        }
    if( textures[ 1 ].get() == texture )
        {
        return true;
        }
    return false;
    }


Float32 TerrainBuild::VertexInfo::get_texture_weight
    (
    TextureDataObject * texture
    ) const
    {
    hope( texture );
    if( textures[ 0 ].get() == texture )
        {
        return weights[ 0 ];
        }
    if( textures[ 1 ].get() == texture )
        {
        return weights[ 1 ];
        }
    return 0.0f;
    }


// class TerrainBuild::ArrayVertex

TerrainBuild::ArrayVertex::ArrayVertex
    (
    int the_x,
    int the_y
    ) : x( the_x ),
        y( the_y )
    {
    }


// class TerrainBuild::Triangle

TerrainBuild::Triangle::Triangle
    (
    int the_v1,
    int the_v2,
    int the_v3
    ) : v1( the_v1 ),
        v2( the_v2 ),
        v3( the_v3 )
    {
    }


// class TerrainBuild::TriangleStrip

TerrainBuild::TriangleStrip::TriangleStrip
    (
    int     vertex_count,
    GLenum  the_front_face
    ) : indices( vertex_count ),
        front_face( the_front_face )
    {
    }


// class TerrainBuild

TerrainBuild::VertexInfo & TerrainBuild::get_vertex_info
    (
    int x,
    int y
    )
    {
    hope( x >= 0 );
    hope( x < heightmap.x_size );
    hope( y >= 0 );
    hope( y < heightmap.y_size );
    return vertex_infos[ y * heightmap.x_size + x ];
    }


const TerrainBuild::VertexInfo & TerrainBuild::get_vertex_info
    (
    int x,
    int y
    ) const
    {
    hope( x >= 0 );
    hope( x < heightmap.x_size );
    hope( y >= 0 );
    hope( y < heightmap.y_size );
    return vertex_infos[ y * heightmap.x_size + x ];
    }


void TerrainBuild::strip_vertex_coordinates
    (
    int &   x,
    int &   y,
    int     strip_index,
    int     vertex_index
    ) const
    {
    hope( strip_index >= 0 );
    hope( strip_index < heightmap.y_size - 1 );
    hope( vertex_index >= 0 );
    hope( vertex_index < 2 * heightmap.x_size );

    if( vertex_index & 1 )
        {
        // Northern line.

        x = ( vertex_index - 1 ) / 2;
        y = strip_index + 1;
        }
    else
        {
        // Southern line.

        x = vertex_index / 2;
        y = strip_index;
        }
    }


void TerrainBuild::strip_triangle_vertices
    (
    int &   i1,
    int &   i2,
    int &   i3,
    int     triangle_index
    ) const
    {
    hope( triangle_index >= 0 );
    hope( triangle_index < 2 * ( heightmap.x_size - 1 ) );

    if( triangle_index & 1 )
        {
        i1 = triangle_index;
        i2 = triangle_index + 2;
        i3 = triangle_index + 1;
        }
    else
        {
        i1 = triangle_index;
        i2 = triangle_index + 1;
        i3 = triangle_index + 2;
        }
    }


OpenGL::Vector3 TerrainBuild::get_heightmap_opengl_vertex
    (
    int     x,
    int     y,
    Float32 height
    )
    {
    return OpenGL::Vector3
        (
        static_cast< GLfloat >( x ),
        static_cast< GLfloat >( y ),
        static_cast< GLfloat >( height )
        );
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo
