/** @file terrain_inline.h
 * Heightmap-based 3D terrain (inline methods).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo {
namespace Client {
namespace Renderer {


// class Heightmap

Float32 Heightmap::get_height
    (
    int x,
    int y
    ) const
    {
    hope( x >= -1 );
    hope( x <= x_size );
    hope( y >= -1 );
    hope( y <= y_size );
    return elements[ ( y + 1 ) * ( x_size + 2 ) + ( x + 1 ) ].height;
    }


Int32 Heightmap::get_material
    (
    int x,
    int y
    ) const
    {
    hope( x >= 0 );
    hope( x < x_size );
    hope( y >= 0 );
    hope( y < y_size );
    return elements[ ( y + 1 ) * ( x_size + 2 ) + ( x + 1 ) ].material_id;
    }


Int32 Heightmap::get_tile_flags
    (
    int x,
    int y
    ) const
    {
    hope( x >= 0 );
    hope( x < x_size - 1 );
    hope( y >= 0 );
    hope( y < y_size - 1 );
    return elements[ ( y + 1 ) * ( x_size + 2 ) + ( x + 1 ) ].tile_flags;
    }

    
void Heightmap::set_height
    (
    int     x,
    int     y,
    Float32 height
    )
    {
    hope( x >= -1 );
    hope( x <= x_size );
    hope( y >= -1 );
    hope( y <= y_size );
    elements[ ( y + 1 ) * ( x_size + 2 ) + ( x + 1 ) ].height = height;
    }


void Heightmap::set_material
    (
    int     x,
    int     y,
    Int32   material
    )
    {
    hope( x >= 0 );
    hope( x < x_size );
    hope( y >= 0 );
    hope( y < y_size );
    elements[ ( y + 1 ) * ( x_size + 2 ) + ( x + 1 ) ].material_id = material;
    }


void Heightmap::set_tile_flags
    (
    int     x,
    int     y,
    Int32   tile_flags
    )
    {
    hope( x >= 0 );
    hope( x < x_size - 1 );
    hope( y >= 0 );
    hope( y < y_size - 1);
    elements[ ( y + 1 ) * ( x_size + 2 ) + ( x + 1 ) ].tile_flags = tile_flags;
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo
