/** @file terrain_material_inline.h
 * Terrain material description and material database (inline methods).
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo {
namespace Client {
namespace Renderer {


// class TerrainMaterial::VertexEquation

TerrainMaterial::VertexEquation::VertexEquation()
      : height_factor( 0.0f ),
        slope_factor( 0.0f ),
        delta( 0.0f )
    {
    }


// class TerrainMaterial::GradientElement

TerrainMaterial::GradientElement::GradientElement()
      : index( 0.0f )
    {
    }


TerrainMaterial::GradientElement::GradientElement
    (
    Float32                         the_index,
    TextureDataObject::Reference    the_texture
    ) : index( the_index ),
        texture( the_texture )
    {
    }


// class TerrainMaterial

TerrainMaterial::TerrainMaterial()
    {
    }


TerrainMaterial::TerrainMaterial
    (
    const TerrainMaterial & other
    ) : equation( other.equation ),
        texture_gradient( other.texture_gradient )
    {
    }


TerrainMaterial & TerrainMaterial::operator=
    (
    const TerrainMaterial & other
    )
    {
    equation = other.equation;
    texture_gradient = other.texture_gradient;
    return *this;
    }


const TerrainMaterial::VertexEquation & TerrainMaterial::get_vertex_equation() const
    {
    return equation;
    }


void TerrainMaterial::set_vertex_equation
    (
    const VertexEquation & new_equation
    )
    {
    equation = new_equation;
    }


const TerrainMaterial::TextureGradient & TerrainMaterial::get_texture_gradient() const
    {
    return texture_gradient;
    }


void TerrainMaterial::clear_texture_gradient()
    {
    texture_gradient.clear();
    }


void TerrainMaterial::add_texture_gradient_element
    (
    const GradientElement & element
    )
    {
    hope( texture_gradient.empty() || texture_gradient.back().index <= element.index );
    hope( !element.texture.is_null() );
    texture_gradient.push_back( element );
    }


// class TerrainMaterialDatabase::MaterialInfo

const SystemTime & TerrainMaterialDatabase::MaterialInfo::get_update_time() const
    {
    return update_time;
    }


// class TerrainMaterialDatabase

const SystemTime & TerrainMaterialDatabase::get_update_time() const
    {
    return update_time;
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo
