/** @file terrain_settings.cpp
 * Terrain settings (implementation).
 */

/*
 * begin        : Dec 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/terrain.h"
#include "renderer/terrain_settings.h"


namespace Demo {
namespace Client {
namespace Renderer {


TerrainSettings::TerrainSettings()
      : ::Massiv::Core::Settings( "Client/Renderer/Terrain" ),
        enable_fog( access_boolean( "enable_fog", true ) ),
        fog_red( access_float( "fog_red", 0.5f ) ),
        fog_green( access_float( "fog_green", 0.55f ) ),
        fog_blue( access_float( "fog_blue", 0.6f ) ),
        fog_start( access_float( "fog_start", 6.0f ) ),
        fog_end( access_float( "fog_end", 25.0f ) ),
        enable_texture( access_boolean( "enable_texture", true ) ),
        enable_light( access_boolean( "enable_light", true ) ),
        draw_marks( access_boolean( "draw_marks", false ) ),
        draw_wireframe( access_boolean( "draw_wireframe", false ) ),
        force_even_strips( access_boolean( "force_even_strips", false ) ),
        min_strip_length( access_integer( "min_strip_length", 6 ) ),
        depth_offset_factor( access_float( "depth_offset_factor", -0.5f ) ),
        depth_offset_bias( access_float( "depth_offset_bias", -0.5f ) )
    {
    }


bool TerrainSettings::settings_changed
    (
    const SystemTime & time
    ) const
    {
    return
        enable_fog.was_modified( time ) ||
        fog_red.was_modified( time ) ||
        fog_green.was_modified( time ) ||
        fog_blue.was_modified( time ) ||
        fog_start.was_modified( time ) ||
        fog_end.was_modified( time ) ||
        enable_texture.was_modified( time ) ||
        enable_light.was_modified( time ) ||
        draw_marks.was_modified( time ) ||
        draw_wireframe.was_modified( time ) ||
        force_even_strips.was_modified( time ) ||
        min_strip_length.was_modified( time ) ||
        depth_offset_factor.was_modified( time ) ||
        depth_offset_bias.was_modified( time );
    }


void TerrainSettings::apply_fog
    (
    PipelineState & state
    ) const
    {
    if( enable_fog )
        {
        state.fog.enable = true;
        state.fog.mode = GL_LINEAR;
        state.fog.start = fog_start;
        state.fog.end = fog_end;
        state.fog.color = OpenGL::FloatColor( fog_red, fog_green, fog_blue );
        }
    else
        {
        state.fog.enable = false;
        }
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo


#ifndef ENABLE_INLINE
#include "terrain_settings_inline.h"
#endif
