/** @file terrain_settings.h
 * Terrain settings.
 */

/*
 * begin        : Dec 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_TERRAIN_SETTINGS_H
#define CLIENT_TERRAIN_SETTINGS_H


#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#ifndef MASSIV_STATISTICS_H
#include "registry/statistics.h"
#endif


namespace Demo {
namespace Client {
namespace Renderer {


/**
 * Terrain renderer settings.
 */
class TerrainSettings : public ::Massiv::Core::Settings
    {
public:

    /**
     * Constructor.
     */
    TerrainSettings();

    /**
     * Anything changed since @a time?
     */
    bool settings_changed
        (
        const SystemTime & time
        ) const;

    /**
     * Apply fog settings to given pipeline state.
     */
    void apply_fog
        (
        PipelineState & state
        ) const;

public:

    Boolean &   enable_fog;
        /**< Enable terrain fog. */

    /**
     * Fog color.
     *
     * @todo The color must be black, otherwise additive terrain blending may add
     *       the fog color mulitiple times. Colored fog would have to be implemented
     *       as another terrain pass.
     */
    //@{
    Float &     fog_red;
    Float &     fog_green;
    Float &     fog_blue;
    //@}

    Float &     fog_start;
        /**< Fog start distance. */

    Float &     fog_end;
        /**< Fog end distance. */

    Boolean &   enable_texture;
        /**< Enable texturing. */

    Boolean &   enable_light;
        /**< Enable lighting. */

    Boolean &   draw_marks;
        /**< Render tile marks. */

    Boolean &   draw_wireframe;
        /**< Render wireframe terrain. */

    Boolean &   force_even_strips;
        /**< Force strips to start at even triangles and end at odd triangles. */

    Integer &   min_strip_length;
        /**< Use separate triangles instead of strips shorter than this.
             Set to 0 or 1 to always draw terrain using strips, use huge numbers
             to always draw terrain as separate triangles. */

    Float &     depth_offset_factor;
        /**< Factor of polygon depth offset.
             Should be zero or negative number. */

    Float &     depth_offset_bias;
        /**< Bias of polygon depth offset.
             Should be zero or negative number. */

    }; // class TerrainSettings


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo


#ifdef ENABLE_INLINE
#include "terrain_settings.h"
#endif


#endif // CLIENT_TERRAIN_SETTINGS_H
