/** @file text_rectangle_inline.h
 * Overlay object - text rectangle (inline methods).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo {
namespace Client {
namespace Renderer {


Font * TextRectangle::get_font() const
    {
    return font;
    }


int TextRectangle::get_columns() const
    {
    return columns;
    }


int TextRectangle::get_rows() const
    {
    return rows;
    }


const OpenGL::Color & TextRectangle::get_text_color() const
    {
    return font_color;
    }


Float32 TextRectangle::get_cell_width() const
    {
    hope( columns > 0 );
    return get_current_size().x() / columns;
    }


Float32 TextRectangle::get_cell_height() const
    {
    hope( rows > 0 );
    return get_current_size().y() / rows;
    }


void TextRectangle::set_font
    (
    Font * new_font
    )
    {
    hope( new_font );
    font = new_font;
    font_version = 0;
    }


void TextRectangle::set_columns_rows
    (
    int new_columns,
    int new_rows
    )
    {
    hope( new_columns > 0 );
    hope( new_rows > 0 );

    columns = new_columns;
    rows = new_rows;
    cleared = true;
    dirty = true;
    }


void TextRectangle::set_text_color
    (
    const SColor & new_color
    )
    {
    font_color = OpenGL::Color( new_color );
    }


void TextRectangle::print_line
    (
    const std::string & text
    )
    {
    const int length = text.size();
    set_columns_rows( length ? length : 1, 1 );
    set_size_from_font();
    print( 0, 0, text );
    }


void TextRectangle::clear()
    {
    cleared = true;
    }


TextRectangle::Cell::Cell()
      : character( 0 )
    {
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo

