/** @file vertex_array_dump.cpp
 * Vertex arrays - dumping.
 */

/*
 * begin        : Jul 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "renderer/vertex_array.h"
#include <iostream>


namespace Demo {
namespace Client {
namespace Renderer {


void VertexArray::dump
    (
    std::ostream &      stream,
    const std::string & indent
    )
    {
    stream << indent << "Vertex array, " << size << " elements.";

    const StreamSelection & selection = selected_streams[ VERTEX_ARRAY ];
    if( selection.index == -1 )
        {
        stream << "\n" << indent << "No vertex array selected.";
        }
    else if( selection.type == TYPE_VECTOR2 )
        {
        Vector2Stream vertices;
        lock_vectors2( vertices, selection.index );

        std::size_t i;
        for( i = 0; i < size; i++ )
            {
            const OpenGL::Vector2 & vertex = vertices[ i ];
            stream << "\n" << indent << "    " << i << ": ( " <<
                vertex.x() << ", " << vertex.y() << " )";
            }
        
        unlock_vectors2( selection.index );
        }
    else
        {
        hope( selection.type == TYPE_VECTOR3 );
        Vector3Stream vertices;
        lock_vectors3( vertices, selection.index );

        std::size_t i;
        for( i = 0; i < size; i++ )
            {
            const OpenGL::Vector3 & vertex = vertices[ i ];
            stream << "\n" << indent << "    " << i << ": ( " << vertex.x() <<
                ", " << vertex.y() << ", " << vertex.z() << " )";
            }

        unlock_vectors3( selection.index );
        }
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo
