/** @file vertex_array_inline.h
 * Vertex arrays (inline methods).
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

namespace Demo {
namespace Client {
namespace Renderer {


VertexArray::StreamTarget VertexArray::texcoord_array_target
    (
    std::size_t unit
    )
    {
    // Number of texcoords array is hardcoded in VertexArray and must match
    // MAX_TEXTURE_UNITS. We use the hardcoded value (2) here.

    hope( unit < 2 );
    switch( unit )
        {
        case 0:
            return TEXCOORD_ARRAY_0;
        case 1:
        default:
            return TEXCOORD_ARRAY_1;
        }
    }


void VertexArray::set_stream_count
    (
    StreamType  type,
    std::size_t count
    )
    {
    hope( type >= 0 );
    hope( type < TYPE_COUNT );
    hope( !memory );
    allocated_streams[ type ] = count;
    }


std::size_t VertexArray::get_stream_count
    (
    StreamType  type
    ) const
    {
    hope( type >= 0 );
    hope( type < TYPE_COUNT );
    return allocated_streams[ type ];
    }


std::size_t VertexArray::get_size() const
    {
    return size;
    }


void VertexArray::unlock_vectors2
    (
    std::size_t index
    )
    {
    unlock( TYPE_VECTOR2, index );
    }


void VertexArray::unlock_vectors3
    (
    std::size_t index
    )
    {
    unlock( TYPE_VECTOR3, index );
    }


void VertexArray::unlock_colors
    (
    std::size_t index
    )
    {
    unlock( TYPE_COLOR, index );
    }


} // namespace Demo::Client::Renderer
} // namespace Demo::Client
} // namespace Demo

