/** @file client_sdl.h
 * Helper SDL functions.
 */

/*
 * begin        : Sun Mar 9 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef CLIENT_SDL_H
#define CLIENT_SDL_H

#ifndef CLIENT_COMMON_H
#include "client_common.h"
#endif

#include <SDL.h>


namespace Demo {
namespace Client {
namespace SDL {


/**
 * Initialize SDL.
 *
 * @pre SDL is not initialized.
 */
void initialize();

/**
 * Shutdown SDL.
 *
 * No-op if SDL is initialized.
 */
void shutdown();

/**
 * Open video mode.
 * @param width Video mode width in pixels.
 * @param height Video mode height in pixels.
 * @param bpp Bit per pixel. Must be 0, 16 or 32.
 *      Set to zero for windowed mode.
 *
 * @pre Video mode is not already opened. Use close_video_mode()
 *      to close opened mode.
 */
void open_video_mode
    (
    int width,
    int height,
    int bpp
    );

/**
 * Close video mode.
 * Nothing is done if no video mode is opened.
 */
void close_video_mode();

/**
 * Returns screen width.
 */
int get_screen_width();

/**
 * Returns screen height.
 */
int get_screen_height();

/**
 * Flip video buffers.
 */
void flip();

/**
 * Process pending events.
 * @return True if quit event was received ie. user wants the application
 *      to quit.
 */
bool process_events();

/**
 * Converts @a key to ascii character. If the @a key doesn't
 * correspond to any ascii character, zero is returned.
 * @param caps If true, ascii char is returned in uppercase.
 * @param shift If true, uppercase or SHIFT value returned.
 */
int key_to_ascii_character
    (
    SDLKey  key,
    bool    caps,
    bool    shift
    );

/**
 * Is SDL initialized.
 */
bool is_sdl_initialized();

/**
 * Return state of video mode.
 * @return true when video opened.
 */
bool is_video_mode_opened();

} // namespace SDL
} // namespace Client
} // namespace Demo


#endif // CLIENT_SDL_H
