/** @file stream_rwops.cpp
 * IOStream SDL_RWops (implementation).
 *
 * Currently read-only. It's assumed that the stream does not throw exceptions
 * on fail and eof. No exceptions are caught, but it's probably not wise
 * to throw exceptions through SDL. Could be fixed in future.
 */

/*
 * begin        : Jun 2003
 * copyright    : (C) 2003 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "sdl/stream_rwops.h"


namespace Demo {
namespace Client {
namespace SDL {


namespace {


static const Uint32 RWOPS_TYPE = 0xbeedee;


/** Get stream from RWops */
std::istream & istream_get
    (
    SDL_RWops *     context
    )
    {
    hope( context );
    hope( context->type == RWOPS_TYPE );
    hope( context->hidden.unknown.data1 );
    return *reinterpret_cast< std::istream * >( context->hidden.unknown.data1 );
    }


/** SDL_RWops seek() */
int istream_seek
    (
    SDL_RWops *     context,
    int             offset,
    int             whence
    )
    {
    std::istream & stream = istream_get( context );
    std::ios_base::seekdir dir;
    switch( whence )
        {
        case SEEK_SET:
            dir = std::ios::beg;
            break;
        case SEEK_CUR:
            dir = std::ios::cur;
            break;
        case SEEK_END:
            dir = std::ios::end;
            break;
        default:
            panic( "Invalid whence" );
        }
    stream.clear();
    stream.seekg( offset, dir );
    return stream.tellg();
    }


/** SDL_RWops read() */
int istream_read
    (
    SDL_RWops *     context,
    void *          ptr,
    int             size,
    int             maxnum
    )
    {
    std::istream & stream = istream_get( context );
    std::istream::char_type * dest = reinterpret_cast< std::istream::char_type * >( ptr );

    stream.clear();
    int num = 0;
    while( ( num < maxnum ) && stream.read( dest, size ) )
        {
        num++;
        dest += size;
        }
    if( num == 0 )
        {
        // FIXME: I don't really understand when -1 should be returned.
        // On failure other than eof?

        num = -1;
        }

    return num;
    }


/** SDL_RWops close() */
int istream_close
    (
    SDL_RWops * context
    )
    {
    // do not close the stream
    SDL_FreeRW( context );
    return 0;
    }


} // anonymous namespace


SDL_RWops * create_stream_rwops
    (
    std::istream & stream
    )
    {
    SDL_RWops * ops = SDL_AllocRW();
    hope( ops );
    ops->seek = &istream_seek;
    ops->read = &istream_read;
    ops->write = NULL;
    ops->close = &istream_close;
    ops->type = RWOPS_TYPE;
    ops->hidden.unknown.data1 = &stream;
    return ops;
    }


} // namespace Demo::Client::SDL
} // namespace Demo::Client
} // namespace Demo
