/** @file local_console.h
 * Local console.
 */

/*
 * begin        : Sat Jun 28 2003
 * copyright    : (C) 2003 by Ondrej Pecta, Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_NODE_CONSOLE_H
#define DEMO_NODE_CONSOLE_H

#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_CLIENT_FWD_H
#include "lib/client/client_fwd.h"
#endif

#ifndef DEMO_CONSOLE_H
#include "lib/shared/console.h"
#endif

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Local console.
 * Handles local commands (commands used to manage the client's console or 
 * commands to be executed locally).
 * Always paired with the local ClientNode.
 */
class LocalConsole : public Console
    {
public:

    typedef std::vector< std::string >  StringList;

    /**
     * Represents one script that can be run from the client's console.
     */
    struct Script
        {
        
        std::string     name;
            /** Name of script. */
        
        StringList      command_list;
            /**< Script's command list. Each command can contain substrings
                 %1 .. %9 - these substrings will be replaced be real parameters of script. */
        
        }; // Script

    /**
     * Default constructor.
     */
    LocalConsole();
    
    /**
     * Default destructor.
     */
    ~LocalConsole();

public:

    /** Help command. */
    static void execute_help
        (
        const Console::CommandContext & context
        );

    /** Quit command. */
    static void execute_quit
        (
        const Console::CommandContext & context
        );

    /** Cls command. */
    static void execute_cls
        (
        const Console::CommandContext & context
        );

    /** Connect command. */
    static void execute_connect
        (
        const Console::CommandContext & context
        );
    
    /** Editor command. */
    static void execute_editor
        (
        const Console::CommandContext & context
        );

    /** Exec command - run local script. */
    static void execute_exec
        (
        const Console::CommandContext & context
        );

    /** Writefile command. */
    static void execute_writefile
        (
        const Console::CommandContext & context
        );

public:
    
    /**
     * Initialize script list from data object.
     */
    static void initialize_script_list();

    /**
     * Fill @a script_list from data object @a script_do.
     */
    static void update_script_list();
        
    /**
     * Find script from script list. 
     * @param script_name script name to find.
     * @param script reference to script structure which will be filled.
     * @return true if script was found.
     */
    static bool find_script
        (
        const std::string &     script_name,
        Script &                script
        );

private:

    static std::vector< Script >           script_list;
        /**< Script list. Initialized from data object. */

    static Massiv::Core::DataObject *      script_do;
        /**< Script list data object. */
    
    }; // LocalConsole

} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_LOCAL_CONSOLE_H
