/** @file local_console_commands.cpp
 * Local console commands implementation.
 */

/*
 * begin        : Oct 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "core.h"
#include "lib/client/local_console.h"
#include "lib/client/client_node.h"
#include "client/client_global.h"
#include "client/gui/client_full_console.h"
#include "client/gui/client_chat_console.h"
#include "client/gui/editor.h"

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


void LocalConsole::execute_help
    (
    const CommandContext &  context
    )
    {
    std::string cmd;
    std::string help;
     
    try
        {
        cmd = context.get_string( 0 );
        }
    catch( ... )
        {
        }

    if( cmd.size() )
        {        
        help = "Detail info about command " + cmd + ":\n"
            + context.console->print_command_help( cmd );
        }
    else
        {
        help = "Commands supported by local console:\n" 
            + context.console->print_command_help();
        }
        
    context.print( help );
    }


void LocalConsole::execute_quit
    (
    const CommandContext &  context
    )
    {
    Massiv::System::request_quit();
    }


void LocalConsole::execute_cls
    (
    const CommandContext &  context
    )
    {
    Demo::Client::Global::get_full_console().clear_display();
    Demo::Client::Global::get_chat_console().clear_display();
    }


void LocalConsole::execute_connect
    (
    const CommandContext &  context
    )
    {
    WeakPointer<ClientNode> client = Massiv::System::get_local_node_id();
    client->connect_console_to( context.get_object_id( 0 ) );
    }


void LocalConsole::execute_editor
    (
    const Console::CommandContext & context
    )
    {
    // Show/hide editor.
    Demo::Client::Editor & editor = Demo::Client::Global::get_editor();
    editor.set_visibility( !editor.is_visible() );
    }


void LocalConsole::execute_writefile
    (
    const Console::CommandContext & context
    )
    {
    WeakPointer<ClientNode> client = Massiv::System::get_local_node_id();
    std::string             filename = context.get_string( 0 );
    if( filename != "" )
        {
        Demo::Client::Global::get_full_console().set_write_filename( filename );
        }
    else
        {
        Demo::Client::Global::get_full_console().set_write_filename( "" );
        }
    }


/**
 * Find @param find_text in string @param text and replace all
 * occurences by string @param replace_text.
 */
void find_and_replace
    (
    std::string &           text,
    const std::string &     find_text,
    const std::string &     replace_text
    )
    {
    std::string::size_type pos = 0;
    int length = find_text.size();
    while( ( pos = text.find( find_text ) ) != std::string::npos )
        {
        // Replace one occurence.
        text.replace( pos, length, replace_text );
        }
    }
    

void LocalConsole::execute_exec
    (
    const Console::CommandContext & context
    )
    {
    std::string  temp =  context.get_string( 0 );
    std::vector< std::string >      param;
    int i;
    
    // Temporary hack - inicializace pole parametru
        {
        std::istringstream  ss( temp );
        while( ss.good() )
            {
            std::string s;
            ss >> quote_string( s );
            param.push_back( s );
            }
        }
        
    Script script;
    if( !find_script( param[ 0 ], script ) )
        {
        context.print( std::string( "Error: cannot find script with name " ) + param[ 0 ] );
        return;
        }
    
    // Get local copy of command list.
    StringList command_list = script.command_list;
    
    // Replace occurence of %n
    for( i = 0; i < ( signed int )command_list.size(); i++ )
        {
        for( int j = 1; j < ( signed int )param.size(); j++ )
            {
            std::ostringstream  ss;
            int tmp = j;
            if( tmp > 9 )
                {
                std::string str = "a";
                str[ 0 ] += tmp - 10;
                ss << "%" << str;
                }
            else
                {
                ss << "%" << tmp;
                }
            find_and_replace( command_list[ i ], ss.str(), param[ j ] );
            }
        }

    // FIXME - jde to do full console.
    Demo::Client::Console & fc = Demo::Client::Global::get_full_console();
    for( i = ( signed int )command_list.size() - 1; i >= 0 ; i-- )
        {
        // Add commands to buffered command list.
        fc.push_front_command_to_buffered_command_list( command_list[ i ] );
        }    
    
#if 1
    // Print command list.
    std::string print = "/exec substitution:\n";
    for( i = 0; i < ( signed int )command_list.size(); i++ )
        {
        // Add commands to buffered command list.
        print += command_list[ i ] + "\n";
        }    
    context.print( print );
#endif
        
    }


} // namespace Demo::Lib
} // namespace Demo
