/** @file ability_modifier.cpp
 * AbilityModifier (implementation).
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/server/ability_modifier.h"
#include "lib/server/entity.h"
#include "lib/server/player.h"
#include "lib/shared/client_entity.h" // ability_to_string()
#include <sstream>


namespace Demo {
namespace Lib {


void AbilityModifier::initialize
    (
    Pointer< Entity >           the_entity,
    AbilityId                   the_ability,
    Int32                       the_ability_value_delta,
    Int32                       the_ability_upper_limit_delta,
    const STime &               the_duration
    )
    {
    hope( the_entity != null );

    // Initialize attributes.
    entity = the_entity;
    ability = the_ability;
    ability_value_delta = the_ability_value_delta;
    ability_upper_limit_delta = the_ability_upper_limit_delta;
    
    // Modify the ability.
    entity->client_entity->add_ability_upper_limit( ability, ability_upper_limit_delta );
    entity->client_entity->add_ability_value( ability, ability_value_delta );
    
    // Schedule self to revert the ability change.
    STime time = STime::current_time() + the_duration;
    migrate_to( entity, time );
    }


void AbilityModifier::delivered_to
    (
    WeakPointer< Object >   object,
    const STime &           time
    )
    {
    hope( entity != null );
    hope( entity == object );
    
    // Revert ability change.
    entity->client_entity->add_ability_upper_limit( ability, -ability_upper_limit_delta );
    
    if( entity->is_player() )
        {
        Int32 new_value = entity->client_entity->get_ability_value( ability );
        Int32 new_upper_limit = entity->client_entity->get_ability_upper_limit( ability );
        Pointer< Player > player = entity.convert();
        std::ostringstream ss;
        ss  << "Effect lost. "
            << "Your " << ClientEntity::ability_to_string( ability )
            << " changed to " << new_value << "/" << new_upper_limit << ".";
        player->log_game_message( ss.str() );
        }
    }


} // namespace Demo::Lib
} // namespace Demo
