/** @file ability_modifier.h
 * AbilityModifier.
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_ABILITY_MODIFIER_H
#define DEMO_ABILITY_MODIFIER_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SERVER_FWD_H
#include "lib/server/server_fwd.h"
#endif

#ifndef DEMO_GAME_DEFS_H
#include "lib/shared/game_defs.h"
#endif


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * AbilityModifier is object that can be used to temporary
 * modify some ability of an entity for limited time duration.
 */
class AbilityModifier : public ::Massiv::Core::Object
    {
public:
    
    MASSIV_OBJECT( AbilityModifier );

    /**
     * Initialize the object.
     * @param entity Entity whose ability will be modified.
     * @param ability Ability to modify.
     * @param ability_value_delta This value is added to current value
     *      of the ability.
     * @param ability_upper_limit_delta This value is added to current
     *      upper limit of the ability.
     * @param duration Duration of the ability change in seconds.
     * @note Upper limit of the ability is changes before uptaing
     *      current value of the ability.
     */
    void initialize
        (
        Pointer< Entity >           entity,
        AbilityId                   ability,
        Int32                       ability_value_delta,
        Int32                       ability_upper_limit_delta,
        const STime &               duration
        );

private:

    /**
     * Called by system when the ability modifier should disable its efect.
     * @c object is assumed to be an Entity.
     */
    virtual void delivered_to
        (
        WeakPointer< Object >   object,
        const STime &           time
        );

protected:

    PPointer< Entity >                  entity;
        /**< Entity whose ability is modified. This pointer ensures that
             we will migrate with the entity. */
    
    PEnum< AbilityId >                  ability;
        /**< Ability id. */
    
    PVlInt32                            ability_value_delta;
        /**< Delta change of ability value. */
    
    PVlInt32                            ability_upper_limit_delta;
        /**< Delta change of ability upper limit. */
    
    }; // AbilityModifier


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_ABILITY_MODIFIER_H
