/** @file account_console.cpp
 * Account console implementation.
 */

/*
 * begin        : Sat Jun 28 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "core.h"
#include "lib/server/account_console.h"

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();

AccountConsole::AccountConsole
    (
    )
    {
    ConsoleCommand * cmd;

    // Command help.

    cmd = command_list.new_command
        (
        "help",
        "Help.",
        ConsoleCommand::FLAGS_ROOT_ACCOUNT,
        &execute_help
        );

    cmd->new_argument
        (
        "Command to show detail info about.",
        CommandArgument::FLAGS_STRING | CommandArgument::FLAGS_ARG_OPTIONAL
        );

    // Command setname.

    cmd = command_list.new_command
        (
        "setname",
        "Sets player's name.",
        ConsoleCommand::FLAGS_ROOT_ACCOUNT,
        &execute_setname
        );

    cmd->new_argument
        (
        "Player's name.",
        CommandArgument::FLAGS_STRING
        );
        
    // Command setcolor.

    cmd = command_list.new_command
        (
        "setcolor",
        "Sets player's chat color.",
        ConsoleCommand::FLAGS_ROOT_ACCOUNT,
        &execute_setcolor
        );

    cmd->new_argument
        (
        "RED.",
        CommandArgument::FLAGS_FLOAT
        );  
        
    cmd->new_argument
        (
        "GREEN.",
        CommandArgument::FLAGS_FLOAT
        ); 
        
    cmd->new_argument
        (
        "BLUE.",
        CommandArgument::FLAGS_FLOAT
        );
    
    // Modify elevation command.

    cmd = command_list.new_command
        (
        "modify_elevation",
        "Modify terrain elevation.",
        // "\n    <x> <y> <map_kind> <map_function> <map_operation> <radius> <strength>",
        ConsoleCommand::FLAGS_ROOT_ACCOUNT,
        &execute_modify_elevation
        );

    cmd->new_argument( "Center x-coordinate.", CommandArgument::FLAGS_FLOAT );
    cmd->new_argument( "Center y-coordinate.", CommandArgument::FLAGS_FLOAT );
    cmd->new_argument( "Map kind.", CommandArgument::FLAGS_STRING );
    cmd->new_argument( "Map function.", CommandArgument::FLAGS_STRING );
    cmd->new_argument( "Map operation.", CommandArgument::FLAGS_STRING );
    cmd->new_argument( "Radius.", CommandArgument::FLAGS_FLOAT );
    cmd->new_argument( "Strength.", CommandArgument::FLAGS_FLOAT );

    // Toggle path finding command.

    cmd = command_list.new_command
        (
        "toggle_path_finding",
        "Enable or disable path finding for the selected player.",
        0,
        &execute_toggle_path_finding
        );
    }


AccountConsole::~AccountConsole()
    {
    }


Pointer<AccountObjectImpl> AccountConsole::get_account
    (
    const CommandContext & context
    )
    {
    return context.account.convert();
    }

    
} // namespace Demo::Lib
} // namespace Demo
