/** @file account_console.h
 * Account console.
 */

/*
 * begin        : Sat Jun 28 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_ACCOUNT_CONSOLE_H
#define DEMO_ACCOUNT_CONSOLE_H

#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_CONSOLE_H
#include "lib/shared/console.h"
#endif

#ifndef DEMO_SERVER_FWD_H
#include "lib/server/server_fwd.h"
#endif

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Account console.
 * Handles commands specific to a particular AccountObject (commands used to 
 * manage that account or commands to be executed on behalf of that account). 
 */
class AccountConsole : public Console
    {
public:

    /**
     * Default constructor.
     */
    AccountConsole();
    
    /**
     * Default destructor.
     */
    ~AccountConsole();

public:

    /** Help command. */
    static void execute_help
        (
        const CommandContext & context
        );
    
    /** Setname command. */
    static void execute_setname
        (
        const CommandContext & context
        );
       
    /** Setcolor command. */
    static void execute_setcolor
        (
        const CommandContext & context
        );
    
    /** Modify elevation command. */
    static void execute_modify_elevation
        (
        const CommandContext & context
        );

    /** Toggle path finding command. */
    static void execute_toggle_path_finding
        (
        const CommandContext & context
        );

private:

    /** Get pointer to account object. */
    static Pointer<AccountObjectImpl> get_account
        (
        const CommandContext & context
        );

    }; // AccountConsole

} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_ACCOUNT_CONSOLE_H
