/** @file account_console_commands.cpp
 * Node console commands implementation.
 */

/*
 * begin        : Oct 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "core.h"
#include "lib/server/account_console.h"
#include "lib/client/client_node.h"
#include "lib/server/account_object_impl.h"


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


void AccountConsole::execute_help
    (
    const CommandContext &  context
    )
    {
    std::string cmd;
    std::string help;
     
    try
        {
        cmd = context.get_string( 0 );
        }
    catch( ... )
        {
        }

    if( cmd.size() )
        {        
        help = "Detail info about command " + cmd + ":\n"
            + context.console->print_command_help( cmd );
        }
    else
        {
        help = "Commands supported by account console:\n" 
            + context.console->print_command_help();
        }
        
    context.print( help );
    }


void AccountConsole::execute_setname
    (
    const CommandContext & context
    )    
    {
    std::string name = context.get_string( 0 );
    get_account( context )->set_name( name );
    std::ostringstream oss;
    oss << "Player name set to " << name << ".";
    context.print( oss.str() );
    }
    
    
void AccountConsole::execute_setcolor
    (
    const CommandContext & context
    )    
    {
    Float32 red = context.get_float( 0 );
    Float32 green = context.get_float( 1 );
    Float32 blue = context.get_float( 2 );
    
    if( red < 0.0 || red > 1.0 || 
        green < 0.0 || green > 1.0 ||
        blue < 0.0 || blue > 1.0 )
        {
        context.print
            ( 
            "Color components must be within range ( 0.0f, 1.0f ).",
             SColor( 1.0f, 0.0f, 0.0f ) 
            );
        return;
        }
    
    get_account( context )->set_chat_color( SColor( red, green, blue ) );
    context.print( "This is your new chat color", SColor( red, green, blue ) );
    }


void AccountConsole::execute_modify_elevation
    (
    const CommandContext & context
    )
    {
    SVector2                            center;
    ClientSector::ElevationMapKind      map_kind;
    ClientSector::ElevationMapFunction  function;
    ClientSector::ElevationMapOperation operation;
    Float32                             radius;
    Float32                             strength;
    
    center = SVector2( context.get_float( 0 ), context.get_float( 1 ) );
    
    if( !ClientSector::string_to_map_kind( context.get_string( 2 ), map_kind ) )
        {
        context.print( "Invalid map kind." );
        return;
        }
    
    if( !ClientSector::string_to_map_function( context.get_string( 3 ), function ) )
        {
        context.print( "Invalid map function." );
        return;
        }
    
    if( !ClientSector::string_to_map_operation( context.get_string( 4 ), operation ) )
        {
        context.print( "Invalid map operation." );
        return;
        }
    
    radius = context.get_float( 5 );
    strength = context.get_float( 6 );
    
    get_account( context )->modify_elevation_command
        (
        center,
        map_kind,
        function,
        operation,
        radius,
        strength
        );
    }


void AccountConsole::execute_toggle_path_finding
    (
    const CommandContext & context
    )
    {
    get_account( context )->toggle_path_finding();
    }


} // namespace Demo::Lib
} // namespace Demo
