/** @file collision_info.h
 * Collision info.
 */

/*
 * begin        : Sun Nov 16 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_COLLISION_INFO_H
#define DEMO_COLLISION_INFO_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SERVER_FWD_H
#include "lib/server/server_fwd.h"
#endif

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Structure that holds all information neccessary to figure out why entity
 * could not move to a target position.
 */
class CollisionInfo : public ValueType
    {
public:
        
    /** Collision reason. */
    enum CollisionReason
        {
        REASON_NO_SECTOR =      0,
            /**< Target position refers to a non-existing sector. */
        REASON_COLLISION =      1,
            /**< Target position is occupied by an Entity. */
        REASON_SLOPE_TOO_HIGH = 2,
            /**< Terrain slope is too high. */
        REASON_OBSTACLE =       3
            /**< There is an obstacle at target position. */
        }; // CollisionReason

    /** Default constructor. */
    CollisionInfo();

    /** Copy constructor. */
    CollisionInfo
        (
        const CollisionInfo & info
        );

public:

    PEnum<CollisionReason>              reason;
        /**< Collision reason. */
    PRemote<Entity>                     entity;
        /**< The entity that collided with us. Valid if @c reason == 
             REASON_COLLISION. */
    PVlInt32                            entity_flags;
        /**< Entity class flags. Valid if @c reason == REASON_COLLISION. */

    }; // CollisionInfo


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_COLLISION_INFO_H
