/** @file device.cpp
 * Device implementation.
 */

/*
 * begin        : Mon June 13 2003
 * copyright    : (C) 2003 by Martin Havlista, Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "core.h"
#include "lib/server/device.h"
#include "lib/shared/client_entity.h"


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


// Implementation of Device
// ~~~~~~~~~~~~~~~~~~~~~~~~

void Device::initialize()
    {
    Entity::initialize();
    client_entity->type = ClientEntity::TYPE_LOGIC;
    create_input( get_input_name() );
    }


Logic::SocketId Device::get_input_id()
    {
    // Device has hardwired default input called "input"
    // whose socket id is always 0.
    
    return 0;
    }


std::string Device::get_input_name()
    {
    return "input";
    }


Pointer<Logic::Packet> Device::get_input()
    {
    return Logic::get_input( get_input_id() );
    }


Logic::SocketId Device::translate_input
    (
    const std::string & input_name
    )
    {
    if( input_name == get_input_name() )
        {
        return get_input_id();
        }

    return Logic::translate_input( input_name );
    }


Logic::SocketId Device::translate_output
    (
    const std::string & output_name
    )
    {
    throw DeviceHasNoOutputsException();
    }


Logic::SocketId Device::create_input
    (
    const std::string & dynamic_name
    )
    {
    if( dynamic_name != get_input_name() || inputs.size() )
        {
        throw DeviceHasDefaultInputOnlyException();
        }
    
    SocketId id = Logic::create_input();
    hope( id == get_input_id() );

    return id;
    }


Logic::SocketId Device::create_output
    (
    const std::string & dynamic_name
    )
    {
    throw DeviceHasNoOutputsException();
    }


// Implementation of PrintMessageDevice
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void PrintMessageDevice::initialize()
    {
    Device::initialize();
    }


void PrintMessageDevice::operate_synced
    (
    const STime & delivery_time
    )
    {
    Pointer<Packet> packet = get_input().convert();

    std::cout   << "PrintMessageDevice input has changed."
                << " Time: " << delivery_time
                << " Value: " << packet->to_string()
                << std::endl;
    }


} // namespace Demo::Lib
} // namespace Demo
