/** @file device.h
 * Logic device.
 */

/*
 * begin        : Mon June 13 2003
 * copyright    : (C) 2003 by Martin Havlista, Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_DEVICE_H
#define DEMO_DEVICE_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SERVER_FWD_H
#include "lib/server/server_fwd.h"
#endif

#ifndef DEMO_ENTITY_H
#include "lib/server/entity.h"
#endif


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();

BASIC_LIB_EXCEPTION( DeviceErrorException, ::Massiv::Core::Lib::RuntimeException );
BASIC_LIB_EXCEPTION( DeviceHasDefaultInputOnlyException, DeviceErrorException );
BASIC_LIB_EXCEPTION( DeviceHasNoOutputsException, DeviceErrorException );


/**
 * Device object.
 * An entity whose logic has one default input ("input") and no outputs.
 */
class Device : public Entity
    {
protected:

    //
    // Device interface.
    //

    /** Initializes object. */
    virtual void initialize();

public:                

    /**
     * Returns default input socket id.
     */
    static SocketId get_input_id();

    /**
     * Returns default input name.
     */
    static std::string get_input_name();

    /**
     * Returns device input.
     */
    Pointer<Packet> get_input();

protected:
    
    //
    // Logic interface implementation.
    //

    /**
     */
    virtual Logic::SocketId translate_input
        (
        const std::string & input_name
        );
    
    /**
     */
    virtual Logic::SocketId translate_output
        (
        const std::string & output_name
        );
    
    /**
     */
    virtual SocketId create_input
        (
        const std::string & dynamic_name = ""
        );
    
    /**
     */
    virtual SocketId create_output
        (
        const std::string & dynamic_name = ""
        );

    }; // Device


/**
 * PrintMessageDevice.
 * Prints message on std::cout when an input is changed.
 */
class PrintMessageDevice : public Device
    {
public:

    /** Initializes object.*/
    virtual void initialize();

protected:

    /**
     */
    virtual void operate_synced
        (
        const STime & delivery_time
        );
                
    }; // PrintMessageDevice


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_DEVICE_H
