/** @file entity_factory.cpp
 * Entity factory.
 */

/*
 * begin        : Sat Nov 30 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "core_management.h"
#include "lib/server/entity_factory.h"
#include "lib/server/entity.h"
#include "lib/server/entity_rpc.h"
#include "lib/server/layer.h"
#include "lib/shared/client_entity.h"
#include "lib/shared/client_sector.h"

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


void EntityFactory::initialize()
    {
    }


Remote<Entity> EntityFactory::create_entity
    (
    const std::string & entity_class
    )
    {
    const ObjectFactory * factory = Global::class_manager().get_factory( entity_class );
    Pointer<Entity> entity = factory->create_object()->get_object_id();
    entity->initialize();
    return entity;
    }


Remote<Entity> EntityFactory::create_entity_in_layer
    (
    const std::string &     entity_class,
    const SVector2 &        origin,
    const SOrientation &    orientation,
    Remote<Layer>           layer,
    bool                    solid,
    const SVector2 &        bounding_box_size,
    bool                    bounding_box_axis_aligned,
    const std::string &     model_tags
    )
    {
    Pointer<Entity> entity = create_entity( entity_class );

    if( !bounding_box_size.epsilon_zero() )
        {
        SVector2        box = bounding_box_size;
        const Float32   epsilon = FloatTraits<Float32>::epsilon();

        if( box.x() < 1.0f )
            {
            box.set_x( 1.0f );
            }

        if( box.x() > ClientSector::NUM_X_TILES - epsilon )
            {
            box.set_x( ClientSector::NUM_X_TILES - epsilon );
            }
        
        if( box.y() < 1.0f )
            {
            box.set_y( 1.0f );
            }
        
        if( box.y() > ClientSector::NUM_Y_TILES - epsilon )
            {
            box.set_y( ClientSector::NUM_Y_TILES - epsilon );
            }

        entity->physical_info.bounding_box_size = box;
        }

    entity->physical_info.physical_class = solid ? PhysicalInfo::CLASS_SOLID : PhysicalInfo::CLASS_PENETRABLE;
    entity->physical_info.bounding_box_axis_aligned = bounding_box_axis_aligned;
    entity->put_into_layer( EntityPosition( origin, orientation, layer ) );
    entity->client_entity->model_tags = model_tags;

    return entity;
    }


} // namespace Demo::Lib
} // namespace Demo
