/** @file entity_factory.h
 * Entity factory.
 */

/*
 * begin        : Sat Nov 30 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_ENTITY_FACTORY_H
#define DEMO_ENTITY_FACTORY_H

#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SERVER_FWD_H
#include "lib/server/server_fwd.h"
#endif

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Factory for Entity objects.
 * To be used by world creator from the console/editor.
 */
class EntityFactory : public Object
    {
public:

    MASSIV_OBJECT( EntityFactory );

    /** Initializes object. */
    void initialize();

public:

    /**
     * Creates and initializes a generic entity.
     * Relies on the fact that all Entity subclasses implement @c initialize()
     * methods properly.
     * @c entity_class refers to IDL name of the entity being created.
     */
    Remote<Entity> create_entity
        (
        const std::string & entity_class
        );

    /**
     * Creates, initializes and places the entity to layer.
     * Allows to parametrize entity's physical properties and visual representation.
     */
    Remote<Entity> create_entity_in_layer
        (
        const std::string &     entity_class,
        const SVector2 &        origin,
        const SOrientation &    orientation,
        Remote<Layer>           layer,
        bool                    physical,
        const SVector2 &        bounding_box_size,
        bool                    bounding_box_axis_aligned,
        const std::string &     model_tags
        );

    };

} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_ENTITY_FACTORY_H
