/** @file item.cpp
 * Item (implementation).
 */

/*
 * begin        : Tue Aug 5 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/server/item.h"
#include "lib/server/monster.h"
#include "lib/server/player.h"
#include "lib/server/boots.h"
#include "lib/server/garbage.h"
#include "lib/server/potion.h"
#include "lib/server/weapon.h"
#include "lib/shared/client_item.h"


namespace Demo {
namespace Lib {


void Item::initialize()
    {
    client_item = CreateObject< ClientItem >();
    }


Pointer< Item > Item::create
    (
    ItemType   item_type,
    int        item_variant
    )
    {
    switch( item_type )
        {
        case ITEM_POTION:
            return CreateObject< Potion >( static_cast< PotionType >( item_variant ) );

        case ITEM_GARBAGE:
            return CreateObject< Garbage >( static_cast< GarbageType >( item_variant ) );

        case ITEM_BOOTS:
            return CreateObject< Boots >( static_cast< BootsType >( item_variant ) );

        case ITEM_WEAPON:
            return CreateObject< Weapon >( static_cast< WeaponType >( item_variant ) );
            
        default:
            return null;
        }
    }


Pointer< Item > Item::use_on_monster
    (
    Pointer< Player >       player,
    WeakPointer< Monster >  monster
    )
    {
    // Compose message.

    std::string message = "doesn't know how to use ";
    message += client_item->get_long_name();
    message += " on ";
    if( monster.is_local() )
        {
        message += monster->get_name();
        }
    else
        {
        message += "an entity";
        }
    player->say( message, Player::MESSAGE_ACTION );
    return this;
    }


Pointer< Item > Item::use_on_sector
    (
    Pointer< Player >       player
    )
    {
    player->say( std::string( "wonders what to do with " ) + client_item->get_long_name(), Player::MESSAGE_ACTION );
    return this;
    }


Pointer< Item > Item::equip
    (
    Pointer< Monster > monster
    )
    {
    return this;
    }


Pointer< Item > Item::unequip
    (
    Pointer< Monster > monster
    )
    {
    return this;
    }


void Item::set_equipped
    (
    bool the_equipped
    )
    {
    client_item->equipped = the_equipped;
    }


bool Item::is_equipped() const
    {
    return client_item->is_equipped();
    }


bool Item::is_equipable() const
    {
    return client_item->is_equipable();
    }


} // namespace Demo::Lib
} // namespace Demo
