/** @file item_slot_container.cpp
 * Item container (implementation).
 */

/*
 * begin        : Tue Aug 5 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/server/item_slot_container.h"
#include "lib/shared/client_item_slot_container.h"
#include "lib/server/item.h"


namespace Demo {
namespace Lib {


void ItemSlotContainer::initialize
    (
    int the_slot_columns,
    int the_slot_rows
    )
    {
    hope( the_slot_columns > 0 && the_slot_rows > 0 );
    client_item_slot_container = CreateObject< ClientItemSlotContainer >( the_slot_columns, the_slot_rows );
    slots.resize( client_item_slot_container->slot_columns * client_item_slot_container->slot_rows );
    }


void ItemSlotContainer::put_item
    (
    int slot_index,
    Pointer< Item > item
    )
    {
    hope( is_valid_slot_index( slot_index ) );
    if( item != null )
        {
        client_item_slot_container->slots[ slot_index ] = item->client_item;
        slots[ slot_index ] = item;
        }
    else
        {
        client_item_slot_container->slots[ slot_index ] = null;
        slots[ slot_index ] = null;
        }
    }


Pointer< Item > ItemSlotContainer::get_item
    (
    int slot_index
    )
    {
    hope( is_valid_slot_index( slot_index ) );
    return slots[ slot_index ];
    }


bool ItemSlotContainer::is_valid_slot_index
    (
    int slot_index
    ) const
    {
    return client_item_slot_container->is_valid_slot_index( slot_index );
    }


} // namespace Demo::Lib
} // namespace Demo
