/** @file item_slot_container.h
 * Item container.
 */

/*
 * begin        : Tue Aug 5 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_ITEM_SLOT_CONTAINER_H
#define DEMO_ITEM_SLOT_CONTAINER_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SERVER_FWD_H
#include "lib/server/server_fwd.h"
#endif


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Item slot container holds items in slots.
 * One slot can hold one or none item.
 * Item container has fixed number of slots as specified
 * by parameters of @c initialize().
 * Slots are organized in slot table with specified width (columns)
 * and height (rows). Slots are indexed by position in this table.
 */
class ItemSlotContainer : public Object
    {
public:
    
    MASSIV_OBJECT( ItemSlotContainer );
    friend class Player;

    typedef PArray< PPointer< Item > >          Slots;
    typedef PPointer< ClientItemSlotContainer > Container;
    
public:

    /** Initializes the slot container. */
    void initialize
        (
        int slot_columns,
        int slot_rows
        );
        
public:

    /**
     * Put item at specified slot.
     * @a item may be null.
     */
    void put_item
        (
        int slot_index,
        Pointer< Item > item
        );
    
    /**
     * Get item at specified slot.
     */
    Pointer< Item > get_item
        (
        int slot_index
        );

    /**
     * Check whether @a slot_index is valid slot index
     * into @a slots array.
     */
    bool is_valid_slot_index
        (
        int slot_index
        ) const;
    
protected:

    Container   client_item_slot_container;
        /**< Client visible part of the item. */
    Slots       slots;
        /**< Array of slots. */
    
    }; // ItemSlotContainer


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_ITEM_SLOT_CONTAINER_H
