/** @file layer.cpp
 * Layer implementation.
 */

/*
 * begin        : Mon Jun 31 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "core.h"
#include "lib/server/layer.h"
#include "lib/server/world.h"
#include "lib/server/layer_rpc.h"
#include "lib/server/sector.h"
#include "lib/server/lock.h"


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


// Implementation of Layer
// ~~~~~~~~~~~~~~~~~~~~~~~

void Layer::initialize()
    {
    }


Remote<Sector> Layer::get_sector
    (
    const SectorAddress & address
    )
    {
    SectorList::Iterator it = sector_list.find( address );

    hope( address.layer == WeakPointer< Object >( this ) );

    if( it == sector_list.end() )
        {
        return null;
        }
    else
        {
        return it->second;
        }
    }


void Layer::register_sector
    (
    Pointer<Sector>         sector,
    const SectorAddress &   address
    )
    {
    int i;
    
    // Lock @c sector_list.

    OPTIMISTIC_LOCK( sector_list_lock, "Layer is busy." );

    // Register sector to the global sector map.
    
    hope( address.layer == WeakPointer< Object >( this ) );

    if( sector_list.contains( address ) )
        {
        throw SectorAlreadyDefinedException();
        }

    sector_list.insert( address, sector );

    // Initialize sector.

    sector->sector_origin = address.get_layer_position();
    sector->set_address( address );

    // Link sector with its neighbors.
    
    for( i = 0; i < 8; i++ )
        {
        Direction               direction = static_cast<Direction>( i );
        SectorAddress           neighbor = address.get_neighbor( direction );
        SectorList::Iterator    it = sector_list.find( neighbor );

        if( it != sector_list.end() )
            {
            Remote<Sector> other_sector = it->second;
            sector->double_link_neighbor( direction, other_sector );
            }
        }
    }


void Layer::register_new_sector
    (
    int column,
    int row
    )
    {
    Pointer<Sector> sector = CreateObject<Sector>();
    register_sector( sector, SectorAddress( this, column, row ) );
    }


} // namespace Demo::Lib
} // namespace Demo
