/** @file naming_service.cpp
 * Naming service (implementation).
 */

/*
 * begin        : Sun Oct 12 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/server/naming_service.h"
#include <sstream>


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


void NamingService::initialize()
    {
    }


WeakPointer< Object > NamingService::get_object
    (
    const std::string & name
    )
    {
    NameToObject::Iterator it = name_to_object.find( name );
    if( it != name_to_object.end() )
        {
        return it->second;
        }
    else
        {
        return null;
        }
    }


bool NamingService::register_object
    (
    const std::string &     name,
    WeakPointer< Object >   object,
    bool                    replace
    )
    {
    bool result = true;
    NameToObject::Iterator it = name_to_object.find( name );
    if( it != name_to_object.end() )
        {
        // Already registered.

        if( it->second != object )
            {
            // Different object registered. Leave it or replace it.

            std::ostringstream oss;
            oss << "Naming service " << get_object_id()
                << " collision: Trying to register two different objects "
                << "(old = " << it->second.get_object_id() << ", new = "
                << object.get_object_id() << ")"
                << " under same name '" << name << "'.";

            if( replace )
                {
                oss << " The mapping to " << it->second.get_object_id() << " will be lost.";
                }
            else
                {
                oss << " Object " << object.get_object_id() << " won't be registered.";
                }

            Global::log_info
                (
                Status::FACILITY_LIB,
                Status::PRIORITY_NORMAL,
                oss.str()
                );

            if( replace )
                {
                it->second = object;
                }
            else
                {
                result = false;
                }
            }
        }
    else
        {
        name_to_object.insert( name, object );
        }

    return result;
    }


bool NamingService::unregister_object
    (
    const std::string &     name,
    WeakPointer< Object >   object
    )
    {
    NameToObject::Iterator it = name_to_object.find( name );
    if( it != name_to_object.end() )
        {
        // Something registered under @a name. Is it the @a object?

        if( it->second == object )
            {
            // Ok, unregister.

            name_to_object.erase( name );
            return true;
            }
        else
            {
            // Nope. Leave the registration.

            return false;
            }
        }

    // No such registration.

    return false;
    }


} // namespace Demo::Lib
} // namespace Demo
