/** @file physical_info.cpp
 * Physical info implementation.
 */

/*
 * begin        : Mon Jul 28 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/server/physical_info.h"

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();

/**
 * Define this to enable solid entities by default.
 */
//#define ENTITIES_ARE_SOLID_BY_DEFAULT


PhysicalInfo::PhysicalInfo() :
    #ifdef ENTITIES_ARE_SOLID_BY_DEFAULT
    physical_class( CLASS_SOLID ),
    #endif
    max_slope( 1.0f )
    {
    initialize_object();
    
    physical_class.default_constructed();
    max_slope.default_constructed();
    default_constructed();
    }


PhysicalInfo::PhysicalInfo
    (
    const PhysicalInfo & info
    ) : ValueType( info ),
        physical_class( info.physical_class ),
        bounding_box_size( info.bounding_box_size ),
        max_slope( info.max_slope ),
        mass( info.mass )
    {
    initialize_object();
    }


bool PhysicalInfo::collides_with
    (
    const PhysicalInfo & info
    ) const
    {
    return collides_with( info.physical_class );
    }


bool PhysicalInfo::collides_with
    (
    PhysicalClass other_class
    ) const
    {
    if( physical_class == CLASS_SOLID )
        {
        // Solid entities collide with any entity class.
        return true;
        }
    else
    if( physical_class == CLASS_PENETRABLE )
        {
        // Penetrable entities collide with non-penetrable entities.
        return other_class != CLASS_PENETRABLE;
        }
    else
    if( physical_class == CLASS_FLYING )
        {
        // Flying entities do not collide with anything.
        // Possible collisions must be handled by the entity's logic.

        return false;
        }
    
    panic( "Unknown physical class." );
    }


bool PhysicalInfo::can_handle_slope
    (
    Float32 slope
    ) const
    {
    if( physical_class == CLASS_FLYING )
        {
        return true;
        }
    else
        {
        return slope <= max_slope;
        }
    }


} // namespace Demo::Lib
} // namespace Demo
