/** @file potion.cpp
 * Potion (implementation).
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/server/potion.h"
#include "lib/server/player.h"
#include "lib/server/monster.h"
#include "lib/server/garbage.h"
#include "lib/server/monster_rpc.h"

namespace Demo {
namespace Lib {


void Potion::initialize
    (
    PotionType the_potion_type
    )
    {
    // Initialize Item class.
    Item::initialize();
    client_item->item_type = ITEM_POTION;
    client_item->item_variant = the_potion_type;
    }


Pointer< Item > Potion::use_on_monster
    (
    Pointer< Player >       player,
    WeakPointer< Monster >  monster
    )
    {
    // Apply the potion.
    Pointer< Item > result_item;
    switch( client_item->item_variant )
        {
        case POTION_HEALTH:
            monster.remote()->async_apply_powerup( ABILITY_HEALTH,
                30, 0, Entity::POWERUP_PERMANENT, STime() );
            result_item = CreateObject< Garbage >( GARBAGE_FLASK );
            break;

        case POTION_SPEED:
            monster.remote()->async_apply_powerup( ABILITY_SPEED,
                30, 30, Entity::POWERUP_TEMPORARY, STime( 15.0 ) );
            result_item = CreateObject< Garbage >( GARBAGE_FLASK );
            break;

        case POTION_MEAT:
            monster.remote()->async_apply_powerup( ABILITY_HEALTH,
                30, 0, Entity::POWERUP_PERMANENT, STime() );
            result_item = CreateObject< Garbage >( GARBAGE_BONE );
            break;
        }

    if( player == monster )
        {

        player->say( std::string( client_item->item_variant == POTION_MEAT ? "eats " : "drinks " ) +
                     client_item->get_long_name(), Player::MESSAGE_ACTION );
        }
    else
        {
        std::string monster_name;
        if( monster.is_local() )
            {
            monster_name = monster->get_name();
            }
        else
            {
            monster_name = "an entity";
            }
        player->say( std::string( "uses " ) + client_item->get_long_name() + " on " + monster_name, Player::MESSAGE_ACTION );
        }

    return result_item;
    }


} // namespace Demo::Lib
} // namespace Demo
