/** @file remote_console_command.cpp
 * Remote console command implementation.
 */

/*
 * begin        : Tue Dec 12 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "core.h"
#include "lib/shared/console.h"
#include "lib/shared/node_object.h"
#include "lib/server/remote_console_command.h"
#include "lib/shared/account_object_rpc.h"

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


void RemoteConsoleCommand::initialize
    (
    WeakPointer<Object>         object,
    WeakPointer<AccountObject>  my_account,
    const std::string &         remote_command
    )
    {
    account = my_account;
    command = remote_command;
    migrate_to( object, STime() );
    }

    
void RemoteConsoleCommand::delivered_to
    (
    WeakPointer<Object>,
    const STime &
    )
    {
    Pointer<NodeObject>     node = Massiv::System::get_local_node_id();
    Console::Environment    dummy_environment;
    PValueArray<SString>    errors;
    std::string             message;
    std::size_t             i;

    // Process the command.

    errors = node->process_node_command( account, command, dummy_environment );
    
    // Send back error log.
    // This must be done manually because "/rcommand" operates asynchronously
    // and client's NodeObject thinks that it completed already.

    if( !errors.empty() )
        {
        message = "Remote \"" + command.read() + "\" outputs:";

        if( !Console::is_unknown_command_error( errors ) )
            {
            for( i = 0; i < errors.size(); i++ )
                {
                message += "\n";
                message += errors[ i ];
                }
            }
        else
            {
            message += "\nUnknown command.";
            }
        
        account->async_print_to_console( node, message, SColor::RED );
        }
    }


} // namespace Demo::Lib
} // namespace Demo
