/** @file remote_console_command.h
 * Remote console command.
 */

/*
 * begin        : Tue Dec 12 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_REMOTE_CONSOLE_COMMAND_H
#define DEMO_REMOTE_CONSOLE_COMMAND_H

#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SHARED_FWD_H
#include "lib/shared/shared_fwd.h"
#endif

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Simple object carrying command for a remote NodeConsole.
 * Its purpose is to migrate to a specified object and then process the command
 * locally by the corresponding local NodeConsole.
 */
class RemoteConsoleCommand : public Object
    {
public:

    /**
     * Initializes remote console command request.
     * Fills in the command and migrates the request to @c object.
     */
    void initialize
        (
        WeakPointer<Object>         object,
        WeakPointer<AccountObject>  my_account,
        const std::string &         remote_command
        );

    /**
     * Processes @c command by local node's @c NodeObject.
     * Note that this is the node where @c object lives because it is local now.
     */
    virtual void delivered_to
        (
        WeakPointer<Object> object,
        const STime &       time
        );

public:
    
    PRemote<AccountObject>  account;
        /**< Who submits the command. */
    PString                 command;
        /**< Command to be executed by a remote NodeConsole. */
    }; // RemoteConsoleCommand


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_REMOTE_CONSOLE_COMMAND_H
