/** @file teleport.h
 * Teleport.
 */

/*
 * begin        : Mon Sep 1 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_TELEPORT_H
#define DEMO_TELEPORT_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_AREA_H
#include "lib/server/area.h"
#endif


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Simple teleport.
 * An @c Area that teleports its visitors to one of teleport's targets
 * (which are chosen randomly). The teleport is triggered upon entity enter
 * to the monitored area. If the entity can not be teleported it is not
 * teleported until the entity leaves and re-enters the area.
 * Only physicals are teleported.
 */
class Teleport : public Area
    {
protected:
    
    MASSIV_OBJECT( Teleport );

    typedef PArray<EntityPosition> TeleportTargets;

public:

    //
    // Teleport interface.
    //

    /** Default-initializes teleport object. */
    virtual void initialize();

    /** Initializes object. */
    void initialize
        (
        const ConvexArea &  the_area,
        int                 the_entity_flags
        );

    /** Adds @c target to teleport target list. */
    void add_target
        (
        const EntityPosition & target
        );

protected:

    //
    // Area interface implementation.
    //

    /**
     */
    virtual void entered
        (
        WeakPointer<Physical> visitor
        );

protected:

    TeleportTargets targets;
        /**< Teleport targets. */
        
    }; // Teleport


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_TELEPORT_H
