/** @file weapon.cpp
 * Weapon item (implementation).
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys, Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/server/weapon.h"
#include "lib/server/potion.h"
#include "lib/server/player.h"
#include "lib/server/player_rpc.h"
#include "lib/shared/client_entity.h"

namespace Demo {
namespace Lib {


void Weapon::initialize
    (
    WeaponType the_weapon_type
    )
    {
    // Initialize Item class.
    Item::initialize();
    client_item->item_type = ITEM_WEAPON;
    client_item->item_variant = the_weapon_type;
    }


Pointer< Item > Weapon::use_on_monster
    (
    Pointer< Player >       player,
    WeakPointer< Monster >  monster
    )
    {
    if( player == monster )
        {
        player->say( "tries to attack self, but changes mind", Player::MESSAGE_ACTION );
        return this;
        }

    std::string monster_name;
    if( monster.is_local() )
        {
        monster_name = monster->get_name(); // FIXME: get it via SRPC?
        }
    else
        {
        monster_name = "an entity";
        }

    if( player->get_client_entity()->get_ability_value( ABILITY_HEALTH ) <= 0 )
        {
        player->say( std::string( "tries to attack " ) + monster_name + ", but is too weak", Player::MESSAGE_ACTION );
        return this;
        }

    // Attack the monster.

    PPair< PEnum< ItemType >, PVlInt32 > acquired_item;
    const Monster::AttackResult attack_result = monster.remote()->sync_attack_by
        (
        acquired_item,
        player->get_client_entity()->current_position,
        static_cast< WeaponType >( client_item->item_variant.read() )
        );

    if( attack_result == Monster::ATTACK_HIT || attack_result == Monster::ATTACK_KILLED )
        {
        player->say( std::string( attack_result == Monster::ATTACK_KILLED ? "kills " : "hits " ) +
                     monster_name + " with " + client_item->get_long_name(), Player::MESSAGE_ACTION );
        Pointer< Item > new_item = Item::create( acquired_item.first, acquired_item.second );

        if( new_item != null )
            {
            // We got a new item.
            // Schedule the message to the future.

            if( player->put_item_to_inventory( new_item ) )
                {
                player.remote()->param( RPC_DELAYED, 0.5f ).async_say( std::string( "obtains " ) +
                    new_item->client_item->get_long_name(), Player::MESSAGE_ACTION );
                }
            else
                {
                player.remote()->param( RPC_DELAYED, 0.5f ).async_say( std::string( "nearly obtains " ) +
                    new_item->client_item->get_long_name() + ", but drops it because of full inventory", Player::MESSAGE_ACTION );
                }
            }
        }
    else
        {
        if( attack_result == Monster::ATTACK_TOO_FAR )
            {
            player->say( std::string( "swings " ) + client_item->get_long_name() + " towards " +
                         monster_name + ", but can't reach", Player::MESSAGE_ACTION );
            }
        else if( attack_result == Monster::ATTACK_ALREADY_DEAD )
            {
            player->say( std::string( "hits dead body of " ) + monster_name + " with " +
                         client_item->get_long_name(), Player::MESSAGE_ACTION );
            }
        else
            {
            player->say( std::string( "tries to attack " ) + monster_name + " with " +
                         client_item->get_long_name() + ", but fails", Player::MESSAGE_ACTION );
            }
        }

    return this;
    }


} // namespace Demo::Lib
} // namespace Demo
