/** @file world.h
 * World.
 */

/*
 * begin        : Mon Jun 31 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_WORLD_H
#define DEMO_WORLD_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SERVER_FWD_H
#include "lib/server/server_fwd.h"
#endif



namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * World object.
 * Represents a virtual world consisting of so called layers.
 * Each layer has its own coordinate system and internal representation.
 */
class World : public Object
    {
public:

    MASSIV_OBJECT( World );
    friend class Layer;

    //
    // World interface.
    //

    /** Initializes world object. */
    void initialize();
    
    /** Registers this layer to a World. */
    void register_layer
        (
        Pointer<Layer> layer
        );

    /** Unregisters the layer from a World. */
    void unregister_layer
        (
        Pointer<Layer> layer
        );

protected:

    PPointer<List> layer_list;
        /**< List of world layers. */

    }; // World


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_WORLD_H
