/** @file ability.h
 * Ability.
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_ABILITY_H
#define DEMO_ABILITY_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Ability represents some entity ability that
 * is represented by a number.
 * Ability is a simple container that holds two
 * integer values. The first stores current value
 * of the ability, the second holds upper limit of
 * the ability.
 */
class Ability : public ::Massiv::Core::ValueType
    {
public:

    /** Default constructor. */
    Ability();
    
    /** Constructor. */
    Ability
        (
        Int32   value,
        Int32   upper_limit
        );
    
    /** Copy constructor. */
    Ability
        (
        const Ability & other
        );

public:

    /**
     * Set value of the ability.
     * The value is clampled against upper limit.
     */
    void set_value
        (
        Int32   new_value
        );

    /**
     * Add @a delta to current value of the ability.
     * The @a delta may be negative.
     * The value is clampled against upper limit.
     */
    void add_value
        (
        Int32   delta
        );
    
    /**
     * Set upper limit.
     * Current value of the ability is clampled against
     * the new upper limit.
     */
    void set_upper_limit
        (
        Int32   new_upper_limit
        );
    
    /**
     * Add @a delta to current upper limit.
     * The @a delta may be negative.
     * Current value of the ability is clampled against
     * the new upper limit.
     */
    void add_upper_limit
        (
        Int32   delta
        );
        
public:

    PVlInt32        value;
        /**< Current value. */
    
    PVlInt32        upper_limit;
        /**< Upper value limit of the ability. */
        
    }; // Ability


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_ABILITY_H
