/** @file client_entity.cpp
 * ClientEntity (implementation).
 */

/*
 * begin        : Tue Aug 5 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/shared/client_entity.h"


namespace Demo {
namespace Lib {


void ClientEntity::initialize
    (
    Pointer<Object> the_entity
    )
    {
    entity = the_entity;
    type = TYPE_DEFAULT;
    stopped = true;
    }


bool ClientEntity::supports_ability
    (
    AbilityId   ability
    ) const
    {
    if( ability >= 0 && static_cast< unsigned int >( ability ) < abilities.size() )
        {
        return true;
        }
    else
        {
        return false;
        }
    }


std::string ClientEntity::ability_to_string
    (
    AbilityId ability
    )
    {
    switch( ability )
        {
        case ABILITY_HEALTH:
            return "health";

        case ABILITY_SPEED:
            return "speed";

        case ABILITY_COUNT:
            break;
        }

    return "unknown";
    }


Int32 ClientEntity::get_ability_value
    (
    AbilityId   ability
    ) const
    {
    if( supports_ability( ability ) )
        {
        return abilities[ ability ].value;
        }
    else
        {
        return 0;
        }
    }


Int32 ClientEntity::get_ability_upper_limit
    (
    AbilityId   ability
    ) const
    {
    if( supports_ability( ability ) )
        {
        return abilities[ ability ].upper_limit;
        }
    else
        {
        return 0;
        }
    }


void ClientEntity::set_ability_value
    (
    AbilityId   ability,
    Int32       new_value
    )
    {
    if( supports_ability( ability ) )
        {
        abilities[ ability ].set_value( new_value );
        }
    }


void ClientEntity::add_ability_value
    (
    AbilityId   ability,
    Int32       delta
    )
    {
    if( supports_ability( ability ) )
        {
        abilities[ ability ].add_value( delta );
        }
    }


void ClientEntity::set_ability_upper_limit
    (
    AbilityId   ability,
    Int32       new_upper_limit
    )
    {
    if( supports_ability( ability ) )
        {
        abilities[ ability ].set_upper_limit( new_upper_limit );
        }
    }


void ClientEntity::add_ability_upper_limit
    (
    AbilityId   ability,
    Int32       delta
    )
    {
    if( supports_ability( ability ) )
        {
        abilities[ ability ].add_upper_limit( delta );
        }
    }


bool ClientEntity::is_visual
    (
    bool admin_mode
    ) const
    {
    // Logic entities have no visual representation.

    if( !admin_mode && ( type == TYPE_LOGIC || type == TYPE_LAMP ||
        type == TYPE_COLOR_GENERATOR ) )
        {
        return false;
        }
    return true;
    }


std::string ClientEntity::get_model_tags() const
    {
    hope( is_visual( true ) );

    // Start with optional tags.

    std::string tags( model_tags );

    // Add type-specific tags.

    bool use_name = false;
    bool has_blood = false;
    switch( type )
        {
        case TYPE_MONSTER:
            // Use monster entity name as a tag. Monster names must be
            // chosen carefully to prevent weird tag matches (don't call
            // monster "player" for example).

            tags += " monster";
            use_name = true;
            has_blood = true;
            break;

        case TYPE_PLAYER:
            // Do NOT use player name as a tag as it could break tag matching.

            tags += " player";
            has_blood = true;
            break;

        default:
            // Other types add no tags...
            break;
        }

    // Some types use entity name as a tag too.

    if( use_name )
        {
        tags += " ";
        tags += name;
        }

    // Some entities have bloody skins.

    if( has_blood && ( get_ability_value( ABILITY_HEALTH ) < get_ability_upper_limit( ABILITY_HEALTH ) / 2 ) )
        {
        tags += " blood";
        }

    // Done.

    return tags;
    }


} // namespace Demo::Lib
} // namespace Demo
