/** @file client_item.cpp
 * ClientItem (implementation).
 */

/*
 * begin        : Tue Aug 5 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/shared/client_item.h"


namespace Demo {
namespace Lib {


void ClientItem::initialize()
    {
    item_type = ITEM_UNKNOWN;
    item_variant = -1;
    equip_type = EQUIP_NONE;
    equipped = false;
    }


bool ClientItem::is_equipped() const
    {
    return equipped;
    }


bool ClientItem::is_equipable() const
    {
    return ( equip_type != EQUIP_NONE );
    }


std::string ClientItem::get_name() const
    {
    std::string desc;

    switch( item_type )
        {
        case ITEM_POTION:
            switch( item_variant )
                {
                case POTION_HEALTH:
                    desc = "health potion";
                    break;
                case POTION_SPEED:
                    desc = "speed potion";
                    break;
                case POTION_MEAT:
                    desc = "meat";
                    break;
                default:
                    desc = "unknown potion";
                }
            break;

        case ITEM_GARBAGE:
            switch( item_variant )
                {
                case GARBAGE_FLASK:
                    desc = "empty flask";
                    break;
                case GARBAGE_BONE:
                    desc = "bone";
                    break;
                default:
                    desc = "garbage";
                }
            break;

        case ITEM_BOOTS:
            switch( item_variant )
                {
                case BOOTS_SPEED:
                    desc = "magic boots";
                    break;
                default:
                    desc = "unknown boots";
                }
            break;

        case ITEM_WEAPON:
            switch( item_variant )
                {
                case WEAPON_AXE:
                    desc = "axe";
                    break;
                default:
                    desc = "unknown weapon";
                }
            break;

        default:
            desc = "unkown item";
        }

    return desc;
    }


std::string ClientItem::get_long_name() const
    {
    std::string desc;

    switch( item_type )
        {
        case ITEM_POTION:
            switch( item_variant )
                {
                case POTION_HEALTH:
                    desc = "a health potion";
                    break;
                case POTION_SPEED:
                    desc = "a speed potion";
                    break;
                case POTION_MEAT:
                    desc = "tasty meat";
                    break;
                default:
                    desc = "an unknown potion";
                }
            break;

        case ITEM_GARBAGE:
            switch( item_variant )
                {
                case GARBAGE_FLASK:
                    desc = "an empty flask";
                    break;
                case GARBAGE_BONE:
                    desc = "a meatless bone";
                    break;
                default:
                    desc = "garbage";
                }
            break;

        case ITEM_BOOTS:
            switch( item_variant )
                {
                case BOOTS_SPEED:
                    desc = "magic boots";
                    break;
                default:
                    desc = "unknown boots";
                }
            break;

        case ITEM_WEAPON:
            switch( item_variant )
                {
                case WEAPON_AXE:
                    desc = "an axe";
                    break;
                default:
                    desc = "an unknown weapon";
                }
            break;

        default:
            desc = "an unkown item";
        }

    // Use short name as default.

    if( desc.empty() )
        {
        desc = "a ";
        desc += get_name();
        }

    return desc;
    }


} // namespace Demo::Lib
} // namespace Demo
