/** @file client_item.h
 * ClientItem.
 */

/*
 * begin        : Tue Aug 5 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_CLIENT_ITEM_H
#define DEMO_CLIENT_ITEM_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_GAME_DEFS_H
#include "lib/shared/game_defs.h"
#endif


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Client visible part of inventory item.
 */
class ClientItem : public Object
    {
public:

    MASSIV_OBJECT( ClientItem );

public:

    /** Initialize the object. */
    void initialize();

public:

    /**
     * Is the item equipped?
     */
    bool is_equipped() const;

    /**
     * Is the item equipable?
     * @return True if @a equip_type is not @c EQUIP_NONE.
     */
    bool is_equipable() const;

    /**
     * Get short item name.
     */
    std::string get_name() const;

    /**
     * Get longer and nicer item name.
     */
    std::string get_long_name() const;

public:

    PEnum< ItemType >   item_type;
        /**< Type of the item. */

    PVlInt32            item_variant;
        /**< Variant of the item. Value depends on @a item_type. */

    PEnum< EquipType >  equip_type;
        /**< Equip cathegory of the item. */

    PBool               equipped;
        /**< Is the item equipped? */

    }; // ClientItem


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_CLIENT_ITEM_H
