/** @file client_light.h
 * Optional ClientEntity light source parameters.
 */

/*
 * begin        : Nov 2003
 * copyright    : (C) 2003 by Ondrej Pecta, Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_CLIENT_LIGHT_H
#define DEMO_CLIENT_LIGHT_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SHARED_FWD_H
#include "lib/shared/shared_fwd.h"
#endif


namespace Demo {
namespace Lib {


INJECT_CORE_SYMBOLS();


/**
 * Parameters of omnidirectional light source.
 *
 * ClientLight is optional part of ClientEntity - any entity may be light
 * source as well, all light sources are entities. See
 * Demo::Client::Renderer::Light for more info about light parameter semantics
 * and their default values.
 */
class ClientLight : public Object
    {
public:
    
    MASSIV_OBJECT( ClientLight );

public:

    /**
     * Initialize the object.
     */
    void initialize();

public:

    //@{
    /** Getter. */
    const SVector3 get_offset() const;

    const SColor get_ambient_intensity() const;

    const SColor get_diffuse_intensity() const;

    const SColor get_specular_intensity() const;

    const SVector3 get_attenuation() const;
    //@}

    //@{
    /** Setter. */
    void set_offset
        (
        const SVector3 new_offset
        );

    void set_ambient_intensity
        (
        const SColor new_ambient_intensity
        );

    void set_diffuse_intensity
        (
        const SColor new_diffuse_intensity
        );

    void set_specular_intensity
        (
        const SColor new_diffuse_intensity
        );

    void set_attenuation
        (
        const SVector3 new_attenuation
        );
    //@}

private:

    PVector3        offset;
        /**< Offset of light position, relative to entity origin position.
             It will be rotated by entity orientation. Default value is zero
             vector, but it should be always set to something else, because
             entity origin usually lies on the ground. */

    PColor          ambient_intensity;
        /**< Ambient intensity of the light. */

    PColor          diffuse_intensity;
        /**< Diffuse intensity of the light. */

    PColor          specular_intensity;
        /**< Specular intensity of the light. */

    PVector3        attenuation;
        /**< Light intensity attenuation factors. */

    }; // ClientLight


} // namespace Demo::Lib
} // namespace Demo


#endif // DEMO_CLIENT_LIGHT_H
