/** @file client_sector.cpp
 * ClientSector (implementation).
 */

/*
 * begin        : Aug 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/shared/client_sector.h"


namespace Demo {
namespace Lib {


const Float32 ClientSector::min_height = -8.0f;
const Float32 ClientSector::max_height = 8.0f;

const Float32 ClientSector::HeightTraits::min_value = ClientSector::min_height;
const Float32 ClientSector::HeightTraits::max_value = ClientSector::max_height;


void ClientSector::initialize()
    {
    }


std::string ClientSector::map_kind_to_string
    (
    ElevationMapKind map_kind
    )
    {
    switch( map_kind )
        {
        case KIND_ELEVATION_MAP:
            return "elevation_map";
        case KIND_DELTA_ELEVATION_MAP:
            return "delta_elevation_map";
        }
    return "unknown";
    }


bool ClientSector::string_to_map_kind
    (
    const std::string & str,
    ElevationMapKind &  map_kind
    )
    {
    if( str == "elevation_map" )
        {
        map_kind = KIND_ELEVATION_MAP;
        return true;
        }
    else if( str == "delta_elevation_map" )
        {
        map_kind = KIND_DELTA_ELEVATION_MAP;
        return true;
        }
    
    return false;
    }


std::string ClientSector::map_function_to_string
    (
    ElevationMapFunction map_function
    )
    {
    switch( map_function )
        {
        case FUNCTION_CIRCLE:
            return "circle";
        case FUNCTION_FUZZY_CIRCLE:
            return "fuzzy_circle";
        case FUNCTION_GAUSSIAN:
            return "gaussian";
        }
    
    return "unknown";
    }


bool ClientSector::string_to_map_function
    (
    const std::string &     str,
    ElevationMapFunction &  map_function
    )
    {
    if( str == "circle" )
        {
        map_function = FUNCTION_CIRCLE;
        return true;
        }
    if( str == "fuzzy_circle" )
        {
        map_function = FUNCTION_FUZZY_CIRCLE;
        return true;
        }
    if( str == "gaussian" )
        {
        map_function = FUNCTION_GAUSSIAN;
        return true;
        }
    
    return false;
    }


std::string ClientSector::map_operation_to_string
    (
    ElevationMapOperation map_operation
    )
    {
    switch( map_operation )
        {
        case OPERATION_SET:
            return "set";
        case OPERATION_ADD:
            return "add";
        case OPERATION_MULTIPLY:
            return "multiply";
        }
    
    return "multiply";
    }


bool ClientSector::string_to_map_operation
    (
    const std::string &     str,
    ElevationMapOperation & map_operation
    )
    {
    if( str == "set" )
        {
        map_operation = OPERATION_SET;
        return true;
        }
    else if( str == "add" )
        {
        map_operation = OPERATION_ADD;
        return true;
        }
    else if( str == "multiply" )
        {
        map_operation = OPERATION_MULTIPLY;
        return true;
        }
        
    return false;
    }


} // namespace Demo::Lib
} // namespace Demo
