/** @file container.cpp
 * Default container implementation.
 */

/*
 * begin        : Wed Feb 19 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/shared/container.h"

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


// Implementation of BasicEnumerator
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void BasicEnumerator::erase_and_next()
    {
    Pointer<BasicEnumerator> e = clone().convert();
    e->next();
    erase();
    copy( e );
    }


// Implementation of BasicContainer
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

int BasicContainer::capacity()
    {
    return size();
    }


bool BasicContainer::empty()
    {
    return !size();
    }


// Implementation of Container
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~

bool Container::contains
    (
    const WeakPointer<Object> & object
    )
    {
    Pointer<Enumerator> e = equal_range( object );

    if( !e->is_end() )
        {
        return true;
        }

    return false;
    }


bool Container::erase
    (
    const WeakPointer<Object> & object
    )
    {
    Pointer<Enumerator> e = equal_range( object );

    if( !e->is_end() )
        {
        e->erase();
        return true;
        }

    return false;
    }


int Container::erase_all
    (
    const WeakPointer<Object> & object
    )
    {
    Pointer<Enumerator> e = equal_range( object );
    int count = 0;

    while( !e->is_end() )
        {
        e->erase_and_next();
        count++;
        }

    return false;
    }


bool Container::insert
    (
    const WeakPointer<Object> & object
    )
    {
    Pointer<Enumerator> e;

    insert( object, e );
    return !e->is_end();
    }


// Implementation of AssociativeContainer
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

bool AssociativeContainer::contains
    (
    const WeakPointer<Object> & key
    )
    {
    Pointer<AssociativeEnumerator> e = equal_range( key );

    if( !e->is_end() )
        {
        return true;
        }

    return false;
    }


bool AssociativeContainer::contains
    (
    const WeakPointer<Object> & key,
    const WeakPointer<Object> & data
    )
    {
    Pointer<AssociativeEnumerator> e = equal_range( key );

    while( !e->is_end() )
        {
        WeakPointer<Object> o = e->current_data();

        if( equal_data( data, o ) )
            {
            return true;
            }

        e->next();
        }

    return false;
    }


WeakPointer<Object> AssociativeContainer::look_up
    (
    const WeakPointer<Object> & key
    )
    {
    Pointer<AssociativeEnumerator> e = equal_range( key );

    if( !e->is_end() )
        {
        return e->current_data();
        }

    return null;
    }


bool AssociativeContainer::erase
    (
    const WeakPointer<Object> & key
    )
    {
    Pointer<AssociativeEnumerator> e = equal_range( key );

    if( !e->is_end() )
        {
        e->erase();
        return true;
        }

    return false;
    }


bool AssociativeContainer::erase
    (
    const WeakPointer<Object> & key,
    const WeakPointer<Object> & data
    )
    {
    Pointer<AssociativeEnumerator> e = equal_range( key );

    while( !e->is_end() )
        {
        WeakPointer<Object> o = e->current_data();

        if( equal_data( data, o ) )
            {
            e->erase();
            return true;
            }
        }

    return false;
    }


int AssociativeContainer::erase_all
    (
    const WeakPointer<Object> & key
    )
    {
    Pointer<AssociativeEnumerator> e = equal_range( key );
    int count = 0;

    while( !e->is_end() )
        {
        e->erase_and_next();
        count++;
        }

    return count;
    }


bool AssociativeContainer::insert
    (
    const WeakPointer<Object> & key,
    const WeakPointer<Object> & data
    )
    {
    Pointer<AssociativeEnumerator> e;
    insert( key, data, e );
    return !e->is_end();
    }


} // namespace Demo::Lib
} // namespace Demo
