/** @file entity_position.cpp
 * Entity position implementation.
 */

/*
 * begin        : Mon Jul 28 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/shared/entity_position.h"
#include "lib/shared/sector_address.h"

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


EntityPosition::EntityPosition()
    {
    initialize_object();
    default_constructed();
    }


EntityPosition::EntityPosition
    (
    const SVector2 &        origin,
    const SOrientation &    orientation,
    WeakPointer<Object>     layer
    ) : origin( origin ),
        orientation( orientation.heading(), orientation.pitch(), orientation.bank() ), // FIXME
        layer( layer )
    {
    initialize_object();
    }


EntityPosition::EntityPosition
    (
    const EntityPosition & position
    ) : ValueType( position ),
        origin( position.origin ),
        orientation( position.orientation ),
        layer( position.layer )
    {
    initialize_object();
    }


bool EntityPosition::is_same_sector
    (
    const EntityPosition & other
    ) const
    {
    SectorAddress address( other );
    
    return is_same_sector( address );
    }


bool EntityPosition::is_same_sector
    (
    const SectorAddress & address
    ) const
    {
    SectorAddress my_address( *this );

    return my_address == address;
    }


bool EntityPosition::epsilon_equals
    (
    const EntityPosition & position
    ) const
    {
    if( !origin.epsilon_equals( position.origin ) )
        {
        return false;
        }

    if( orientation != position.orientation )
        {
        return false;
        }

    if( layer != position.layer )
        {
        return false;
        }

    return true;
    }


} // namespace Demo::Lib
} // namespace Demo
