/** @file entity_position.h
 * Entity position.
 */

/*
 * begin        : Mon Jul 28 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_ENTITY_POSITION_H
#define DEMO_ENTITY_POSITION_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SHARED_FWD_H
#include "lib/shared/shared_fwd.h"
#endif


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Entity position & orientation.
 */
class EntityPosition : public ::Massiv::Core::ValueType
    {
public:

    /** Default constructor. */
    EntityPosition();

    /** Constructor. */
    EntityPosition
        (
        const SVector2 &        origin,
        const SOrientation &    orientation,
        WeakPointer<Object>     layer
        );

    /** Copy constructor. */
    EntityPosition
        (
        const EntityPosition & position
        );

    /** Tests if @c other refers to the same sector as this Position. */
    bool is_same_sector
        (
        const EntityPosition & other
        ) const;

    /** Tests if this EntityPosition refers to @c address sector. */
    bool is_same_sector
        (
        const SectorAddress & address
        ) const;

    /** Tests if @c position is the same like this EntityPosition. */
    bool epsilon_equals
        (
        const EntityPosition & position
        ) const;

public:
        
    PVector2                origin;
        /**< Entity origin in layer 2D coordinates. */
    POrientation            orientation;
        /**< Entity orientation. */
    PWeakPointer<Object>    layer;
        /**< Layer the entity belongs to. */
    }; // EntityPosition


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_ENTITY_POSITION_H
