/** @file hash_table_container.cpp
 * Hash table container implementation.
 */

/*
 * begin        : Wed Feb 19 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/shared/hash_table_container.h"


namespace Demo {
namespace Lib {


// Implementation of HashTable::TailChainEntry
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void HashTable::TailChainEntry::initialize()
    {
    }


WeakPointer<Object> HashTable::TailChainEntry::get_key()
    {
    return null;
    }


WeakPointer<Object> HashTable::TailChainEntry::get_data()
    {
    return null;
    }


// Implementation of HashTable::PureWeakChainEntry
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void HashTable::PureWeakChainEntry::initialize
    (
    const WeakPointer<Object> & the_key,
    const WeakPointer<Object> & the_data
    )
    {
    key = the_key;
    data = the_data;
    }


WeakPointer<Object> HashTable::PureWeakChainEntry::get_key()
    {
    return key;
    }


WeakPointer<Object> HashTable::PureWeakChainEntry::get_data()
    {
    return data;
    }


// Implementation of HashTable::WeakChainEntry
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void HashTable::WeakChainEntry::initialize
    (
    const WeakPointer<Object> & the_key,
    const WeakPointer<Object> & the_data
    )
    {
    key = the_key;
    data = the_data;
    }


WeakPointer<Object> HashTable::WeakChainEntry::get_key()
    {
    return key;
    }


WeakPointer<Object> HashTable::WeakChainEntry::get_data()
    {
    return data;
    }


// Implementation of HashTable::StrongChainEntry
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void HashTable::StrongChainEntry::initialize
    (
    const WeakPointer<Object> & the_key,
    const WeakPointer<Object> & the_data
    )
    {
    key = the_key;
    data = the_data;
    }


WeakPointer<Object> HashTable::StrongChainEntry::get_key()
    {
    return key;
    }


WeakPointer<Object> HashTable::StrongChainEntry::get_data()
    {
    return data;
    }


// Implementation of HashTable::ChainEntryEnumerator
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void HashTable::ChainEntryEnumerator::initialize()
    {
    collision_chain = INVALID_CHAIN;
    }


void HashTable::ChainEntryEnumerator::initialize
    (
    const Pointer<HashTable> &  table,
    Address                     chain,
    BeginTag
    )
    {
    hash_table = table;
    collision_chain = chain;

    if( collision_chain != INVALID_FIRST_CHAIN )
        {
        chain_tail = hash_table->buckets[ collision_chain ];
        entry = chain_tail->next;
        }
    }


void HashTable::ChainEntryEnumerator::initialize
    (
    const Pointer<HashTable> &  table,
    Address                     chain,
    EndTag
    )
    {
    hash_table = table;
    collision_chain = chain;

    if( collision_chain != INVALID_LAST_CHAIN )
        {
        chain_tail = hash_table->buckets[ collision_chain ];
        entry = chain_tail;
        }
    }


void HashTable::ChainEntryEnumerator::initialize
    (
    const Pointer<HashTable> &  table,
    Address                     chain,
    const Pointer<ChainEntry> & this_entry
    )
    {
    hash_table = table;
    collision_chain = chain;

    chain_tail = hash_table->buckets[ collision_chain ];
    entry = this_entry;
    }


bool HashTable::ChainEntryEnumerator::is_end()
    {
    return entry == chain_tail;
    }


bool HashTable::ChainEntryEnumerator::is_begin()
    {
    return entry->prev == chain_tail;
    }


void HashTable::ChainEntryEnumerator::next()
    {
    entry = entry->next;
    }


void HashTable::ChainEntryEnumerator::prev()
    {
    entry = entry->prev;
    }


void HashTable::ChainEntryEnumerator::erase()
    {
    // Erase element.

    entry->prev->next = entry->next;
    entry->next->prev = entry->prev;

    update_table();

    // Invalidate enumerator.

    hash_table = null;
    entry = null;
    chain_tail = null;
    }


void HashTable::ChainEntryEnumerator::erase_and_next()
    {
    Pointer<ChainEntry> next = entry->next;

    // Erase element.

    entry->prev->next = next;
    entry->next->prev = entry->prev;

    update_table();

    // Move enumerator.

    entry = next;
    }


WeakPointer<Object> HashTable::ChainEntryEnumerator::current_key()
    {
    return entry->get_key();
    }


WeakPointer<Object> HashTable::ChainEntryEnumerator::current_data()
    {
    return entry->get_data();
    }


void HashTable::ChainEntryEnumerator::update_table()
    {
    hash_table->num_items--;

    if( chain_tail == chain_tail->next )
        {
        // This was the last element in the collision chain.
        // The chain must be deleted because it contains
        // tail only.
        // Note: @c chain_tail should be left unmodified.

        hash_table->delete_chain( collision_chain );
        }
    }


// Implementation of HashTable::RangeEnumerator
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void HashTable::RangeEnumerator::initialize
    (
    const Pointer<HashTable> &  table,
    const Pointer<Object> &     search_key,
    Address                     chain
    )
    {
    ChainEntryEnumerator::initialize( table, chain, BeginTag() );
    key = search_key;

    // Move forward until element that matches @c key is found.

    while( !is_end() && !table->equal_key( current_key(), key ) )
        {
        ChainEntryEnumerator::next();
        }

    // Now chain entry enumerator points to the first element
    // of RangeEnumerator's sequence.

    begin = entry;
    }


bool HashTable::RangeEnumerator::is_begin()
    {
    return entry == begin;
    }


void HashTable::RangeEnumerator::next()
    {
    ChainEntryEnumerator::next();

    while( !is_end() && !hash_table->equal_key( current_key(), key ) )
        {
        ChainEntryEnumerator::next();
        }
    }


void HashTable::RangeEnumerator::prev()
    {
    ChainEntryEnumerator::prev();

    while( !is_begin() && !hash_table->equal_key( current_key(), key ) )
        {
        ChainEntryEnumerator::prev();
        }
    }


// Implementation of HashTable::HashTableEnumerator
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void HashTable::HashTableEnumerator::initialize
    (
    const Pointer<HashTable> &  table,
    BeginTag
    )
    {
    ChainEntryEnumerator::initialize( table, table->first_chain, BeginTag() );
    }


void HashTable::HashTableEnumerator::initialize
    (
    const Pointer<HashTable> &  table,
    EndTag
    )
    {
    ChainEntryEnumerator::initialize( table, table->last_chain, EndTag() );
    }


bool HashTable::HashTableEnumerator::is_end()
    {
    if( ChainEntryEnumerator::is_end() )
        {
        // We are the end the collision chain.
        // If this is the last non-empty collision chain
        // we are at the end of the whole enumerated sequence.

        return collision_chain == hash_table->last_chain;
        }
    else
        {
        return false;
        }
    }


bool HashTable::HashTableEnumerator::is_begin()
    {
    if( ChainEntryEnumerator::is_begin() )
        {
        // We are at the beginning of the collision chain.
        // If this is the first non-empty collision chain
        // we are at the beginning of the whole enumerated
        // sequence.

        return collision_chain == hash_table->first_chain;
        }
    else
        {
        return false;
        }
    }


void HashTable::HashTableEnumerator::next()
    {
    ChainEntryEnumerator::next();
    next_chain();
    }


void HashTable::HashTableEnumerator::prev()
    {
    prev_chain();
    ChainEntryEnumerator::prev();
    }


void HashTable::HashTableEnumerator::erase_and_next()
    {
    ChainEntryEnumerator::erase_and_next();
    next_chain();
    }


void HashTable::HashTableEnumerator::next_chain()
    {
    if( ChainEntryEnumerator::is_end() && collision_chain < hash_table->last_chain )
        {
        // We are at the end of the collision chain.
        // Move enumerator to the beginning of the next non-empty chain.

        do
            {
            chain_tail = hash_table->buckets[ ++collision_chain ];
            }
        while( chain_tail == null );

        entry = chain_tail->next;
        }
    }


void HashTable::HashTableEnumerator::prev_chain()
    {
    if( ChainEntryEnumerator::is_begin() && collision_chain > hash_table->first_chain )
        {
        // We are at the beginning of a collision chain.
        // Move enumerator to the end of the previous non-empty chain.

        do
            {
            chain_tail = hash_table->buckets[ --collision_chain ];
            }
        while( chain_tail == null );

        entry = chain_tail;
        }
    }


// Implementation of HashTable
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~

void HashTable::initialize
    (
    int container_flags,
    int table_size
    )
    {
    flags = container_flags;

    num_items = 0;
    first_chain = INVALID_FIRST_CHAIN;
    last_chain = INVALID_LAST_CHAIN;

    buckets.clear();
    buckets.resize( table_size );
    }


int HashTable::size()
    {
    return num_items;
    }


void HashTable::clear()
    {
    initialize( flags, buckets.size() );
    }


Pointer<AssociativeEnumerator> HashTable::begin()
    {
    return CreateObject<HashTableEnumerator>( this, BeginTag() );
    }


Pointer<AssociativeEnumerator> HashTable::end()
    {
    return CreateObject<HashTableEnumerator>( this, EndTag() );
    }


bool HashTable::contains
    (
    const WeakPointer<Object> & key
    )
    {
    Address             chain = get_address( key );
    Pointer<ChainEntry> tail = buckets[ chain ];

    if( tail == null )
        {
        return false;
        }

    Pointer<ChainEntry> entry = tail->next;

    while( entry != tail )
        {
        if( equal_key( entry->get_key(), key ) )
            {
            return true;
            }

        entry = entry->next;
        }

    return false;
    }


bool HashTable::contains
    (
    const WeakPointer<Object> & key,
    const WeakPointer<Object> & data
    )
    {
    Address             chain = get_address( key );
    Pointer<ChainEntry> tail = buckets[ chain ];

    if( tail == null )
        {
        return false;
        }

    Pointer<ChainEntry> entry = tail->next;

    while( entry != tail )
        {
        if( equal_key( entry->get_key(), key ) &&
            equal_data( entry->get_data(), data ) )
            {
            return true;
            }

        entry = entry->next;
        }

    return false;
    }


WeakPointer<Object> HashTable::look_up
    (
    const WeakPointer<Object> & key
    )
    {
    Address             chain = get_address( key );
    Pointer<ChainEntry> tail = buckets[ chain ];

    if( tail == null )
        {
        return null;
        }

    Pointer<ChainEntry> entry = tail->next;

    while( entry != tail )
        {
        if( equal_key( entry->get_key(), key ) )
            {
            return entry->get_data();
            }

        entry = entry->next;
        }

    return null;
    }


bool HashTable::erase
    (
    const WeakPointer<Object> & key
    )
    {
    Address             chain = get_address( key );
    Pointer<ChainEntry> tail = buckets[ chain ];

    if( tail == null )
        {
        return false;
        }

    Pointer<ChainEntry> entry = tail->next;

    while( entry != tail )
        {
        if( equal_key( entry->get_key(), key ) )
            {
            entry->prev->next = entry->next;
            entry->next->prev = entry->prev;

            if( tail->next == tail )
                {
                delete_chain( chain );
                }

            num_items--;
            return true;
            }

        entry = entry->next;
        }

    return false;
    }


bool HashTable::erase
    (
    const WeakPointer<Object> & key,
    const WeakPointer<Object> & data
    )
    {
    Address             chain = get_address( key );
    Pointer<ChainEntry> tail = buckets[ chain ];

    if( tail == null )
        {
        return false;
        }

    Pointer<ChainEntry> entry = tail->next;

    while( entry != tail )
        {
        if( equal_key( entry->get_key(), key ) &&
            equal_data( entry->get_key(), data ) )
            {
            entry->prev->next = entry->next;
            entry->next->prev = entry->prev;

            if( tail->next == tail )
                {
                delete_chain( chain );
                }

            num_items--;
            return true;
            }

        entry = entry->next;
        }

    return false;
    }


int HashTable::erase_all
    (
    const WeakPointer<Object> & key
    )
    {
    Address             chain = get_address( key );
    Pointer<ChainEntry> tail = buckets[ chain ];
    int                 count = 0;

    if( tail == null )
        {
        return false;
        }

    Pointer<ChainEntry> entry = tail->next;

    while( entry != tail )
        {
        if( equal_key( entry->get_key(), key ) )
            {
            Pointer<ChainEntry> next_entry = entry->next;

            entry->prev->next = next_entry;
            entry->next->prev = entry->prev;
            entry = next_entry;

            count++;

            if( tail->next == tail )
                {
                delete_chain( chain );
                break;
                }
            }
        else
            {
            entry = entry->next;
            }
        }

    num_items -= count;
    return count;
    }


Pointer<AssociativeEnumerator> HashTable::equal_range
    (
    const WeakPointer<Object> & key
    )
    {
    Address chain = get_address( key );

    if( buckets[ chain ] != null )
        {
        return CreateObject<RangeEnumerator>( this, key, chain );
        }
    else
        {
        return CreateObject<ChainEntryEnumerator>();
        }
    }


bool HashTable::insert
    (
    const WeakPointer<Object> & key,
    const WeakPointer<Object> & data
    )
    {
    Address chain = get_address( key );

    if( !flags_on( flags, FLAGS_ALLOW_NULL_DATA ) && data == null )
        {
        return false;
        }

    if( buckets[ chain ] == null )
        {
        create_chain( chain, key, data );
        }
    else
        {
        // Test uniqueness.

        if( !flags_on( flags, FLAGS_ALLOW_DUPLICATE_KEYS ) )
            {
            Pointer<ChainEntry> entry = buckets[ chain ]->next;

            while( entry != buckets[ chain ] )
                {
                if( equal_key( entry->get_key(), key ) )
                    {
                    return false;
                    }

                entry = entry->next;
                }
            }

        // Create new entry and link it to the beginning of the collision chain.

        Pointer<ChainEntry> entry = create_entry( key, data );

        entry->next = buckets[ chain ]->next;
        entry->prev = buckets[ chain ];
        buckets[ chain ]->next = entry;

        num_items++;
        }

    return true;
    }


void HashTable::insert
    (
    const WeakPointer<Object> &         key,
    const WeakPointer<Object> &         data,
    Pointer<AssociativeEnumerator> &    enumerator
    )
    {
    Address             chain = get_address( key );
    Pointer<ChainEntry> entry;

    if( !flags_on( flags, FLAGS_ALLOW_NULL_DATA ) && data == null )
        {
        enumerator = CreateObject<ChainEntryEnumerator>();
        return;
        }

    if( buckets[ chain ] == null )
        {
        create_chain( chain, key, data );
        entry = buckets[ chain ]->next;
        }
    else
        {
        // Test uniqueness.

        if( !flags_on( flags, FLAGS_ALLOW_DUPLICATE_KEYS ) )
            {
            entry = buckets[ chain ]->next;

            while( entry != buckets[ chain ] )
                {
                if( equal_key( entry->get_key(), key ) )
                    {
                    enumerator = CreateObject<ChainEntryEnumerator>();
                    return;
                    }

                entry = entry->next;
                }
            }

        // Create new entry and link it to the beginning of the collision chain.

        entry = create_entry( key, data );

        entry->next = buckets[ chain ]->next;
        entry->prev = buckets[ chain ];
        buckets[ chain ]->next = entry;

        num_items++;
        }

    enumerator = CreateObject<ChainEntryEnumerator>( this, chain, entry );
    }


bool HashTable::equal_key
    (
    const WeakPointer<Object> & key1,
    const WeakPointer<Object> & key2
    ) const
    {
    if( flags_on( flags, FLAGS_STRONG_REFERENCED_KEYS ) )
        return key1->equals( key2 );
    else
        return key1 == key2;
    }


bool HashTable::equal_data
    (
    const WeakPointer<Object> & data1,
    const WeakPointer<Object> & data2
    ) const
    {
    if( flags_on( flags, FLAGS_STRONG_REFERENCED_DATA ) )
        return data1->equals( data2 );
    else
        return data1 == data2;
    }
    

int HashTable::hash_key
    (
    const WeakPointer<Object> & key
    ) const
    {
    if( flags_on( flags, FLAGS_STRONG_REFERENCED_KEYS ) )
        return key->hash();
    else
        return ObjectId::Hash()( key.get_object_id() );
    }


Pointer<HashTable::ChainEntry> HashTable::create_entry
    (
    const WeakPointer<Object> & key,
    const WeakPointer<Object> & data
    )
    {
    if( flags_on( flags, FLAGS_STRONG_REFERENCED_KEYS ) )
        {
        if( flags_on( flags, FLAGS_STRONG_REFERENCED_DATA ) )
            return CreateObject<StrongChainEntry>( key, data );
        else
            return CreateObject<WeakChainEntry>( key, data );
        }
    else
        return CreateObject<PureWeakChainEntry>( key, data );
    }


HashTable::Address HashTable::get_address
    (
    const WeakPointer<Object> & key
    ) const
    {
    return hash_key( key ) % buckets.size();
    }


void HashTable::create_chain
    (
    Address                     chain,
    const WeakPointer<Object> & key,
    const WeakPointer<Object> & data
    )
    {
    // Create chain.

    Pointer<ChainEntry> tail = CreateObject<TailChainEntry>();
    Pointer<ChainEntry> entry = create_entry( key, data );

    tail->next = entry;
    tail->prev = entry;
    entry->next = tail;
    entry->prev = tail;

    buckets[ chain ] = tail;

    // Update table.

    if( chain < first_chain )
        {
        first_chain = chain;
        }

    if( chain > last_chain )
        {
        last_chain = chain;
        }

    num_items++;
    }


void HashTable::delete_chain
    (
    Address chain
    )
    {
    // Delete chain.

    buckets[ chain ] = null;

    // Update table.

    if( chain == first_chain )
        {
        while( ++chain < last_chain )
            {
            if( buckets[ chain ] != null )
                {
                first_chain = chain;
                return;
                }
            }

        first_chain = INVALID_FIRST_CHAIN;
        }
    else
    if( chain == last_chain )
        {
        while( --chain > first_chain )
            {
            if( buckets[ chain ] != null )
                {
                last_chain = chain;
                return;
                }
            }

        last_chain = INVALID_LAST_CHAIN;
        }
    }


} // namespace Demo::Lib
} // namespace Demo
