/** @file lib.h
 * Lib main include file.
 * Include this as a first include in each header file.
 */

/*
 * begin        : Fri Jul 4 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_LIB_H
#define DEMO_LIB_H


#ifndef MASSIV_CORE_H
#error "core.h must be included from each .cpp as the first include!"
#endif


/*
 * Lib conventions:
 *
 * Each class attribute declared as public may be accessed read-only
 * (even by subclasses), unless otherwise noted.
 * Setter method must be created if the attribute may be modified
 * out-of-class or by subclasses. If the attribute may be modified
 * by subclasses only, make the setter method protected.
 */

/**
 * Macro that imports core symbols to current scope.
 * Used by the lib. This is provided as a replacement for simple
 * "using namespace Massiv::Core" that does not (often) work properly
 * under MSVC 6.0 (some symbols need be explicitly imported by
 * "using symbol").
 * The list of imported symbols is not complete. Feel free to extend it
 * as needed.
 * Sorted in an alphabetic order.
 */
#define INJECT_CORE_SYMBOLS() \
    using Massiv::Core::CreateObject; \
    using Massiv::Core::Float32; \
    using Massiv::Core::Int32; \
    using Massiv::Core::UInt32; \
    using Massiv::Core::MetaObject; \
    using Massiv::Core::null; \
    using Massiv::Core::Object; \
    using Massiv::Core::ObjectId; \
    using Massiv::Core::PArray; \
    using Massiv::Core::PBool; \
    using Massiv::Core::PColor; \
    using Massiv::Core::PDictionary; \
    using Massiv::Core::PEnum; \
    using Massiv::Core::PFloat32; \
    using Massiv::Core::PInt; \
    using Massiv::Core::PInt32; \
    using Massiv::Core::POrientation; \
    using Massiv::Core::PPointer; \
    using Massiv::Core::PRemote; \
    using Massiv::Core::Property; \
    using Massiv::Core::PString; \
    using Massiv::Core::PTime; \
    using Massiv::Core::PValueArray; \
    using Massiv::Core::PVector2; \
    using Massiv::Core::PVector3; \
    using Massiv::Core::PVlInt32; \
    using Massiv::Core::PWeakPointer; \
    using Massiv::Core::Pointer; \
    using Massiv::Core::PtrToObject; \
    using Massiv::Core::Remote; \
    using Massiv::Core::SBool; \
    using Massiv::Core::SColor; \
    using Massiv::Core::SInt; \
    using Massiv::Core::SInt32; \
    using Massiv::Core::SInt64; \
    using Massiv::Core::SFloat32; \
    using Massiv::Core::SFloatQ; \
    using Massiv::Core::SOrientation; \
    using Massiv::Core::SString; \
    using Massiv::Core::SString; \
    using Massiv::Core::STime; \
    using Massiv::Core::SVector2; \
    using Massiv::Core::SVector3; \
    using Massiv::Core::VariantPointer; \
    using Massiv::Core::ValueType; \
    using Massiv::Core::WeakPointer; \
    using namespace Massiv::Core;


#endif // DEMO_LIB_H
