/** @file node_console.h
 * Node console.
 */

/*
 * begin        : Sat Jun 28 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_NODE_CONSOLE_H
#define DEMO_NODE_CONSOLE_H

#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_CONSOLE_H
#include "lib/shared/console.h"
#endif

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Node console.
 * Handles commands specific to a particular node (commands used to manage that
 * node or commands to be executed on that node).   
 * Always paired with a NodeObject.
 */
class NodeConsole : public Console
    {
public:

    /**
     * Default constructor.
     */
    NodeConsole();
    
    /**
     * Default destructor.
     */
    ~NodeConsole();

public:

    // Command execution.

    static void execute_help
        (
        const CommandContext & context
        );

    static void execute_stat
        (
        const CommandContext & context
        );

    static void execute_archive
        (
        const CommandContext & context
        );

    static void execute_call
        (
        const CommandContext & context
        );

    static void execute_rcall
        (
        const CommandContext & context
        );
        
    static void execute_get_naming_service
        (
        const CommandContext & context
        );

    static void execute_echo
        (
        const CommandContext & context
        );

    static void execute_classinfo
        (
        const CommandContext & context
        );

    static void execute_copy
        (
        const CommandContext & context
        );

    static void execute_createbound
        (
        const CommandContext & context
        );

    static void execute_createobject
        (
        const CommandContext & context
        );

    static void execute_createpinned
        (
        const CommandContext & context
        );

    static void execute_dump
        (
        const CommandContext & context
        );

    static void execute_eraseobject
        (
        const CommandContext & context
        );

    static void execute_gc
        (
        const CommandContext & context
        );

    static void execute_get
        (
        const CommandContext & context
        );

    static void execute_getvar
        (
        const CommandContext & context
        );

    static void execute_mgroup
        (
        const CommandContext & context
        );

    static void execute_name
        (
        const CommandContext & context
        );

    static void execute_objects
        (
        const CommandContext & context
        );

    static void execute_rgroup
        (
        const CommandContext & context
        );

    static void execute_savesettings
        (
        const CommandContext & context
        );

    static void execute_set
        (
        const CommandContext & context
        );

    static void execute_setvar
        (
        const CommandContext & context
        );

    static void execute_snapmemory
        (
        const CommandContext & context
        );

    static void execute_swapin
        (
        const CommandContext & context
        );

    static void execute_swapinall
        (
        const CommandContext & context
        );

    static void execute_swapout
        (
        const CommandContext & context
        );
    
    static void execute_pin
        (
        const CommandContext & context
        );

    static void execute_unpin
        (
        const CommandContext & context
        );
    
    static void execute_migrate
        (
        const CommandContext & context
        );

    static void execute_data_update
        (
        const CommandContext & context
        );
        
    static void execute_log
        (
        const CommandContext & context
        );
    
    static void execute_server_quit
        (
        const Console::CommandContext & context
        );

    static void execute_rcommand
        (
        const Console::CommandContext & context
        );

    static void execute_gather_replicated
        (
        const Console::CommandContext & context
        );

    static void execute_replication_info
        (
        const Console::CommandContext & context
        );
        
    }; // NodeConsole


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_NODE_CONSOLE_H
