/** @file node_object.cpp
 * Node object implementation.
 */

/*
 * begin        : Mon Feb 17 2003
 * copyright    : (C) 2003 by Marek Vondrak, Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "core.h"
#include "lib/shared/node_object.h"
#include "lib/shared/account_object.h"
#include "lib/shared/node_console.h"


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


// Implementation of NodeObject
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NodeObject::NodeObject() :
    console( NULL )
    {
    }


NodeObject::~NodeObject()
    {
    delete console;
    }


void NodeObject::initialize()
    {
    console = new NodeConsole();
    }


Pointer< Massiv::Core::Lib::AccountObjectInterface > NodeObject::create_account_object()
    {
    // Supported only by server nodes, see ServerNode object.
    return null;
    }


PValueArray<SString> NodeObject::process_node_command
    (
    const WeakPointer<AccountObject> &  account,
    const std::string &                 command,
    Console::Environment &              env
    )
    {
    PValueArray<SString> errors;

    console->process_command
        (
        account,
        command,
        false,
        errors,
        env
        );

    return errors;
    }


PValueArray<SString> NodeObject::auto_complete_node_command
    (
    const WeakPointer<AccountObject> &  account,
    const std::string &                 partial_command
    )
    {
    PValueArray<SString> completions;
    
    Console::Environment    env; // FIXME: je to tu na nic, jen kvuli kompilaci
    
    console->process_command
        (
        account,
        partial_command,
        true,
        completions,
        env
        );

    return completions;
    }


bool NodeObject::can_connect
    (
    const WeakPointer<AccountObject> &  account
    )
    {
    return true;
    }


} // namespace Demo::Lib
} // namespace Demo
