/** @file node_object.h
 * Node object.
 */

/*
 * begin        : Mon Feb 17 2003
 * copyright    : (C) 2003 by Marek Vondrak, Petr Tovarys, Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_NODE_OBJECT_H
#define DEMO_NODE_OBJECT_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SHARED_FWD_H
#include "lib/shared/shared_fwd.h"
#endif

#ifndef DEMO_CONSOLE_H
#include "lib/shared/console.h"
#endif

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Object that represents connected node.
 * Defines node management interface that is used to implement client's
 * console.
 */
class NodeObject : public Massiv::Core::Lib::NodeObjectInterface
    {
    MASSIV_OBJECT( NodeObject );

protected:

    /**
     * Constructor.
     */
    NodeObject();

    /**
     * Destructor.
     */
    ~NodeObject();

public:

    /**
     * Initializes object.
     */
    void initialize();
    
public:

    // From NodeObjectInterface.
    virtual Pointer< Massiv::Core::Lib::AccountObjectInterface > create_account_object();

public:

    //
    // Node management.
    //

    /**
     * Processes @c command on the local node.
     * The command is processed on behalf of @c account.
     * Returns error log generated during command parsing.
     */
    PValueArray<SString> process_node_command
        (
        const WeakPointer<AccountObject> &  account,
        const std::string &                 command,
        Console::Environment &              env
        );
    
    /**
     * Returns possible command completions for @c partial_command.
     */
    PValueArray<SString> auto_complete_node_command
        (
        const WeakPointer<AccountObject> &  account,
        const std::string &                 partial_command
        );

    /**
     * Tests if client console can connect to this node under
     * this @c account.
     */
    bool can_connect
        (
        const WeakPointer<AccountObject> &  account
        );

private:

    NodeConsole * console;
        /**< Owning pointer to console instance that manages the node.
             This is not @c std::auto_ptr to prevent compilers from instantiating
             NodeConsole destructor whenever they see/use NodeObject - we don't want
             to include node_console.h into every header. */
    
    }; // NodeObject


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_NODE_OBJECT_H
