/** @file random.h
 * Random number generator functions.
 */

/*
 * begin        : Mon Dec 1 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_RANDOM_H
#define DEMO_RANDOM_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Random number generator functions.
 */
class Random
    {
public:

    /** 
     * Returns a random number from the [ 0, 1 ] range such that the returned 
     * numbers are distributed uniformly in this range.
     */
    static Float32 random();

    /** 
     * Returns a random integer from the [ 0, max_value ] range such that the
     * returned numbers are distributed uniformly in this range.
     */
    static unsigned random_integer
        (
        unsigned max_value
        );

    /**
     * Returns a random vector such that its components are distributed uniformly
     * in the [ -1, 1 ] range.
     */
    static SVector2 random_vector();

    }; // Random


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_RANDOM_H
