/** @file sector_address.cpp
 * Sector address implementation.
 */

/*
 * begin        : Mon Jul 28 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "lib/shared/sector_address.h"
#include "lib/shared/entity_position.h"
#include "lib/shared/client_sector.h"

namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


SectorAddress::SectorAddress()
    {
    initialize_object();
    default_constructed();
    }


SectorAddress::SectorAddress
    (
    WeakPointer<Object> the_layer,
    int                 the_column,
    int                 the_row
    ) : layer( the_layer ),
        column( the_column ),
        row( the_row )
    {
    initialize_object();
    }


SectorAddress::SectorAddress
    (
    WeakPointer<Object> the_layer,
    const SVector2 &    the_layer_position
    ) : layer
        ( the_layer ),
        column
        ( FloatTraits<Float32>::floor_int( the_layer_position.x() / ClientSector::SECTOR_X_SIZE ) ),
        row
        ( FloatTraits<Float32>::floor_int( the_layer_position.y() / ClientSector::SECTOR_Y_SIZE ) )
    {
    initialize_object();
    }


SectorAddress::SectorAddress
    (
    const EntityPosition & position
    ) : layer( position.layer ),
        column
        ( FloatTraits<Float32>::floor_int( position.origin.x() / ClientSector::SECTOR_X_SIZE ) ),
        row
        ( FloatTraits<Float32>::floor_int( position.origin.y() / ClientSector::SECTOR_Y_SIZE ) )
    {
    initialize_object();
    }


SectorAddress::SectorAddress
    (
    const SectorAddress & address
    ) : ValueType( address ),
        layer( address.layer ),
        column( address.column ),
        row( address.row )
    {
    initialize_object();
    }


SectorAddress SectorAddress::get_neighbor
    (
    Direction direction
    ) const
    {
    switch( direction )
        {
        case EAST:
            return SectorAddress( layer, column + 1, row );
        case NORTH_EAST:
            return SectorAddress( layer, column + 1, row + 1 );
        case NORTH:
            return SectorAddress( layer, column, row + 1 );
        case NORTH_WEST:
            return SectorAddress( layer, column - 1, row + 1 );
        case WEST:
            return SectorAddress( layer, column - 1, row );
        case SOUTH_WEST:
            return SectorAddress( layer, column - 1, row - 1 );
        case SOUTH:
            return SectorAddress( layer, column, row - 1 );
        case SOUTH_EAST:
            return SectorAddress( layer, column + 1, row - 1 );
        }
    
    panic( "Invalid direction." );
    }


SVector2 SectorAddress::get_layer_position() const
    {
    return SVector2
        (
        static_cast<Float32>( column * ClientSector::SECTOR_X_SIZE ),
        static_cast<Float32>( row * ClientSector::SECTOR_Y_SIZE )
        );
    }


bool SectorAddress::operator==
    (
    const SectorAddress & address
    ) const
    {
    return  layer == address.layer &&
            row == address.row &&
            column == address.column;    
    }


bool SectorAddress::operator!=
    (
    const SectorAddress & address
    ) const
    {
    return !operator==( address );
    }


int SectorAddress::hash() const
    {
    return row * 12345678 + column * 87654321;
    }

    
} // namespace Demo::Lib
} // namespace Demo
