/** @file sector_address.h
 * Sector address.
 */

/*
 * begin        : Mon Jul 28 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef DEMO_SECTOR_ADDRESS_H
#define DEMO_SECTOR_ADDRESS_H


#ifndef DEMO_LIB_H
#include "lib/shared/lib.h"
#endif

#ifndef DEMO_SHARED_FWD_H
#include "lib/shared/shared_fwd.h"
#endif

#ifndef DEMO_DIRECTION_H
#include "lib/shared/direction.h"
#endif


namespace Demo {
namespace Lib {

INJECT_CORE_SYMBOLS();


/**
 * Address of a sector.
 * The address uniquely identifies the sector and gives information
 * about sector position in a layer.
 */
class SectorAddress : public ValueType
    {
public:

    /** Default constructor. */
    SectorAddress();

    /** Constructor. */
    SectorAddress
        (
        WeakPointer<Object> the_layer,
        int                 the_column,
        int                 the_row
        );

    /**
     * Constructs the address from @c layer_position.
     * Creates address of a sector @c layer_position should belong to.
     */
    SectorAddress
        (
        WeakPointer<Object> the_layer,
        const SVector2 &    the_layer_position
        );

    /**
     * Constructs the address from entity position.
     */
    SectorAddress
        (
        const EntityPosition & position
        );

    /** Copy constructor. */
    SectorAddress
        (
        const SectorAddress & address
        );

    /** Returns address of a neighboring sector in given @c direction. */
    SectorAddress get_neighbor
        (
        Direction direction
        ) const;
    
    /**
     * Returns origin of sector with this address in layer coordinates.
     * Sector origin refers to [ 0, 0 ] point in sector coordinates.
     */
    SVector2 get_layer_position() const;

    /**
     * Returns true if @c address refers to the same sector as this
     * SectorAddress.
     */
    bool operator==
        (
        const SectorAddress & address
        ) const;
    
    /**
     * Returns true if @c address does not refer to the same sector as this
     * SectorAddress.
     */
    bool operator!=
        (
        const SectorAddress & address
        ) const;

    /**
     * Computes hash code.
     */
    virtual int hash() const;

public:
        
    PWeakPointer<Object>    layer;
        /**< Layer. */
    PInt32                  column;
        /**< Index of the sector in east direction. */
    PInt32                  row;
        /**< Index of the sector in north direction. */
    }; // SectorAddress


} // namespace Demo::Lib
} // namespace Demo

#endif // DEMO_SECTOR_ADDRESS_H
