/** @file server_global.h
 * Global server stuff.
 */

/*
 * begin        : Oct 2003
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#ifndef SERVER_GLOBAL_H
#define SERVER_GLOBAL_H

#ifndef MASSIV_CORE_H
#include "core.h"
#endif

#include <memory>


namespace Demo {

namespace Lib {
class AccountConsole;
} // Demo::Lib

namespace Server {

/**
 * Server global stuff.
 */
class Global
    {
public:

    /**
     * Initialize.
     */
    static void initialize
        (
        );

    /**
     * Shutdown.
     */
    static void shutdown();

public:

    /**
     * Is server initialized?
     */
    static INLINE bool is_initialized();

// getters
public:

    //@{
    /**
     * Getter.
     * It's an error to call this function if the global object does not exist.
     */
    static INLINE Demo::Lib::AccountConsole & get_account_console();

private:
        
    /**
     * Global objects.
     */
    struct Objects
        {
        /** Allocate global objects. */
        Objects();
        
        /** Destroy global objects. */
        ~Objects();

        /** Initialize global objects. */
        void initialize();

    public:

        std::auto_ptr< Demo::Lib::AccountConsole >  account_console;
            /**< The account console. */
                   
        }; // struct Objects

    static Objects * objects;
        /**< Global objects.
             Stupid wrapper, we don't want to have static auto_ptrs. */

    }; // Global


} // namespace Server
} // namespace Demo


#ifdef ENABLE_INLINE
#include "server_global_inline.h"
#endif


#endif // SERVER_GLOBAL_H
