/** @file service_main.cpp
 * Demo service node.
 */

/*
 * begin        : Wed Jun 2 2003
 * copyright    : (C) 2003 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "core.h"
#include "cmdline/cmdline_parser.h"
#include "data/path_manager.h"
#include "service_main.h"
#include "config.h"
#include <iosfwd>
#include <fstream>

using namespace Massiv;


namespace Demo {
namespace Service {

// Anonymous namespace.
namespace {


std::string read_local_node_config()
    {
    std::string         config_file_name = Massiv::Core::Global::path_manager().
        get_login_directory() + "/default";
    std::ifstream       file( config_file_name.c_str() );
    std::ostringstream  oss;
    oss << file.rdbuf();
    if( !file.good() )
        {
        std::ostringstream ss;
        ss  << "Error reading '" << config_file_name << "' file.";
        throw Massiv::Core::ExIOError( ss.str() );
        }

    return oss.str();
    }


void next_tick_callback()
    {
    }
  

std::string config_file_name = CONFIG_MASSIV_CONF_PATH;
    /**< Config file name variable. Can be changed from command line. */


void initialize()
    {
    std::cout << "Initializing Massiv core..." << std::endl;

    System::initialize( System::StartUpInfo( 
        Massiv::Core::NodeId::TYPE_SERVICE, config_file_name, NULL, NULL, 
        next_tick_callback ) );
    }

    
} // Anonymous namespace.


void service_main
    (
    int             argc,
    const char **   argv
    )
    {
    // Parse command line.

    hope( argv );

    {
    Massiv::CmdLineParser cmd_line( argc, argv, "h?", "c" );
    bool parse = true;
    while( parse )
        {
        std::string parameter;
        const char option = cmd_line.get_next( parameter );
        switch( option )
            {
            case '-':
            case '\0':
                parse = false;
                break;

            case 'c':
                config_file_name = parameter;
                break;

            case 'h':
            case '?':
                std::cout << "Usage:\n" <<
                    argv[ 0 ] << " [options]\n\n" <<
                    "Options:\n" <<
                    "  -h (-?)          This help\n" <<
                    "  -c <file name>   Override default config file name.\n" <<
                    std::endl;
                return;

            default:
                panic( "Invalid command line option." );
            }
        }
    }

    initialize();
    
    try
        {
        std::string         config = read_local_node_config();
        std::istringstream  config_stream( config );
            
        System::download_data();
        System::connect( config_stream );
        std::cout << "------------------------ RUNNING ------------------------" << std::endl;
        System::run();
        System::disconnect();
        }
    catch( Massiv::Core::Exception & )
        {
        }

    System::shutdown();
    }


} // namespace Service
} // namespace Demo
