/** @file
 * "Creating Managed Class" example.
 * Implementation of the Hello class.
 */

/*
 * begin        : Feb 2004
 * copyright    : (C) 2004 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

/*LINE 1*/
/*BEGIN all*/
#include "core.h"
#include "hello.h"
#include "database/well_known_object_id_database.h"
#include <sstream>

using namespace Massiv::Core;

namespace example {

/*BEGIN initialize*/
void Hello::initialize
    (
    const std::string & the_name
    )
    {
    name = the_name;
    }
/*END initialize*/

/*BEGIN hello*/
std::string Hello::hello
    (
    const std::string & callee_name
    )
    {
    std::ostringstream oss;
    oss << "Hello to " << callee_name << " from " << name << std::endl;
    oss << "(called " << total_call_count << " times, " <<
           current_call_count << " since startup)" << std::endl;
    const std::string s = oss.str();

    total_call_count++;
    current_call_count++;

    Global::log_info( Status::FACILITY_LIB, Status::PRIORITY_LOW, s );
    return s;
    }
/*END hello*/

/*BEGIN register_to_naming*/
/*BEGIN naming.start*/
bool Hello::register_to_naming
    (
    const std::string & the_name
    )
    {
    initialize( the_name );
/*END naming.start*/

/*BEGIN naming.locals*/
    const WeakPointer< Object > self( this );
    bool failed = true;
/*END naming.locals*/
/*BEGIN naming.try*/
    try
        {
/*END naming.try*/
/*BEGIN naming.naming*/
        const ObjectId naming_id = Global::well_known_object_id_database().
            get_naming_service_object();
        const Remote< Object > naming( naming_id );
/*END naming.naming*/

/*BEGIN naming.metaobject*/
        const MetaObject * const metaobject = Global::class_manager().
            get_metaobject( "Demo::Lib::NamingService" );
/*END naming.metaobject*/

/*BEGIN naming.arguments*/
        std::stringstream ss;
        {
        TextWriter tw( ss );
        const Serializer::Description desc;
        name.text_write( tw, desc );
        tw.write_space();
        self.text_write( tw, desc );
        tw.write_space();
        const SBool replace = false;
        replace.text_write( tw, desc );
        }
/*END naming.arguments*/

/*BEGIN naming.call*/
        TextReader tr( ss );
        std::auto_ptr< MethodPacket > results = metaobject->
            remote_call_method( naming, "register_object", tr );
/*END naming.call*/

/*BEGIN naming.result*/
        if( results->get_argument_value( -1 ) == "true" )
            {
            failed = false;
            }
        else
            {
            std::ostringstream oss;
            oss << "Object " << name << " already registered "
                   " to the global naming service." << std::endl;
            Global::log_warning( Status::FACILITY_LIB,
                Status::PRIORITY_HIGH, oss.str() );
            }
        }
/*END naming.result*/
/*BEGIN naming.catch*/
    catch( std::exception & e )
        {
        std::ostringstream oss;
        oss << "Failed to register " << name << " to naming service: "
            << e.what();
        Global::log_warning( Status::FACILITY_LIB,
            Status::PRIORITY_HIGH, oss.str() );
        }
/*END naming.catch*/

/*BEGIN naming.return*/
    if( failed )
        {
        Massiv::System::dispose_gc_root( self );
        }
    return failed ? false : true;
    }
/*END naming.return*/
/*END register_to_naming*/

/*BEGIN object_updated*/
void Hello::object_updated
    (
    UpdateReason reason
    )
    {
    if( reason == SIMULATION_STARTUP )
        {
        current_call_count = 0;
        }
    }
/*END object_updated*/

} // namespace example
/*END all*/
