/** @file
 * "Creating Managed Class" example.
 * Interface of the Hello class.
 */

/*
 * begin        : Feb 2004
 * copyright    : (C) 2004 by Stepan Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

/*LINE 1*/
/*BEGIN all*/
#ifndef EXAMPLE_HELLO_H
#define EXAMPLE_HELLO_H

#ifndef MASSIV_CORE_H
#include "core.h"
#endif

#ifndef EXAMPLE_HELLO_INTERFACE_H
#include "hello_interface.h"
#endif

namespace example {

/**
 * Simple implementation of the HelloInterface.
 */
class Hello : public HelloInterface
    {
public:

    /**
     * Object initialization.
     *
     * The internal name of the object (not to be confused with
     * name under which it's registered to the garbarge collector)
     * will be set to @a the_name.
     */
    void initialize
        (
        const std::string & the_name
        );

    /**
     * The "say hello" method.
     *
     * The message generated by this implementation will contain
     * the callee name, name of the Hello object, and counters
     * indicating how many times this method has been called since
     * creation of this object and since last simulation startup.
     */
    virtual std::string hello
        (
        const std::string & callee_name
        );

    /**
     * Register the object to global naming service.
     *
     * This method will change name of the object to @a the_name,
     * and try to register the object to the global naming service
     * under that name. If the registration fails it will destroy
     * the object.
     *
     * @note This method will fail unless the example library is
     *       linked with the Massiv Demo. However, because it uses
     *       dynamic RPC, it can be compiled without the library.
     */
    bool register_to_naming
        (
        const std::string & the_name
        );

protected:

    /**
     * Object update callback.
     */
    virtual void object_updated
        (
        UpdateReason reason
        );

public:

/*BEGIN properties*/
    Massiv::Core::PString   name;
        /**< Internal name of the object.
             It's used to print the hello message and to register
             the object to the global naming service. */

    Massiv::Core::PVlInt32  total_call_count;
        /**< Number of calls to hello() since object creation. */

    Massiv::Core::PVlInt32  current_call_count;
        /**< Number of calls to hello() since node startup. */
/*END properties*/

    }; // class Hello

} // namespace example

#endif // EXAMPLE_HELLO_H
/*END all*/
