/** @file server_main.cpp
 * Testing server.
 */

/*
 * begin        : Sat Dec 8 2002
 * copyright    : (C) 2002 by Stepan Vondrak, Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "core.h"
#include "server_main.h"
#include "server_common.h"
#include "class_list.h"
#include "objects.h"
#include "object/object_manager.h"
#include <iostream>


using namespace Massiv::Core;
using namespace Massiv::Generated;


namespace Massiv {
namespace ServerTest {
namespace PhilosophersRPC {

namespace { // anonymous namespace

void run
    (
    float period
    )
    {
    const SystemTime old_time = System::system_time();

    while( System::system_time() < ( old_time + period ) )
        {
        SystemTime          t = System::system_time();
        std::ostringstream  ss;
        static int          counter;

        while( System::system_time() < ( old_time + period ) &&
                System::system_time() < ( t + 5.0 ) )
            {
            System::next_tick();
            }

        ss << "memory_" << counter++ << ".dot";
        Global::garbage_collector().save_memory_snapshot( ss.str() );
        }
    }

} // anonymous namespace

enum Parameters
    {
    NUM_PHILOSOPHERS    =   4
    };

void server_main()
    {
    // Initialize core.

    std::cout << "Initializing core..." << std::endl;

    System::register_classes( &register_classes );

    if( ServerCommon::options().create_initial_archive )
        {
        System::initialize
            (
            0
            | System::INIT_COUT_LOGGER_DESTINATION
            | System::INIT_STANDALONE_NODE
            );

        UInt32 provider_id = ServerCommon::options().initial_provider_id;

        std::cout << "Creating initial provider (" << provider_id << ")..." << std::endl;
        System::create_initial_object_provider( provider_id );

        std::cout << "Creating initial archive..." << std::endl;
        System::create_initial_archive();

        std::cout << "Initial archive successfully created. Exiting..." << std::endl;
        return;
        }
    else
        {
        System::initialize
            (
            0
            | System::INIT_COUT_LOGGER_DESTINATION
            );
        }

    std::cout << "Connecting..." << std::endl;

    System::connect();

    while( System::get_phase() != System::PHASE_CONNECTED )
        {
        System::next_tick();
        }

    // Create GC root objects.

    { // sob
    std::cout << "Creating objects..." << std::endl;

    Pointer<Table> table = CreateObject< Table >::create();

    for( int i = 0; i < NUM_PHILOSOPHERS; i++ )
        {
        Pointer<Philosopher> philosopher = CreateObject< Philosopher >::create
            ( 1.0 + i * 0.3, 5.0, 1.0 );
        philosopher->try_to_sit( table, STime( System::time().get_value() + i * 0.003 ) );
        }
    } // eob

    // Run for a while.

    run( 60.0 );

    // Done.
    System::shutdown();
    }


} // namespace Massiv::ServerTest::PhilosophersRPC
} // namespace Massiv::ServerTest
} // namespace Massiv
