/** @file objects.cpp
 * Testing objects for pointer tests.
 */

/*
 * begin        : Fri Jan 17 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "objects.h"
#include <iostream>


using namespace Massiv::Core;


namespace Massiv {
namespace ServerTest {
namespace PointerTest {


// Implementation of List::Entry
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void List::Entry::initialize()
    {
    }


// Implementation of List
// ~~~~~~~~~~~~~~~~~~~~~~

void List::initialize()
    {
    Pointer<Entry>  tail = CreateObject<Entry>::create();

    tail->next = tail->prev = tail;

    first = tail;
    last = tail;
    }


void List::push_back
    (
    const Pointer<Object> & object
    )
    {
    Pointer<Entry> entry = CreateObject<Entry>::create();

    entry->next = last;
    entry->prev = last->prev;
    entry->prev->next = entry;
    last->prev = entry;
    entry->object = object;

    if( first == last )
        first = entry;

    num_items++;
    }


void List::push_front
    (
    const Pointer<Object> & object
    )
    {
    Pointer<Entry> entry = CreateObject<Entry>::create();

    entry->next = first;
    entry->prev = last;
    first->prev = entry;
    last->next = entry;
    first = entry;
    entry->object = object;

    num_items++;
    }


Pointer<Enumerator> List::elements()
    {
    Pointer<ListEnumerator> e = CreateObject<ListEnumerator>::create();

    e->entry = last;
    e->list = this;

    return e;
    }


bool List::contains
    (
    const Pointer<Object> & object
    )
    {
    Pointer<Enumerator> e = elements();

    while( e->has_next() )
        {
        Pointer<Object> o = e->next();

        if( o == object )
            return true;
        }

    return false;
    }


void List::clear()
    {
    first = last;
    }


// Implementation of ListEnumerator
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void ListEnumerator::initialize()
    {
    }


bool ListEnumerator::has_next()
    {
    return entry->next != list->last;
    }


Pointer<Object> ListEnumerator::next()
    {
    Pointer<Object> o( entry->object );

    entry = entry->next;

    return o;
    }


// Implementation of ObjectA
// ~~~~~~~~~~~~~~~~~~~~~~~~~

void ObjectA::initialize()
    {
    }


// Implementation of ObjectB
// ~~~~~~~~~~~~~~~~~~~~~~~~~

void ObjectB::initialize()
    {
    ObjectA::initialize();
    }


// Implementation of ObjectC
// ~~~~~~~~~~~~~~~~~~~~~~~~~

void ObjectC::initialize()
    {
    ObjectB::initialize();
    }


} // namespace Massiv::ServerTest::PointerTest
} // namespace Massiv::ServerTest
} // namespace Massiv
