/** @file objects.h
 * Testing objects for pointer test.
 */

/*
 * begin        : Fri Jan 17 2003
 * copyright    : (C) 2003 by Marek Vondrak
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef _OBJECTS_H_
#define _OBJECTS_H_


#include "core.h"


using namespace Massiv::Core;


namespace Massiv {
namespace ServerTest {
namespace PointerTest {


/**
 * Abstract enumerator.
 */
class Enumerator : public Object
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::PointerTest//Enumerator
        parent Massiv::Core::Object
        abstract
    endclass
    IDL_END
    */

public:


    /** Can @c next() be called? */
    virtual bool            has_next() = 0;

    /** Returns next object stored in the container. */
    virtual Pointer<Object> next() = 0;

    }; // Enumerator

/**
 * Abstract container.
 */
class Container : public Object
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::PointerTest//Container
        parent Massiv::Core::Object
        abstract
    endclass
    IDL_END
    */

public:

        /** Returns enumerator. */
    virtual Pointer<Enumerator> elements() = 0;

        /** Does container contain @c object? */
    virtual bool contains
        (
        const Pointer<Object> & object
        ) = 0;

        /** Clears container. */
    virtual void clear() = 0;

    }; // Container

/**
 * List.
 */
class List : public Container
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::PointerTest//List
        parent Massiv::ServerTest::PointerTest::Container
        property first : ptr_repflags = MIGRATE
        property last : ptr_repflags = MIGRATE
        property num_items
    endclass
    IDL_END
    */

public:
    /**
     * List entry.
     */
    class Entry : public Object
        {
        /*
        IDL_BEGIN
        class Massiv::ServerTest::PointerTest//List::Entry
            parent Massiv::Core::Object
            property next : ptr_repflags = MIGRATE
            property prev : ptr_repflags = MIGRATE
            property object : ptr_repflags = MIGRATE
        endclass
        IDL_END
        */

    public:

        /** Initializes object. */
        void initialize();

    public:

        PPointer<Entry>     next;
            /**< Next entry. */
        PPointer<Entry>     prev;
            /**< Previous entry. */
        PPointer<Object>    object;
            /**< Stored object. */
        }; // class Entry

public:

    /** Initializes object. */
    void initialize();

    /** Puts object to the end of the list. */
    void push_back
        (
        const Pointer<Object> & object
        );

    /** Puts object to the front of the list. */
    void push_front
        (
        const Pointer<Object> & object
        );

    /**
     */
    virtual Pointer<Enumerator> elements();

    /**
     */
    virtual bool contains
        (
        const Pointer<Object> & object
        );

    /**
     */
    virtual void clear();

public:

    PPointer<Entry>     first;
        /**< First entry. */
    PPointer<Entry>     last;
        /**< List tail. Unused entry. */
    PInt32              num_items;
        /**< Number of objects stored in the list. */
    }; // List

/**
 * List enumerator.
 */
class ListEnumerator : public Enumerator
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::PointerTest//ListEnumerator
        parent Massiv::ServerTest::PointerTest::Enumerator
        property entry : ptr_repflags = MIGRATE
        property list : ptr_repflags = MIGRATE
    endclass
    IDL_END
    */

public:

    /** Initializes object. */
    void initialize();

    /**
     */
    virtual bool has_next();

    /**
     */
    virtual Pointer<Object> next();

public:

    PPointer<List::Entry> entry;
        /**< Current entry the enumerator points to. */
    PPointer<List>      list;
        /**< List. */
    }; // ListEnumerator

/**
 * Testing object A.
 */
class ObjectA : public Object
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::PointerTest//ObjectA
        parent Massiv::Core::Object
    endclass
    IDL_END
    */

public:

    /** Initializes object. */
    void initialize();

    }; // ObjectA

/**
 * Testing object B.
 */
class ObjectB : public ObjectA
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::PointerTest//ObjectB
        parent Massiv::ServerTest::PointerTest::ObjectA
    endclass
    IDL_END
    */

public:

    /** Initializes object. */
    void initialize();

    }; // ObjectB

/**
 * Testing object C.
 */
class ObjectC : public ObjectB
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::PointerTest//ObjectC
        parent Massiv::ServerTest::PointerTest::ObjectB
    endclass
    IDL_END
    */

public:

    /** Initializes object. */
    void initialize();

    }; // ObjectC

} // namespace Massiv::ServerTest::PointerTest
} // namespace Massiv::ServerTest
} // namespace Massiv


#endif // _OBJECTS_H_
