/** @file server_common.cpp
 * Common settings for server tests.
 */

/*
 * begin        : Wed Dec 18 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "global/common.h"
#include "server_common.h"
#include "utility/debug.h"
#include <string>
#include <sstream>


namespace Massiv {
namespace ServerTest {


// Global stuff.
static ServerCommon::Options      g_options;


ServerCommon::Options::Options()
    {
    create_initial_archive       = false;
    enable_archivation           = false;
    }


void ServerCommon::init
    (
    int             argc,
    const char **   argv
    )
    {
    int i;
    for( i = 1; i < argc; ++i )
        {
        std::string arg = argv[ i ];
        
        if( arg == "-i" || arg == "--create-initial-archive" )
            {
            ++i;
            if( i < argc )
                {
                Massiv::Core::UInt32 provider_id = 0;
                arg = argv[ i ];
                std::istringstream ss( arg );
                ss >> provider_id;
                if( !ss.fail() )
                    {
                    g_options.create_initial_archive = true;
                    g_options.initial_provider_id = provider_id;
                    }
                else
                    {
                    panic( "Argument -i requires provider id." );
                    }
                }
            else
                {
                panic( "Argument -i requires provider id." );
                }
            }
        else if( arg == "-a" || arg == "--enable-archivation" )
            {
            g_options.enable_archivation = true;
            }
        else
            {
            /** @todo Throw exception. */
            panic( "Unknown program argument." );
            }
        }
    }


const ServerCommon::Options & ServerCommon::options()
    {
    return g_options;
    }


} // namespace Massiv::ServerTest
} // namespace Massiv
