/** @file server_main.cpp
 * Testing signal elements.
 */

/*
 * begin        : Thu Mar 11 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "server_main.h"
#include "server_common.h"
#include "class_list.h"
#include "signal_elements.h"
#include "signal_elements_rpc.h"

#include <iostream>


using namespace Massiv::Core;
using namespace Massiv::Generated;


namespace Massiv {
namespace ServerTest {
namespace Signal {

namespace { // anonymous namespace

void run
    (
    float period
    )
    {
    const SystemTime old_time = System::system_time();

    while( System::system_time() < ( old_time + period ) )
        {
        SystemTime          t = System::system_time();
        std::ostringstream  ss;
        static int          counter;

        while( System::system_time() < ( old_time + period ) &&
                System::system_time() < ( t + 5.0 ) )
            {
            System::next_tick();
            }

        ss << "memory_" << counter++ << ".dot";
        Global::garbage_collector().save_memory_snapshot( ss.str() );
        }
    }

} // anonymous namespace

void server_main()
    {
    // Initialize core.

    std::cout << "Initializing core..." << std::endl;
    
    System::register_classes( &register_classes );

    if( ServerCommon::options().create_initial_archive )
        {
        System::initialize
            (
            0
            | System::INIT_COUT_LOGGER_DESTINATION
            | System::INIT_STANDALONE_NODE
            );
            
        UInt32 provider_id = ServerCommon::options().initial_provider_id;
        
        std::cout << "Creating initial provider (" << provider_id << ")..." << std::endl;
        System::create_initial_object_provider( provider_id );
        
        std::cout << "Creating initial archive..." << std::endl;
        System::create_initial_archive();
        
        std::cout << "Initial archive successfully created. Exiting..." << std::endl;
        return;
        }
    else
        {
        System::initialize
            (
            0
            | System::INIT_COUT_LOGGER_DESTINATION
            );
        }
        
    std::cout << "Connecting..." << std::endl;
    
    System::connect();

    while( System::get_phase() != System::PHASE_CONNECTED )
        {
        System::next_tick();
        }

    // Register classes.

    std::cout << "Registering classes..." << std::endl;

    register_classes();

    // Create GC root objects.

    { // sob

    std::cout << "Creating objects..." << std::endl;
    
    Pointer< SignalWait > sig_wait = CreateObject< SignalWait >::create();
    Pointer< SignalPrint > sig_print = CreateObject< SignalPrint >::create();
    Pointer< SignalBool > sig_bool = CreateObject< SignalBool >::create();
    
    sig_wait->register_signalized_element( sig_bool );
    sig_wait->setup( 3.0 );
    sig_wait->signal( 0 );
    
    sig_bool->register_signalized_element( sig_print );
    sig_bool->setup( true, 3.0 );

    } // eob

    // Run for a while.
    run( 10.0 );

    // Done.
    System::shutdown();
    }


} // namespace Massiv::ServerTest::Signal
} // namespace Massiv::ServerTest
} // namespace Massiv
