/** @file signal_element_base.h
 * Signal element base classes.
 */

/*
 * begin        : Tue Mar 11 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef _SIGNAL_ELEMENT_BASE_H_
#define _SIGNAL_ELEMENT_BASE_H_


#include "core.h"

using namespace Massiv::Core;


namespace Massiv {
namespace ServerTest {
namespace Signal {


/**
 * SignalElement.
 * Every signal element is derivered from this class.
 * SignalElement defines virtual function called signal() - it
 * can be called via RPC.
 * When signal() is called on a signal element, it means 
 * "signal is received".
 */
class SignalElement : public Object
    {
public:

    /**
     * Send signal to this element.
     */
    virtual void signal
        (
        int     sig_num
        );

public:

    // No attributes.

    }; // SignalElement

/**
 * SignalSendElement.
 * When signal is received, this class (and its childs) always send signal to
 * registered signal element(s).
 * To register signal element to be signalized,
 * call @method register_signalized_element().
 */
class SignalSendElement : public SignalElement
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::Signal::SignalSendElement
        parent Massiv::ServerTest::Signal::SignalElement

        property sig_element

    endclass
    IDL_END
    */

public:

    /**
     * Constructor.
     */
    SignalSendElement();

    /**
     * Send signal to this element.
     */
    virtual void signal
        (
        int     sig_num
        );

    /**
     * Set element to be signalized when signal is received.
     */
    void register_signalized_element
        (
        WeakPointer< SignalElement >    signalized_element
        );
    
    /**
     * Send signal to all registered signal element(s).
     * At least one element must be registered.
     * @todo: zatim muze byt registrovan pouze jeden. V budoucnu se pouzije asi PArray 
     *        signal elementu.
     */
     void signal_all
        (
        int     sig_num
        );

public:

    PRemote< SignalElement >    sig_element;
        /**< Element to be signalized. */
    PBool                       registered;
        /**< True when at least one signal element is registerd to be signalized. */

    }; // SignalSendElement


} // namespace Massiv::ServerTest::Signal
} // namespace Massiv::ServerTest
} // namespace Massiv

#endif // _SIGNAL_ELEMENT_BASE_H_
