/** @file signal_elements.cpp
 * Signal elements implementation.
 */

/*
 * begin        : Thu Mar 11 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "signal_element_base.h"
#include "signal_element_base_rpc.h"
#include "signal_elements.h"
#include "signal_elements_rpc.h"
#include "utility/debug.h"
#include <iostream>


namespace Massiv {
namespace ServerTest {
namespace Signal {


// class SignalWait

void SignalWait::initialize()
    {
    }

SignalWait::SignalWait() :
    wait_time( 0 )
    {
    }


void SignalWait::signal
    (
    int     sig_num
    )
    {
    if( !registered )
        {
        panic( "No signal element is registered in SignalWait object." );
        }
    std::cout << "~~~~ SignalWait " << get_object_id() << " signalized(signal n. \"" 
              << sig_num << "\")" << std::endl;
    sig_element->delayed_signal( wait_time, sig_num );
    }


void SignalWait::setup
    (
    const STime &   time
    )
    {
    wait_time = time;
    }


// class SignalPrint

void SignalPrint::initialize()
    {
    }

void SignalPrint::signal
    (
    int     sig_num
    )
    {
    std::cout << "~~~~ SignalPrint " << get_object_id() << " signalized(signal n. \"" 
              << sig_num << "\")" << std::endl;
    print( sig_num );
    }


void SignalPrint::print
    (
    int     sig_num
    )
    {
    std::cout << "SignalPrint::print() : \"" << sig_num << "\"" << std::endl;
    }


// class SignalBool

void SignalBool::initialize()
    {
    }


SignalBool::SignalBool() :
    wait_time( 0 ),
    value( true )
    {    
    }

#define SIGNAL_BOOL_ADD         128

void SignalBool::signal
    (
    int     sig_num
    )
    {    
    std::cout << "~~~~ SignalBool " << get_object_id() << " signalized(signal n. \"" 
              << sig_num << "\")" << std::endl;
    switch( sig_num )
        {
        case 0:
        case 1:
            {
            // Initial signal
            signal_all( sig_num );
            Remote< SignalBool > remote_sig_bool( this );
            // Signalize itself.
            remote_sig_bool->delayed_signal( wait_time, sig_num + SIGNAL_BOOL_ADD );
            break;
            }
        case SIGNAL_BOOL_ADD:
        case SIGNAL_BOOL_ADD + 1:
            {
            // Second signal
            signal_all( !( sig_num - SIGNAL_BOOL_ADD ) );
            break;
            }
        default:
            {
            std::ostringstream  ss;
            ss << "SignalBool: unexpected signal(" << sig_num << ")";
            panic( ss.str().c_str() );
            }
        }
    
    }


void SignalBool::setup
    (
    bool            init_value,
    const STime &   time
    )
    {
    value = init_value;
    wait_time = time;
    }


} // namespace Massiv::ServerTest::Signal
} // namespace Massiv::ServerTest
} // namespace Massiv
