/** @file signal_elements.h
 * Signal element objects.
 */

/*
 * begin        : Tue Mar 11 2003
 * copyright    : (C) 2003 by Martin Havlista
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#ifndef _SIGNAL_ELEMENTS_H_
#define _SIGNAL_ELEMENTS_H_


#include "core.h"
#include "signal_element_base.h"


using namespace Massiv::Core;


namespace Massiv {
namespace ServerTest {
namespace Signal {


/**
 * SignalWait.
 * Element is waiting specified amount of time and then sends signal
 * to registered elements.
 */
class SignalWait : public SignalSendElement
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::Signal::SignalWait
        parent Massiv::ServerTest::Signal::SignalSendElement

    endclass
    IDL_END
    */

public:

    /**
     * Initializes object.
     */
    void initialize();


public:

    /**
     * Constructor.
     */
    SignalWait();

    /**
     * Send signal to this element.
     */
    virtual void signal
        (
        int     sig_num
        );

    /**
     * Set wait time before signal is sent to registered signalized element(s).
     */
    void setup
        (
        const STime &   time
        );

public:

    PTime                       wait_time;
        /**< Time to wait before signal is sent to registered signal element(s). */

    }; // SignalWait


/**
 * SignalPrint.
 * Element prints message with number of signal when signal is received.
 */
class SignalPrint : public SignalElement
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::Signal::SignalPrint
        parent Massiv::ServerTest::Signal::SignalElement

    endclass
    IDL_END
    */

public:

    /**
     * Initializes object.
     */
    void initialize();

public:

    /**
     * Send signal to this element.
     */
    virtual void signal
        (
        int     sig_num
        );

    /**
     * Print message when signal is received. Message depends on @param sig_num.
     */
    void print
        (
        int     sig_num
        );

public:

    // No attributes.
    
    }; // SignalPrint


/**
 * SignalBool.
 * Element can receive only signal 0 or 1. Else assert is invoke.
 * When signal is received, is sends signal with the same number
 * to registered elements, waits specified amout of time, and then sends
 * the second signal with negative number to registered element.
 * Example: 0 is received -\> sends 0 -\> waiting -\> sends 1
 */
class SignalBool : public SignalSendElement
    {
    /*
    IDL_BEGIN
    class Massiv::ServerTest::Signal::SignalBool
        parent Massiv::ServerTest::Signal::SignalSendElement

    endclass
    IDL_END
    */

public:

    /**
     * Initializes object.
     */
    void initialize();

public:

    /**
     * Constructor.
     */
    SignalBool();

    /**
     * Send signal to this element.
     */
    void signal
        (
        int     sig_num
        );

    /**
     * Setup.
     * @param init_value - initial value registered signal element(s) are signalized with.
     * @param time - after this time negation of @param init_value is sent again
     *               to registered signal element(s).
     */
    void setup
        (
        bool            init_value,
        const STime &   time
        );

public:

    PTime                       wait_time;
        /**< Time to wait before signal is sent to registered signal element(s). */
    PBool                       value;
        /**< When signal is received, this value is immediately sent, after 
             wait_time is its negation sent. Dafault value is true( 1 ). */

    }; // SignalBool


} // namespace Massiv::ServerTest::Signal
} // namespace Massiv::ServerTest
} // namespace Massiv


#endif // _SIGNAL_ELEMENTS_H_
