/** @file server_main.cpp
 * Testing server.
 */

/*
 * begin        : Tue Nov 19 2002
 * copyright    : (C) 2002 by Petr Tovarys
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/


#include "server_common.h"
#include "server_main.h"
#include "class_list.h"
#include "objects.h"
#include "object/class_manager.h"
#include <iostream>


using namespace Massiv::Core;
using namespace Massiv::Generated;


namespace Massiv {
namespace ServerTest {
namespace Startup {


void server_main()
    {
    // Initialize core.

    std::cout << "Initializing core..." << std::endl;

    System::register_classes( &register_classes );

    if( ServerCommon::options().create_initial_archive )
        {
        System::initialize
            (
            0
            | System::INIT_COUT_LOGGER_DESTINATION
            | System::INIT_STANDALONE_NODE
            );

        UInt32 provider_id = ServerCommon::options().initial_provider_id;

        std::cout << "Creating initial provider (" << provider_id << ")..." << std::endl;
        System::create_initial_object_provider( provider_id );

        std::cout << "Creating initial archive..." << std::endl;
        System::create_initial_archive();

        std::cout << "Initial archive successfully created. Exiting..." << std::endl;
        return;
        }
    else
        {
        System::initialize
            (
            0
            | System::INIT_COUT_LOGGER_DESTINATION
            );
        }

    std::cout << "Connecting..." << std::endl;

    System::connect();

    while( System::get_phase() != System::PHASE_CONNECTED )
        {
        System::next_tick();
        }

    // Instantiate factories.

    std::cout << "Instantiating factories..." << std::endl;

    register_classes();

    // Create objects.

    std::cout << "Creating objects..." << std::endl;

    WeakPointer<Root> root;
    hope( root.is_null() );

    { // sob
    Pointer<Null> null;
    hope( null.is_null() );

    null = CreateObject<Null>::create();
    hope( ! null.is_null() );

    root = CreateObject<Root>::create();
    hope( ! root.is_null() );

    // Run for a while. StackStrong null should keep the Null alive.

    std::cout << "Running with Null referenced from the stack..." << std::endl;
    const SystemTime old_time = System::system_time();
    while( System::system_time() < ( old_time + 2.0 ) )
        {
        System::next_tick();
        }

    // Reference Null from Root.

    root->null = null;

    // Forget stack reference.

    std::cout << "Forgetting stack reference to the Null..." << std::endl;
    } // eob

    // Run for a while. Root should keep the Null alive.

    {
    std::cout << "Running with Null referenced from the Root..." << std::endl;
    const SystemTime old_time = System::system_time();
    while( System::system_time() < ( old_time + 2.0 ) )
        {
        System::next_tick();
        }
    } // eob

    // Forget the reference.

    std::cout << "Forgetting reference from the Root to the Null..." << std::endl;
    root->null.set_null();

    // Run for a while. Null should die.

    {
    std::cout << "Running with Null unreferenced..." << std::endl;
    const SystemTime old_time = System::system_time();
    while( System::system_time() < ( old_time + 5.0 ) )
        {
        System::next_tick();
        }
    } // eob

    // Done.
    System::shutdown();
    }


} // namespace Massiv::ServerTest::Startup
} // namespace Massiv::ServerTest
} // namespace Massiv
