/** @file pwd_encrypt.cpp
 * Command line utility for encrypting files.
 */

/*
 * begin        : Mon Aug 10 2002
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "global/common.h"
#include "data/account_manager.h"
#include "system/system.h"

#include <iostream>
#include <sstream>
#include <fstream>

using namespace Massiv::Core;

int main
    (
    int        argc,
    char **    argv
    )
    {
    try
        {
        if( argc < 2 || argc > 3 )
            {
            std::cout << "Invalid argument count( " << argc - 1 << " )" << std::endl;
            std::cout << "Usage: " << argv[ 0 ] << " filename [ password ]" << std::endl;
            std::cout << "Encrypts file with a given password." << std::endl;
            std::cout << "Encrypted file is saved as filename.crypted" << std::endl;
            return( 1 );
            }
        std::string file         = argv[ 1 ];
        std::string password;
        
        if( argc < 3 )
            password = "";
        else
            password = argv[ 2 ];
            
        std::string crypted( "" );
        
        std::ifstream ifile( file.c_str(), std::ios_base::binary );
        if( !ifile.good() ) 
            {
            std::cout << "Bad source filename!";
            return 2;
            }
        
        std::ostringstream  oss;
        oss << ifile.rdbuf();
        
        
        std::string new_file = file + std::string( ".crypted" );
        
        // Initialize massiv core.
        Massiv::System::StartUpInfo info;
        info.base_services = true;
        info.file_systems = true;
        Massiv::System::initialize( Massiv::System::StartUpInfo( info ) );

        AccountManager::encrypt_file( new_file, password, oss.str() );
        
        // Shutdown massiv core.
        Massiv::System::shutdown();

        ifile.close();
        
        return( 0 );
        }
    catch( std::exception & e )
        {
        std::cout << "Error: " << e.what() << std::endl; 
        return( 1 ); 
        }
    }
